package com.incubator.common.game;

/**
 * 游戏服务
 * 
 * @author johnc
 *
 */
public interface IService {

	/**
	 * 启动服务
	 */
	public void startService();

	/**
	 * 关闭服务
	 */
	public void closeService();

	/**
	 * 获取服务状态
	 * 
	 * @return
	 */
	public ServiceState getState();

	/**
	 * 链接是否存在
	 * @param key
	 * @return
	 */
	public boolean hasAttribute(String key);

	/**
	 * 增加一个链接
	 * @param key
	 * @param value
	 */
	public void setAttribute(String key, Object value);

	/**
	 * 获取一个链接
	 * @param key
	 * @return
	 */
	public Object getAttribute(String key);

	/**
	 * 移除一个链接
	 * @param key
	 * @return
	 */
	public Object removeAttribute(String key);

	/**
	 * 清空链接
	 */
	public void clearAttributes();

	/**
	 * 服务器状态
	 * @author johnc
	 *
	 */
	public static enum ServiceState {

		/** 停止 **/
		STOPED(1),
		/** 启动中 **/
		STARTING(2),
		/** 进行中 **/
		ACTIVE(3),
		/** 结束中 */
		STOPING(4);

		private final int value;

		private ServiceState(int value) {
			this.value = value;
		}

		public int getValue() {
			return value;
		}

		public static ServiceState valueOf(int value) {
			ServiceState state = STOPED;
			for (ServiceState rs : values()) {
				if (rs.value == value) {
					state = rs;
				}
			}
			return state;
		}
	}

}
