package com.incubator.common.net;

import java.net.InetSocketAddress;
import java.util.Set;

/**
 * 网络连接接口
 */
public interface Connection {
	
	/**
	 * 获取玩家ID
	 * @return
	 */
	String getUid();

	/**
	 * 设置玩家ID
	 * @param uid
	 */
	void setUid(String uid);

	/**
	 * 获取角色id
	 * @return
	 */
	String getPlayerId();

	/**
	 * 设置角色id
	 * @param playerId
	 */
	void setPlayerId(String playerId);

	/**
	 * 获取链连的hashcode
	 * @return
	 */
	int getHashCode();

	/**
	 * 是否活跃打开可写
	 * 
	 * @return
	 */
	public boolean isActive();

	/**
	 * 是否有属性
	 * 
	 * @param key
	 * @return
	 */
	public boolean hasAttribute(String key);

	/**
	 * 设置属性
	 * 
	 * @param key
	 * @param value
	 */
	public void setAttribute(String key, Object value);

	/**
	 * 获取属性key集合
	 * 
	 * @return
	 */
	public Set<String> getAttributeNames();

	/**
	 * 获取属性
	 * 
	 * @param key
	 * @return
	 */
	public Object getAttribute(String key);

	/**
	 * 移除属性
	 * 
	 * @param key
	 * @return
	 */
	public Object removeAttribute(String key);

	/**
	 * 清空属性
	 */
	public void clearAttributes();

	/**
	 * 获取附件
	 * 
	 * @return
	 */
	public <T> T getAttachment();

	/**
	 * 设置附件
	 * 
	 * @param attachment
	 */
	public <T> void setAttachment(T attachment);

	/**
	 * 获取连接远程地址
	 * 
	 * @return
	 */
	public InetSocketAddress getRemoteAddress();

	/**
	 * 获取连接本地地址
	 * 
	 * @return
	 */
	public InetSocketAddress getLocalAddress();

	/**
	 * 关闭链接
	 */
	public void close();

	/**
	 * 发送信息
	 * 
	 * @param obj
	 */
	public void write(Object obj);

	/**
	 * 发送 bytes 数据
	 * 
	 * @param bytes
	 */
	public void write(byte[] bytes);

	/**
	 * 发送 obj 数据
	 * 
	 * @param obj
	 */
	public void writeAndFlush(Object obj);

	/**
	 * 发送 bytes 数据
	 * 
	 * @param bytes
	 */
	public void writeAndFlush(byte[] bytes);

	/**
	 * @param bytes
	 */
	public void flush();

}
