package com.incubator.common.net;

/**
 * 带心跳操作的监听器
 * @author johnc
 *
 * @param <T>
 */
public interface ConnectionHeartbeatListener<T> extends ConnectionListener<T> {

	/**
	 * 读超时时间
	 * (保证5-10秒内有消息来回) 没有任何请求或者3次心跳请求没有回复则关闭连接(2次会有临界点,如果回复比较快)
	 * @return
	 */
	public long getReaderTimeMills();

	/**
	 * 写超时时间
	 * 主动心跳请求发送
	 * @return
	 */
	public long getWriterTimeMills();

	/**
	 * 读写超时时间
	 * 3次心跳时间没有操作则关闭连接
	 * @return
	 */
	public long getAllTimeMills();

	/**
	 * 读写空闲超时事件
	 * 如果心跳超时则关闭连接
	 * @param connection
	 */
	public void event_AllTimeOut(Connection connection);

	/**
	 * 读空闲超时
	 * 
	 * @param connection
	 */
	public void event_ReadTimeOut(Connection connection);

	/**
	 * 写空闲超时
	 * 
	 * @param connection
	 */
	public void event_WriteTimeOut(Connection connection);

}
