package com.incubator.common.net.netty;

import com.incubator.common.net.Connection;
import com.incubator.common.net.ConnectionFactory;
import io.netty.channel.Channel;

/**
 * 连接实例工厂
 */
public class NettyConnectionFactory implements ConnectionFactory {

	// 使用静态内部类实现单例模式，线程安全且延迟加载
	private static class Holder {
		private static final NettyConnectionFactory INSTANCE = new NettyConnectionFactory();
	}

	/**
	 * 获取工厂实例（线程安全）
	 *
	 * @return 单例的 NettyConnectionFactory 实例
	 */
	public static NettyConnectionFactory getInstance() {
		return Holder.INSTANCE;
	}

	// 私有构造函数，防止外部实例化
	private NettyConnectionFactory() {
	}

	@Override
	public Connection buildConnection() {
		// 提供默认构造方法返回一个 NettyConnection（带有空 Channel）
		return new NettyConnection(null);
	}

	@Override
	public Connection buildConnection(Object obj) {
		// 检查输入对象类型是否为 Channel，避免不必要的实例化
		if (obj instanceof Channel) {
			return new NettyConnection((Channel) obj);
		}
		throw new IllegalArgumentException("Invalid object type. Expected io.netty.channel.Channel.");
	}
}
