package com.incubator.common.net.netty.handler.websocket.binary;

import com.incubator.common.net.netty.handler.websocket.WebSocketServerInitializer;
import com.incubator.common.net.netty.handler.websocket.binary.codec.WebSocketBinaryFrameByteBufAdapter;
import io.netty.channel.ChannelHandler;
import io.netty.channel.ChannelHandlerContext;

/**
 * WebSocket 二进制帧处理服务端适配器
 */
public class WebSocketServerBinaryAdapterHandler extends WebSocketServerInitializer {

	private final ChannelHandler handler;

	public WebSocketServerBinaryAdapterHandler(String uri, ChannelHandler handler) {
		super(uri);
		this.handler = handler;
	}

	@Override
	protected void webSocketHandComplete(ChannelHandlerContext ctx) {
		// 优化管道操作的顺序，避免不必要的事件触发
		ctx.pipeline()
				.addLast(new WebSocketBinaryFrameByteBufAdapter()) // 添加适配器
				.addLast(handler); // 添加自定义业务处理器

		// 触发必要的事件，确保管道的完整性
		triggerChannelEvents(ctx);
	}

	/**
	 * 手动触发通道事件，确保新加入的处理器可以正常处理事件
	 *
	 * @param ctx 通道上下文
	 */
	private void triggerChannelEvents(ChannelHandlerContext ctx) {
		try {
			ctx.fireChannelRegistered(); // 通道注册事件
			ctx.fireChannelActive();     // 通道激活事件
		} catch (Exception e) {
			// 增加异常捕获，避免事件传播导致未处理的异常中断高并发流程
			e.printStackTrace();
		}
	}
}
