
package com.incubator.common.util;

import java.util.Random;

/**
 * 随机数工具类
 *
 * @author agui
 */
public final class RandomUtil {

	/**
	 * 随机数生成器
	 */
	private static final Random __RANDOM__ = new Random();

	/**
	 * 获取列表的随机索引或数组的随机下标 : [0, size)
	 */
	public static int getIndex(int size) {
		if (size <= 1) {
			return 0;
		}
		return __RANDOM__.nextInt(size);
	}

	/**
	 * 获取从1开始到范围上限内的随机整数 : [1, N]
	 */
	public static int getInt(int limit) {
		if (limit <= 1) {
			return 1;
		}
		return __RANDOM__.nextInt(limit) + 1;
	}

	/**
	 * 获取范围内的随机整数[1, N]
	 */
	public static int getInt(int lower, int limit) {
		if (limit <= lower) {
			return lower;
		}
		return __RANDOM__.nextInt(limit - lower + 1) + lower;
	}

	/**
	 * 获取范围内的随机整数[1, N)
	 */
	public static int getIntExt(int lower, int limit) {
		if (limit <= lower) {
			return lower;
		}
		return __RANDOM__.nextInt(limit - lower) + lower;
	}

	/**
	 * 获取随机小数[N)
	 */
	public static float getFloat() {
		return __RANDOM__.nextFloat();
	}


	/**
	 * 获取随机数[0 - max)
	 */
	public static int random(int max) {
		return __RANDOM__.nextInt(max);
	}


	/**
	 * 获取随机数(0.0 - 0.9)
	 *
	 * @return
	 */
	public static double randomDouble() {
		return __RANDOM__.nextDouble();
	}


	public static int randomValue(int range) {
		if (range <= 0) {
			return 0;
		}
		return __RANDOM__.nextInt(range);
	}

	public static boolean hasHitRate(int range, int rate) {
		int cos = randomValue(range);
		if (cos < rate) {
			return true;
		}
		return false;
	}

	/**
	 * 生成一个6位数 100000 到 999999 之间的随机数
	 *
	 * @return
	 */
	public static long randomSixDigits() {
		return 100000L + RandomUtil.random(900000);
	}

	/**
	 * 生成一个7位数 1000000 到 9999999 之间的随机数
	 *
	 * @return
	 */
	public static long randomSevenDigits() {
		return 1000000L + RandomUtil.random(9000000);
	}

	/**
	 * 百分比
	 *
	 * @param per
	 * @return
	 */
	public static boolean randomPercent(int per) {
		int ran = RandomUtil.getInt(1, 100);
		return (ran >= 1 && ran <= per);
	}

	/**
	 * 万分比
	 *
	 * @param per
	 * @return
	 */
	public static boolean randomTenThousand(int per) {
		int ran = RandomUtil.getInt(1, 10000);
		return (ran >= 1 && ran <= per);
	}

	/**
	 * 生成格式化的用户名，格式为 user_123456
	 *
	 * @return 格式化的用户名
	 */
	public static String generateUsername() {
		int randomNumber = 100000 + getIndex(900000); // 生成6位随机数
		return "user_" + randomNumber;
	}
}
