package com.incubator.core.exception;

/**
 * 业务异常基类，所有业务异常都必须继承于此异常 .
 * 
 * @author johnc
 * 
 */
public class MessageException extends RuntimeException {

	/**
	 * 
	 */
	private static final long serialVersionUID = -2560623316827874232L;

	/**
	 * 异常信息
	 */
	protected String msg;

	/**
	 * 具体异常码
	 */
	protected int code;

	/**
	 * 未知异常
	 */
	public static final int UNKNOWN_ERROR = 10000;
	/**
	 * 唯一键
	 */
	public static final int KEY_ERROR = 10001;
	/**
	 * 关联约束
	 */
	public static final int DATAINTEGRITYVIOLATION = 10002;
	/**
	 * 为超级管理员，不能删除
	 */
	public static final int USERNAMEADMIN_ERROR = 10003;

	/**
	 * 原密码错误请重新输入。
	 */
	public static final int USERNAMEPWD_ERROR = 10004;

	/**
	 * Token 验证不通过
	 */
	public static final int TOKEN_IS_ILLICIT = 10005;
	
	/**
	 * 锁错误
	 */
	public static final int LOCK_ERROR = 10006;

	public MessageException() {
		super();
	}

	public MessageException(int code, String msgFormat, Object... args) {
		super(String.format(msgFormat, args));
		this.code = code;
		this.msg = String.format(msgFormat, args);
	}

	public MessageException(String message, Throwable cause) {
		super(message, cause);
	}

	public MessageException(Throwable cause) {
		super(cause);
	}

	public MessageException(String message) {
		super(message);
	}

	public MessageException newInstance(String msgFormat, Object... args) {
		return new MessageException(this.code, msgFormat, args);
	}

	public String getMsg() {
		return msg;
	}

	public int getCode() {
		return code;
	}

}
