package com.incubator.core.net.handler;

import com.incubator.common.net.ConnectionListener;
import com.incubator.common.net.netty.handler.http.HttpServerInitHandler;
import io.netty.handler.codec.http.HttpRequest;
import io.netty.handler.ssl.SslContext;

/**
 * HTTP 服务端初始化处理器
 *
 * 提供支持 SSL 和非 SSL 模式的 HTTP 服务初始化器。
 *
 * @author johnc
 */
public class InitalizerHttpServerHandler extends HttpServerInitHandler {

	/**
	 * 构造函数（支持 SSL）
	 *
	 * @param listener 连接监听器
	 * @param sslCtx   SSL 上下文
	 */
	public InitalizerHttpServerHandler(ConnectionListener<HttpRequest> listener, SslContext sslCtx) {
		super(listener, sslCtx);
	}

	/**
	 * 构造函数（不使用 SSL）
	 *
	 * @param listener 连接监听器
	 */
	public InitalizerHttpServerHandler(ConnectionListener<HttpRequest> listener) {
		super(listener);
	}

	/**
	 * 可在子类中进一步扩展逻辑，如添加特定的 HTTP 处理逻辑。
	 */
	@Override
	protected void initChannel(io.netty.channel.socket.SocketChannel ch) throws Exception {
		super.initChannel(ch); // 调用父类初始化管道
		// 可在此处添加额外的自定义逻辑
		// 如添加业务处理器:
		// ch.pipeline().addLast(new CustomHttpHandler());
	}
}
