package com.incubator.core.net.handler;

import com.incubator.common.net.ConnectionHeartbeatListener;
import com.incubator.common.net.netty.listener.adapter.ListenerHandlerHeartbeatAdapter;
import io.netty.channel.ChannelInitializer;
import io.netty.channel.socket.SocketChannel;

/**
 * 协议处理器初始化类
 *
 * 用于初始化心跳监听器，支持高并发环境下的高效处理。
 */
public class InitalizerProtocolHandler extends ChannelInitializer<SocketChannel> {

	private final ConnectionHeartbeatListener<Object> listener;

	/**
	 * 构造函数
	 *
	 * @param listener 心跳连接监听器
	 */
	public InitalizerProtocolHandler(ConnectionHeartbeatListener<Object> listener) {
		if (listener == null) {
			throw new IllegalArgumentException("listener cannot be null");
		}
		this.listener = listener;
	}

	@Override
	protected void initChannel(SocketChannel ch) {
		// 添加心跳监听适配器到管道
		ch.pipeline().addLast(new ListenerHandlerHeartbeatAdapter<>(listener));
	}

	/**
	 * 获取心跳监听器（可扩展以支持动态调整逻辑）
	 *
	 * @return 心跳连接监听器
	 */
	public ConnectionHeartbeatListener<Object> getListener() {
		return listener;
	}
}
