package com.incubator.core.util;

import com.alibaba.fastjson2.JSONArray;
import com.incubator.common.data.GameJsonFilter;
import com.incubator.common.log4j.Log;

import java.io.File;
import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.util.HashMap;
import java.util.Map;
import java.util.Objects;

/**
 * 配置脚本统一实现
 */
public final class DataUtil {

	public static Map<String, String> Classes = new HashMap<>();
	/** 配置需要继承父类的类 */
	public static Map<String, String> superClasses = new HashMap<>();
	
	public static String extendClass = "Ext";
	public static String pakName = "Ext";
	
	// 加载扩展类
	public static void addExtClass(String pakName) {
		DataUtil.pakName = pakName;
		File file = new File(DIR + File.separator + "ext" + File.separator);
		if (file.exists() && file.isDirectory()) {
			for (String f : Objects.requireNonNull(file.list())) {
				String name = f.substring(0, f.indexOf(".java"));
				Classes.put(name, pakName + ".ext." + name);
			}
		}
	}

	public static String getClassName(String name) {
		if (name.contains(File.separator)) {
			name = name.substring(name.lastIndexOf(File.separator) + 1);
		}
		StringBuilder builder = new StringBuilder();
		builder.append(Character.toUpperCase(name.charAt(0)));
		for (int i = 1; i < name.length(); ++i) {
			if (name.charAt(i) == '.') break;
			if (name.charAt(i) == '-') {
				builder.append("_");
				continue; 
			}
			if (name.charAt(i) == '_') {
				if (name.charAt(i + 1) != '.') {
					builder.append(Character.toUpperCase(name.charAt(i + 1)));
					++i;
				}
			} else {
				builder.append(name.charAt(i));
			}
		}
		return builder.toString();
	}

	public static String getFieldName(String name) {
		if (name.length() < 1) {
			return null;
		}
		name = name.trim();
		StringBuilder builder = new StringBuilder();
		builder.append(Character.toLowerCase(name.charAt(0)));
		for (int i = 1; i < name.length(); ++i) {
			if (name.charAt(i) == ' ') continue; 
			if (name.charAt(i) == '-') {
				builder.append("_");
				continue; 
			}
			if (name.charAt(i) == '_') {
				if (name.length() > i + 1) {
					builder.append("_").append(Character.toLowerCase(name.charAt(i + 1)));
					++i;
				}
			} else {
				builder.append(name.charAt(i));
			}
		}
		String field = builder.toString();
		switch (field) {
			case "long":
				field = "long_v";
				break;
			case "goto":
				field = "goto_v";
				break;
			case "default":
				field = "default_v";
				break;
			case "new":
				field = "new_v";
				break;
			case "try":
				field = "try_v";
				break;
			case "final":
				field = "final_v";
				break;
		}
		return field;
	}

	private static String DIR = "/";

	public static void setDir(String path) {
		File toDir = new File(path);
		toDir.mkdirs();
		DIR = toDir.getAbsolutePath() + File.separator;
	}

	private static String type(Object type) {
		if ("NUMBER".equals(type)) {
			return "int";
		} else if ("STRING".equals(type)) {
			return ("String");
		} else if ("FLOAT".equals(type)) {
			return ("float");
		} else {
			Log.error(" undefine type : {}", type);
		}
		return "Object";
	}
	
	private static final Map<String, String> DataNames = new HashMap<>();
	private static StringBuilder BaseBuilder;
	public static void writeClass(File file) {
		try {
			String baseName = getClassName(file.getName());
			if (GameJsonFilter.isFilter(baseName, file.getPath())) {
				return;
			}
			if (BaseBuilder == null) {
				BaseBuilder = new StringBuilder();
				BaseBuilder.append("package ").append(pakName).append("; \r\n\r\n");
				for (String fullName : Classes.values()) {
					BaseBuilder.append("import ").append(fullName).append(";\r\n");
				}
				BaseBuilder.append("import java.util.Map; \r\n");
				BaseBuilder.append("import java.util.HashMap; \r\n\r\n");
				BaseBuilder.append("import java.util.function.Predicate; \r\n\n");
				BaseBuilder.append("import java.util.List; \r\n\r\n");
				BaseBuilder.append("import java.util.ArrayList; \r\n\r\n");
				BaseBuilder.append("/** 游戏-策划配置 **/\n");
				BaseBuilder.append("public final class GameData { \r\n\r\n");
			}
			StringBuilder builder = new StringBuilder();
			builder.append("package ").append(pakName).append("; \r\n\r\n");
			
			String clsName = baseName + "CO"; 
			if (DataNames.containsKey(clsName)) {
				Log.error(baseName, " 表名冲突：\n", file.getAbsolutePath(), "\n", DataNames.get(clsName));
				return;
			}
			builder.append("/** 游戏-策划配置 **/\r\n\r\n");
			
			DataNames.put(clsName, file.getAbsolutePath());
			builder.append("public class ").append(clsName);
			if (superClasses.containsKey(clsName)) {
				builder.append(" extends ").append(superClasses.get(clsName));
			}
			builder.append(" { \r\n\r\n");

			String w = FileUtil.readText(file);
			JSONArray doc = JSONArray.parseArray(w);
			if (doc.size() < 3) {
				Log.error("空数据: {}", file.getAbsolutePath());
				return;
			}
			JSONArray descs = (JSONArray) doc.get(0);
			JSONArray keys = (JSONArray) doc.get(1);
			JSONArray types = (JSONArray) doc.get(2);
			String k_type = type(types.get(0));			
			
			if (!superClasses.containsKey(clsName)) {
				for (int i = 0; i < keys.size(); i++) {
					String field = getFieldName(keys.get(i).toString());
					if (field == null) {
						Log.warn("空列: {}", file.getAbsolutePath());
						continue;
					}
					builder.append("\t/** ").append(descs.get(i)).append(" */\r\n");
					builder.append("\tpublic ").append(type(types.get(i)));
					builder.append(" ").append(field).append(";\r\n");
				}
				builder.append(String.format("\r\n\t/** 主键 */\r\n\tpublic %s getKey() {\r\n\t\treturn this.%s; \r\n\t}\r\n", k_type,
						getFieldName(keys.get(0).toString())));

				builder.append("\r\n\t/** 构造属性 */\r\n\tpublic void initProperty() { }\r\n");
			}
			
			builder.append("\r\n\t/** 构造前置属性 */\r\n\tpublic void beforeProperty() { }\r\n");

			builder.append("\r\n}");

			String keyName = clsName;
			String classNameExt = baseName + extendClass;
			if (Classes.containsKey(classNameExt)) {
				keyName = classNameExt;
			}
			
			if (k_type.equals("int")) {
				k_type = "Integer";
			}
			BaseBuilder.append(String.format("\tpublic static Map<%s, %s> %ss = new HashMap<%s, %s>();\n",
					k_type, keyName, baseName, k_type, keyName));
			
			
			BaseBuilder.append("\t");
			BaseBuilder.append("public static List<").append(keyName).append("> find").append(baseName).append("s(Predicate<").append(keyName).append("> pre) {\n");
			BaseBuilder.append("\t\tList<").append(keyName).append("> results = new ArrayList<>();\n");
			BaseBuilder.append("\t\tfor(").append(keyName).append(" t : ").append(baseName).append("s.values()) {\n");
			BaseBuilder.append("\t\t\tif(pre.test(t)) {\n");
			BaseBuilder.append("\t\t\t\tresults.add(t);\n");
			BaseBuilder.append("\t\t\t}\n\t\t}\n");
			BaseBuilder.append("\t\treturn results;\n");
			BaseBuilder.append("\t}\r\n\n");

			BaseBuilder.append("");
			clsName += ".java";
			//Out.info("生成：", clsName);
			FileUtil.write(new File(DIR + clsName), builder.toString());
		} catch (Exception e) {
			Log.error("DataUtil writeclass catch - ", e);
			Log.error(file.getAbsolutePath());
		}
	}

	public static void writeGameData() {
		if (BaseBuilder != null) {
			BaseBuilder.append("\n}");
			FileUtil.write(new File(DIR + "GameData.java"), BaseBuilder.toString());
		}
	}

	@SuppressWarnings("rawtypes")
	private static final Map<String, Map> Datas = new HashMap<>();
	
	@SuppressWarnings({ "rawtypes", "unchecked" })
	public static Map loadData(File file, boolean reload) {
		Map loadData = null;
		String fieldName = null;
		String name = "";
		int row = 0;
		int col = 0;
		try {
			name = getClassName(file.getName());
			if (GameJsonFilter.isFilter(name, file.getPath())) {
				return null;
			}
			
			String className = pakName + "." + name + "CO";
			String classNameExt = name + extendClass;
			if (Classes.containsKey(classNameExt)) {
				className = Classes.get(classNameExt);
			}
			String dataName = name + "s";
			JSONArray doc = JSONArray.parseArray(FileUtil.readText(file));
			if (doc.size() < 3) {//前面3行数据是表头
				Log.debug(file.getAbsolutePath() , " json doc row < 3");
				return null;
			}

			loadData = (Map) ClassUtil.getStaticProperty(Class.forName(pakName + ".GameData"), dataName);
			if (reload || !Datas.containsKey(dataName)) {
				Datas.put(dataName, loadData);
			} else {
				Log.error(dataName, " 数据表名冲突 -> ", file.getAbsolutePath());
				return null;
			}
			JSONArray keys = (JSONArray) doc.get(1);
			Class clz = null;
			// 存在子类,则 存放子类到GameData 中
			try {
				clz = Class.forName(pakName + ".ext." + classNameExt);
			} catch (Exception e) {
				clz = Class.forName(className);
			}
			Object owner = null;
			Field field = null;
			Map<String, Field> fields = new HashMap<>();
			for (int i = 3; i < doc.size(); i++) {
				row = i;
				owner = clz.newInstance();
				JSONArray values = (JSONArray) doc.get(i);
				for (int k = 0; k < keys.size(); k++) {
					col = k;
					fieldName = getFieldName(keys.get(k).toString());
					if (fieldName == null) {
						//Out.debug(file.getAbsolutePath() , " " , keys.get(k) , " 没有这个字段名");
						continue;
					}
					Object o = values.get(k);
					field = fields.get(fieldName);
					if (field == null) {
						field = ClassUtil.getDeclaredField(owner, fieldName);
						fields.put(fieldName, field);
					}
					if (field == null) {
						//Out.warn(className, " not exists ", fieldName);
						continue;
					}
					String typeName = field.getType().getSimpleName();
					if ("int".equals(typeName)) {
						field.set(owner, ((Number) o).intValue());
					} else if ("float".equals(typeName)) {
						field.set(owner, ((Number) o).floatValue());
					} else {
						field.set(owner, o);
					}
				}
				Object key = ClassUtil.invokeMethod(owner, "getKey");
				if (loadData.containsKey(key) && !name.equals("Suffix")) {
					Log.warn("表：" + file + ", 重复键值：" + key);
				}	

				loadData.put(key, owner);
			}
			fields.clear();
		} catch (Exception e) {
			Log.error(file.getAbsolutePath() , " : " , fieldName , "&row=",row," col=" , col, ", catch - ", e);
		}
		return loadData;
	}

	@SuppressWarnings("rawtypes")
	private static void initProperty(Map loadData, String methodName) {
		try {
			Method method = null;
			for (Object o : loadData.values()) {
				if (method == null) {
					Class<?> ownerClass = o.getClass();
					method = ownerClass.getMethod(methodName);
				}
				method.invoke(o);
			}
		} catch (Exception e) {
			Log.error("DataUtil initProperty", e);
		}
	}
	
	@SuppressWarnings("rawtypes")
	public static void initProperty() {		
		for (Map.Entry<String, Map> entry : Datas.entrySet()) {
			initProperty(entry.getValue(), "beforeProperty");
		}
		
		for (Map.Entry<String, Map> entry : Datas.entrySet()) {
			initProperty(entry.getValue(), "initProperty");
		}
		Datas.clear();
	}

}
