package com.incubator.game.contest;

import cn.hutool.json.JSONArray;
import cn.hutool.json.JSONException;
import cn.hutool.json.JSONObject;
import com.incubator.common.log4j.Log;
import com.incubator.game.constant.RedisKeyConstant;
import com.incubator.game.data.jedis.RedisUtil;
import com.incubator.game.util.ContestUtil;
import org.apache.commons.lang3.StringUtils;

import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;

/**
 * Author:  xhb
 * Date:  2025-01-14 18:44
 * 赛事调度器，用于处理赛事的并发执行和资源分配等任务。
 */
public class ContestThreadedScheduler {

    // 创建一个具有固定线程池
    public static final ScheduledExecutorService scheduler = Executors.newScheduledThreadPool(4); // 线程池大小可根据需要调整

    public static void startScheduler() {
        Runnable task = () -> {
            Log.info("开始扫描赛事...");
            //执行任务
            try {
                contestStart();
            } catch (Exception e) {
                e.printStackTrace();
                Log.error("赛事定时器执行错误:{} " + e.getMessage());
            }
            Log.info("赛事创建完成...");
        };

        // 10秒扫描一次赛事状态
        scheduler.scheduleAtFixedRate(task, 0, 10, TimeUnit.SECONDS);
    }

    public static void contestStart() {
        // 赛事逻辑
        String competeListStr = RedisUtil.get(RedisKeyConstant.COMPETE_LIST);
        if (StringUtils.isEmpty(competeListStr)) {
            Log.info("赛事信息为空，跳过后续执行...");
            return;
        }
        JSONArray jsonArray = null;
        try {
            jsonArray = new JSONArray(competeListStr);
        } catch (JSONException e) {
            e.printStackTrace();
            Log.error("JSON信息转换异常: {}", e.getMessage(), e);
            return;
        }
        for (Object o : jsonArray) {
            JSONObject jsonObject = (JSONObject) o;
            ContestUtil.createJsonContest(jsonObject);
        }
    }

    public void stopScheduler() {
        // 关闭任务
        scheduler.shutdown();
        try {
            // 等待所有任务完成，最多等待1小时
            if (!scheduler.awaitTermination(1, TimeUnit.HOURS)) {
                // 超时后强制关闭
                scheduler.shutdownNow();
            }
        } catch (InterruptedException e) {
            scheduler.shutdownNow();
            Thread.currentThread().interrupt(); // 恢复中断状态
        }
    }
}
