package com.incubator.game.data.jdbc;

import com.incubator.common.log4j.Log;
import com.incubator.core.db.handler.DBHandler;
import com.incubator.game.data.po.AccountInfoPO;
import com.incubator.game.data.po.GameLogPO;
import com.incubator.game.data.po.GlobalInfoPO;
import com.incubator.game.data.po.PlayerInfoPO;

import java.sql.Timestamp;
import java.util.Date;
import java.util.List;

public class LSDBHandler extends DBHandler {

	/** 记录用户注册 */
	public int userRegister(long uid, String playerId, int platformId, int channelId, int subChannelId, int gameId,
							String phoneNumber, String emailAddress, String walletAddress, String walletChain,
							String shareCode, int lastGameId, String lastLoginIp, Date lastLoginTime, String token) {

		String sql = "call user_register(?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)";
		int nRes = excuteProcessForInt(sql, uid, playerId, platformId, channelId, subChannelId, gameId, phoneNumber, emailAddress, walletAddress, walletChain, shareCode, lastGameId, lastLoginIp, lastLoginTime.toString(), token);

		Log.info("user_register : ", playerId, ", ", platformId, ", ", channelId, ", ", subChannelId, ", ", gameId, ", ", phoneNumber, ", res = ", nRes);
		return nRes;
	}

	/**
	 * 获取账号信息
	 **/
	public AccountInfoPO getAccountByUid(long uid) {
		String sql = "SELECT * FROM `account_info` WHERE uid = ?";
		return this.queryForObject(sql, new DBData.AccountMapper(), uid);
	}

	public PlayerInfoPO getPlayerByUid(String playerId) {
		String sql = "SELECT * FROM player_info WHERE player_id = ?";
		return this.queryForObject(sql, new DBData.PlayerMapper(), playerId);
	}

	public String saveGameLog(String playerId, int gameId, String content, Timestamp time) {
		String sql = "call save_gameLog(?, ?, ?, ?)";
		return this.excuteProcessForString(sql, playerId, gameId, content, time);
	}

	public List<GameLogPO> getGameLogListByPlayerId(String playerId) {
		String sql = "select * from game_log where player_id = ?";
		return this.query(sql, new DBData.GameLogMapper(), playerId);
	}

	public int insertGlobal(GlobalInfoPO po) {
		String sql = "call save_global(?, ?)";
		return this.excuteProcessForInt(sql, po.server_id, po.allBlobData);
	}

	public GlobalInfoPO getGlobalInfoByServer(int serverId) {
		String sql = "select * from global_info where server_id = ?";
		return this.queryForObject(sql, new DBData.GlobalMapper(), serverId);
	}

	public int updateByGlobal(GlobalInfoPO po) {
		String sql = "UPDATE global_info SET allBlobData = ? where server_id = ?";
		return this.update(sql, po.allBlobData, po.server_id);
	}

	public int saveStatisticsLog(long day, String content) {
		String sql = "call save_statisticsLog(?, ?)";
		return this.excuteProcessForInt(sql, day, content);
	}
}