package com.incubator.game.data.mongo;

import com.incubator.common.log4j.Log;
import com.incubator.game.GGame;
import com.incubator.common.util.PropertiesUtil;
import com.mongodb.*;
import com.mongodb.client.FindIterable;
import com.mongodb.client.MongoCollection;
import com.mongodb.client.MongoDatabase;
import com.mongodb.client.MongoIterable;
import com.mongodb.client.model.Filters;
import com.mongodb.client.result.DeleteResult;
import org.bson.Document;
import org.bson.conversions.Bson;
import org.bson.types.ObjectId;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.function.Consumer;

public class MongoUtil {

	private MongoClient mongoClient = null;

	private static String HOST;

	private static Integer POST;

	private static String DBNAME;

	private static MongoUtil mongoUtil = null;

	private MongoUtil() {
        try {
			PropertiesUtil dbMongo = GGame.DB_SETTINGS;
			HOST = dbMongo.getProperty("mongodb.host");
			POST = Integer.valueOf(Objects.requireNonNull(dbMongo.getProperty("mongodb.port")));
			DBNAME = dbMongo.getProperty("mongodb.dbName");

			MongoClientOptions.Builder build = new MongoClientOptions.Builder();
			build.maxWaitTime(Integer.parseInt(Objects.requireNonNull(dbMongo.getProperty("mongodb.maxWaitTime"))));
			build.connectTimeout(Integer.parseInt(Objects.requireNonNull(dbMongo.getProperty("mongodb.connectTimeout"))));
			build.socketTimeout(Integer.parseInt(Objects.requireNonNull(dbMongo.getProperty("mongodb.socketTimeout"))));
			build.connectionsPerHost(Integer.parseInt(Objects.requireNonNull(dbMongo.getProperty("mongodb.connectionsNumbersPerHost"))));
			build.writeConcern(WriteConcern.ACKNOWLEDGED);
			MongoClientOptions myOptions = build.build();
			mongoClient = new MongoClient(new ServerAddress(HOST, POST), myOptions);
        } catch (MongoException e) {
            Log.error("初始化Mongo客户端失败...", e);
        }
    }

	public synchronized static MongoUtil getInstance() {
		if (mongoUtil == null) {
			mongoUtil = new MongoUtil();
		}
		return mongoUtil;
	}

	public boolean isExits(String collectionName, Map<String, Object> filterMap) {
		if (filterMap != null) {
			FindIterable<Document> docs = mongoClient.getDatabase(DBNAME).getCollection(collectionName).find(new Document(filterMap));
			Document doc = docs.first();
			return doc != null;
		}
		return false;
	}

	public boolean insert(String collectionName, Document doc) {
		if (doc != null) {
			mongoClient.getDatabase(DBNAME).getCollection(collectionName).insertOne(doc);
			return true;
		}
		return false;
	}
	
	public boolean insertMany(String collectionName, List<Document> doc) {
		if (doc != null && !doc.isEmpty()) {
			mongoClient.getDatabase(DBNAME).getCollection(collectionName).insertMany(doc);
			return true;
		}		
		return false;
	}

	public boolean deleteById(String collectionName, String _id) {
		ObjectId objectId = new ObjectId(_id);
		Bson filter = Filters.eq("_id", objectId);
		DeleteResult deleteResult = getDatabase().getCollection(collectionName).deleteOne(filter);
		long deletedCount = deleteResult.getDeletedCount();
		return deletedCount > 0;
	}

	public List<Document> find(String collectionName, Bson filter) {
		List<Document> resultList = new ArrayList<>();
		// 获取数据库和集合
		MongoCollection<Document> collection = mongoClient.getDatabase(DBNAME).getCollection(collectionName);
		// 如果过滤器为空，使用默认过滤器
		filter = (filter != null) ? filter : new Document();
		// 查询集合并收集结果
		FindIterable<Document> docs = collection.find(filter);
		docs.forEach((Consumer<Document>) resultList::add);

		return resultList;
	}

	public Document findById(String collectionName, String _id) {
		ObjectId objectId = new ObjectId(_id);
		return getDatabase().getCollection(collectionName).find(Filters.eq("_id", objectId)).first();
	}


	public MongoCollection<Document> getCollection(String collectionName) {
		return mongoClient.getDatabase(DBNAME).getCollection(collectionName);
	}

	public MongoDatabase getDatabase() {
		return mongoClient.getDatabase(DBNAME);
	}

	public long getCount(String collectionName) {
		return getDatabase().getCollection(collectionName).countDocuments();
	}

	/**
	 * 查询dbName下的所有表名
	 */
	public List<String> getAllCollections() {
		MongoIterable<String> cols = getDatabase().listCollectionNames();
		List<String> _list = new ArrayList<>();
		for (String s : cols) {
			_list.add(s);
		}
		return _list;
	}

	/**
	 * 获取所有数据库名称列表
	 *
	 * @return
	 */
	public MongoIterable<String> getAllDatabaseName() {
		return mongoClient.listDatabaseNames();
	}

	/**
	 * 删除一个数据库
	 */
	public void dropDatabase() {
		getDatabase().drop();
	}

	/**
	 * 删除collection
	 */
	public void dropCollection(String collectionName) {
		getDatabase().getCollection(collectionName).drop();
	}

	/**
	 * 关闭
	 */
	public void close() {
		if (mongoClient != null) {
			mongoClient.close();
			mongoClient = null;
		}
	}

}
