package com.incubator.game.data.mq;

import com.incubator.common.log4j.Log;
import com.incubator.common.util.PropertiesUtil;
import com.incubator.game.GGame;
import com.rabbitmq.client.Channel;
import com.rabbitmq.client.Connection;
import com.rabbitmq.client.ConnectionFactory;

import java.util.Objects;

public class RabbitMQUtil {

    private static String EXCHANGE_NAME;
    private static String ROUTING_KEY;

    private static Connection connection;
    private static Channel channel;

    static {
        try {
            // 初始化连接和通道
            initializeConnectionAndChannel();
        } catch (Exception e) {
            Log.error("RabbitMQUtil 初始化连接和通道出错...{}", e);
        }
    }

    private static void initializeConnectionAndChannel() throws Exception {
        PropertiesUtil dbRabbitmq = GGame.DB_SETTINGS;
        String HOST = dbRabbitmq.getProperty("rabbitmq.host", "127.0.0.1");
        int PORT = Integer.parseInt(Objects.requireNonNull(dbRabbitmq.getProperty("rabbitmq.port", "5672")));
        String username = dbRabbitmq.getProperty("rabbitmq.username", "guest1");
        String password = dbRabbitmq.getProperty("rabbitmq.password", "guest1");

        EXCHANGE_NAME = dbRabbitmq.getProperty("rabbitmq.exchangeName", "zj_exchange");
        String QUEUE_NAME = dbRabbitmq.getProperty("rabbitmq.queueName", "zj_queue");
        ROUTING_KEY = dbRabbitmq.getProperty("rabbitmq.routingKey", "zj_routing_key");

        ConnectionFactory factory = new ConnectionFactory();
        factory.setHost(HOST);
        factory.setPort(PORT);
        factory.setUsername(username);
        factory.setPassword(password);

        connection = factory.newConnection();
        channel = connection.createChannel();

        // 声明交换机
        channel.exchangeDeclare(EXCHANGE_NAME, "direct",true);

        // 声明队列
        channel.queueDeclare(QUEUE_NAME, true, false, false, null);

        // 绑定队列到交换机
        channel.queueBind(QUEUE_NAME, EXCHANGE_NAME, ROUTING_KEY);
    }

    public static void sendMessage(String message) throws Exception {
        // 发送消息
        channel.basicPublish(EXCHANGE_NAME, ROUTING_KEY, null, message.getBytes());
        Log.info("发送消息 =====> RabbitMQ : " + message);
    }

    public static void closeConnection() {
        try {
            if (channel != null && channel.isOpen()) {
                channel.close();
            }
            if (connection != null && connection.isOpen()) {
                connection.close();
            }
        } catch (Exception e) {
            Log.error("RabbitMQUtil - closeConnection() 出错...{}", e);
        }
    }
}