package com.incubator.game.handler.address;

import cn.hutool.json.JSONArray;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.incubator.common.MessageHandler;
import com.incubator.common.net.Connection;
import com.incubator.core.net.ws.NetHandler;
import com.incubator.core.net.ws.WSRequest;
import com.incubator.core.net.ws.WSResponse;
import com.incubator.game.constant.RedisKeyConstant;
import com.incubator.game.data.entity.ZjMemberAddr;
import com.incubator.game.data.jedis.RedisUtil;
import com.incubator.game.data.mq.RabbitMQUtil;
import com.incubator.game.data.po.ZjMemberAddrPO;
import com.incubator.game.util.ObjectMapperUtil;
import com.incubator.game.util.UtilCore;
import com.incubator.message.proto.CommonProto;

import java.util.*;
import java.util.stream.Collectors;


/**
 * Author:  xhb
 * Date:  2024-12-10 15:06
 */
@MessageHandler(id = CommonProto.Cmd.SaveAddressReq_VALUE)
public class SaveAddress extends NetHandler {

    @Override
    public void onDate(Connection session, WSRequest request, WSResponse response) {
        response.setCmd(CommonProto.Cmd.SaveAddressRes_VALUE);
        String receiver = request.getDataValue("receiver", String.class, null);
        String mobile = request.getDataValue("mobile", String.class, null);
        String addr = request.getDataValue("addr", String.class, null);
        String userId = session.getPlayerId();
        String status = request.getDataValue("status", String.class, null);
        String commonAddr = request.getDataValue("commonAddr", String.class, null);
        String addrId = request.getDataValue("addrId", String.class, null);

        ZjMemberAddrPO zjMemberAddrPO = new ZjMemberAddrPO();
        zjMemberAddrPO.setReceiver(receiver);
        zjMemberAddrPO.setMobile(mobile);
        zjMemberAddrPO.setAddr(addr);
        zjMemberAddrPO.setUserId(userId);
        zjMemberAddrPO.setStatus(Integer.parseInt(status));

        if (addrId != null) {
            zjMemberAddrPO.setAddrId(addrId);
        }

        // 获取用户信息并解析
        String userInfoJson = RedisUtil.get(RedisKeyConstant.USER_BASE + userId);
        if (userInfoJson == null) {
            throw new RuntimeException("User info not found in Redis");
        }
        JSONObject userObject = JSONUtil.parseObj(userInfoJson);
        JSONArray userAddressArray = userObject.getJSONArray("userAddress");


        // 更新地址列表
        List<ZjMemberAddrPO> zjMemberAddrPOList = userAddressArray.toList(ZjMemberAddrPO.class);
        if (commonAddr!=null){
            zjMemberAddrPO.setCommonAddr(Integer.parseInt(commonAddr));
            // 更新默认地址状态
            if (zjMemberAddrPO.getCommonAddr() == 1) {
                zjMemberAddrPOList.forEach(po -> po.setCommonAddr(po.getAddrId().equals(zjMemberAddrPO.getAddrId()) ? 1 : 0));
            }
        }
        ZjMemberAddr zjMemberAddr = new ZjMemberAddr();

        boolean isNewAddress = zjMemberAddrPO.getAddrId() == null;
        if (isNewAddress) {
            zjMemberAddrPO.setAddrId(UtilCore.getDateRandomId());
            zjMemberAddrPO.setCreateTime(new Date());
            zjMemberAddrPOList.add(zjMemberAddrPO);

            zjMemberAddr.setAddrId(zjMemberAddrPO.getAddrId());
            zjMemberAddr.setCreateTime(zjMemberAddrPO.getCreateTime());
            zjMemberAddr.setAddr(zjMemberAddrPO.getAddr());
            zjMemberAddr.setReceiver(zjMemberAddrPO.getReceiver());
            zjMemberAddr.setMobile(zjMemberAddrPO.getMobile());
            zjMemberAddr.setStatus(zjMemberAddrPO.getStatus());
            zjMemberAddr.setCommonAddr(zjMemberAddrPO.getCommonAddr());
            zjMemberAddr.setUserId(zjMemberAddrPO.getUserId());
        } else {
            Optional<ZjMemberAddrPO> addressToUpdate = zjMemberAddrPOList.stream()
                    .filter(p -> p.getAddrId().equals(zjMemberAddrPO.getAddrId()))
                    .findFirst();
            if (!addressToUpdate.isPresent()){
                return;
            }
            addressToUpdate.ifPresent(address -> {
                address.setUpdateTime(new Date());
                address.setStatus(zjMemberAddrPO.getStatus());
                address.setReceiver(zjMemberAddrPO.getReceiver());
                address.setMobile(zjMemberAddrPO.getMobile());
                address.setCommonAddr(zjMemberAddrPO.getCommonAddr());
                address.setAddr(zjMemberAddrPO.getAddr());
                zjMemberAddr.setUpdateTime(address.getUpdateTime());
                zjMemberAddr.setCommonAddr(address.getCommonAddr());
                zjMemberAddr.setAddrId(address.getAddrId());
                zjMemberAddr.setReceiver(address.getReceiver());
                zjMemberAddr.setMobile(address.getMobile());
                zjMemberAddr.setStatus(address.getStatus());
                zjMemberAddr.setAddr(address.getAddr());
                zjMemberAddr.setUserId(address.getUserId());
            });
        }
        JSONObject object = new JSONObject();
        object.put("type", "saveAddr");
        object.put("data", zjMemberAddr);
        try {
            RabbitMQUtil.sendMessage(object.toString());
        } catch (Exception e) {
            e.printStackTrace();
        }


        // 更新Redis中的用户信息
        userObject.put("userAddress", zjMemberAddrPOList);
        RedisUtil.set(RedisKeyConstant.USER_BASE + userId, userObject.toString());
        List<ZjMemberAddrPO> collect = zjMemberAddrPOList.stream()
                .filter(po -> po.getStatus() == 1)
                .collect(Collectors.toList());
        Map<String,Object> map = new HashMap<>();
        List<Map<String, Object>> maps = ObjectMapperUtil.convertListToMapList(collect);
        map.put("addressList",maps);
        response.setData(map);
    }
}
