package com.incubator.game.handler.backpack;

import cn.hutool.json.JSONArray;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.incubator.common.MessageHandler;
import com.incubator.common.net.Connection;
import com.incubator.core.net.ws.NetHandler;
import com.incubator.core.net.ws.WSRequest;
import com.incubator.core.net.ws.WSResponse;
import com.incubator.game.constant.RedisKeyConstant;
import com.incubator.game.data.jedis.RedisUtil;
import com.incubator.game.data.po.ZjBackpackPO;
import com.incubator.game.data.po.ZjOrderPO;
import com.incubator.game.util.UtilCore;
import com.incubator.message.proto.CommonProto;

import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;

/**
 * Author:  xhb
 * Date:  2024-12-11 9:11
 */
@MessageHandler(id = CommonProto.Cmd.BagUseReq_VALUE)
public class BagUse extends NetHandler {

    private final List<String> numGoods = new ArrayList<>();//数量道具

    private final List<String> disposableGoods = new ArrayList<>();//一次性道具

    private final List<String> repeatedlyGoods = new ArrayList<>();//重复性道具


    public BagUse() {
        initializeGoodsLists();
    }

    private void initializeGoodsLists() {
        numGoods.add("6"); // 红玫瑰
        numGoods.add("7"); // 蓝玫瑰
        numGoods.add("8"); // 鸡蛋
        numGoods.add("9"); // 鞭炮

        disposableGoods.add("14"); // 大米
        disposableGoods.add("15"); // 手机

        repeatedlyGoods.add("10"); // 天数记牌器
    }



    @Override
    public void onDate(Connection session, WSRequest request, WSResponse response) {
        response.setCmd(CommonProto.Cmd.BagUseRes_VALUE);

        String userId = session.getPlayerId();
        String backId = request.getDataValue("backId", String.class, "");
        String receiveId = request.getDataValue("receiveId", String.class, "");//接收者id
        String addressee = request.getDataValue("addressee", String.class, "");//收件人
        String phone = request.getDataValue("phone", String.class, "");//电话
        String address = request.getDataValue("address", String.class, "");//地址
        String addressInfo = request.getDataValue("addressInfo", String.class, "");//详细地址
        String userInfo = RedisUtil.get(RedisKeyConstant.USER_BASE + userId);
        JSONObject userObject = JSONUtil.parseObj(userInfo);
        JSONArray userBackpack = userObject.getJSONArray("userBackpack");
        List<ZjBackpackPO> zjBackpackPOS = userBackpack.toList(ZjBackpackPO.class);
        JSONArray orderList = userObject.getJSONArray("orderList");

        List<ZjOrderPO> zjOrderPOS = orderList.toList(ZjOrderPO.class);


        Optional<ZjBackpackPO> first = zjBackpackPOS.stream()
                .filter(item -> item.getBackId().equalsIgnoreCase(backId))
                .findFirst();
        if (first.isPresent()){
            ZjBackpackPO zjBackpackPO = first.get();
            String goodsId = zjBackpackPO.getGoodsId();
            boolean isInNumGoods = numGoods.contains(goodsId);
            boolean isInDisposableGoods = disposableGoods.contains(goodsId);
            boolean isInRepeatablyGoods = repeatedlyGoods.contains(goodsId);

            if (isInNumGoods) {
                System.out.println(goodsId + " 属于数量道具");
                zjBackpackPO.setGoodsNum(zjBackpackPO.getGoodsNum()-1);
                if (zjBackpackPO.getGoodsNum() == 0){
                    userBackpack.remove(zjBackpackPO);
                }
            } else if (isInDisposableGoods) {
                System.out.println(goodsId + " 属于一次性道具");
                zjBackpackPO.setIsEmploy("Y");
                //兑换大米或者手机 生成订单
                // 生成订单
                ZjOrderPO zjOrder = new ZjOrderPO();
                zjOrder.setOrderId(UtilCore.getDateRandomId());
                String orderNo = UtilCore.getDateRandomId();//订单号
                zjOrder.setOrderNo(orderNo);
                zjOrder.setProductName("实物道具");
                zjOrder.setOrderStatus("3");
                zjOrder.setPayType("2");
                zjOrder.setOrderType("5");
                zjOrder.setName(addressee);
                zjOrder.setPhoneNumber(phone);
                zjOrder.setShippingAddress(addressInfo);
                zjOrder.setInvoiceMode("1");
                zjOrder.setReceiveGoodsMode("1");
                zjOrder.setDistributionMode("1");
                LocalDateTime now = LocalDateTime.now();
                DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
                String dateString = now.format(formatter);
                zjOrder.setCreateTime(dateString);
                zjOrderPOS.add(zjOrder);
                userObject.put("orderList", orderList);
                RedisUtil.set(RedisKeyConstant.USER_BASE + userId,userObject.toString());
            } else if (isInRepeatablyGoods) {
                System.out.println(goodsId + " 属于重复性道具");

            } else {
                System.out.println(goodsId + " 不属于任何道具集合");
            }
        }
        userObject.set("userBackpack", zjBackpackPOS);
        RedisUtil.set(RedisKeyConstant.USER_BASE + userId, userObject.toString());
        response.setMessage("使用成功！");
    }
}
