package com.incubator.game.handler.club;

import cn.hutool.json.JSONArray;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.incubator.common.MessageHandler;
import com.incubator.common.net.Connection;
import com.incubator.core.net.ws.NetHandler;
import com.incubator.core.net.ws.WSRequest;
import com.incubator.core.net.ws.WSResponse;
import com.incubator.game.constant.RedisKeyConstant;
import com.incubator.game.data.entity.ZjClub;
import com.incubator.game.data.entity.ZjClubMember;
import com.incubator.game.data.jedis.RedisUtil;
import com.incubator.game.data.mq.RabbitMQUtil;
import com.incubator.game.data.po.ZjClubMemberPo;
import com.incubator.game.data.po.ZjClubPo;
import com.incubator.game.data.po.ZjUserLobbyPo;
import com.incubator.message.proto.CommonProto;

import java.util.*;
import java.util.stream.Collectors;

/**
 * Author:  xhb
 * Date:  2024-12-09 14:27
 * 同意或拒绝加入俱乐部
 */
@MessageHandler(id = CommonProto.Cmd.AgreeOrRefuseJoinReq_VALUE)
public class AgreeOrRefuseJoin extends NetHandler {

    @Override
    public void onDate(Connection session, WSRequest request, WSResponse response) {
        response.setCmd(CommonProto.Cmd.AgreeOrRefuseJoinRes_VALUE);

        String ids = request.getDataValue("ids", String.class, "");
        String status = request.getDataValue("memberStatus", String.class, "");
        String clubId = request.getDataValue("clubId", String.class, "");

        // 查询指定俱乐部
        String clubListStr = RedisUtil.get(RedisKeyConstant.CLUB_BASE);
        JSONObject clubJson = JSONUtil.parseObj(clubListStr);
        JSONArray clubListArray = clubJson.getJSONArray("clubList");
        List<ZjClubPo> clubList = clubListArray.toList(ZjClubPo.class);
        ZjClubPo zjClub = clubList.stream()
                .filter(club -> clubId.equals(club.getClubId()))
                .findFirst()
                .orElse(null);
        if (zjClub == null) {
            response.setCode(102);
            response.setMessage("俱乐部不存在");
            return;
        }

        String memberListStr = RedisUtil.get(RedisKeyConstant.CLUB_MEMBER + clubId);
        JSONObject memberObject = JSONUtil.parseObj(memberListStr);
        JSONArray clubMemberArray = memberObject.getJSONArray("clubMember");
        List<ZjClubMemberPo> clubMember = clubMemberArray.toList(ZjClubMemberPo.class);
        Map<String, ZjClubMemberPo> memberMap = new HashMap<>();
        for (ZjClubMemberPo member : clubMember) {
            if(member.getMemberStatus().equals("0")){
                memberMap.put(member.getUserId(), member);
            }

        }
        String[] userIds = ids.split(",");
        Date now = new Date(); // 创建一次，循环中重用
        for (String userId : userIds) {
            ZjClubMemberPo member = memberMap.get(userId);
            if (member != null) {
                member.setMemberStatus(status);
                ZjClubMember zjClubMember = new ZjClubMember();
                zjClubMember.setMemberId(member.getMemberId());
                zjClubMember.setMemberStatus(member.getMemberStatus());
                zjClubMember.setUserId(member.getUserId());
                zjClubMember.setClubId(member.getClubId());
                if ("1".equals(status)) {
                    zjClub.setClubMemberNum(zjClub.getClubMemberNum() + 1);
                    ZjClub zjClub1 = new ZjClub();
                    zjClub1.setClubId(zjClub.getClubId());
                    zjClub1.setClubMemberNum(zjClub.getClubMemberNum());
                    JSONObject object = new JSONObject();
                    object.put("type", "saveClub");
                    object.put("data", zjClub1);
                    try {
                        RabbitMQUtil.sendMessage(object.toString());
                    } catch (Exception e) {
                        e.printStackTrace();
                    }

                    member.setIsMaster("N");
                    member.setJoinTime(now);
                    zjClubMember.setJoinTime(now);


                    String userInfo = RedisUtil.get(RedisKeyConstant.USER_BASE + userId);
                    JSONObject userObject = JSONUtil.parseObj(userInfo);
                    JSONArray appJoinList = userObject.getJSONArray("appJoinList");
                    List<String> strings = appJoinList.toList(String.class);
                    strings.remove(clubId);
                    userObject.put("appJoinList",strings);

                    ZjUserLobbyPo lobbyUser = JSONUtil.toBean(userObject.getJSONObject("lobbyUser"), ZjUserLobbyPo.class);
                    member.setIsMaster("N");
                    member.setMasterScore(lobbyUser.getMasterScore());
                    member.setSevenDayMasterScore((Integer) userObject.get("sevenScore")==null?0:(Integer)userObject.get("sevenScore"));
                    if (lobbyUser.getUserClub() == null || lobbyUser.getUserClub().isEmpty()) {
                        lobbyUser.setUserClub(clubId);
                        member.setIsMain("Y");
                        userObject.put("lobbyUser", lobbyUser);
                        RedisUtil.set(RedisKeyConstant.USER_BASE + userId, userObject.toString());
                        JSONObject object1 = new JSONObject();
                        object1.put("type", "saveMainClub");
                        object1.put("data", lobbyUser);
                        try {
                            RabbitMQUtil.sendMessage(object1.toString());
                        } catch (Exception e) {
                            e.printStackTrace();
                        }
                    }
                }
                JSONObject object = new JSONObject();
                object.put("type", "saveClubMember");
                object.put("data", zjClubMember);
                try {
                    RabbitMQUtil.sendMessage(object.toString());
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
        }

        memberObject.put("clubMember", clubMember);
        RedisUtil.set(RedisKeyConstant.CLUB_MEMBER + clubId, memberObject.toString());
        RedisUtil.set(RedisKeyConstant.CLUB_BASE, clubJson.toString());
        response.setMessage("操作成功！");
    }
}
