package com.incubator.game.handler.club;

import cn.hutool.json.JSONArray;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.incubator.common.MessageHandler;
import com.incubator.common.net.Connection;
import com.incubator.core.net.ws.NetHandler;
import com.incubator.core.net.ws.WSRequest;
import com.incubator.core.net.ws.WSResponse;
import com.incubator.game.constant.RedisKeyConstant;
import com.incubator.game.data.entity.ZjClubMember;
import com.incubator.game.data.jedis.RedisUtil;
import com.incubator.game.data.mq.RabbitMQUtil;
import com.incubator.game.data.po.ZjClubMemberPo;
import com.incubator.game.data.po.ZjClubPo;
import com.incubator.game.data.po.ZjUserLobbyPo;
import com.incubator.game.util.UtilCore;
import com.incubator.message.proto.CommonProto;

import java.util.List;


/**
 * Author:  xhb
 * Date:  2024-12-09 14:27
 * 请求加入俱乐部请求
 */
@MessageHandler(id = CommonProto.Cmd.ApplyJoinClubReq_VALUE)
public class ApplyJoinClub extends NetHandler {

    @Override
    public void onDate(Connection session, WSRequest request, WSResponse response) {
        response.setCmd(CommonProto.Cmd.ApplyJoinClubRes_VALUE);
        /**
         *memberId,userId,clubId,memberStatus,requestExplain,requestChannel
         * {
         *     "userId":"123",用户id
         *     "clubId":"123",俱乐部id
         *     "memberStatus":"123",加入状态 0:申请加入 1:同意加入 2:拒绝加入 3:退出俱乐部 4:删除俱乐部
         *     "requestExplain":"123", 申请理由
         *     "requestChannel":"123",申请渠道
         * }
         */
        String userId = session.getPlayerId();
        String clubId = request.getDataValue("clubId", String.class, "");
        String memberStatus = request.getDataValue("memberStatus", String.class, "");
        String requestExplain = request.getDataValue("requestExplain", String.class, "");
        String requestChannel = request.getDataValue("requestChannel", String.class, "");
        ZjClubMemberPo newClubMember =new ZjClubMemberPo();
        newClubMember.setUserId(userId);
        newClubMember.setClubId(clubId);
        newClubMember.setRequestExplain(requestExplain);
        newClubMember.setRequestChannel(requestChannel);
        newClubMember.setMemberStatus(memberStatus);
        String userInfo = RedisUtil.get(RedisKeyConstant.USER_BASE + newClubMember.getUserId());
        JSONObject userObject = JSONUtil.parseObj(userInfo);
        Object lobbyUserObj = userObject.get("lobbyUser");
        ZjUserLobbyPo lobbyUser = JSONUtil.toBean((JSONObject) lobbyUserObj, ZjUserLobbyPo.class);

        JSONArray appJoinList = userObject.getJSONArray("appJoinList");
        if (appJoinList != null&&appJoinList.size()>0) {
            appJoinList.add(clubId);
            userObject.put("appJoinList", appJoinList);

        }else {
            appJoinList = new JSONArray();
            appJoinList.add(clubId);
        }
        userObject.put("appJoinList", appJoinList);

        //创建成员信息
        newClubMember.setMemberId(UtilCore.getDateRandomId());
        newClubMember.setMemberStatus("0");//加入状态 0:申请加入 1:同意加入 2:拒绝加入 3:退出俱乐部 4:删除俱乐部
        newClubMember.setNickName(lobbyUser.getNickName());
        newClubMember.setAvatarUrl(lobbyUser.getAvatarUrl());
        newClubMember.setWinRate(lobbyUser.getTraditionChance());
        String memBerList = RedisUtil.get(RedisKeyConstant.CLUB_MEMBER + newClubMember.getClubId());
        JSONArray clubMember = JSONUtil.parseObj(memBerList).getJSONArray("clubMember");
        List<ZjClubMemberPo> clubMemberPos = clubMember.toList(ZjClubMemberPo.class);
        if (clubMemberPos.stream().anyMatch(x -> x.getUserId().equals(userId)&&x.getMemberStatus().equals("0"))) {
            response.setCode(CommonProto.Code.PARAMETER_ERR_VALUE);
            response.setMessage("您已经申请过该俱乐部了！");
            return;
        }
        clubMember.add(newClubMember);

        ZjClubMember zjClubMember = new ZjClubMember();
        zjClubMember.setUserId(newClubMember.getUserId());
        zjClubMember.setClubId(newClubMember.getClubId());
        zjClubMember.setMemberStatus(newClubMember.getMemberStatus());
        zjClubMember.setRequestChannel(newClubMember.getRequestChannel()==null?"":newClubMember.getRequestChannel());
        zjClubMember.setRequestExplain(newClubMember.getRequestExplain()==null?"":newClubMember.getRequestExplain());
        zjClubMember.setMemberId(newClubMember.getMemberId());
        JSONObject object1 = new JSONObject();
        object1.put("type", "saveClubMember");
        object1.put("data", zjClubMember);
        try {
            RabbitMQUtil.sendMessage(object1.toString());
        } catch (Exception e) {
            e.printStackTrace();
        }
        JSONObject object = new JSONObject();
        object.put("clubMember", clubMember);
        RedisUtil.set(RedisKeyConstant.CLUB_MEMBER + newClubMember.getClubId(), object.toString());
        RedisUtil.set(RedisKeyConstant.USER_BASE + newClubMember.getUserId(), userObject.toString());
        response.setMessage("申请成功！");
    }
}
