package com.incubator.game.handler.club;

import cn.hutool.json.JSONArray;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.incubator.common.MessageHandler;
import com.incubator.common.net.Connection;
import com.incubator.core.net.ws.NetHandler;
import com.incubator.core.net.ws.WSRequest;
import com.incubator.core.net.ws.WSResponse;
import com.incubator.game.constant.RedisKeyConstant;
import com.incubator.game.data.entity.ZjClub;
import com.incubator.game.data.entity.ZjClubMember;
import com.incubator.game.data.jedis.RedisUtil;
import com.incubator.game.data.mq.RabbitMQUtil;
import com.incubator.game.data.po.ZjClubMemberPo;
import com.incubator.game.data.po.ZjClubPo;
import com.incubator.game.data.po.ZjUserLobbyPo;
import com.incubator.game.util.UtilCore;
import com.incubator.message.proto.CommonProto;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

/**
 * Author:  xhb
 * Date:  2024-12-09 14:27
 * 创建俱乐部
 */
@MessageHandler(id = CommonProto.Cmd.CreateClubReq_VALUE)
public class CreateClub extends NetHandler {

    @Override
    public void onDate(Connection session, WSRequest request, WSResponse response) {
        response.setCmd(CommonProto.Cmd.CreateClubRes_VALUE);

        String userId = session.getPlayerId();
        String userInfo = RedisUtil.get(RedisKeyConstant.USER_BASE + userId);
        JSONObject userObject = JSONUtil.parseObj(userInfo);
//        int s = (int) userObject.get("sevenScore");
        Object sevenScore = userObject.get("sevenScore");
        int s = sevenScore == null ? 0 : (int) sevenScore;
        Object lobbyUserObj = userObject.get("lobbyUser");
        ZjUserLobbyPo lobbyUser = JSONUtil.toBean((JSONObject) lobbyUserObj, ZjUserLobbyPo.class);


        String clubId = request.getDataValue("clubId", String.class, null);
        String clubName = request.getDataValue("clubName", String.class, null);
        String clubImg = request.getDataValue("clubImg", String.class, null);
        String clubDescription = request.getDataValue("clubDescription", String.class, null);
        String clubAddress = request.getDataValue("clubAddress", String.class, null);

        ZjClubPo club = new ZjClubPo();
        if (clubId!=null){
            club.setClubId(clubId);
        }
        club.setClubName(clubName);
        club.setClubImg(clubImg);
        club.setClubDescription(clubDescription);
        club.setClubAddress(clubAddress);
        JSONObject jsonObject = getClubListFromRedis();
        JSONArray clubArray = jsonObject.getJSONArray("clubList");
        List<ZjClubPo> clubList = new ArrayList<>();
        if(clubArray != null) {
            clubList = clubArray.toList(ZjClubPo.class).stream()
                    .filter(clubPo -> clubPo.getClubStatus().equals("1"))
                    .collect(Collectors.toList());

        }else{
            clubArray = new JSONArray();
        }


        if (club.getClubId()==null){
            if (isUserAlreadyCreatedClub(clubList, userId)) {
                response.setCode(CommonProto.Code.PARAMETER_ERR_VALUE);
                response.setMessage("您已创建过俱乐部");
                return;
            }
            ZjClubPo newClub = createNewClub(club, userId);
            newClub.setRank(clubList.size()+1);
            newClub.setCreateTime(new Date());
            newClub.setSevenRank(clubList.size()+1);
            newClub.setSevenDayMasterScore(s);
            newClub.setMasterScore(lobbyUser.getMasterScore());
            newClub.setMasterName(lobbyUser.getNickName());
            newClub.setClubMemberNum(1);

            ZjClub zjClub = new ZjClub();
            zjClub.setClubId(newClub.getClubId());
            zjClub.setClubName(newClub.getClubName());
            zjClub.setClubImg(newClub.getClubImg());
            zjClub.setClubDescription(newClub.getClubDescription());
            zjClub.setClubAddress(newClub.getClubAddress());
            zjClub.setCreateTime(new Date());
            zjClub.setClubStatus("1");
            zjClub.setClubCreator(userId);
            zjClub.setClubMemberNum(newClub.getClubMemberNum());
            JSONObject object = new JSONObject();
            object.put("type","saveClub");
            object.put("data",zjClub);
            try {
                RabbitMQUtil.sendMessage(object.toString());
            } catch (Exception e) {
                e.printStackTrace();
            }


            addClubToRedis(jsonObject, clubArray, newClub);//添加俱乐部到redis

            addClubMemberToRedis(userObject, newClub);//添加俱乐部成员到redis

            updateUserLobbyClub(lobbyUser, newClub);//设置主俱乐部

            userObject.put("lobbyUser", lobbyUser);
            ArrayList<ZjClubPo> zjClubPoArrarList = new ArrayList<>();
            zjClubPoArrarList.add(newClub);
            userObject.put("clubList", zjClubPoArrarList);
            RedisUtil.set(RedisKeyConstant.USER_BASE + userId, userObject.toString());

        }else {
            Optional<ZjClubPo> existingClub = clubList.stream()
                    .filter(clubPo -> clubPo.getClubId().equals(club.getClubId()))
                    .findFirst();
            existingClub.ifPresent(c -> {
                if (c.getClubStatus().equals("2")){
                    response.setCode(CommonProto.Code.PARAMETER_ERR_VALUE);
                    response.setMessage("您的俱乐部已被冻结");
                    return;
                }
                c.setClubName(club.getClubName());
                c.setClubImg(club.getClubImg());
                c.setClubDescription(club.getClubDescription());
                c.setClubAddress(club.getClubAddress());
            });
            ZjClubPo clubPo = existingClub.get();
            ZjClub zjClub = new ZjClub();
            zjClub.setClubId(clubPo.getClubId());
            zjClub.setClubName(clubPo.getClubName());
            zjClub.setClubImg(clubPo.getClubImg());
            zjClub.setClubDescription(clubPo.getClubDescription());
            zjClub.setClubAddress(clubPo.getClubAddress());
            zjClub.setCreateTime(new Date());
            zjClub.setClubStatus("1");
            zjClub.setClubCreator(clubPo.getClubCreator());
            zjClub.setClubMemberNum(clubPo.getClubMemberNum());
            JSONObject object = new JSONObject();
            object.put("type","saveClub");
            object.put("data",zjClub);
            try {
                RabbitMQUtil.sendMessage(object.toString());
            } catch (Exception e) {
                e.printStackTrace();
            }


            jsonObject.put("clubList", clubList);
            RedisUtil.set(RedisKeyConstant.CLUB_BASE, jsonObject.toString());
        }
    }

    private void addClubMemberToRedis(JSONObject userObject, ZjClubPo newClub) {
        Object lobbyUserObj = userObject.get("lobbyUser");
        ZjUserLobbyPo lobbyUser = JSONUtil.toBean((JSONObject) lobbyUserObj, ZjUserLobbyPo.class);
        ZjClubMemberPo zjClubMemberPo = new ZjClubMemberPo();
        zjClubMemberPo.setMemberId(UtilCore.getDateRandomId());
        zjClubMemberPo.setUserId(lobbyUser.getUserId());
        zjClubMemberPo.setClubId(newClub.getClubId());
        zjClubMemberPo.setIsMain("Y");
        zjClubMemberPo.setIsMaster("Y");
        zjClubMemberPo.setMemberStatus("1");
        zjClubMemberPo.setAvatarUrl(lobbyUser.getAvatarUrl());
        zjClubMemberPo.setNickName(lobbyUser.getNickName());
        zjClubMemberPo.setJoinTime(new Date());
        zjClubMemberPo.setSevenDayMasterScore((Integer) userObject.get("sevenScore"));
        zjClubMemberPo.setRequestExplain("掌门人");
        zjClubMemberPo.setRequestChannel("2");
        JSONObject object = new JSONObject();
        ArrayList<ZjClubMemberPo> objects = new ArrayList<>();
        objects.add(zjClubMemberPo);
        object.put("clubMember",objects);
        RedisUtil.set(RedisKeyConstant.CLUB_MEMBER + newClub.getClubId(), object.toString());

        ZjClubMember zjClubMember = new ZjClubMember();
        zjClubMember.setUserId(zjClubMemberPo.getUserId());
        zjClubMember.setClubId(zjClubMemberPo.getClubId());
        zjClubMember.setMemberStatus(zjClubMemberPo.getMemberStatus());
        zjClubMember.setRequestChannel(zjClubMemberPo.getRequestChannel()==null?"":zjClubMemberPo.getRequestChannel());
        zjClubMember.setRequestExplain(zjClubMemberPo.getRequestExplain()==null?"":zjClubMemberPo.getRequestExplain());
        zjClubMember.setMemberId(zjClubMemberPo.getMemberId());
        JSONObject object1 = new JSONObject();
        object1.put("type", "saveClubMember");
        object1.put("data", zjClubMember);
        try {
            RabbitMQUtil.sendMessage(object1.toString());
        } catch (Exception e) {
            e.printStackTrace();
        }




    }

    private JSONObject getClubListFromRedis() {
        String clubListStr = RedisUtil.get(RedisKeyConstant.CLUB_BASE);
        return JSONUtil.parseObj(clubListStr);
    }

    private boolean isUserAlreadyCreatedClub(List<ZjClubPo> clubList, String userId) {
        return clubList.stream().anyMatch(club -> club.getClubCreator().equals(userId));
    }

    private ZjClubPo createNewClub(ZjClubPo newClub , String userId) {
        newClub.setClubId(UtilCore.getDateRandomId());
        newClub.setClubCreator(userId);
        newClub.setOnlineCount(1);
        newClub.setClubStatus("1");
        newClub.setClubLevel(1);
        return newClub;
    }

    private void addClubToRedis(JSONObject jsonObject, JSONArray clubArray, ZjClubPo newClub) {
        clubArray.add(newClub);
        jsonObject.set("clubList", clubArray);
        RedisUtil.set(RedisKeyConstant.CLUB_BASE, jsonObject.toString());
    }

    private void updateUserLobbyClub(ZjUserLobbyPo lobbyUser, ZjClubPo newClub) {

        //查询当前用户的主俱乐部
        String userClub = lobbyUser.getUserClub();
        lobbyUser.setUserClub(newClub.getClubId());
        if (userClub != null&&!userClub.equals("")) {
            String oldClub = RedisUtil.get(RedisKeyConstant.CLUB_MEMBER + userClub);
            JSONObject oldClubJO = JSONUtil.parseObj(oldClub);
            JSONArray clubArray = oldClubJO.getJSONArray("clubMember");
            List<ZjClubMemberPo> clubMembers = clubArray.toList(ZjClubMemberPo.class);
            List<ZjClubMemberPo> updatedClubMembers = clubMembers.stream()
                    .peek(member -> {
                        if (member.getUserId().equals(lobbyUser.getUserId())) {
                            member.setIsMain("N");
                        }
                    })
                    .collect(Collectors.toList());
            oldClubJO.put("clubMember",updatedClubMembers);
            RedisUtil.set(RedisKeyConstant.CLUB_MEMBER + userClub,oldClubJO.toString());
        }
        JSONObject object = new JSONObject();
        object.put("type", "saveMainClub");
        object.put("data", lobbyUser);
        try {
            RabbitMQUtil.sendMessage(object.toString());
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
