package com.incubator.game.handler.club;

import cn.hutool.json.JSONArray;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.incubator.common.MessageHandler;
import com.incubator.common.net.Connection;
import com.incubator.core.net.ws.NetHandler;
import com.incubator.core.net.ws.WSRequest;
import com.incubator.core.net.ws.WSResponse;
import com.incubator.game.constant.RedisKeyConstant;
import com.incubator.game.data.jedis.RedisUtil;
import com.incubator.game.data.po.ZjClubPo;
import com.incubator.game.util.ObjectMapperUtil;
import com.incubator.message.proto.CommonProto;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * Author:  xhb
 * Date:  2024-12-12 16:22
 */
@MessageHandler(id = CommonProto.Cmd.CreateClubListReq_VALUE)
public class CreateClubList extends NetHandler {

    @Override
    public void onDate(Connection session, WSRequest request, WSResponse response) {
        response.setCmd(CommonProto.Cmd.CreateClubListRes_VALUE);

        String userId = session.getPlayerId();
        String clubs = RedisUtil.get(RedisKeyConstant.CLUB_BASE);
        JSONObject clubListObject = JSONUtil.parseObj(clubs);
        JSONArray clubListArray = clubListObject.getJSONArray("clubList");
        if (clubListArray == null) {
            response.setData(new HashMap<>());
            return;
        }
        List<ZjClubPo> clubList = clubListArray.toList(ZjClubPo.class);

        List<ZjClubPo> collect = clubList.stream()
                .filter(club -> club.getClubCreator().equals(userId)&&club.getClubStatus().equals("1"))
                .collect(Collectors.toList());
        List<Map<String, Object>> maps = ObjectMapperUtil.convertListToMapList(collect);
        Map<String, Object> map = new HashMap<>();
        map.put("CreateClubList", maps);
        response.setData(map);
    }
}
