package com.incubator.game.handler.club;

import cn.hutool.json.JSONArray;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.incubator.common.MessageHandler;
import com.incubator.common.net.Connection;
import com.incubator.core.net.ws.NetHandler;
import com.incubator.core.net.ws.WSRequest;
import com.incubator.core.net.ws.WSResponse;
import com.incubator.game.constant.RedisKeyConstant;
import com.incubator.game.data.entity.ZjClub;
import com.incubator.game.data.entity.ZjClubMember;
import com.incubator.game.data.jedis.RedisUtil;
import com.incubator.game.data.mq.RabbitMQUtil;
import com.incubator.game.data.po.ZjClubMemberPo;
import com.incubator.game.data.po.ZjClubPo;
import com.incubator.game.data.po.ZjUserLobbyPo;
import com.incubator.message.proto.CommonProto;

import java.util.*;
import java.util.stream.Collectors;

/**
 * Author:  xhb
 * Date:  2024-12-09 14:28
 */
@MessageHandler(id = CommonProto.Cmd.DeleteMemberReq_VALUE)
public class DeleteMember extends NetHandler {

    @Override
    public void onDate(Connection session, WSRequest request, WSResponse response) {
        response.setCmd(CommonProto.Cmd.DeleteMemberRes_VALUE);

        String clubId = request.getDataValue("clubId", String.class, "");
        String userId = request.getDataValue("userId", String.class, "");

        // 获取俱乐部列表和成员列表
        String clubListStr = RedisUtil.get(RedisKeyConstant.CLUB_BASE);
        String memberList = RedisUtil.get(RedisKeyConstant.CLUB_MEMBER + clubId);

        JSONObject clubJson = JSONUtil.parseObj(clubListStr);
        JSONArray clubListArray = clubJson.getJSONArray("clubList");
        List<ZjClubPo> clubList = clubListArray.toList(ZjClubPo.class);
        ZjClubPo zjClub = clubList.stream()
                .filter(club -> clubId.equals(club.getClubId()))
                .findFirst()
                .orElse(null);
        if (zjClub == null) {
            response.setCode(102);
            response.setMessage("俱乐部不存在");
            return;
        }

        JSONObject memberObject = JSONUtil.parseObj(memberList);
        JSONArray clubMemberArray = memberObject.getJSONArray("clubMember");
        List<ZjClubMemberPo> clubMember = clubMemberArray.toList(ZjClubMemberPo.class);

        Optional<ZjClubMemberPo> result = clubMember.stream()
                .filter(member -> userId.equals(member.getUserId()))
                .findFirst();
        if (result.isPresent()) {
            ZjClubMemberPo zjClubMember = result.get();
            zjClubMember.setMemberStatus("3");
            zjClubMember.setQuitTime(new Date());
            zjClub.setClubMemberNum(zjClub.getClubMemberNum() - 1);

            ZjClubMember zjClubMember1 = new ZjClubMember();
            zjClubMember1.setMemberId(zjClubMember.getMemberId());
            zjClubMember1.setMemberStatus(zjClubMember.getMemberStatus());
            JSONObject object1 = new JSONObject();
            object1.put("type", "saveClubMember");
            object1.put("data", zjClubMember1);
            try {
                RabbitMQUtil.sendMessage(object1.toString());
            } catch (Exception e) {
                e.printStackTrace();
            }

            ZjClub zjClub1 = new ZjClub();
            zjClub1.setClubId(zjClub.getClubId());
            zjClub1.setClubMemberNum(zjClub.getClubMemberNum());
            JSONObject object2 = new JSONObject();
            object2.put("type", "saveClub");
            object2.put("data", zjClub1);
            try {
                RabbitMQUtil.sendMessage(object2.toString());
            } catch (Exception e) {
                e.printStackTrace();
            }

            // 更新俱乐部列表和成员列表
            clubJson.set("clubList", clubList);
            memberObject.set("clubMember", clubMember);

            // 使用Redis事务保证原子性
            RedisUtil.set(RedisKeyConstant.CLUB_BASE, clubJson.toString());
            RedisUtil.set(RedisKeyConstant.CLUB_MEMBER + clubId, memberObject.toString());

            // 设置主俱乐部
            String userInfo = RedisUtil.get(RedisKeyConstant.USER_BASE + userId);
            JSONObject userObject = JSONUtil.parseObj(userInfo);
            ZjUserLobbyPo lobbyUser = JSONUtil.toBean(userObject.getJSONObject("lobbyUser"), ZjUserLobbyPo.class);
            JSONArray userClubList = userObject.getJSONArray("clubList");
            List<ZjClubPo> clubPoList = userClubList.toList(ZjClubPo.class);
            clubPoList.removeIf(clubPo -> clubId.equals(clubPo.getClubId()));

            if (lobbyUser.getUserClub().equals(clubId)) {
                if (clubPoList.isEmpty()) {
                    lobbyUser.setUserClub("");
                } else {
                    ZjClubPo newClub = clubPoList.get(0); // 直接选择第一个作为主俱乐部
                    lobbyUser.setUserClub(newClub.getClubId());
                }
                userObject.put("lobbyUser", lobbyUser);

                JSONObject object = new JSONObject();
                object.put("type", "saveMainClub");
                object.put("data", lobbyUser);
                try {
                    RabbitMQUtil.sendMessage(object.toString());
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
            userObject.put("clubList", clubPoList);
            RedisUtil.set(RedisKeyConstant.USER_BASE + userId, userObject.toString());
        }
        response.setMessage("删除成功");
    }
}
