package com.incubator.game.handler.club;

import cn.hutool.json.JSONArray;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.incubator.common.MessageHandler;
import com.incubator.common.net.Connection;
import com.incubator.core.net.ws.NetHandler;
import com.incubator.core.net.ws.WSRequest;
import com.incubator.core.net.ws.WSResponse;
import com.incubator.game.constant.RedisKeyConstant;
import com.incubator.game.data.entity.ZjClub;
import com.incubator.game.data.entity.ZjClubMember;
import com.incubator.game.data.jedis.RedisUtil;
import com.incubator.game.data.mq.RabbitMQUtil;
import com.incubator.game.data.po.ZjClubMemberPo;
import com.incubator.game.data.po.ZjClubPo;
import com.incubator.game.data.po.ZjUserLobbyPo;
import com.incubator.message.proto.CommonProto;

import java.util.List;
import java.util.Objects;
import java.util.Random;
import java.util.stream.Collectors;

/**
 * Author:  xhb
 * Date:  2024-12-09 14:28
 */
@MessageHandler(id = CommonProto.Cmd.DissolveClubReq_VALUE)
public class DissolveClub extends NetHandler {
    @Override
    public void onDate(Connection session, WSRequest request, WSResponse response) {
        response.setCode(CommonProto.Cmd.DissolveClubRes_VALUE);

        String clubId = request.getDataValue("clubId", String.class, "");
        String clubListStr = RedisUtil.get(RedisKeyConstant.CLUB_BASE);
        JSONObject clubJson = JSONUtil.parseObj(clubListStr);
        JSONArray clubListArray = clubJson.getJSONArray("clubList");
        List<ZjClubPo> clubList = clubListArray.toList(ZjClubPo.class);
        ZjClubPo zjClub = clubList.stream()
                .filter(club -> clubId.equals(club.getClubId()))
                .findFirst()
                .orElse(null);
        if (zjClub == null) {
            response.setCode(102);
            response.setMessage("俱乐部不存在");
            return;
        }
        // 更新俱乐部状态
        zjClub.setClubStatus("0"); // 解散俱乐部

        ZjClub zjClub1 = new ZjClub();
        zjClub1.setClubId(zjClub.getClubId());
        zjClub1.setClubStatus(zjClub.getClubStatus());
        JSONObject object2 = new JSONObject();
        object2.put("type", "saveClub");
        object2.put("data", zjClub1);
        try {
            RabbitMQUtil.sendMessage(object2.toString());
        } catch (Exception e) {
            e.printStackTrace();
        }


        clubJson.put("clubList", clubList);
        RedisUtil.set(RedisKeyConstant.CLUB_BASE, clubJson.toString());

        // 查询俱乐部成员，并踢出俱乐部
        String memberList = RedisUtil.get(RedisKeyConstant.CLUB_MEMBER + clubId);
        JSONObject memberObject = JSONUtil.parseObj(memberList);
        JSONArray clubMemberArray = memberObject.getJSONArray("clubMember");
        List<ZjClubMemberPo> zjClubMembers = clubMemberArray.toList(ZjClubMemberPo.class);

        for (ZjClubMemberPo zjClubMember : zjClubMembers) {
            // 设置主俱乐部
            String userId = zjClubMember.getUserId();
            String userInfo = RedisUtil.get(RedisKeyConstant.USER_BASE + userId);
            JSONObject userObject = JSONUtil.parseObj(userInfo);
            ZjUserLobbyPo lobbyUser = JSONUtil.toBean(userObject.getJSONObject("lobbyUser"), ZjUserLobbyPo.class);

            if (zjClubMember.getMemberStatus().equals("0")){
                JSONArray appJoinList = userObject.getJSONArray("appJoinList");
                List<String> strings = appJoinList.toList(String.class);
                strings.remove(clubId);
                userObject.put("appJoinList",strings);
            }
            zjClubMember.setMemberStatus("3");

            ZjClubMember zjClubMember1 = new ZjClubMember();
            zjClubMember1.setMemberId(zjClubMember.getMemberId());
            zjClubMember1.setMemberStatus(zjClubMember.getMemberStatus());
            JSONObject object1 = new JSONObject();
            object1.put("type", "saveClubMember");
            object1.put("data", zjClubMember1);
            try {
                RabbitMQUtil.sendMessage(object1.toString());
            } catch (Exception e) {
                e.printStackTrace();
            }




            JSONArray userClubList = userObject.getJSONArray("clubList");
            List<ZjClubPo> userClubPoList = userClubList.toList(ZjClubPo.class);
            userClubPoList.removeIf(clubPo -> clubId.equals(clubPo.getClubId()));
            if (lobbyUser.getUserClub().equals(clubId)) {
                if (userClubPoList.isEmpty()) {
                    lobbyUser.setUserClub("");
                } else {
                    ZjClubPo newClub = userClubPoList.get(0); // 直接选择第一个作为主俱乐部
                    lobbyUser.setUserClub(newClub.getClubId());
                }
                JSONObject object = new JSONObject();
                object.put("type", "saveMainClub");
                object.put("data", lobbyUser);
                try {
                    RabbitMQUtil.sendMessage(object.toString());
                } catch (Exception e) {
                    e.printStackTrace();
                }
                userObject.put("lobbyUser", lobbyUser);
            }
            userObject.put("clubList", userClubList);
            RedisUtil.set(RedisKeyConstant.USER_BASE + userId, userObject.toString());
        }
        memberObject.put("clubMember", zjClubMembers);
        RedisUtil.set(RedisKeyConstant.CLUB_MEMBER + clubId, memberObject.toString());
        response.setMessage("解散成功");
    }
}
