package com.incubator.game.handler.club;

import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.incubator.common.MessageHandler;
import com.incubator.common.net.Connection;
import com.incubator.core.net.ws.NetHandler;
import com.incubator.core.net.ws.WSRequest;
import com.incubator.core.net.ws.WSResponse;
import com.incubator.game.constant.RedisKeyConstant;
import com.incubator.game.data.jedis.RedisUtil;
import com.incubator.game.data.po.ZjClubMemberPo;
import com.incubator.game.data.po.ZjUserLobbyPo;
import com.incubator.game.util.ObjectMapperUtil;
import com.incubator.message.proto.CommonProto;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * Author:  xhb
 * Date:  2024-12-09 14:27
 * 获取申请列表
 */
@MessageHandler(id = CommonProto.Cmd.GetApplyListReq_VALUE)
public class GetApplyList extends NetHandler {

    @Override
    public void onDate(Connection session, WSRequest request, WSResponse response) {
        response.setCmd(CommonProto.Cmd.GetApplyListRes_VALUE);
        String userId = session.getPlayerId();

        String userInfo = RedisUtil.get(RedisKeyConstant.USER_BASE + userId);
        JSONObject userObject = JSONUtil.parseObj(userInfo);
        Object lobbyUserObj = userObject.get("lobbyUser");
        ZjUserLobbyPo lobbyUser = JSONUtil.toBean((JSONObject) lobbyUserObj, ZjUserLobbyPo.class);
        String memberList = RedisUtil.get(RedisKeyConstant.CLUB_MEMBER + lobbyUser.getUserClub());
        if (memberList == null) {
            response.setMessage("您当前为管理俱乐部");
            response.setCode(CommonProto.Code.PARAMETER_ERR_VALUE);
            return;
        }
        List<ZjClubMemberPo> clubMember = JSONUtil.parseObj(memberList).getJSONArray("clubMember").toList(ZjClubMemberPo.class);
        //提取status属性为"0"的元素，并组成新的集合
        List<ZjClubMemberPo> filteredClubMember = clubMember.stream()
                .filter(member -> "0".equals(member.getMemberStatus()))
                .collect(Collectors.toList());
        List<Map<String, Object>> maps = ObjectMapperUtil.convertListToMapList(filteredClubMember);
        Map<String,Object> map=new HashMap<>();
        map.put("applyList",maps);
        response.setData(map);
    }
}
