package com.incubator.game.handler.club;

import cn.hutool.json.JSONArray;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import cn.hutool.jwt.JWT;
import cn.hutool.jwt.JWTUtil;
import com.incubator.common.MessageHandler;
import com.incubator.common.net.Connection;
import com.incubator.core.net.ws.NetHandler;
import com.incubator.core.net.ws.WSRequest;
import com.incubator.core.net.ws.WSResponse;
import com.incubator.game.constant.RedisKeyConstant;
import com.incubator.game.data.jedis.RedisUtil;
import com.incubator.game.data.po.ZjClubPo;
import com.incubator.game.util.ObjectMapperUtil;
import com.incubator.message.proto.CommonProto;

import java.util.*;
import java.util.stream.Collectors;

/**
 * Author:  xhb
 * Date:  2024-12-09 13:19
 * 获取俱乐部列表信息
 */
@MessageHandler(id = CommonProto.Cmd.ClubInfoReq_VALUE)
public class GetClubListHandler extends NetHandler {


    @Override
    public void onDate(Connection session, WSRequest request, WSResponse response) {
        response.setCmd(CommonProto.Cmd.ClubInfoRes_VALUE);

        //提取我加入的俱乐部列表
        String userId = session.getPlayerId();
        String userInfo = RedisUtil.get(RedisKeyConstant.USER_BASE + userId);
        JSONObject Object = JSONUtil.parseObj(userInfo);
        JSONArray clubListArray = JSONUtil.parseObj(Object).getJSONArray("clubList");
        if (clubListArray == null) {
            response.setData(new HashMap<>());
            return;
        }
        List<ZjClubPo> myClubList = clubListArray.toList(ZjClubPo.class);


        //取出俱乐部列表
        String clubs = RedisUtil.get(RedisKeyConstant.CLUB_BASE);
        JSONObject jsonObject = JSONUtil.parseObj(clubs);
        JSONArray clubArray = jsonObject.getJSONArray("clubList");
        if (clubArray == null) {
            response.setData(new HashMap<>());
            return;
        }
        List<ZjClubPo> clubList = clubArray.toList(ZjClubPo.class);
        List<ZjClubPo> clubPoList = clubList.stream()
                .filter(club -> club.getClubStatus().equals("1"))
                .collect(Collectors.toList());


        Set<String> clubIdSet = myClubList.stream()
                .map(ZjClubPo::getClubId).collect(Collectors.toSet());

        List<ZjClubPo> zjClubPoList;
        JSONArray appJoinArray = JSONUtil.parseObj(Object).getJSONArray("appJoinList");
        if (appJoinArray != null&&appJoinArray.size()>0) {
            Set<String> strings = new HashSet<>(appJoinArray.toList(String.class));
            zjClubPoList = clubPoList.stream()
                    .peek(clubPo -> {
                        if (clubIdSet.contains(clubPo.getClubId())) {
                            clubPo.setIsJoin("3");
                        } else if(strings.contains(clubPo.getClubId())) {
                            clubPo.setIsJoin("2");
                        }else {
                            clubPo.setIsJoin("1");
                        }
                    })
                    .collect(Collectors.toList());

        } else {
            zjClubPoList = clubPoList.stream()
                    .peek(clubPo -> {
                        if (clubIdSet.contains(clubPo.getClubId())) {
                            clubPo.setIsJoin("3");
                        } else {
                            clubPo.setIsJoin("1");
                        }
                    })
                    .collect(Collectors.toList());
        }


        List<ZjClubPo> rankList = zjClubPoList.stream()
                .sorted(Comparator.comparingInt(ZjClubPo::getRank))
                .collect(Collectors.toList());


        List<Map<String, Object>> rankMaps = ObjectMapperUtil.convertListToMapList(rankList);

        List<ZjClubPo> sevenRankList = zjClubPoList.stream()
                .sorted(Comparator.comparingInt(ZjClubPo::getSevenRank))
                .collect(Collectors.toList());

        List<Map<String, Object>> maps = ObjectMapperUtil.convertListToMapList(sevenRankList);
        Map<String, Object> map = new HashMap<>();
        map.put("rankList", rankMaps);
        map.put("sevenRankList", maps);
        response.setData(map);
    }
}
