package com.incubator.game.handler.club;

import cn.hutool.json.JSONArray;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import cn.hutool.jwt.JWT;
import cn.hutool.jwt.JWTUtil;
import com.incubator.common.MessageHandler;
import com.incubator.common.net.Connection;
import com.incubator.core.net.ws.NetHandler;
import com.incubator.core.net.ws.WSRequest;
import com.incubator.core.net.ws.WSResponse;
import com.incubator.game.constant.RedisKeyConstant;
import com.incubator.game.data.jedis.RedisUtil;
import com.incubator.game.data.po.ZjClubMemberPo;
import com.incubator.game.data.po.ZjClubPo;
import com.incubator.game.data.po.ZjUserLobbyPo;
import com.incubator.game.util.ObjectMapperUtil;
import com.incubator.message.proto.CommonProto;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.stream.Collectors;

/**
 * Author:  xhb
 * Date:  2024-12-09 13:38
 * 获取我的俱乐部信息
 */
@MessageHandler(id = CommonProto.Cmd.MyClubReq_VALUE)
public class GetMyClub extends NetHandler {


    @Override
    public void onDate(Connection session, WSRequest request, WSResponse response) {
        response.setCmd(CommonProto.Cmd.MyClubRes_VALUE);
        String userId = session.getPlayerId();
        String userInfo = RedisUtil.get(RedisKeyConstant.USER_BASE + userId);
        JSONObject userObject = JSONUtil.parseObj(userInfo);
        Object lobbyUserObj = userObject.get("lobbyUser");
        ZjUserLobbyPo lobbyUser = JSONUtil.toBean((JSONObject) lobbyUserObj, ZjUserLobbyPo.class);
        if (lobbyUser.getUserClub() == null||lobbyUser.getUserClub().equals("")) {
            response.setCode(CommonProto.Code.PARAMETER_ERR_VALUE);
            response.setMessage("您当前未加入俱乐部");
            return;
        }
        String club = RedisUtil.get(RedisKeyConstant.CLUB_MEMBER + lobbyUser.getUserClub());
        if (club == null) {
            response.setCode(CommonProto.Code.PARAMETER_ERR_VALUE);
            response.setMessage("您当前未加入俱乐部");
            return;
        }
        JSONObject clubObject = JSONUtil.parseObj(club);
        JSONArray clubMemberArray = clubObject.getJSONArray("clubMember");
        List<ZjClubMemberPo> clubMemberList = clubMemberArray.toList(ZjClubMemberPo.class);
        List<ZjClubMemberPo> clubMemberPos = clubMemberList.stream()
                .filter(item -> item.getMemberStatus().equals("1"))
                .collect(Collectors.toList());


        String clubs = RedisUtil.get(RedisKeyConstant.CLUB_BASE);
        JSONObject clubListObject = JSONUtil.parseObj(clubs);
        JSONArray clubListArray = clubListObject.getJSONArray("clubList");
        List<ZjClubPo> clubList = clubListArray.toList(ZjClubPo.class);
        Optional<ZjClubPo> first = clubList.stream()
                .filter(c -> c.getClubStatus().equals("1") && c.getClubId().equals(lobbyUser.getUserClub()))
                .findFirst();
        if (!first.isPresent()) {
            return;
        }
        ZjClubPo clubPo = first.get();

        List<Map<String, Object>> maps = ObjectMapperUtil.convertListToMapList(clubMemberPos);
        Map<String, Object> map = new HashMap<>();
        map.put("clubMemberList", maps);
        map.put("member", clubMemberPos.size());
        map.put("rank", clubPo.getRank());
        map.put("level", clubPo.getClubLevel());
        map.put("mainClubMember", clubMemberPos.stream()
                .filter(member -> "Y".equals(member.getIsMain()))
                .count());
        long count = clubMemberList.stream()
                .filter(item -> item.getMemberStatus().equals("0"))
                .count();
        map.put("sh", count);
        response.setData(map);
    }
}
