package com.incubator.game.handler.club;

import cn.hutool.json.JSONArray;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.incubator.common.MessageHandler;
import com.incubator.common.net.Connection;
import com.incubator.core.net.ws.NetHandler;
import com.incubator.core.net.ws.WSRequest;
import com.incubator.core.net.ws.WSResponse;
import com.incubator.game.constant.RedisKeyConstant;
import com.incubator.game.data.jedis.RedisUtil;
import com.incubator.game.data.po.ZjClubPo;
import com.incubator.game.data.po.ZjUserLobbyPo;
import com.incubator.game.util.ObjectMapperUtil;
import com.incubator.message.proto.CommonProto;

import java.util.*;
import java.util.stream.Collectors;

/**
 * Author:  xhb
 * Date:  2024-12-10 14:14
 * 获取我的俱乐部列表消息
 */
@MessageHandler(id = CommonProto.Cmd.GetMyClubListReq_VALUE)
public class GetMyClubList extends NetHandler {

    @Override
    public void onDate(Connection session, WSRequest request, WSResponse response) {
        response.setCmd(CommonProto.Cmd.GetMyClubListRes_VALUE);

        String userId = session.getPlayerId();
        String userInfo = RedisUtil.get(RedisKeyConstant.USER_BASE + userId);
        JSONObject jsonObject = JSONUtil.parseObj(userInfo);
        Object lobbyUserObj = jsonObject.get("lobbyUser");
        ZjUserLobbyPo lobbyUser = JSONUtil.toBean((JSONObject) lobbyUserObj, ZjUserLobbyPo.class);
        JSONArray clubListArray = JSONUtil.parseObj(jsonObject).getJSONArray("clubList");
        if (clubListArray == null) {
            response.setData(new HashMap<>());
            return;
        }
        List<ZjClubPo> myClubList = clubListArray.toList(ZjClubPo.class);


        List<ZjClubPo> clubPoList = myClubList.stream()
                .filter(club -> club.getClubStatus().equals("1"))
                .collect(Collectors.toList());

        clubPoList = clubPoList.stream()
                .peek(clubPo -> {
                    clubPo.setIsMain(lobbyUser.getUserClub().equals(clubPo.getClubId()) ? "Y" : "N");
                    clubPo.setIsCreate(lobbyUser.getUserId().equals(clubPo.getClubCreator()) ? "Y" : "N");
                })
                .collect(Collectors.toList());

        List<Map<String, Object>> maps = ObjectMapperUtil.convertListToMapList(clubPoList);
        Map<String,Object> hashMap = new HashMap<>();
        hashMap.put("clubList", maps);
        response.setData(hashMap);
    }
}
