package com.incubator.game.handler.club;

import cn.hutool.json.JSONArray;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.incubator.common.MessageHandler;
import com.incubator.common.log4j.Log;
import com.incubator.common.net.Connection;
import com.incubator.core.net.ws.NetHandler;
import com.incubator.core.net.ws.WSRequest;
import com.incubator.core.net.ws.WSResponse;
import com.incubator.game.constant.RedisKeyConstant;
import com.incubator.game.data.entity.ZjClub;
import com.incubator.game.data.entity.ZjClubMember;
import com.incubator.game.data.jedis.RedisUtil;
import com.incubator.game.data.mq.RabbitMQUtil;
import com.incubator.game.data.po.ZjClubMemberPo;
import com.incubator.game.data.po.ZjClubPo;
import com.incubator.game.data.po.ZjUserLobbyPo;
import com.incubator.message.proto.CommonProto;

import java.util.Date;
import java.util.List;
import java.util.Objects;
import java.util.Random;
import java.util.stream.Collectors;

/**
 * Author:  xhb
 * Date:  2024-12-09 14:27
 */
@MessageHandler(id = CommonProto.Cmd.QuitClubReq_VALUE)
public class QuitClub extends NetHandler {

    @Override
    public void onDate(Connection session, WSRequest request, WSResponse response) {
        response.setCmd(CommonProto.Cmd.QuitClubRes_VALUE);

        String userId = session.getPlayerId();
        String clubId = request.getDataValue("clubId", String.class, "");


        String clubListStr = RedisUtil.get(RedisKeyConstant.CLUB_BASE);

        JSONObject clubJson = JSONUtil.parseObj(clubListStr);
        JSONArray clubListArray = clubJson.getJSONArray("clubList");
        List<ZjClubPo> clubList = clubListArray.toList(ZjClubPo.class);
        ZjClubPo zjClub = clubList.stream()
                .filter(club -> clubId.equals(club.getClubId()))
                .findFirst()
                .orElse(null);
        if (zjClub == null) {
            response.setCode(102);
            response.setMessage("俱乐部不存在");
            return;
        }

        ZjClub zjClub1 = new ZjClub();
        zjClub1.setClubId(zjClub.getClubId());
        zjClub1.setClubMemberNum(zjClub.getClubMemberNum());
        JSONObject object2 = new JSONObject();
        object2.put("type", "saveClub");
        object2.put("data", zjClub1);
        try {
            RabbitMQUtil.sendMessage(object2.toString());
        } catch (Exception e) {
            e.printStackTrace();
        }


        try {
            // 更新用户在俱乐部的状态
            updateClubMemberStatus(clubId, userId);

            // 更新用户的主俱乐部信息
            updateUserMainClub(userId, clubId);
            response.setMessage("退出成功！");
        } catch (Exception e) {
            // 异常处理
            Log.error("系统异常 " + userId, e);
            response.setCmd(CommonProto.Code.SYSTEM_ERR_VALUE);
        }
    }

    private void updateClubMemberStatus(String clubId, String userId) {
        String zjClubMember = RedisUtil.get(RedisKeyConstant.CLUB_MEMBER + clubId);
        JSONArray clubArray = JSONUtil.parseObj(zjClubMember).getJSONArray("clubMember");
        List<ZjClubMemberPo> clubMemberList = clubArray.toList(ZjClubMemberPo.class);

        for (ZjClubMemberPo clubPo : clubMemberList) {
            if (clubPo.getUserId().equals(userId)) {
                clubPo.setMemberStatus("3");
                clubPo.setQuitTime(new Date());

                ZjClubMember zjClubMember1 = new ZjClubMember();
                zjClubMember1.setMemberId(clubPo.getMemberId());
                zjClubMember1.setMemberStatus(zjClubMember1.getMemberStatus());
                JSONObject object1 = new JSONObject();
                object1.put("type", "saveClubMember");
                object1.put("data", zjClubMember1);
                try {
                    RabbitMQUtil.sendMessage(object1.toString());
                } catch (Exception e) {
                    e.printStackTrace();
                }

                break;
            }
        }

        JSONObject object = new JSONObject();
        object.put("clubMember", clubMemberList);
        RedisUtil.set(RedisKeyConstant.CLUB_MEMBER + clubId, object.toString());
    }

    private void updateUserMainClub(String userId, String clubId) {
        String userInfo = RedisUtil.get(RedisKeyConstant.USER_BASE + userId);
        JSONObject jsonObject = JSONUtil.parseObj(userInfo);
        ZjUserLobbyPo lobbyUser = JSONUtil.toBean(jsonObject.getJSONObject("lobbyUser"), ZjUserLobbyPo.class);

        JSONArray clubListArray = JSONUtil.parseObj(jsonObject).getJSONArray("clubList");
        List<ZjClubPo> clubList = clubListArray.toList(ZjClubPo.class);
        clubList.removeIf(clubPo -> clubId.equals(clubPo.getClubId()));
        String userClub = lobbyUser.getUserClub();
        if (userClub != null && userClub.equals(clubId)) {
            if (clubList.isEmpty()) {
                lobbyUser.setUserClub("");
            } else {
                Random random = new Random();
                int randomIndex = random.nextInt(clubList.size());
                String newClubId = clubList.get(randomIndex).getClubId();
                lobbyUser.setUserClub(newClubId);
            }
            jsonObject.put("lobbyUser", lobbyUser);

            JSONObject object = new JSONObject();
            object.put("type", "saveMainClub");
            object.put("data", lobbyUser);
            try {
                RabbitMQUtil.sendMessage(object.toString());
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        jsonObject.put("clubList", clubList);
        RedisUtil.set(RedisKeyConstant.USER_BASE + userId, jsonObject.toString());
    }
}
