package com.incubator.game.handler.club;

import cn.hutool.json.JSONArray;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.incubator.common.MessageHandler;
import com.incubator.common.net.Connection;
import com.incubator.core.net.ws.NetHandler;
import com.incubator.core.net.ws.WSRequest;
import com.incubator.core.net.ws.WSResponse;
import com.incubator.game.constant.RedisKeyConstant;
import com.incubator.game.contest.Contest;
import com.incubator.game.contest.ContestService;
import com.incubator.game.data.jedis.RedisUtil;
import com.incubator.game.data.po.ZjClubMemberPo;
import com.incubator.game.data.po.ZjClubPo;
import com.incubator.game.data.po.ZjUserLobbyPo;
import com.incubator.game.handler.contest.GetClubCompetitionListHandler;
import com.incubator.game.util.ObjectMapperUtil;
import com.incubator.message.proto.CommonProto;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * Author:  xhb
 * Date:  2025-01-02 13:06
 */
@MessageHandler(id = CommonProto.Cmd.SearchClubReq_VALUE)
public class SearchClub extends NetHandler {

    @Override
    public void onDate(Connection session, WSRequest request, WSResponse response) {
        response.setCmd(CommonProto.Cmd.SearchClubRes_VALUE);

        String userId = session.getPlayerId();
        String type = request.getDataValue("type", String.class, "");
        String name = request.getDataValue("searchText", String.class, "");
        Map<String, Object> map = new HashMap<>();
        switch (type){
            case "club":
                String clubs = RedisUtil.get(RedisKeyConstant.CLUB_BASE);
                JSONObject jsonObject = JSONUtil.parseObj(clubs);
                JSONArray clubArray = jsonObject.getJSONArray("clubList");
                if (clubArray == null) {
                    response.setData(new HashMap<>());
                    return;
                }
                List<ZjClubPo> clubList = clubArray.toList(ZjClubPo.class);
                List<ZjClubPo> clubPoList = clubList.stream()
                        .filter(club -> club.getClubStatus().equals("1")&&club.getClubName().contains(name))
                        .collect(Collectors.toList());
                List<Map<String, Object>> mapList = ObjectMapperUtil.convertListToMapList(clubPoList);
                map.put("clubList", mapList);
                response.setData(map);
                return;
            case "player":
                String userInfo = RedisUtil.get(RedisKeyConstant.USER_BASE + userId);
                JSONObject userObject = JSONUtil.parseObj(userInfo);
                Object lobbyUserObj = userObject.get("lobbyUser");
                ZjUserLobbyPo lobbyUser = JSONUtil.toBean((JSONObject) lobbyUserObj, ZjUserLobbyPo.class);
                if (lobbyUser.getUserClub() == null||lobbyUser.getUserClub().equals("")) {
                    response.setCode(CommonProto.Code.PARAMETER_ERR_VALUE);
                    response.setMessage("您当前未加入俱乐部");
                    return;
                }
                String club = RedisUtil.get(RedisKeyConstant.CLUB_MEMBER + lobbyUser.getUserClub());
                if (club == null) {
                    response.setCode(CommonProto.Code.PARAMETER_ERR_VALUE);
                    response.setMessage("您当前未加入俱乐部");
                    return;
                }
                JSONObject clubObject = JSONUtil.parseObj(club);
                JSONArray clubMemberArray = clubObject.getJSONArray("clubMember");
                List<ZjClubMemberPo> clubMemberList = clubMemberArray.toList(ZjClubMemberPo.class);
                List<ZjClubMemberPo> clubMemberPos = clubMemberList.stream()
                        .filter(item -> item.getMemberStatus().equals("1")&&item.getNickName().contains(name))
                        .collect(Collectors.toList());
                List<Map<String, Object>> convertListToMapList = ObjectMapperUtil.convertListToMapList(clubMemberPos);
                map.put("playerList", convertListToMapList);
                response.setData(map);
                return;
            case "contest":
                // 俱乐部比赛信息列表
                List<Object> list = new ArrayList<>();
                for (Contest contest : ContestService.getInstance().contestMap.values()) {
                    if (contest != null&&contest.prop.getCompeteType() == 2&&contest.prop.getDesc().contains(name)) {
                        list.add(GetClubCompetitionListHandler.contestInfoToMessageByUId(contest,userId));
                    }
                }
                map.put("contestInfo", list);
                response.setData(map);
        }
    }
}