package com.incubator.game.handler.club;

import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.incubator.common.MessageHandler;
import com.incubator.common.net.Connection;
import com.incubator.core.net.ws.NetHandler;
import com.incubator.core.net.ws.WSRequest;
import com.incubator.core.net.ws.WSResponse;
import com.incubator.game.constant.RedisKeyConstant;
import com.incubator.game.data.jedis.RedisUtil;
import com.incubator.game.data.mq.RabbitMQUtil;
import com.incubator.game.data.po.ZjUserLobbyPo;
import com.incubator.message.proto.CommonProto;


/**
 * Author:  xhb
 * Date:  2024-12-09 14:27
 * 设置我的主社团
 */
@MessageHandler(id = CommonProto.Cmd.SetMainClubReq_VALUE)
public class SetMainClub extends NetHandler {

    @Override
    public void onDate(Connection session, WSRequest request, WSResponse response) {
        response.setCmd(CommonProto.Cmd.SetMainClubRes_VALUE);

        String userId = session.getPlayerId();
        String clubId = request.getDataValue("clubId", String.class, "");

        // 根据token获取用户信息
        String userInfo = RedisUtil.get(RedisKeyConstant.USER_BASE + userId);
        JSONObject jsonObject = JSONUtil.parseObj(userInfo);
        Object lobbyUserObj = jsonObject.get("lobbyUser");
        ZjUserLobbyPo lobbyUser = JSONUtil.toBean((JSONObject) lobbyUserObj, ZjUserLobbyPo.class);

        //查询上次设置主俱乐部的时间是否到期
        long ttl = RedisUtil.ttl(RedisKeyConstant.SET_MAIN_CLUB);
        if (ttl>0) {
            response.setCode(CommonProto.Code.SYSTEM_ERR_VALUE);
            response.setMessage("您已设置过主俱乐部，请在"+(int) Math.ceil((double) ttl / (24 * 60 * 60))+"天后再次设置！");
            return ;
        }
        //查询我的主俱乐部
        lobbyUser.setUserClub(clubId);
        RedisUtil.setEx(RedisKeyConstant.SET_MAIN_CLUB+userId, clubId, Math.toIntExact(30 * 24 * 60 * 60));//设置用户的主俱乐部为30天
        response.setCode(CommonProto.Code.OK_VALUE);
        response.setMessage("设置成功！");

        JSONObject object = new JSONObject();
        object.put("type", "saveMainClub");
        object.put("data", lobbyUser);
        try {
            RabbitMQUtil.sendMessage(object.toString());
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
