package com.incubator.game.handler.contest;

import cn.hutool.json.JSONArray;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.incubator.common.MessageHandler;
import com.incubator.common.net.Connection;
import com.incubator.core.net.ws.NetHandler;
import com.incubator.core.net.ws.WSRequest;
import com.incubator.core.net.ws.WSResponse;
import com.incubator.game.constant.RedisKeyConstant;
import com.incubator.game.contest.Contest;
import com.incubator.game.contest.ContestService;
import com.incubator.game.data.jedis.RedisUtil;
import com.incubator.game.data.po.ZjClubPo;
import com.incubator.game.player.Player;
import com.incubator.game.util.JsonToMapUtil;
import com.incubator.game.util.PlayerUtil;
import com.incubator.game.util.ProtoUtil;
import com.incubator.message.proto.CommonProto;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * Author:  xhb
 * Date:  2024-12-26 17:05
 * Description: 获取俱乐部比赛列表
 */
@MessageHandler(id = CommonProto.Cmd.GetClubCompetitionListReq_VALUE)
public class GetClubCompetitionListHandler extends NetHandler {

    @Override
    public void onDate(Connection session, WSRequest request, WSResponse response) {
        response.setCmd(CommonProto.Cmd.GetClubCompetitionListRes_VALUE);

        String uid = session.getPlayerId();
        Player player = (Player) PlayerUtil.getOnlinePlayer(uid);
        Map<String, Object> map = new HashMap<>();
        // 俱乐部比赛信息列表

        map.put("contestInfo", contestInfoList(player));
        response.setData(map);
    }

    public static List<Object> contestInfoList( Player player){
        List<Object> list = new ArrayList<>();
        for (Contest contest : ContestService.getInstance().contestMap.values()) {
            if (contest != null&&contest.prop.getCompeteType() == 2) {
                list.add(contestInfoToMessageByUId(contest,player.getUid()));
            }
        }
        return list;
    }


    public static Map<String, Object> contestInfoToMessageByUId(Contest contest,String uid) {
        Map<String, Object> data = new HashMap<>();
        if (contest != null) {
            if (contest.prop.getCompeteType() == 2){
                data.put("contestId", contest.data.contestId);
                data.put("contestRule", contest.prop.contestSystem);
                data.put("contestLevel", contest.prop.getLevel());
                data.put("contestTime", contest.data.startTime);
                data.put("contestState", contest.state);
                String clubId = contest.prop.getClubId();
                data.put("clubId", clubId);
                //取出俱乐部列表
                String clubs = RedisUtil.get(RedisKeyConstant.CLUB_BASE);
                JSONObject jsonObject = JSONUtil.parseObj(clubs);
                JSONArray clubArray = jsonObject.getJSONArray("clubList");
                List<ZjClubPo> clubList = clubArray.toList(ZjClubPo.class);
                List<ZjClubPo> clubPoList = clubList.stream()
                        .filter(club -> club.getClubId().equals(clubId))
                        .collect(Collectors.toList());
                ZjClubPo clubPo = clubPoList.get(0);
                data.put("briefly", clubPo.getClubDescription());
                data.put("avatar", clubPo.getClubImg());
                data.put("name", clubPo.getClubName());
                data.put("max", 100);
                data.put("current", clubPo.getClubMemberNum());
                JSONArray rewards = contest.prop.rewards;
                List<Map<String, Object>> mapList1 = JsonToMapUtil.parseJsonArrayToMapList(rewards);
                data.put("rewards",mapList1);
                List<Map<String, Object>> mapList = ProtoUtil.contestRegisListToMessage(contest);
                data.put("regisList", mapList);
                data.put("isSign", false);
                for (Map<String, Object> map : mapList) {
                    if (map.get("id").equals(uid)){
                        data.put("isSign", true);
                    }
                }
            }
        }
        return data;
    }
}
