package com.incubator.game.handler.contest;

import cn.hutool.json.JSONArray;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.incubator.common.net.Connection;
import com.incubator.core.net.ws.NetHandler;
import com.incubator.core.net.ws.WSRequest;
import com.incubator.core.net.ws.WSResponse;
import com.incubator.game.constant.RedisKeyConstant;
import com.incubator.game.contest.Contest;
import com.incubator.game.contest.ContestService;
import com.incubator.game.data.jedis.RedisUtil;
import com.incubator.game.data.po.ZjClubPo;
import com.incubator.game.util.JsonToMapUtil;
import com.incubator.game.util.ProtoUtil;
import com.incubator.message.proto.CommonProto;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * Author:  xhb
 * Date:  2024-12-26 17:11
 * Description: 获取俱乐部联赛比赛列表
 */
public class GetClubLeagueMatchListHandler extends NetHandler {

    @Override
    public void onDate(Connection session, WSRequest request, WSResponse response) {
        response.setCmd(CommonProto.Cmd.GetContestListRes_VALUE);

        String uid = session.getPlayerId();
        Map<String, Object> map = new HashMap<>();
        // 官方比赛信息列表
        List<Object> list = new ArrayList<>();
        for (Contest contest : ContestService.getInstance().contestMap.values()) {
            if (contest != null) {
                list.add(contestInfoToMessageByUId(contest,uid));
            }
        }
        map.put("contestInfo", list);
        response.setData(map);
    }

    public static Map<String, Object> contestInfoToMessageByUId(Contest contest,String uid) {
        Map<String, Object> data = new HashMap<>();
        if (contest != null) {
            if (contest.prop.getCompeteType() == 1&&contest.prop.getClubMode()==2){
                data.put("raceId", contest.data.contestId);
                data.put("raceType", contest.prop.getLevel());
                data.put("raceTime", contest.data.startTime);
                data.put("raceStatus", contest.state);
                String clubId = contest.prop.getClubId();
                data.put("clubId", clubId);
                //取出俱乐部列表
                String clubs = RedisUtil.get(RedisKeyConstant.CLUB_BASE);
                JSONObject jsonObject = JSONUtil.parseObj(clubs);
                JSONArray clubArray = jsonObject.getJSONArray("clubList");
                List<ZjClubPo> clubList = clubArray.toList(ZjClubPo.class);
                List<ZjClubPo> clubPoList = clubList.stream()
                        .filter(club -> club.getClubId().equals(clubId))
                        .collect(Collectors.toList());
                ZjClubPo clubPo = clubPoList.get(0);
                data.put("briefly", clubPo.getClubDescription());
                data.put("avatar", clubPo.getClubImg());
                data.put("name", clubPo.getClubName());
                data.put("max", 100);
                data.put("current", clubPo.getClubMemberNum());
                JSONArray rewards = contest.prop.rewards;
                List<Map<String, Object>> mapList1 = JsonToMapUtil.parseJsonArrayToMapList(rewards);
                data.put("rewards",mapList1);
                List<Map<String, Object>> mapList = ProtoUtil.contestRegisListToMessage(contest);
                data.put("regisList", mapList);
                data.put("isSign", false);
                for (Map<String, Object> map : mapList) {
                    if (map.containsKey(uid)){
                        data.put("isSign", true);
                    }
                }
            }
        }
        return data;
    }
}
