package com.incubator.game.handler.contest;

import cn.hutool.json.JSONArray;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.incubator.common.MessageHandler;
import com.incubator.common.log4j.Log;
import com.incubator.common.net.Connection;
import com.incubator.core.net.ws.NetHandler;
import com.incubator.core.net.ws.WSRequest;
import com.incubator.core.net.ws.WSResponse;
import com.incubator.game.constant.RedisKeyConstant;
import com.incubator.game.contest.Contest;
import com.incubator.game.contest.ContestService;
import com.incubator.game.data.jedis.RedisUtil;
import com.incubator.game.data.po.ZjBackpackPO;
import com.incubator.game.player.Player;
import com.incubator.game.util.JsonToMapUtil;
import com.incubator.game.util.PlayerUtil;
import com.incubator.game.util.ProtoUtil;
import com.incubator.message.proto.CommonProto;

import java.util.*;

/**
 * Author:  xhb
 * Date:  2024-12-25 10:26
 * Description: 获取比赛列表处理器
 */
@MessageHandler(id = CommonProto.Cmd.GetContestListReq_VALUE)
public class GetContestListHandler extends NetHandler {
    @Override
    public void onDate(Connection session, WSRequest request, WSResponse response) {
        response.setCmd(CommonProto.Cmd.GetContestListRes_VALUE);

        String uid = session.getPlayerId();
        Player player = (Player) PlayerUtil.getOnlinePlayer(uid);
        Map<String, Object> map = new HashMap<>();
        map.put("contestInfo", contestInfoList(player));
        response.setData(map);
    }


    public static List<Object> contestInfoList(Player player) {
        List<Object> list = new ArrayList<>();
        for (Contest contest : ContestService.getInstance().contestMap.values()) {
            if (contest.prop != null && contest.prop.getCompeteType() == 0 && contest.prop.runStatus.equals("Y")) {
                list.add(contestInfoToMessageByUId(contest, player.getId()));
            }
        }
        return list;
    }

    public static Map<String, Object> contestInfoToMessageByUId(Contest contest, String uid) {
        Map<String, Object> data = new HashMap<>();
        if (contest != null) {
            data.put("contestId", contest.data.contestId);
            data.put("contestName", contest.prop.desc);
            data.put("contestType", contest.data.contestType);
            data.put("contestPlayType", contest.prop.gameType);
            data.put("contestLevel", contest.data.level);
            data.put("contestTime", contest.data.startTime);
            data.put("contestState", contest.state);
            data.put("contestPhotoUrl", contest.prop.url);
            data.put("contestSystem", contest.prop.contestSystem);
            JSONArray rewards = contest.prop.rewards;
            List<Map<String, Object>> mapList1 = JsonToMapUtil.parseJsonArrayToMapList(rewards);
            data.put("rewards", mapList1);
            data.put("regisCount", 0);
            data.put("regisMoney", "");
            data.put("regisMoneyType", "");
            List<Map<String, Object>> mapList = ProtoUtil.contestRegisListToMessage(contest);
            data.put("regisList", mapList);
            data.put("isSign", false);
            if (!mapList.isEmpty()){
                for (Map<String, Object> map : mapList) {
                    if (map.get("id").equals(uid)) {
                        data.put("isSign", true);
                    }
                }
            }
            //查询背包是否有参赛卷
            String userInfo = RedisUtil.get(RedisKeyConstant.USER_BASE + uid);
            JSONObject userObject = JSONUtil.parseObj(userInfo);
            JSONArray userBackpack = userObject.getJSONArray("userBackpack");
            List<ZjBackpackPO> zjBackpackPOS = userBackpack.toList(ZjBackpackPO.class);
            Optional<ZjBackpackPO> first = zjBackpackPOS.stream()
                    .filter(goods -> goods.getGoodsId().equals("402"))
                    .findFirst();
            if (first.isPresent()){
                data.put("isTicket", true);
            }else {
                data.put("isTicket", false);
            }
//            }
        }
        return data;
    }
}
