package com.incubator.game.handler.contest;

import cn.hutool.json.JSONObject;
import com.incubator.common.MessageHandler;
import com.incubator.common.log4j.Log;
import com.incubator.common.net.Connection;
import com.incubator.core.net.ws.NetHandler;
import com.incubator.core.net.ws.WSRequest;
import com.incubator.core.net.ws.WSResponse;
import com.incubator.game.contest.Contest;
import com.incubator.game.contest.ContestService;
import com.incubator.game.data.mq.RabbitMQUtil;
import com.incubator.game.player.Player;
import com.incubator.game.util.PlayerUtil;
import com.incubator.game.util.ProtoUtil;
import com.incubator.message.proto.CommonProto;

import java.math.BigDecimal;

/**
 * Author:  xhb
 * Date:  2025-01-04 0:38
 * Description: 加入候选人处理器
 */
@MessageHandler(id = CommonProto.Cmd.JoinCandidateReq_VALUE)
public class JoinCandidateHandler extends NetHandler {

    @Override
    public void onDate(Connection session, WSRequest request, WSResponse response) {
        response.setCmd(CommonProto.Cmd.JoinCandidateRes_VALUE);

        Player player = (Player) PlayerUtil.getOnlinePlayer(session.getPlayerId());
        if (player == null) {
            Log.info("创建房间失败，找不到玩家");
            response.setCode(CommonProto.Code.SYSTEM_ERR_VALUE);
            response.setMessage("操作失败...");
            return;
        }

        // 判断参数
        long contestId = request.getDataValue("contestId", Long.class, 0L);
        if (contestId <= 0L) {
            Log.info("参数错误");
            response.setCode(CommonProto.Code.PARAMETER_ERR_VALUE);
            response.setMessage("参数错误...");
            return;
        }

        // 比赛是否存在
        Contest contest = ContestService.getInstance().contestMap.getOrDefault(contestId, null);
        if (contest == null) {
            Log.info("比赛不存在或已结束");
            response.setCode(CommonProto.Code.CONTEST_NOT_EXIST_ERR_VALUE);
            response.setMessage("操作失败，比赛不存在或已结束...");
            return;
        }

        // 扣门票或者报名费
        String currencyType = "2";
        BigDecimal cost = contest.prop.getSubstituteCost();
        String errorMessage = "";

        if (player.data.diamond < cost.longValue()) {
            errorMessage = "钻石不足";

        }
        player.data.diamond -= cost.longValue();

        if (!errorMessage.isEmpty()) {
            Log.info(errorMessage);
            response.setCode(CommonProto.Code.BALANCE_NOT_ENOUGH_VALUE);
            response.setMessage("候补失败，您的" + errorMessage);
            return;
        }
        //报名成功 发送扣费消息
        JSONObject object = new JSONObject();
        object.put("type","JoinContest");
        object.put("userId",player.data.playerId);
        object.put("competeId",contest.data.contestId);
        object.put("goodsId",currencyType);
        object.put("goodsNum",cost);
        try {
            RabbitMQUtil.sendMessage(object.toString());
        } catch (Exception e) {
            e.printStackTrace();
        }


        // 报名加入候补队列
        contest = ContestService.getInstance().JoinCandidate(player, contest.data.contestId);

        // 正常返回
        response.setData(ProtoUtil.contestInfoToMessage(contest));

    }
}
