package com.incubator.game.handler.http;

import com.incubator.common.MessageHandler;
import com.incubator.core.net.http.HttpHandler;
import com.incubator.core.net.http.HttpRequest;
import com.incubator.core.net.http.HttpResponse;
import com.incubator.game.GGame;
import com.incubator.game.contest.Contest;
import com.incubator.game.contest.ContestService;
import com.incubator.game.data.data.ext.ContestExt;
import com.incubator.game.handler.contest.GetContestListHandler;
import com.incubator.game.player.GPlayer;
import com.incubator.game.player.Player;
import com.incubator.game.util.ProtoUtil;
import com.incubator.message.proto.CommonProto;

import java.util.HashMap;
import java.util.Map;

/**
 * Author:  xhb
 * Date:  2025-01-07 11:12
 */
@MessageHandler(httpPath = "/updateCompete")
public class UpdateCompeteHandler extends HttpHandler {

    /**
     * 修改官方赛事回执信息
     **/
    @Override
    public HttpResponse onDate(HttpRequest request, Map<String, String> params) throws Exception {
        HttpResponse httpResponse = new HttpResponse();
        httpResponse.setCode(200);
        httpResponse.setMessage("成功！");
        long contestId = Long.parseLong(params.get("iD"));
        Contest contest = ContestService.getInstance().contestMap.getOrDefault(contestId, null);
        if (contest == null) {
            httpResponse.setMessage("比赛不存在");
            return httpResponse;
        }
        ContestExt prop = contest.prop;
        String runStatus = params.get("runStatus");
        prop.setRunStatus(runStatus);
        if (runStatus.equals("Y")){//如果是Y 开启主线程处理
            contest.startContestTask();
        }else { // 如果不是运行状态则踢出所有玩家告知当前比赛异常结束
            contest.actors.forEach((playerId, player1) -> {
                player1.data.contestId = 0;
                player1.data.roomId = 0;
                Map<String, Object> data = ProtoUtil.contestInfoToMessage(contest);
                data.put("message", "当前比赛异常结束,参加的费用将会原路返回");
                player1.receive(CommonProto.Cmd.EndOfRace_VALUE, data);
            });
        }
        //给在线玩家发送比赛列表
        Map<String, GPlayer> onlinePlayers = GGame.onlinePlayers;
        for (GPlayer player : onlinePlayers.values()) {
            Map<String, Object> map = new HashMap<>();
            // 官方比赛信息列表
            map.put("contestInfo", GetContestListHandler.contestInfoList((Player) player));
            player.receive(CommonProto.Cmd.GetContestListRes_VALUE, map);
        }
        return httpResponse;
    }
}
