package com.incubator.game.handler.login;

import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import cn.hutool.jwt.JWT;
import cn.hutool.jwt.JWTUtil;
import com.incubator.common.MessageHandler;
import com.incubator.common.log4j.Log;
import com.incubator.common.net.Connection;
import com.incubator.core.net.ws.NetHandler;
import com.incubator.core.net.ws.WSRequest;
import com.incubator.core.net.ws.WSResponse;
import com.incubator.game.constant.RedisKeyConstant;
import com.incubator.game.data.jedis.RedisUtil;
import com.incubator.game.data.po.PlayerInfoPO;
import com.incubator.game.data.po.ZjUserLobbyPo;
import com.incubator.game.player.Player;
import com.incubator.game.util.ProtoUtil;
import com.incubator.game.util.PlayerUtil;
import com.incubator.message.proto.CommonProto;

import java.util.HashMap;
import java.util.Map;

/**
 * 登录游戏服
 *
 * @author johnc
 */
@MessageHandler(id = CommonProto.Cmd.LoginReq_VALUE)
public class LoginGameHandler extends NetHandler {

	@Override
	public void onDate(Connection session, WSRequest request, WSResponse response) {
		response.setCmd(CommonProto.Cmd.LoginRes_VALUE);

		// 判断参数
		String token = request.getDataValue("token", String.class, "");
//		String userId = request.getDataValue("userId", String.class, "");
		if (token==null) {
			//如果token为空则重新登录
			Log.info("登录token为空");
			response.setCode(CommonProto.Code.SYSTEM_ERR_VALUE);
			return;
		}
		final JWT jwt = JWTUtil.parseToken(token);
		JSONObject payloads = jwt.getPayloads();
		System.out.println("payloads.get(\"userId\") = " + payloads.get("userId"));
		String userId = (String) payloads.get("userId");
		// 根据token获取用户信息
		String userInfo = RedisUtil.get(RedisKeyConstant.USER_BASE + userId);
		JSONObject jsonObject = JSONUtil.parseObj(userInfo);
		Object lobbyUserObj = jsonObject.get("lobbyUser");
		ZjUserLobbyPo lobbyUser = JSONUtil.toBean((JSONObject) lobbyUserObj, ZjUserLobbyPo.class);
		System.out.println("用户信息为 ============== " + userInfo);
		System.out.println("大厅信息为 ============== " + lobbyUserObj);

//		String username = request.getDataValue("username", String.class, "");

		// 创建player
//		String player_id = UUID.randomUUID().toString();
		PlayerInfoPO playerInfoPO = new PlayerInfoPO();
		playerInfoPO.playerId = lobbyUser.getUserId();
		playerInfoPO.name = lobbyUser.getNickName();
		playerInfoPO.avatarUrl = lobbyUser.getAvatarUrl();
		playerInfoPO.sex = lobbyUser.getSex().equals("密")?0:1;
		playerInfoPO.exp = lobbyUser.getMasterScore();
		playerInfoPO.maxExp = lobbyUser.getMasterScoreLevel();
		playerInfoPO.vipLevel = 1;
		playerInfoPO.masterScore = lobbyUser.getMasterScore();
		playerInfoPO.masterScoreLevel = lobbyUser.getHonorLevel();
		playerInfoPO.gold = lobbyUser.getGoldCoin();
		playerInfoPO.diamond = lobbyUser.getDiamond();
		playerInfoPO.accumulatedPoints = 10000L;
		playerInfoPO.startTimeVip = "";
		playerInfoPO.endTimeVip = "";
		playerInfoPO.gps = "";
		playerInfoPO.gpsAddr = "";
		playerInfoPO.foca = lobbyUser.getFoca();
		playerInfoPO.ranking = lobbyUser.getRanking();
		playerInfoPO.figureId = 1;
		playerInfoPO.realName = lobbyUser.getName();
		playerInfoPO.phoneNum = "13800000001";
		playerInfoPO.idCard = lobbyUser.getCertNo();
		playerInfoPO.friendMasterNum = lobbyUser.getFriendNum();
		playerInfoPO.friendMasterRate = lobbyUser.getFriendChance();
		playerInfoPO.tradPoints = lobbyUser.getTraditionMark();
		playerInfoPO.tradPointsRate = lobbyUser.getTraditionChance();
		playerInfoPO.tradMatchNum = lobbyUser.getNormalMark();
		playerInfoPO.tradMatchRate = lobbyUser.getNormalChance();
		playerInfoPO.noReshNum = lobbyUser.getNoShufflingNum();
		playerInfoPO.noReshRate = lobbyUser.getNoShufflingChance();
		playerInfoPO.matchPoints = 0;
//		playerInfoPO.token = token;
		Player player = PlayerUtil.createPlayer(playerInfoPO);

		// 绑定session
		player.bind(session, lobbyUser.getUserId(), player.getId(), token);
		Log.info("玩家进入游戏 playerId={}", player.getId());

		// 登录数据
		player.onLogin();

		// 正常返回
		Map<String, Object> map = new HashMap<>();
		map.put("playerInfo", ProtoUtil.playerInfoToMessage(player));
		// 玩家绑定的房间号
		map.put("roomId", player.data.roomId);
		// 玩家绑定的官方比赛id
		map.put("contestId", player.data.contestId);
		// 官方比赛信息列表
		map.put("contestInfo", ProtoUtil.contestListToMessage());

		//奖励弹窗
		String eventStr = RedisUtil.get(RedisKeyConstant.ZJ_USER_EVENT_KEY + userId);
		map.put("eventList", ProtoUtil.eventToJsonArray(eventStr));


		response.setData(map);
	}
}
