package com.incubator.game.handler.login;

import com.alibaba.fastjson2.JSONObject;
import com.incubator.common.MessageHandler;
import com.incubator.common.log4j.Log;
import com.incubator.common.net.Connection;
import com.incubator.common.util.RandomUtil;
import com.incubator.core.net.ws.NetHandler;
import com.incubator.core.net.ws.WSRequest;
import com.incubator.core.net.ws.WSResponse;
import com.incubator.game.constant.RedisKeyConstant;
import com.incubator.game.constant.RoomStatus;
import com.incubator.game.constant.WkServerStatus;
import com.incubator.game.data.jedis.RedisUtil;
import com.incubator.game.player.Player;
import com.incubator.game.room.Room;
import com.incubator.game.room.RoomService;
import com.incubator.game.util.PlayerUtil;
import com.incubator.game.util.RoomUtil;
import com.incubator.message.proto.CommonProto;

import java.util.*;

/**
 * 创建房间或加入前分服请求
 *
 * @author johnc
 */
@MessageHandler(id = CommonProto.Cmd.SubRoomReq_VALUE)
public class SubRoomHandler extends NetHandler {

	@Override
	public void onDate(Connection session, WSRequest request, WSResponse response) {
		response.setCmd(CommonProto.Cmd.SubRoomRes_VALUE);

		Player player = (Player) PlayerUtil.getOnlinePlayer(session.getPlayerId());
		if (player == null) {
			Log.info("创建房间失败，找不到玩家");
			response.setCode(CommonProto.Code.SYSTEM_ERR_VALUE);
			response.setMessage("操作失败");
			return;
		}
		// 玩家是否存在房间
		HashMap<String, RoomStatus> wkToRedis = RedisUtil.getRoomToRedis(RedisKeyConstant.WK_ROOM_MAP);
		ArrayList<String> pls = new ArrayList<>();
		for (Map.Entry<String, RoomStatus> entry : wkToRedis.entrySet()) {
			String key = entry.getKey();
			RoomStatus value = entry.getValue();
			pls.addAll(value.getPlayers());
		}

		if (pls.contains(player.getId())) {
			Log.info("玩家已在房间，不可创建房间");
			response.setCode(CommonProto.Code.PLAYER_ROOM_EXIST_ERR_VALUE);
			response.setMessage("操作失败");
			return;
		}
		// 判断参数
		long roomId = request.getDataValue("roomId", Long.class, 0L);
		// 判断是否是进入赛事
//		boolean isEvents = request.getDataValue("isEvents", Boolean.class, false);
//		// 进入赛事
//		if (isEvents){
//			// 赛事房间规则
//			HashMap<String, WkServerStatus> wkServes = RedisUtil.getWkToRedis(RedisKeyConstant.WK_SERVE);
//			for (Map.Entry<String, WkServerStatus> entry : wkServes.entrySet()) {
//				String type1 = entry.getValue().getType();
//				if (type1.equals(String.valueOf(1))){
//					WkServerStatus value = entry.getValue();
//					Map<String, Object> map = new HashMap<>();
//					map.put("wkIp", value.getName());
//					// 正常返回
//					response.setData(map);
//					return;
//				}
//			}
//		}


		if (roomId > 0){
			//加入房间
			Room room = RoomService.getInstance().getRoom(roomId);
			HashMap<String, WkServerStatus> wkServes = RedisUtil.getWkToRedis(RedisKeyConstant.WK_ROOM_MAP);
			WkServerStatus wkServerStatus = wkServes.get(String.valueOf(roomId));
			Map<String, Object> map = new HashMap<>();
			map.put("roomId", roomId);
			map.put("isCreate", false);
			map.put("wkIp", wkServerStatus.getName());
			map.put("type", room.data.type);
			//修改房间信息
			RoomStatus roomStatus = wkToRedis.get(String.valueOf(roomId));
			List<String> players = roomStatus.getPlayers();
			players.add(player.getId());
			roomStatus.setPlayers(players);
			RedisUtil.saveRoomToRedis(RedisKeyConstant.WK_ROOM_MAP, roomStatus);
			// 正常返回
			response.setData(map);
		}else {

			// 房间规则
			Map<String, Object> map = new HashMap<>();
			map.put("roomId", randomRoomId(wkToRedis));
			//保存房间信息
			RoomStatus roomStatus = new RoomStatus();
			roomStatus.setRoomId((Long) map.get("roomId"));
			ArrayList<String> list = new ArrayList<>();
			list.add(player.getId());
			roomStatus.setPlayers(list);
			roomStatus.setRoomType(0);
			//查询服务器信息
			HashMap<String, WkServerStatus> wkServes = RedisUtil.getWkToRedis(RedisKeyConstant.WK_SERVE);
			ArrayList<WkServerStatus> wkServerStatuses = new ArrayList<>();
			for (Map.Entry<String, WkServerStatus> entry : wkServes.entrySet()) {
				String type1 = entry.getValue().getType();
				if (type1.equals(String.valueOf(0))){
					wkServerStatuses.add(entry.getValue());
				}
			}
			String name = selectLeastPopulatedServer(wkServerStatuses).get().getName();
			map.put("wkIp", name);
			map.put("isCreate", true);
			roomStatus.setName(name);
			// 保存房间信息
			RedisUtil.saveRoomToRedis(RedisKeyConstant.WK_ROOM_MAP, roomStatus);
			// 正常返回
			response.setData(map);
		}


	}
	/**
	 * 选择在线人数最少的服务器。
	 *
	 * @return 最少在线人数的服务器，如果不存在任何在线服务器，则返回 Optional.empty()。
	 */
	public  static Optional<WkServerStatus> selectLeastPopulatedServer(List<WkServerStatus> fetchOnlineServers){
		// 使用流式 API 找到在线人数最少的服务器
		return fetchOnlineServers.stream()
				.min(Comparator.comparingInt(WkServerStatus::getNum));
	}
	/**
	 * 生成唯一6位数房间号
	 *
	 * @return
	 */
	public static long randomRoomId(HashMap<String, RoomStatus> wkToRedis ) {
		if (wkToRedis == null || wkToRedis.isEmpty()) {
			return RandomUtil.randomSixDigits();
		}
		long roomNumber;
		// 循环生成，直到找到一个唯一的房间号
		do {
			roomNumber = RandomUtil.randomSixDigits();
		} while (wkToRedis.containsKey(roomNumber));
		return roomNumber;
	}

	/**
	 * 生成唯一7位数房间号
	 *
	 * @return
	 */
	public static long randomRoomId7(HashMap<String, RoomStatus> wkToRedis ) {
		if (wkToRedis == null || wkToRedis.isEmpty()) {
			return  RandomUtil.randomSevenDigits();
		}
		long roomNumber;
		// 循环生成，直到找到一个唯一的房间号
		do {
			roomNumber = RandomUtil.randomSevenDigits();
		} while (wkToRedis.containsKey(roomNumber));
		return roomNumber;
	}
}
