package com.incubator.game.handler.master;

import cn.hutool.json.JSONArray;
import com.incubator.common.MessageHandler;
import com.incubator.common.log4j.Log;
import com.incubator.common.net.Connection;
import com.incubator.core.net.ws.NetHandler;
import com.incubator.core.net.ws.WSRequest;
import com.incubator.core.net.ws.WSResponse;
import com.incubator.game.data.jedis.RedisUtil;
import com.incubator.game.data.model.MasterRankModel;
import com.incubator.game.player.Player;
import com.incubator.game.util.ObjectMapperUtil;
import com.incubator.game.util.PlayerUtil;
import com.incubator.message.proto.CommonProto;
import java.util.*;

/**
 * Author:  xhb
 * Date:  2025-01-13 16:16
 * 大师分排名
 */
@MessageHandler(id = CommonProto.Cmd.MasterRankReq_VALUE)
public class MasterRankHandler extends NetHandler {

    @Override
    public void onDate(Connection session, WSRequest request, WSResponse response) {
        response.setCmd(CommonProto.Cmd.MasterRankRes_VALUE);
        String userId = session.getPlayerId();
        Player player =(Player) PlayerUtil.getOnlinePlayer(userId);
        if (player == null) {
            Log.error("玩家不存在: {}", userId);
            return;
        }
        String date = request.getDataValue("date", String.class, null);//时间
        String type = request.getDataValue("type", String.class, null);//时间
        HashMap<String, Object> map = new HashMap<>();
        handleRankRequest(type, date, map,player);
        response.setData(map);
    }


    public void handleRankRequest(String type, String date, Map<String, Object> map, Player player) {
        try {
            RankType rankType = RankType.valueOf(type.toUpperCase());
            String rankData = RedisUtil.get(rankType.getRedisKey() + (rankType == RankType.DAY && date != null ? date : ""));
            if (rankData != null) {
                JSONArray objects = new JSONArray(rankData);
                if (objects.size() > 0) {
                    List<MasterRankModel> masterRankModels = objects.toList(MasterRankModel.class);
                    Optional<MasterRankModel> first = masterRankModels.stream()
                            .filter(masterRankModel -> masterRankModel.getUserId().equals(player.data.playerId))
                            .findFirst();
                    if (first.isPresent()) {
                        MasterRankModel masterRankModel = first.get();
                        map.put("myRank", masterRankModel.getRank());
                        map.put("myNickName", masterRankModel.getNickName());
                        map.put("myAvatar", masterRankModel.getAvatar());
                        map.put("myScore", masterRankModel.getScore());
                        map.put("myUserId", masterRankModel.getUserId());
                    } else {
                        map.put("myRank", "未上榜");
                        map.put("myNickName", player.data.name);
                        map.put("myAvatar", player.data.avatarUrl);
                        map.put("myScore", player.data.masterScore);
                        map.put("myUserId", player.data.playerId);
                    }
                    map.put(rankType.getType() + "Rank", ObjectMapperUtil.convertListToMapList(masterRankModels));
                }
            }
        } catch (IllegalArgumentException e) {
            //无效的rankType
            Log.error("无效的日期类型:{}", type);
        } catch (Exception e) {
            // 处理其他异常
            Log.error("处理排名请求时出错:{}", e.getMessage());
        }
    }
}
