package com.incubator.game.handler.order;

import cn.hutool.json.JSONArray;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.incubator.common.MessageHandler;
import com.incubator.common.net.Connection;
import com.incubator.core.net.ws.NetHandler;
import com.incubator.core.net.ws.WSRequest;
import com.incubator.core.net.ws.WSResponse;
import com.incubator.game.constant.RedisKeyConstant;
import com.incubator.game.data.jedis.RedisUtil;
import com.incubator.game.data.po.ZjMemberAddrPO;
import com.incubator.game.data.po.ZjOrderPO;
import com.incubator.game.util.ObjectMapperUtil;
import com.incubator.message.proto.CommonProto;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * Author:  xhb
 * Date:  2024-12-10 14:57
 */
@MessageHandler(id = CommonProto.Cmd.GetOrderListReq_VALUE)
public class GetOrderList extends NetHandler {

    @Override
    public void onDate(Connection session, WSRequest request, WSResponse response) {
        response.setCmd(CommonProto.Cmd.GetOrderListRes_VALUE);
        String userId = session.getPlayerId();
        String date = request.getDataValue("date", String.class, "");
        String userInfo = RedisUtil.get(RedisKeyConstant.USER_BASE + userId);
        JSONObject userObject = JSONUtil.parseObj(userInfo);
        JSONArray userOrder = userObject.getJSONArray("orderList");
        if (userOrder == null || userOrder.isEmpty()) {
            response.setData(new HashMap<>());
            return;
        }
        List<ZjOrderPO> zjOrderPOList = userOrder.toList(ZjOrderPO.class);

        // 使用Stream API提取orderType为1的数据
        List<ZjOrderPO> orderType1List = zjOrderPOList.stream()
                .filter(order -> order.getOrderType().equals("1")&&order.getCreateTime().contains(date))
                .collect(Collectors.toList());

        // 使用Stream API提取orderType为2的数据
        List<ZjOrderPO> orderType2List = zjOrderPOList.stream()
                .filter(order -> order.getOrderType().equals("2")&&order.getCreateTime().contains(date))
                .collect(Collectors.toList());
        
        List<Map<String, Object>> maps1 = ObjectMapperUtil.convertListToMapList(orderType1List);
        List<Map<String, Object>> maps2 = ObjectMapperUtil.convertListToMapList(orderType2List);
        Map<String,Object> map = new HashMap<>();
        map.put("orderList",maps1);
        map.put("TOrderList",maps2);
        response.setData(map);
    }
}
