package com.incubator.game.handler.room;

import com.alibaba.fastjson2.JSONObject;
import com.incubator.common.MessageHandler;
import com.incubator.common.log4j.Log;
import com.incubator.common.net.Connection;
import com.incubator.core.net.ws.NetHandler;
import com.incubator.core.net.ws.WSRequest;
import com.incubator.core.net.ws.WSResponse;
import com.incubator.game.player.Player;
import com.incubator.game.room.Room;
import com.incubator.game.room.RoomService;
import com.incubator.game.util.PlayerUtil;
import com.incubator.game.util.RoomUtil;
import com.incubator.message.proto.CommonProto;

/**
 * 创建房间请求
 *
 * @author johnc
 */
@MessageHandler(id = CommonProto.Cmd.CreateRoomReq_VALUE)
public class CreatRoomHandler extends NetHandler {

	@Override
	public void onDate(Connection session, WSRequest request, WSResponse response) {
		response.setCmd(CommonProto.Cmd.CreateRoomRes_VALUE);

		Player player = (Player) PlayerUtil.getOnlinePlayer(session.getPlayerId());
		if (player == null) {
			Log.info("创建房间失败，找不到玩家");
			response.setCode(CommonProto.Code.SYSTEM_ERR_VALUE);
			response.setMessage("操作失败");
			return;
		}

		// 玩家是否存在房间
		Room room = RoomService.getInstance().getRoomByPlayerId(player.getId());
		if (room != null) {
			Log.info("玩家已在房间，不可创建房间");
			response.setCode(CommonProto.Code.PLAYER_ROOM_EXIST_ERR_VALUE);
			response.setMessage("操作失败");
			return;
		}

		// 玩法类型 1掼蛋 2转蛋 3复式掼蛋
		int type = request.getDataValue("ty", Integer.class, 0);

		// 玩法 mode 1连续打 2按局数 3按时间
		int mode = request.getDataValue("mode", Integer.class, 0);
		// 玩法具体 modev  14A必打
		int modev = request.getDataValue("modev", Integer.class, 0);


		// 胜利规则 vic 1有头游无末游过A结束 2游头游过A结束 33局不过A判负结束 43局不过A用2开始
		int vic = request.getDataValue("vic", Integer.class, 0);
		// 进贡 jing 0进贡 1不进贡
		int jing = request.getDataValue("jing", Integer.class, -1);
		// 升级 lv  1-123级 2-124级 3-135级 4-246级
		int lv = request.getDataValue("lv", Integer.class, 1);
		// 套圈 lp 0套圈 1不套圈
		int lp = request.getDataValue("lp", Integer.class, -1);

		// 队友 team 0固定队友 1随机搭档 2均匀分配
		int team = request.getDataValue("team", Integer.class, -1);
		// 级牌 tc 0随机打几 1固定打2
		int tc = request.getDataValue("tc", Integer.class, -1);
		// 翻倍 dv 0不翻倍 1加倍 2翻倍
		int dv = request.getDataValue("dv", Integer.class, -1);
		// 积分 pv 0:123 1:124
		int pv = request.getDataValue("pv", Integer.class, -1);


		// 记牌器 count 0允许 1禁用
		int count = request.getDataValue("count", Integer.class, -1);
		// 超时 ot 1过 2等待 3托管
		int ot = request.getDataValue("ot", Integer.class, 0);
		// 语音 vc 0允许 1禁用
		int vc = request.getDataValue("vc", Integer.class, -1);
		// 旁观 pgr 1第一视角 2上帝视角 3上帝&第一视角 4旁观视角
		int pgr = request.getDataValue("pgr", Integer.class, 0);
//		long roomId = request.getDataValue("roomId", Long.class, 0L);

//		if (roomId == 0) {
//			Log.info("房间号空");
//			response.setCode(CommonProto.Code.PLAYER_ROOM_EXIST_ERR_VALUE);
//			response.setMessage("房间号空");
//			return;
//		}
		// 房间规则
		JSONObject jsonData;

		switch (type) {
			case 1:
				// 掼蛋
				jsonData = RoomUtil.gdRoomRule(type, mode, modev, lv, vic, jing, lp);
				break;
			case 2:
				// 转蛋
				if (mode == 1) {
					Log.info("转蛋参数错误，转蛋没有连续打");
					response.setCode(CommonProto.Code.PARAMETER_ERR_VALUE);
					response.setMessage("转蛋参数错误...");
					return;
				}
				jsonData = RoomUtil.zdRoomRule(type, mode, modev, lv, team, tc, dv, pv);
				break;
			case 3:
				// 复式掼蛋
				if (mode == 1 || mode == 3) {
					Log.info("复式掼蛋参数错误，复式掼蛋没有连续打和按时间");
					response.setCode(CommonProto.Code.PARAMETER_ERR_VALUE);
					response.setMessage("复式掼蛋参数错误...");
					return;
				}
				jsonData = RoomUtil.fsgdRoomRule(type, mode, modev, lv, vic, lp, tc);
				// 房间最大局数
				jsonData.put("maxRound", 8);
				break;
			default:
				Log.warn("创建房间 - 未知玩法类型 : {}", type);
				response.setCode(CommonProto.Code.PLAYER_ROOM_EXIST_ERR_VALUE);
				response.setMessage("操作失败");
				return;
		}

		// 记牌器 0允许 1禁用
		jsonData.put("recordType", count);
		// 超时规则 1:过牌 2:待机 3:托管
		jsonData.put("timeoutType", ot);
		// 语音规则 0允许 1禁用
		jsonData.put("voiceType", vc);
		// 观看规则 1第一视角 2上帝视角 3上帝&第一视角 4禁用
		jsonData.put("watchType", pgr);

        // 人数模式 2:2人 4:4人
		jsonData.put("playerNum", 4);
		// 房间类型 0:好友房 1:官方比赛房
		jsonData.put("type", 0);
		// 房间id
		jsonData.put("roomId", RoomService.getInstance().randomRoomId());

		// 创建房间
		room = RoomService.getInstance().creatRoom(player, type, jsonData);

		if (room == null) {
			Log.info("系统错误");
			response.setCode(CommonProto.Code.SYSTEM_ERR_VALUE);
			response.setMessage("操作失败");
			return;
		}

		// 玩家绑定房间
		player.data.roomId = room.data.roomId;

		// 正常返回
		response.setData(room.roomToMessage(player, null));
	}
}
