package com.incubator.game.handler.room;

import com.incubator.common.MessageHandler;
import com.incubator.common.log4j.Log;
import com.incubator.common.net.Connection;
import com.incubator.core.net.ws.NetHandler;
import com.incubator.core.net.ws.WSRequest;
import com.incubator.core.net.ws.WSResponse;
import com.incubator.game.player.Player;
import com.incubator.game.util.GMStruct;
import com.incubator.game.util.GMUtil;
import com.incubator.game.util.PlayerUtil;
import com.incubator.game.util.ProtoUtil;
import com.incubator.message.proto.CommonProto;

/**
 * 获取玩法规则信息 请求
 *
 * @author johnc
 */
@MessageHandler(id = CommonProto.Cmd.RoomRuleInfoReq_VALUE)
public class GetRoomRuleInfoHandler extends NetHandler {

	@Override
	public void onDate(Connection session, WSRequest request, WSResponse response) {
		response.setCmd(CommonProto.Cmd.RoomRuleInfoRes_VALUE);

		Player player = (Player) PlayerUtil.getOnlinePlayer(session.getPlayerId());
		if (player == null) {
			Log.info("创建房间失败，找不到玩家");
			response.setCode(CommonProto.Code.SYSTEM_ERR_VALUE);
			response.setMessage("操作失败...");
			return;
		}

		// 判断参数
		int type = request.getDataValue("ty", Integer.class, 0);
		if (type <= 0) {
			Log.info("参数错误");
			response.setCode(CommonProto.Code.PARAMETER_ERR_VALUE);
			response.setMessage("参数错误");
			return;
		}

		GMStruct.GameRule gameRule = null;
		// 获取玩法类型规则
		switch (type) {
			case 1:
				gameRule = GMUtil.getRule(type);
				break;
			default:
				Log.info("未知玩法类型...{}", type);
				response.setCode(CommonProto.Code.PARAMETER_ERR_VALUE);
				response.setMessage("未知玩法类型");
				return;
		}

		if (gameRule == null) {
			Log.info("获取规则错误");
			response.setCode(10);
			response.setMessage("获取规则错误");
			return;
		}

		// 正常返回
		response.setData(ProtoUtil.gameRuleToMessage(gameRule));
	}
}
