package com.incubator.game.handler.room;

import com.incubator.common.MessageHandler;
import com.incubator.common.log4j.Log;
import com.incubator.common.net.Connection;
import com.incubator.core.net.ws.NetHandler;
import com.incubator.core.net.ws.WSRequest;
import com.incubator.core.net.ws.WSResponse;
import com.incubator.game.player.Player;
import com.incubator.game.room.Room;
import com.incubator.game.room.RoomService;
import com.incubator.game.util.PlayerUtil;
import com.incubator.message.proto.CommonProto;

/**
 * 加入房间请求
 *
 * @author johnc
 */
@MessageHandler(id = CommonProto.Cmd.JoinRoomReq_VALUE)
public class JoinRoomHandler extends NetHandler {

	@Override
	public void onDate(Connection session, WSRequest request, WSResponse response) {
		response.setCmd(CommonProto.Cmd.JoinRoomRes_VALUE);

		Player player = (Player) PlayerUtil.getOnlinePlayer(session.getPlayerId());
		if (player == null) {
			Log.info("创建房间失败，找不到玩家");
			response.setCode(CommonProto.Code.SYSTEM_ERR_VALUE);
			response.setMessage("操作失败...");
			return;
		}

		// 判断参数
		long roomId = request.getDataValue("roomId", Long.class, 0L);
		if (roomId <= 0) {
			Log.info("参数错误");
			response.setCode(CommonProto.Code.PARAMETER_ERR_VALUE);
			response.setMessage("您输入的房间号不存在");
			return;
		}

		// 加入的房间是否存在
		Room room = RoomService.getInstance().getRoom(roomId);
		if (room == null) {
			Log.info("房间已解散");
			response.setCode(CommonProto.Code.ROOM_NOT_EXIST_ERR_VALUE);
			response.setMessage("房间已解散");
			return;
		}

		// 判断玩家绑定房间
		if (player.data.roomId > 0 && player.data.roomId != roomId) {
			Log.info("不可重复加入其他房间");
			response.setCode(CommonProto.Code.JOIN_ROOM_REPEATEDLY_ERR_VALUE);
			response.setMessage("操作失败，不可重复加入其他房间...");
			return;
		}

		// 判断人数是否已满
		if (!room.actors.containsKey(player.getId()) && room.actors.size() >= room.data.maxNum) {
			Log.info("房间人数已满");
			response.setCode(CommonProto.Code.JOIN_ROOM_REPEATEDLY_ERR_VALUE);
			response.setMessage("加入失败,当前房间人数已满");
			return;
		}

		// 执行玩家加入房间逻辑
		if (!room.actors.containsKey(player.getId())) {
            room.joinRoom(player);
        }
		response.setMessage("加入成功");
        // 正常返回
		response.setData(room.roomToMessage(player, null));
	}
}
