package com.incubator.game.handler.room;

import com.incubator.common.MessageHandler;
import com.incubator.common.log4j.Log;
import com.incubator.common.net.Connection;
import com.incubator.core.net.ws.NetHandler;
import com.incubator.core.net.ws.WSRequest;
import com.incubator.core.net.ws.WSResponse;
import com.incubator.game.player.Player;
import com.incubator.game.room.Room;
import com.incubator.game.room.RoomService;
import com.incubator.game.util.PlayerUtil;
import com.incubator.message.proto.CommonProto;

import java.util.HashMap;

/**
 * 玩家离开房间 请求
 *
 * @author johnc
 */
@MessageHandler(id = CommonProto.Cmd.LeaveRoomReq_VALUE)
public class LeaveRoomHandler extends NetHandler {

    @Override
    public synchronized void onDate(Connection session, WSRequest request, WSResponse response) {
        response.setCmd(CommonProto.Cmd.LeaveRoomRes_VALUE);

        Player player = (Player) PlayerUtil.getOnlinePlayer(session.getPlayerId());
        if (player == null) {
            Log.info("离开房间失败，找不到玩家");
            response.setCode(CommonProto.Code.SYSTEM_ERR_VALUE);
            response.setMessage("操作失败...");
            return;
        }

        // 判断玩家绑定房间
        if (player.data.roomId <= 0) {
            Log.info("离开房间失败，玩家不在房间");
            response.setCode(CommonProto.Code.PLAYER_ROOM_NOT_EXIST_ERR_VALUE);
            response.setMessage("操作失败，玩家不在房间...");
            return;
        }

        // 玩家房间是否存在
        Room room = RoomService.getInstance().getRoom(player.data.roomId);
        if (room == null) {
            Log.info("离开房间失败，玩家房间已解散");
            response.setCode(CommonProto.Code.ROOM_NOT_EXIST_ERR_VALUE);
            response.setMessage("操作失败，玩家房间已解散...");
            return;
        }

        // 当前状态是否可操作
//        if (room.state != 0 || player.data.state != 0) {
//            Log.info("离开房间失败，游戏已开始...");
//            response.setCode(7);
//            response.setMessage("操作失败，游戏已开始...");
//            return;
//        }

        // 离开房间
        room.leaveRoom(player);

        // 正常返回
        response.setData(new HashMap<>());
    }
}
