package com.incubator.game.handler.room;

import com.incubator.common.MessageHandler;
import com.incubator.common.log4j.Log;
import com.incubator.common.net.Connection;
import com.incubator.core.net.ws.NetHandler;
import com.incubator.core.net.ws.WSRequest;
import com.incubator.core.net.ws.WSResponse;
import com.incubator.game.player.Player;
import com.incubator.game.room.Room;
import com.incubator.game.room.RoomService;
import com.incubator.game.util.PlayerUtil;
import com.incubator.message.proto.CommonProto;

import java.util.Map;
import java.util.Objects;

/**
 * 准备请求
 *
 * @author johnc
 */
@MessageHandler(id = CommonProto.Cmd.RoomReadyReq_VALUE)
public class ReadyHandler extends NetHandler {

	@Override
	public void onDate(Connection session, WSRequest request, WSResponse response) {
		response.setCmd(CommonProto.Cmd.RoomReadyRes_VALUE);

		Player player = (Player) PlayerUtil.getOnlinePlayer(session.getPlayerId());
		if (player == null) {
			Log.info("创建房间失败，找不到玩家");
			response.setCode(CommonProto.Code.SYSTEM_ERR_VALUE);
			response.setMessage("操作失败...");
			return;
		}

		// 判断玩家绑定房间
		if (player.data.roomId <= 0) {
			Log.info("准备失败，玩家不在房间");
			response.setCode(CommonProto.Code.PLAYER_ROOM_NOT_EXIST_ERR_VALUE);
			response.setMessage("操作失败，玩家不在房间...");
			return;
		}

		// 玩家房间是否存在
		Room room = RoomService.getInstance().getRoom(player.data.roomId);
		if (room == null) {
			Log.info("玩家房间已解散");
			response.setCode(CommonProto.Code.ROOM_NOT_EXIST_ERR_VALUE);
			response.setMessage("操作失败，玩家房间已解散...");
			return;
		}

		// 房间是否已开始
		if (room.state != 0) {
			Log.info("玩家房间已开始，不可准备");
			response.setCode(CommonProto.Code.ROOM_NOT_EXIST_ERR_VALUE);
			response.setMessage("玩家房间已开始，不可准备...");
		}

		// 设置玩家状态
		player.data.state = 1;

		// 广播房间其他3个玩家
		for (Player tmPlayer : room.actors.values()) {
			if (tmPlayer != null && !Objects.equals(tmPlayer.getId(), player.getId())) {
				Map<String, Object> data = room.roomToMessage(tmPlayer, null);
				tmPlayer.receive(CommonProto.Cmd.ReadyStateUpdate_VALUE, data);
			}
		}

		// 正常返回
		response.setData(room.roomToMessage(player, null));
	}
}
