package com.incubator.game.handler.user;

import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.incubator.common.MessageHandler;
import com.incubator.common.net.Connection;
import com.incubator.core.net.ws.NetHandler;
import com.incubator.core.net.ws.WSRequest;
import com.incubator.core.net.ws.WSResponse;
import com.incubator.game.constant.RedisKeyConstant;
import com.incubator.game.data.entity.ZjUser;
import com.incubator.game.data.jedis.RedisUtil;
import com.incubator.game.data.mq.RabbitMQUtil;
import com.incubator.game.data.po.ZjUserLobbyPo;
import com.incubator.message.proto.CommonProto;

/**
 * Author:  xhb
 * Date:  2024-12-10 14:53
 * 修改昵称处理
 */
@MessageHandler(id = CommonProto.Cmd.UpdateNickNameReq_VALUE)
public class UpdateNickNameHandler extends NetHandler {

    @Override
    public void onDate(Connection session, WSRequest request, WSResponse response) {
        response.setCmd(CommonProto.Cmd.UpdateNickNameRes_VALUE);
        String userId = session.getPlayerId();
        String nickName = request.getDataValue("name", String.class, "");

        String userInfo = RedisUtil.get(RedisKeyConstant.USER_BASE + userId);
        JSONObject userObject = JSONUtil.parseObj(userInfo);
        Object lobbyUserObj = userObject.get("lobbyUser");
        ZjUserLobbyPo lobbyUser = JSONUtil.toBean((JSONObject) lobbyUserObj, ZjUserLobbyPo.class);
        lobbyUser.setNickName(nickName);
        userObject.put("lobbyUser", lobbyUser);
        RedisUtil.set(RedisKeyConstant.USER_BASE + userId, userObject.toString());
        response.setMessage("修改成功！");

        ZjUser zjUser = new ZjUser();
        zjUser.setUserId(userId);
        zjUser.setNickName(nickName);
        JSONObject object = new JSONObject();
        object.put("type", "updateUser");
        object.put("data", zjUser);
        try {
            RabbitMQUtil.sendMessage(object.toString());
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

}
