package com.incubator.game.handler.user;

import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.incubator.common.MessageHandler;
import com.incubator.common.net.Connection;
import com.incubator.core.net.ws.NetHandler;
import com.incubator.core.net.ws.WSRequest;
import com.incubator.core.net.ws.WSResponse;
import com.incubator.game.constant.RedisKeyConstant;
import com.incubator.game.data.entity.ZjUser;
import com.incubator.game.data.entity.ZjUserLongin;
import com.incubator.game.data.jedis.RedisUtil;
import com.incubator.game.data.mq.RabbitMQUtil;
import com.incubator.game.data.po.ZjUserLobbyPo;
import com.incubator.message.proto.CommonProto;

/**
 * Author:  xhb
 * Date:  2024-12-10 14:54
 * 绑定手机号
 */
@MessageHandler(id = CommonProto.Cmd.UpdatePhoneReq_VALUE)
public class UpdatePhoneHandler extends NetHandler {

    @Override
    public void onDate(Connection session, WSRequest request, WSResponse response) {
        response.setCmd(CommonProto.Cmd.UpdatePhoneRes_VALUE);
        String userId = session.getPlayerId();
        String name = request.getDataValue("name", String.class, "");
        String phone = request.getDataValue("phone", String.class, "");

        String userInfo = RedisUtil.get(RedisKeyConstant.USER_BASE + userId);
        JSONObject userObject = JSONUtil.parseObj(userInfo);
        Object lobbyUserObj = userObject.get("lobbyUser");
        ZjUserLobbyPo lobbyUser = JSONUtil.toBean((JSONObject) lobbyUserObj, ZjUserLobbyPo.class);
        lobbyUser.setPhone(phone);
        lobbyUser.setName(name);
        userObject.put("lobbyUser", lobbyUser);
        RedisUtil.set(RedisKeyConstant.USER_BASE + userId, userObject.toString());
        response.setMessage("修改成功！");

        ZjUserLongin zjUserLongin = new ZjUserLongin();
        zjUserLongin.setUserId(userId);
        zjUserLongin.setName(name);
        zjUserLongin.setPhoneNumber(phone);
        JSONObject object = new JSONObject();
        object.put("type", "updateUserLogin");
        object.put("data", zjUserLongin);
        try {
            RabbitMQUtil.sendMessage(object.toString());
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
