package com.incubator.game.listener;

import com.incubator.common.log4j.Log;
import com.incubator.core.net.ws.*;
import com.incubator.game.GGame;
import com.incubator.common.net.Connection;
import com.incubator.message.proto.CommonProto;
import com.incubator.core.net.handler.listener.GameClientConnectionListener;
import io.netty.buffer.ByteBuf;

import java.util.HashMap;
import java.util.Map;

/**
 * 中控服消息
 * @author johnc
 *
 */
public class CenterClientListener extends GameClientConnectionListener {

	@Override
	public void connectionOpened(Connection conn) {
		Log.info("中控服链路打开发送注册,注册码:{}", CommonProto.Cmd.S2S_GAME_REG_VALUE);
        try {
            Map<String, Object> data = new HashMap<>();
			data.put("serverId", GGame.SERVER_ID);
			data.put("host", GGame.BIND_HOST);
			data.put("port", GGame.BIND_PORT);
			conn.writeAndFlush(new WSResponse(CommonProto.Cmd.S2S_GAME_REG, data).toBytes());
        } catch (Exception e) {
			Log.info("游戏服注册出错：{}", e.getMessage());
        }
	}

	@Override
	public void connectionClosed(Connection conn) {
		GGame.isRegCenter = false;
	}

	@Override
	public void messageArrived(Connection conn, Object msg) {
		if (msg instanceof ByteBuf) {
			ByteBuf byteBuf = (ByteBuf) msg;

            try {
                byte[] bytes = new byte[byteBuf.readableBytes()];
                byteBuf.readBytes(bytes);
				// 提取请求数据
                WSRequest request = new WSRequest(bytes);
				// 获取处理器
                NetHandler handler = GGame.handlers.get(request.getCmd());
                if (handler == null) {
					Log.info("消息号错误... cmd={}", request.getCmd());
					conn.close();
					return;
                }
				// 处理消息
				Log.info("收到消息 : cmd={}, 请求内容 : {}", request.getCmd(), request.toJson());
				WSResponse response = new WSResponse();
				handler.onDate(conn, request, response);
				Log.info("返回消息 : cmd={}, 返回内容 : {}", request.getCmd(), response.toJson());
            } catch (Exception e) {
				Log.error("处理消息时发生错误 : ", e);
				conn.close();
            }
        }
	}

	@Override
	public boolean doIsHeartReq(Object msg) {
//		try {
//			byte[] bytes = new byte[msg.content().readableBytes()];
//			msg.content().readBytes(bytes);
//			MessageUnpacker unpacker = MessagePack.newDefaultUnpacker(bytes);
//			WSRequest request = super.parseWSRequest(unpacker);
//			return request.getMsgId() == ProtoCommon.MsgId.S2C_Heartbeat_VALUE;
//		} catch (Exception e) {
//			Log.error("心跳请求解析失败", e);
//			return false;
//		}

		return false;

//		String text = msg.text();
//		MsgBase request = JSON.parseObject(text, MsgBase.class);
//		return request.getMsgId() == ProtoCommon.MsgId.S2C_Heartbeat_VALUE;
	}

	@Override
	public void doSendHeartReq(Connection conn) {
        try {
//            HeartbeatPush push = new HeartbeatPush();
//            push.setTime(System.currentTimeMillis());
//            Message msg = new Message(ProtoCommon.MsgId.S2C_Heartbeat, push.toJson());
//            msg.setCode(ProtoCommon.Code.OK_VALUE);
//            conn.writeAndFlush(super.packMessage(msg));
        } catch (Exception e) {
			Log.info("发送心跳请求出错：" + conn.getPlayerId() + e.getMessage());
        }

//		conn.writeAndFlush(new TextWebSocketFrame(msg.toJson()));
	}

	@Override
	public void doSendHeartRsp(Connection conn) {
//		HeartbeatPush push = new HeartbeatPush();
//		push.setTime(System.currentTimeMillis());
//		Message msg = new Message(ProtoCommon.MsgId.S2C_Heartbeat, push.toJson());
//		msg.setCode(ProtoCommon.Code.OK_VALUE);
//		conn.writeAndFlush(new TextWebSocketFrame(msg.toJson()));
	}

}
