package com.incubator.game.room;

import cn.hutool.json.JSONException;
import com.alibaba.fastjson2.JSONObject;
import com.incubator.common.log4j.Log;
import com.incubator.common.util.RandomUtil;
import com.incubator.game.GGame;
import com.incubator.game.constant.RedisKeyConstant;
import com.incubator.game.constant.RoomStatus;
import com.incubator.game.contest.Contest;
import com.incubator.game.contest.ContestService;
import com.incubator.game.data.entity.ZjUserInfolog;
import com.incubator.game.data.jedis.RedisUtil;
import com.incubator.game.data.po.RoomPO;
import com.incubator.game.player.Player;
import com.incubator.game.util.CardUtils;
import com.incubator.game.util.GDUtils;
import com.incubator.game.util.SendGameInfoUtil;
import com.incubator.game.util.Struct;
import com.incubator.message.proto.CommonProto;
import org.apache.commons.lang3.StringUtils;

import java.math.BigDecimal;
import java.util.*;
import java.util.concurrent.locks.ReentrantLock;
import java.util.stream.Collectors;

/**
 * 掼蛋房间
 */
public class GDRoom extends Room {

    public GDRoom() {}

    @Override
    public void init(Player player, JSONObject jsonData) {
        this.jsonData = jsonData;
        // todo 默认配置
        this.data = new RoomPO();
        this.data.roomId = jsonData.getLong("roomId");
        this.data.type = jsonData.getInteger("type");
        this.data.gameType = jsonData.getInteger("gameType");
        this.data.ownerId = player != null? player.getId() : "";
        this.data.maxNum = 4;
        this.data.mode = jsonData.getInteger("mode");
        this.data.modev = jsonData.getInteger("modev");

        // 连续打
        if (this.data.mode == 1) {
            this.data.maxRound = 9999;
        }

        // 按局数
        if (this.data.mode == 2) {
            this.data.maxRound = jsonData.getInteger("modev");
        }

        // 按时间
        if (this.data.mode == 3) {
            this.data.curTime = System.currentTimeMillis();
            long time = jsonData.getInteger("modev");
            this.data.maxTime = this.data.curTime + (time * 60L * 1000L);
            this.data.maxRound = 9999;
        }

        this.data.playerMap = new Player[this.data.maxNum];
        // 是否有进还贡规则
        this.data.isTribute = jsonData.getInteger("advanceType") == 0;

        // 初始化级牌点数
        this.data.curLevelPoint = GDUtils.levelPoint[this.data.curLevelIndex];
        // 初始化牌库
        this.initCardList();
    }

    /**
     * 玩家加入房间
     */
    @Override
    public synchronized void joinRoom(Player player) {
        if (player == null) {
            return;
        }
        lock.lock();
        try {
            if (this.actors.size() >= this.data.maxNum) {
                return; // 房间已满
            }
            // 重置玩家房间数据
            player.resetRoom();
            // 玩家绑定房间
            player.data.roomId = this.data.roomId;

            // 加入房间角色缓存
            if (!this.actors.containsKey(player.getId())) {
                this.actors.put(player.getId(), player);
            }
            // 设置座位号
            for (int i = 0; i < this.data.playerMap.length; i++) {
                Player tmPlayer = this.data.playerMap[i];
                if (tmPlayer == null) {
                    player.data.pos = i;
                    this.data.playerMap[i] = player;
                    break;
                }
            }

            // 推送房间其他3个玩家
            for (Player tmPlayer : this.actors.values()) {
                if (tmPlayer != null && !Objects.equals(tmPlayer.getId(), player.getId())) {
                    Map<String, Object> data = this.roomToMessage(tmPlayer, null);
                    tmPlayer.receive(CommonProto.Cmd.RoomPlayerJoin_VALUE, data);
                }
            }

            // 有人就开启定时器
            if (!this.actors.isEmpty())  {
                this.startRoomTask();
            }
        } finally {
            lock.unlock();
        }
    }

    /** 玩家离开房间 **/
    @Override
    public synchronized void leaveRoom(Player player) {
        if (player == null) {
            return;
        }
        lock.lock();
        try {
            // 清理房间数据
            this.actors.remove(player.getId());
            RoomStatus roomToRedis = RedisUtil.getRoomToRedis(RedisKeyConstant.WK_ROOM_MAP, String.valueOf(this.data.roomId));
            if (roomToRedis != null){
                List<String> players = roomToRedis.getPlayers();
                players.remove(player.getId());
                roomToRedis.setPlayers(players);
            }
            this.data.playerMap[player.data.pos] = null;
            // todo 房主退出
            if (player.getId().equals(this.data.ownerId)) {
//                Player nextPlayer = this.getNextPlayer(player);
//                if (nextPlayer != null) {
//                    this.data.ownerId = nextPlayer.getId();
//                }
            }
            // 重置玩家房间数据
            player.resetRoom();

            // 推送房间其他玩家
            for (Player tmPlayer : this.actors.values()) {
                if (tmPlayer != null && !Objects.equals(tmPlayer.getId(), player.getId())) {
                    Map<String, Object> data = roomToMessage(tmPlayer, null);
                    tmPlayer.receive(CommonProto.Cmd.RoomPlayerLeave_VALUE, data);
                }
            }
        } finally {
            lock.unlock();
        }

        // 广播
        if (this.state == 2) {
            for (Player tmPlayer : this.actors.values()) {
                if (tmPlayer != null) {
                    tmPlayer.receive(CommonProto.Cmd.DisbandRoom_VALUE, new HashMap<>());
                }
            }
            this.destroy();
        }
    }

    /**
     * 初始化牌库
     */
    @Override
    public void initCardList() {
        for (int i = 0; i < 2; i++) {
            this.data.cardList.addAll(Arrays.asList(GDUtils.values));
            Collections.shuffle(this.data.cardList);
        }
    }

    /**
     * 主线程逻辑
     */
    @Override
    public void doUpdate() {
        synchronized (this) {
//            Log.debug("检测: 经典掼蛋房间主线程执行...");

            // 房间没人直接解散
            if (this.actors.isEmpty()) {
                this.destroy();
                return;
            }

            switch (this.state) {
                case 0:
                    // 等待状态
                    if (this.checkReadyStart()) {
                        this.state = 1;
                        this.flag = false;
                        this.time = 0;
                    }

                    // 120秒未开始直接解散
//                    if (this.time >= 120 && this.type != 1) {
//                        // 广播
//                        for (Player tmPlayer : this.actors.values()) {
//                            if (tmPlayer != null) {
//                                tmPlayer.receive(CommonProto.Cmd.DisbandRoom_VALUE, new HashMap<>());
//                            }
//                        }
//                        this.destroy();
//                    }

                    break;
                case 1:
                    //锁
                    RedisUtil.set("roomGambit:"+this.data.roomId, "0");
                    // 开局
                    if (!this.flag) {
                        //开始
                        this.doStart();
                        Log.debug("检测: 房间号:{}, 房间人数:{}, 当前局数:{}, 状态:开局...VS动画...", this.data.roomId, this.actors.size(), this.data.curRound);
                    }
                    break;
                case 2:
                    // 进行中
                    if (!this.flag) {
                        // 检查进贡是否完成
                        if (this.data.tributeState == 1 && this.checkTributeStart()) {
                            // 切换还贡状态
                            this.doChangeBackTribute();
                            this.time = 0;
                        }

                        // 检查还贡是否完成
                        if (this.data.tributeState == 2 && this.checkBackTributeStart()) {
                            // 进还贡换牌
                            this.doChangeTributeCard();
                            this.flag = true;
                            this.time = 0;
                        }

                        // 80秒强制操作进还贡
//                        if (this.time >= 80) {
//                            this.doTribute();
//                            this.flag = true;
//                            this.time = 0;
//                        }
                    } else {
                        // 20秒未操作,强制过
//                        if (this.time >= 20) {
//                            Log.debug("强制操作过...");
//                            this.playerPass(this.data.currentPlayer, true);
//                            this.flag = true;
//                            this.time = 0;
//                        }
                    }
                    break;
                case 3:
                    // 结束
                    if (!this.flag) {
                        this.flag = true;
                        this.time = 0;
                        Log.debug("游戏结束: 房间号 : {}, 房间人数 : {}, 当前局数 : {}", this.data.roomId, this.actors.size(), this.data.curRound);
                    } else {
                        // 根据类型判断结束走向
                        switch (this.data.mode) {
                            case 1:
                                // 连续打
                                if (!this.checkAllOver()) {
                                    this.resetNextRoom();
                                    this.state = 0;
                                    this.flag = false;
                                    this.time = 0;
                                } else {
                                    this.totalSettlement();
                                    try {
                                        Thread.sleep(2000);
                                    } catch (InterruptedException e) {
                                        e.printStackTrace();
                                    }
                                    // 广播
                                    for (Player tmPlayer : this.actors.values()) {
                                        if (tmPlayer != null) {
                                            tmPlayer.receive(CommonProto.Cmd.DisbandRoom_VALUE, new HashMap<>());
                                        }
                                    }
                                    this.destroy();
                                }
                                break;
                            case 2:// 按局数
                                // 小于最大局数,切等待状态
                                if (this.data.curRound < this.data.maxRound) {
//                                    if (!noCircleSettlement()){//不套圈，判断能否判负或判2
                                        this.resetNextRoom();
                                        this.state = 0;
                                        this.flag = false;
                                        this.time = 0;
//                                    }else {
//                                        try {
//                                            Thread.sleep(1000);
//                                        } catch (InterruptedException e) {
//                                            e.printStackTrace();
//                                        }
//                                        this.totalSettlement();
//                                        try {
//                                            Thread.sleep(1000);
//                                        } catch (InterruptedException e) {
//                                            e.printStackTrace();
//                                        }
//                                        // 广播
//                                        for (Player tmPlayer : this.actors.values()) {
//                                            if (tmPlayer != null) {
//                                                tmPlayer.receive(CommonProto.Cmd.DisbandRoom_VALUE, new HashMap<>());
//                                            }
//                                        }
//                                        this.destroy();
//                                    }
                                } else {
                                    //判断锁是否存在 //等待消失
//                                    if ((Integer)this.jsonData.get("ferrule") == 0){
//                                        this.circleSettlement();
//                                    }else {
                                        this.totalSettlement();
//                                    }
                                    try {
                                        Thread.sleep(1000);
                                    } catch (InterruptedException e) {
                                        e.printStackTrace();
                                    }
                                    // 广播
                                    for (Player tmPlayer : this.actors.values()) {
                                        if (tmPlayer != null) {
                                            tmPlayer.receive(CommonProto.Cmd.DisbandRoom_VALUE, new HashMap<>());
                                        }
                                    }
                                    this.destroy();
                                }
                                break;
                            case 3:
                                // 按时间
                                // 小于最大时间,切等待状态
                                if (System.currentTimeMillis() < this.data.maxTime) {
                                    this.resetNextRoom();
                                    this.state = 0;
                                    this.flag = false;
                                    this.time = 0;
                                } else {
                                    //判断锁是否存在 //等待消失

                                        try {
                                            Thread.sleep(2000);
                                        } catch (InterruptedException e) {
                                            e.printStackTrace();
                                        }

                                    // 广播
                                    for (Player tmPlayer : this.actors.values()) {
                                        if (tmPlayer != null) {
                                            tmPlayer.receive(CommonProto.Cmd.DisbandRoom_VALUE, new HashMap<>());
                                        }
                                    }
                                    this.destroy();
                                }
                                break;
                            default:
                                break;
                        }

                        // 120秒未开始直接解散
                        if (this.time >= 120) {
                            // 广播
                            for (Player tmPlayer : this.actors.values()) {
                                if (tmPlayer != null) {
                                    tmPlayer.receive(CommonProto.Cmd.DisbandRoom_VALUE, new HashMap<>());
                                }
                            }
                            this.destroy();
                        }
                    }
                    break;
            }

            this.time += 1;
        }
    }

    /**
     * 总结算
     */
    public void totalSettlement() {
        // 广播结算信息
        for (Player tmPlayer : this.actors.values()) {
            if (tmPlayer != null) {
                Map<String, Object> data = this.roomToMessage(tmPlayer, null);
                // 增加结算玩家信息
                if (!this.data.result.isEmpty()) {
                    Map<Integer, Object> settlementInfo = new HashMap<>();
                    for (Map.Entry<Integer, Struct.SettlementData> entry : this.data.result.entrySet()) {
                        Map<String, Object> settlementData = new HashMap<>();
                        settlementData.put("playerId", entry.getValue().playerId);
                        settlementData.put("score", entry.getValue().score);
                        settlementData.put("levelPoint", this.actors.getOrDefault(entry.getValue().playerId,null).data.levelPointView[0]);
                        settlementInfo.put(entry.getKey(), settlementData);
                    }
                    data.put("settlementInfo", settlementInfo);
                }
                tmPlayer.receive(CommonProto.Cmd.SummaryCalculation_VALUE, data);
            }
        }
    }

    /**
     * 检测是否可以开始 3秒倒计时开始
     *
     * @return
     */
    public boolean checkReadyStart() {
        if (!this.isStart() || this.state != 0) {
            return false;
        }

        // 广播3秒倒计时
        for (Player tmPlayer : this.actors.values()) {
            if (tmPlayer != null) {
                Map<String, Object> data = this.roomToMessage(tmPlayer, null);
                data.put("state", 1);
                data.put("time", 0);
//                data.put("time", 3);
                tmPlayer.receive(CommonProto.Cmd.GameStateChange_VALUE, data);
            }
        }

        return true;
    }

    /**
     * 是否玩家都已准备,可开始
     *
     * @return
     */
    public boolean isStart() {
        int cnt = 0;
        for (Player player : this.actors.values()) {
            if (player != null && player.data.state == 1) {
                cnt += 1;
            }
        }
        return cnt >= this.data.maxNum;
    }

    /**
     * 给玩家发牌，每人27张
     */
    @Override
    public synchronized void doSendCard() {
        for (Player player : this.actors.values()) {
            if (player != null) {
                // 设置用户状态
                player.data.state = 2;
                // 设置级牌点数
                if (this.data.curRound == 1) {
                    // 我方级数
                    player.data.levelPointView[0] = GDUtils.levelPoint[0];
                    // 对方级数
                    player.data.levelPointView[1] = GDUtils.levelPoint[0];
                }
                // 设置队友位置
                if (this.data.maxNum >= 4) {
                    Integer[] nextPos = this.getNextPos(player.data.pos);
                    Player tmpPlayer = this.data.playerMap[nextPos[1]];
                    if (tmpPlayer != null) {
                        player.data.teammateId = tmpPlayer.getId();
                    }
                }
                // 发牌
                for (int i = 0; i < 27; i++) {
                    int card = this.data.cardList.get(0);
                    // 设置当前赖子
                    if (GDUtils.isWildCard(card, this.data.curLevelPoint)) {
                        this.data.wildCard = card;
                    }
                    player.data.remainCards = CardUtils.add(player.data.remainCards, card);
                    this.data.cardList.remove(0);
                }
                // todo 测试代码
                if (this.data.wildCard != 0) {
                    player.data.remainCards = CardUtils.add(player.data.remainCards, this.data.wildCard);
                    player.data.remainCards = CardUtils.add(player.data.remainCards, this.data.wildCard);
                }
            }
        }
    }

    /**
     * 游戏开局
     */
    @Override
    public synchronized void doStart() {
        try {
            this.data.curRound += 1;
            // 定庄
            if (this.data.curRound == 1) {
                // 随机定庄
                this.data.zhuangPos = RandomUtil.getIndex(this.data.maxNum);
            } else {
                // 不进贡头游先出
                if (!this.data.isTribute) {
                    Struct.SettlementData player1Data = this.data.result.getOrDefault(1, null);
                    if (player1Data == null || StringUtils.isEmpty(player1Data.playerId)) {
                        return;
                    }
                    Player p1 = this.actors.getOrDefault(player1Data.playerId, null);
                    if (p1 != null) {
                        this.data.zhuangPos = p1.data.pos;
                    }
                    this.data.result.clear();
                }
            }
            // 发牌
            this.doSendCard();
            // 进还贡数据
            Map<String, Object> tributeInfo = null;
            if (this.data.isTribute) {
                tributeInfo = new HashMap<>();
                this.checkTributeInfo();
                // 定庄
                if (this.isTwoKing()) {
                    // 定庄 头游先出
                    Struct.SettlementData player1Data = this.data.result.getOrDefault(1, null);
                    if (player1Data == null || StringUtils.isEmpty(player1Data.playerId)) {
                        return;
                    }
                    Player p1 = this.actors.getOrDefault(player1Data.playerId, null);
                    if (p1 != null) {
                        this.data.zhuangPos = p1.data.pos;
                    }
                    this.data.result.clear();
                }
                if (this.isTwoKing()) {
                    this.data.tributeState = 3;
                } else {
                    if (!this.data.tributeList.isEmpty() && !this.isTwoKing() && this.data.tributeState == 0) {
                        this.data.tributeState = 1;
                    }
                }
                tributeInfo.put("tributeState", this.data.tributeState);
                tributeInfo.put("tributeList", this.data.tributeList);
                tributeInfo.put("twoKingInfo", this.data.towKingList);
                tributeInfo.put("tributeCardMap", this.data.tributeCardMap);
            }
            // 设置当前操作玩家
            this.data.currentPlayer = this.data.playerMap[this.data.zhuangPos];
            // 广播
            for (Player tmPlayer : this.actors.values()) {
                if (tmPlayer != null) {
                    Map<String, Object> data = this.roomToMessage(tmPlayer, this.data.currentPlayer);
                    data.put("zhuangPos", this.data.zhuangPos);
                    if (this.data.isTribute) {
                        data.put("zhuangPos", !this.data.tributeList.isEmpty() && !this.isTwoKing()? -1 : this.data.zhuangPos);
                    }
                    data.put("tributeInfo", tributeInfo);
                    data.put("state", 2);
                    data.put("time", !this.data.tributeList.isEmpty()? 40 : 20);
                    Log.debug("玩家 : {}, 手牌 : {}", tmPlayer.data.name, tmPlayer.data.remainCards);
                    tmPlayer.receive(CommonProto.Cmd.GameStateChange_VALUE, data);
                }
            }

            // 记录游戏回放
            Map<String, Object> data = this.roomToPlayBack(this.data.currentPlayer);
            this.playBack.put(Long.toString(System.currentTimeMillis()), data);

            // 是否有进还贡数据 或 抗贡
            if (this.data.isTribute && !this.data.tributeList.isEmpty() && !this.isTwoKing()) {
                this.state = 2;
                this.flag = false;
                this.time = 0;
            } else {
                this.state = 2;
                this.flag = true;
                this.time = 0;
                // 清理数据
                this.data.tributeState = 0;
                this.data.result.clear();
                this.data.tributeList.clear();
                this.data.towKingList.clear();
            }
        } catch (Exception e) {
            Log.error("===游戏开局出错...", e);
        }
    }

    /**
     * 开局检测是否进贡、还贡
     *
     * @return
     */
    public synchronized void checkTributeInfo() {
        // 头游
        Struct.SettlementData player1Data = this.data.result.getOrDefault(1, null);
        if (player1Data == null || StringUtils.isEmpty(player1Data.playerId)) {
            return;
        }
        Player p1 = this.actors.getOrDefault(player1Data.playerId, null);
        if (p1 == null) {
            return;
        }
        // 末游
        Player p4 = null;

        // 是否双贡
        boolean isDoubleTribute = false;
        // 头游搭档数据
        Struct.TeammateData teammateData = this.getTeammate(p1.data.teammateId);
        if (teammateData.getRank() == 2) {
            // 进贡, 双下
            isDoubleTribute = true;
        }
        // 进还贡
        Struct.SettlementData p4Data = this.data.result.getOrDefault(4, null);
        if (p4Data == null || StringUtils.isEmpty(p4Data.playerId)) {
            return;
        }
        p4 = this.actors.getOrDefault(p4Data.playerId, null);
        if (p4 != null) {
            this.data.tributeList.add(new String[]{p4.getId() , p1.getId()});
        }
        // 双贡
        if (isDoubleTribute) {
            Struct.SettlementData p3Data = this.data.result.getOrDefault(3, null);
            if (p3Data == null || StringUtils.isEmpty(p3Data.playerId)) {
                return;
            }
            Player p3 = this.actors.getOrDefault(p3Data.playerId, null);
            if (p3 != null) {
                this.data.tributeList.add(new String[]{p3.getId(), teammateData.player.getId()});
            }
        }

        // 双贡情况
        if (this.data.tributeList.size() >= 2) {
            // 1游[0:进贡者, 1:还贡者]
            String[] tribute1 = this.data.tributeList.get(0);
            // 2游[0:进贡者, 1:还贡者]
            String[] tribute2 = this.data.tributeList.get(1);

            // 1游
            Player player1 = this.actors.getOrDefault(tribute1[1], null);
            // 1游进贡者
            Player tributePlayer1 = this.actors.getOrDefault(tribute1[0], null);
            // 2游
            Player player2 = this.actors.getOrDefault(tribute2[1], null);
            // 2游进贡者
            Player tributePlayer2 = this.actors.getOrDefault(tribute2[0], null);

            if (player1 == null || tributePlayer1 == null || player2 == null || tributePlayer2 == null) {
                return;
            }
            // 1游进贡者最大的手牌
            int player1MaxCard = GDUtils.getTributeMaxCard(tributePlayer1.data.remainCards, this.data.curLevelPoint);
            // 2游进贡者最大手牌
            int player2MaxCard = GDUtils.getTributeMaxCard(tributePlayer2.data.remainCards, this.data.curLevelPoint);

            // 1游取牌点大的进贡者
            if (player2MaxCard > player1MaxCard) {
                tribute1[0] = tributePlayer2.getId();
                tribute2[0] = tributePlayer1.getId();
                // 设置庄家
                this.data.zhuangPos = tributePlayer2.data.pos;
            } else if (player2MaxCard == player1MaxCard) {
                // 牌点相同按顺时针进贡
                // 设置1游进贡者为1游下家
                Player nextPlayer = this.getNextPlayer(player1);
                if (nextPlayer != null) {
                    tribute1[0] = nextPlayer.getId();
                }
                //设置2游进贡者为2游下家
                nextPlayer = this.getNextPlayer(player2);
                if (nextPlayer != null) {
                    tribute2[0] = nextPlayer.getId();
                }
                // 设置庄家
                this.data.zhuangPos = tributePlayer2.data.pos;
            } else {
                // 设置庄家
                this.data.zhuangPos = tributePlayer1.data.pos;
            }
        }

        // 单贡情况
        if (this.data.tributeList.size() == 1) {
            // 1游[0:进贡者, 1:还贡者]
            String[] tribute1 = this.data.tributeList.get(0);
            // 1游
            Player player1 = this.actors.getOrDefault(tribute1[1], null);
            // 1游进贡者
            Player tributePlayer1 = this.actors.getOrDefault(tribute1[0], null);
            if (player1 == null || tributePlayer1 == null) {
                return;
            }
            // 设置庄家
            this.data.zhuangPos = tributePlayer1.data.pos;
        }

        // 抗贡
        int num = 0;
        for (String[] tributeId : this.data.tributeList) {
            // 进贡者
            Player tributePlayer = this.actors.getOrDefault(tributeId[0], null);
            if (tributePlayer != null) {
                // 统计手牌数量
                Map<Integer, Integer> cardMap = CardUtils.getCardMap(tributePlayer.data.remainCards);
                if (!cardMap.isEmpty()) {
                    num = cardMap.getOrDefault(GDUtils.BIGKING, 0);
                    if (num > 0 && !this.data.towKingList.contains(tributeId[0])) {
                        // 记录信息
                        this.data.towKingList.add(tributeId[0]);
                    }
                }
            }
        }

//        if (this.data.towKingList.size() < 2) {
//            this.data.towKingList.clear();
//        }
    }

    /**
     * 检测双王抗贡
     *
     * @return
     */
    public boolean isTwoKing() {
        int num = 0;
        for (String[] tributeId : this.data.tributeList) {
            // 进贡者
            Player tributePlayer = this.actors.getOrDefault(tributeId[0], null);
            if (tributePlayer != null) {
                // 统计手牌数量
                Map<Integer, Integer> cardMap = CardUtils.getCardMap(tributePlayer.data.remainCards);
                if (!cardMap.isEmpty()) {
                    num += cardMap.getOrDefault(GDUtils.BIGKING, 0);
                }
            }
        }
        return num > 0 && num % 2 == 0;
    }

    /**
     * 检测进贡是否完成
     *
     * @return
     */
    public synchronized boolean checkTributeStart() {
//        Log.debug("检测进贡是否完成...");
        int cnt = 0;
        for (String[] playerIds : this.data.tributeList) {
            if (!StringUtils.isEmpty(playerIds[0]) && !StringUtils.isEmpty(playerIds[1])) {
                // 进贡者
                Player tributePlayer = this.actors.getOrDefault(playerIds[0], null);
                if (tributePlayer != null && tributePlayer.data.tributeCard > 0) {
                    cnt += 1;
                }
            }
        }
        return cnt == this.data.tributeList.size();
    }

    /**
     * 检测还贡是否完成
     *
     * @return
     */
    @Override
    public synchronized boolean checkBackTributeStart() {
//        Log.debug("检测还贡是否完成...");
        int cnt = 0;
        for (String[] playerIds : this.data.tributeList) {
            if (!StringUtils.isEmpty(playerIds[0]) && !StringUtils.isEmpty(playerIds[1])) {
                // 还贡者
                Player backPlayer = this.actors.getOrDefault(playerIds[1], null);
                if (backPlayer != null && backPlayer.data.tributeCard > 0) {
                    cnt += 1;
                }
            }
        }
        return cnt == this.data.tributeList.size();
    }

    /**
     * 切换还贡状态
     */
    public synchronized void doChangeBackTribute() {
        this.data.tributeState = 2;
        // 进还贡数据
        Map<String, Object> tributeInfo = new HashMap<>();
        tributeInfo.put("tributeState", this.data.tributeState);
        tributeInfo.put("tributeList", this.data.tributeList);
        tributeInfo.put("twoKingInfo", this.data.towKingList);
        tributeInfo.put("tributeCardMap", this.data.tributeCardMap);

        // 广播
        for (Player tmPlayer : this.actors.values()) {
            if (tmPlayer != null) {
                Map<String, Object> data = this.roomToMessage(tmPlayer, this.data.currentPlayer);
                data.put("zhuangPos", !this.data.tributeList.isEmpty() && !this.isTwoKing()? -1 : this.data.zhuangPos);
                data.put("tributeInfo", tributeInfo);
                data.put("state", 2);
                data.put("time", !this.data.tributeList.isEmpty()? 40 : 20);
                Log.debug("玩家 : {}, 手牌 : {}", tmPlayer.data.name, tmPlayer.data.remainCards);
                tmPlayer.receive(CommonProto.Cmd.GameStateChange_VALUE, data);
            }
        }
    }

    /**
     * 强制进贡、还贡
     */
    public synchronized void doTribute() {
        Log.debug("强制进贡、还贡...");
        // 强制选牌
        for (String[] playerIds : this.data.tributeList) {
            if (!StringUtils.isEmpty(playerIds[0]) && !StringUtils.isEmpty(playerIds[1])) {
                // 进贡者
                Player tributePlayer = this.actors.getOrDefault(playerIds[0], null);
                // 还贡者
                Player backPlayer = this.actors.getOrDefault(playerIds[1], null);
                if (tributePlayer == null || backPlayer == null) {
                    continue;
                }
                // 进贡者选牌
                int maxCard = GDUtils.getTributeMaxCard(tributePlayer.data.remainCards, this.data.curLevelPoint);
                this.data.tributeCardMap.put(tributePlayer.getId(), maxCard);
                // 还贡者选牌
                int tributeCard = 0;
                for (Integer card : backPlayer.data.remainCards) {
                    byte low = (byte) (card & 0xf);
                    // 过滤级牌
                    if (low == this.data.curLevelPoint) {
                        continue;
                    }
                    if (low <= 10) {
                        tributeCard = card;
                        break;
                    }
                }
                this.data.tributeCardMap.put(backPlayer.getId(), tributeCard);
                Log.debug("强制进还贡...进贡者 : {}, 选牌 : {}, 还贡者 : {}, 选牌 : {}",
                        tributePlayer.data.name, maxCard, backPlayer.data.name, tributeCard);
            }
        }
        // 换牌
        this.doChangeTributeCard();
    }

    /**
     * 进还贡换牌
     */
    public synchronized void doChangeTributeCard() {
        if (this.data.tributeList.isEmpty()) {
            return;
        }
        Log.debug("进还贡换牌...");
        int tributeCard = 0;
        int backCard = 0;
        for (String[] playerIds : this.data.tributeList) {
            if (!StringUtils.isEmpty(playerIds[0]) && !StringUtils.isEmpty(playerIds[1])) {
                // 进贡者
                Player tributePlayer = this.actors.getOrDefault(playerIds[0], null);
                // 还贡者
                Player backPlayer = this.actors.getOrDefault(playerIds[1], null);
                if (tributePlayer == null || backPlayer == null) {
                    continue;
                }
                // 移除进贡者手牌
                tributeCard = this.data.tributeCardMap.getOrDefault(tributePlayer.getId(), 0);
                if (tributeCard > 0) {
                    // 还贡者添加牌
                    backPlayer.data.remainCards = CardUtils.add(backPlayer.data.remainCards, tributeCard);
                    backPlayer.data.tributeGetCard = tributeCard;
                }

                // 移除还贡者手牌
                backCard = this.data.tributeCardMap.getOrDefault(backPlayer.getId(), 0);
                if (backCard > 0) {
                    // 进贡者添加牌
                    tributePlayer.data.remainCards = CardUtils.add(tributePlayer.data.remainCards, backCard);
                    tributePlayer.data.tributeGetCard = backCard;
                }
                // 更新数据
                this.actors.put(tributePlayer.getId(), tributePlayer);
                this.actors.put(backPlayer.getId(), backPlayer);
            }
        }
        // 清理数据
        this.data.tributeState = 0;
        this.data.result.clear();
        this.data.tributeList.clear();
        this.data.towKingList.clear();
        // 进还贡数据
        Map<String, Object> tributeInfo = new HashMap<>();
        tributeInfo.put("tributeList", this.data.tributeList);
        tributeInfo.put("twoKingInfo", this.data.towKingList);
        tributeInfo.put("tributeState", this.data.tributeState);
        // 进还贡的选牌map
        tributeInfo.put("tributeCardMap", this.data.tributeCardMap);
        // 广播
        for (Player tmPlayer : this.actors.values()) {
            if (tmPlayer != null) {
                Map<String, Object> data = this.roomToMessage(tmPlayer, this.data.currentPlayer);
                data.put("tributeInfo", tributeInfo);
                data.put("state", 2);
                data.put("time", 20);
                Log.debug("进还贡换牌...玩家 : {}, 手牌 : {}", tmPlayer.data.name, tmPlayer.data.remainCards);
                tmPlayer.receive(CommonProto.Cmd.GameStateChange_VALUE, data);
            }
        }
    }

    /**
     * 根据当前玩家位置, 获取下一个玩家位置
     *
     * @param pos 当前操作玩家的位置
     * @return
     */
    public Integer[] getNextPos(int pos) {
        Integer[] nextpos = null;
        switch (pos) {
            case 3:
                nextpos = new Integer[] { 0, 1, 2 };
                break;
            case 2:
                nextpos = new Integer[] { 3, 0, 1 };
                break;
            case 1:
                nextpos = new Integer[] { 2, 3, 0 };
                break;
            case 0:
                nextpos = new Integer[] { 1, 2, 3 };
                break;
        }
        return nextpos;
    }

    /**
     * 根据当前玩家位置, 获取下一个可操作玩家
     *
     * @param player 当前操作玩家
     * @return
     */
    public Player getNextPlayer(Player player) {
        Player nextPlayer = null;
        Integer[] nextPos = this.getNextPos(player.data.pos);
        for (Integer index : nextPos) {
            Player temPlayer = this.data.playerMap[index];
            if (temPlayer.data.state == 2 && !temPlayer.getId().equals(player.getId())) {
                nextPlayer = temPlayer;
                break;
            }
        }
        return nextPlayer;
    }

    /**
     * 玩家换座位
     *
     * @param player
     * @param targetSeat
     */
    @Override
    public synchronized void switchSeat(Player player, int targetSeat) {
        lock.lock();
        try {
            if (targetSeat < 0 || targetSeat >= this.data.maxNum) {
                return; // 目标座位无效
            }
            // 找到玩家当前座位
            int currentSeat = -1;
            for (int i = 0; i < this.data.maxNum; i++) {
                Player tmPlayer = this.data.playerMap[i];
                if (tmPlayer != null && tmPlayer.getId().equals(player.getId())) {
                    currentSeat = i;
                    break;
                }
            }
            if (currentSeat == -1) {
                return; // 玩家不在座位上
            }

            // 检查目标座位
            Player targetPlayer = this.data.playerMap[targetSeat];
            if (targetPlayer == null) {
                player.data.pos = targetSeat;
                // 如果目标座位为空，直接换过去
                this.data.playerMap[targetSeat] = player;
                this.data.playerMap[currentSeat] = null;
                Log.debug("roomId: " + this.data.roomId + ", " + player.data.name + " 换到座位号: " + targetSeat);
            }
            // 如果目标座位已被占用，与目标玩家交换位置
            if (targetPlayer != null && targetPlayer.data.isChangePos) {
                player.data.pos = targetSeat;
                targetPlayer.data.pos = currentSeat;
                this.data.playerMap[targetSeat] = player;
                this.data.playerMap[currentSeat] = targetPlayer;
                Log.debug("roomId: " + this.data.roomId + ", " + player.data.name + " 与 " + targetPlayer.data.name + " 交换了座位");
            }
        } finally {
            lock.unlock();
        }
    }

    /**
     * 玩家出牌
     *
     * @param player    出牌玩家
     * @param disCardList   打出的牌
     * @param disCardType   打出的牌型
     */
    @Override
    public synchronized void playerDisCard(Player player, int[] disCardList, GDUtils.CardType disCardType) {
        // 是否自己回合
        if (!this.data.currentPlayer.getId().equals(player.getId())) {
            return;
        }
        // 出的牌是否都在手上
        if (!CardUtils.contains(player.data.remainCards, disCardList)) {
            return;
        }
        Log.info("出牌...玩家ID:{}, 位置:{}, 出牌牌型:{}, 出牌:{}, 手牌大小:{}, 手牌信息: {}",
                player.getId(), player.data.pos, disCardType, Arrays.toString(disCardList),
                player.data.remainCards.length, Arrays.toString(player.data.remainCards));
        // 从手牌中移除
        for (Integer card : disCardList) {
            player.data.remainCards = CardUtils.remove(player.data.remainCards, card);
        }
        // 设置出牌堆
        this.data.curDisCardList = disCardList;
        this.data.curDisCardType = disCardType;
        // 设置当前出牌的玩家
        this.data.disCardPlayer = player;

        // 清空过玩家集合
        this.data.passList.removeIf(id -> player.getId().equals(id));

        // 设置接风玩家
        this.data.jieFengPlayer = null;
        // 设置下一个操作玩家
        this.data.currentPlayer = this.getNextPlayer(player);
        // 出完牌设置结算数据
        if (player.data.remainCards.length == 0) {
            player.data.state = 3;
            this.data.result.put(this.data.result.size() + 1, new Struct.SettlementData(player.getId(), 0));
        }
        // 设置其他可操作玩家集合
        this.data.otherOpPlayerList.clear();
        for (Player tmPlayer : this.actors.values()) {
            if (tmPlayer != null && !tmPlayer.getId().equals(player.getId())) {
                if (tmPlayer.data.state == 2) {
                    if (!this.data.otherOpPlayerList.contains(tmPlayer.getId())) {
                        this.data.otherOpPlayerList.add(tmPlayer.getId());
                    }
                }
            }
        }

        // 广播其他3人
        for (Player tmPlayer : this.actors.values()) {
            if (tmPlayer != null && !Objects.equals(tmPlayer.getId(), player.getId())) {
                Map<String, Object> data = this.roomToMessage(tmPlayer, this.data.currentPlayer);
                data.put("time", 20);
                tmPlayer.receive(CommonProto.Cmd.OtherPlayCards_VALUE, data);
            }
        }

        boolean isGameOver = false;

        // 4人房2人一队手牌出完,直接结算
        if (this.data.result.size() == 2) {
            // 1游
            Struct.SettlementData player1Data = this.data.result.get(1);
            if (player1Data == null || StringUtils.isEmpty(player1Data.playerId)) {
                return;
            }
            Player player1 = this.actors.getOrDefault(player1Data.playerId, null);
            if (player1 == null) {
                return;
            }

            // 2游
            Struct.SettlementData player2Data = this.data.result.get(2);
            if (player2Data == null || StringUtils.isEmpty(player2Data.playerId)) {
                return;
            }
            Player player2 = this.actors.getOrDefault(player2Data.playerId, null);
            if (player2 == null) {
                return;
            }

            // 1游2游是否一队
            if (player1.data.teammateId.equals(player2.getId())) {
                // 已结算的用户id
                List<String> ids = new ArrayList<>();
                for (Struct.SettlementData settlementData : this.data.result.values()) {
                    if (settlementData != null && !StringUtils.isEmpty(settlementData.playerId)) {
                        ids.add(settlementData.playerId);
                    }
                }
                // 补充3游4游玩家
                for (Player tmPlayer : this.data.playerMap) {
                    if (tmPlayer != null) {
                        tmPlayer.data.state = 3;
                        // 补充最后一个玩家
                        if (!ids.contains(tmPlayer.getId())) {
                            this.data.result.put(this.data.result.size() + 1, new Struct.SettlementData(tmPlayer.getId(), 0));
                        }
                    }
                }
                isGameOver = true;
            }
        }

        // 4人房间3人手牌出完，直接结算
        if (this.data.result.size() >= this.data.maxNum - 1) {
            // 已结算的用户id
            List<String> ids = new ArrayList<>();
            for (Struct.SettlementData settlementData : this.data.result.values()) {
                if (settlementData != null && !StringUtils.isEmpty(settlementData.playerId)) {
                    ids.add(settlementData.playerId);
                }
            }
            for (Player tmPlayer : this.data.playerMap) {
                if (tmPlayer != null) {
                    tmPlayer.data.state = 3;
                    // 补充最后一个玩家
                    if (!ids.contains(tmPlayer.getId())) {
                        this.data.result.put(this.data.result.size() + 1, new Struct.SettlementData(tmPlayer.getId(), 0));
                    }
                }
            }

            isGameOver = true;
        }

        // 记录游戏回放
        Map<String, Object> data = this.roomToPlayBack(this.data.currentPlayer);
        this.playBack.put(Long.toString(System.currentTimeMillis()), data);

        if (isGameOver) {
            // 结算
            try {
                Thread.sleep(2000);
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
            this.settlement();
            // 结束
            this.gameOver();
        }
    }

    /**
     * 玩家过
     *
     * @param player    点击过玩家
     * @param isAuto    是否自动操作过
     */
    @Override
    public synchronized void playerPass(Player player, boolean isAuto) {
        // 当前可操作玩家是否自己
        if (!this.data.currentPlayer.getId().equals(player.getId())) {
            return;
        }
        Log.info("过...玩家ID:{}, 位置:{}, 手牌大小:{}, 手牌信息: {}", player.getId(), player.data.pos, player.data.remainCards.length, Arrays.toString(player.data.remainCards));

        boolean isClear = false;

        // 清掉自己出牌堆
        player.data.discardList = new int[0];

        // 添加过玩家列表
        if (!this.data.passList.contains(player.getId())) {
            this.data.passList.add(player.getId());
        }
        // 设置其他可操作玩家集合
        this.data.otherOpPlayerList.removeIf(id -> player.getId().equals(id));

        // 4人房3人不出，清空出牌堆
        if (this.data.otherOpPlayerList.isEmpty()) {
            // 清空出牌堆
            this.data.curDisCardList = new int[0];
            this.data.curDisCardType = null;

            for (Player p : this.actors.values()) {
                if (p != null && p.data.discardList.length > 0) {
                    p.data.discardList = new int[0];
                }
            }
            // 所有玩家都点过
            if (!this.data.passList.isEmpty()) {
                // 清空过玩家集合
                this.data.passList.clear();
                // 设置接风标识
                if (this.data.maxNum == 4 && this.data.disCardPlayer.data.remainCards.length == 0) {
                    this.data.jieFengPlayer = this.actors.getOrDefault(this.data.disCardPlayer.data.teammateId, null);
                }
            }
            isClear = true;
        }

        // 判断接风
        if (isClear && this.data.jieFengPlayer != null) {
            this.data.currentPlayer = this.actors.getOrDefault(this.data.jieFengPlayer.getId(), null);
            this.data.disCardPlayer = null;
        } else {
            // 设置下一个操作玩家
            this.data.currentPlayer = this.getNextPlayer(player);
        }

        // 当前过玩家的下家清掉出牌堆
        Integer[] nextPos = this.getNextPos(player.data.pos);
        Player nextPlayer = this.data.playerMap[nextPos[0]];
        if (nextPlayer != null && nextPlayer.data.discardList.length > 0) {
            nextPlayer.data.discardList = new int[0];
        }

        // 广播其他3人
        for (Player tmPlayer : this.actors.values()) {
            if (tmPlayer != null && !Objects.equals(tmPlayer.getId(), player.getId())) {
                Map<String, Object> data = this.roomToMessage(tmPlayer, this.data.currentPlayer);
                // 接风玩家
                data.put("jieFengPlayer", isClear && this.data.jieFengPlayer != null? this.data.jieFengPlayer.getId() : "");
                data.put("time", 20);
                tmPlayer.receive(CommonProto.Cmd.OtherPlayCards_VALUE, data);
            }
        }

        // 记录游戏回放
        Map<String, Object> data = this.roomToPlayBack(this.data.currentPlayer);
        this.playBack.put(Long.toString(System.currentTimeMillis()), data);
    }

    /**
     * 获取队友
     *
     * @param playerId
     * @return
     */
    public Struct.TeammateData getTeammate(String playerId) {
        Struct.TeammateData data = new Struct.TeammateData();
        for (Map.Entry<Integer, Struct. SettlementData> entry : this.data.result.entrySet()) {
            if (entry.getValue() != null && !StringUtils.isEmpty(entry.getValue().playerId)) {
                Player tmPlayer = this.actors.getOrDefault(entry.getValue().playerId, null);
                if (tmPlayer != null && tmPlayer.getId().equals(playerId)) {
                    data.player = tmPlayer;
                    data.rank = entry.getKey();
                }
            }
        }
        return data;
    }

    /**
     * 升级
     */
    private synchronized void upgrade() {
        StringBuilder builder = new StringBuilder();

        // 头游
        Struct.SettlementData winPlayerData = this.data.result.getOrDefault(1, null);
        if (winPlayerData == null || StringUtils.isEmpty(winPlayerData.playerId)) {
            return;
        }
        Player winPlayer = this.actors.getOrDefault(winPlayerData.playerId, null);
        if (winPlayer == null) {
            return;
        }

        // 末游
        Player p4 = null;

        // 原级牌点数
        int oldLevelPoint = this.data.curLevelPoint;

        // 4人房
        if (this.data.maxNum == 4) {
            JSONObject jsonData = this.jsonData;
            Integer lv = (Integer)jsonData.get("upgradeType");
            int[] ints = score(lv);
            // 升级数
            int upLevel = 0;
            // 头游搭档数据
            Struct.TeammateData teammateData = this.getTeammate(winPlayer.data.teammateId);
            switch (teammateData.getRank()) {
                case 2:
                    // 搭档2游, 升3级
                    upLevel = ints[2];
                    builder.append("1游玩家: ").append(winPlayer.data.name)
                            .append(", 搭档玩家: ").append(teammateData.player.data.name)
                            .append(" 2游, 升3级");
                    break;
                case 3:
                    // 搭档3游, 升2级
                    upLevel = ints[1];
                    builder.append("1游玩家: ").append(winPlayer.data.name)
                            .append(", 搭档玩家: ").append(teammateData.player.data.name)
                            .append(" 3游, 升2级");
                    break;
                case 4:
                    // 搭档4游, 升1级
                    upLevel = ints[0];
                    builder.append("1游玩家: ").append(winPlayer.data.name)
                            .append(", 搭档玩家: ").append(teammateData.player.data.name)
                            .append(" 4游, 升1级");
                    break;
            }
            //判断当前玩家的最高分值的局数是否存在
            if (winPlayer.data.bestScore==null||winPlayer.data.bestScore.isEmpty()){
                //设置最大值
                winPlayer.data.bestScore.put(this.data.curRound,upLevel);//当前局次，分值
                teammateData.player.data.bestScore.put(this.data.curRound,upLevel);
            }else {
                for (Map.Entry<Integer, Integer> entry : winPlayer.data.bestScore.entrySet()) {
                    if (upLevel>entry.getValue()){
                        winPlayer.data.bestScore.remove(entry.getKey());
                        winPlayer.data.bestScore.put(this.data.curRound,upLevel);
                        teammateData.player.data.bestScore.remove(entry.getKey());
                        teammateData.player.data.bestScore.put(this.data.curRound,upLevel);
                        break;
                    }
                }
            }

            // 根据类型判断结束走向
            if (this.data.mode == 1) {
                if (winPlayer.data.levelPointView[0] >= this.data.modev) {
                    this.data.isOver = true;
                    // 设置全局当前级牌点数
                    this.data.curLevelPoint = this.data.modev;
                    for (Struct.SettlementData settlementData : this.data.result.values()) {
                        if (settlementData != null) {
                            Player tmPlayer = this.actors.getOrDefault(settlementData.playerId, null);
                            if (tmPlayer != null) {
                                if (settlementData.playerId.equals(winPlayer.getId()) || settlementData.playerId.equals(teammateData.player.getId())) {
                                    settlementData.score = upLevel;
                                } else {
                                    settlementData.score = 0;
                                }
                            }
                        }
                    }
                } else {
                    this.data.isOver = false;
                    // 升级
                    winPlayer.data.levelIndex = Math.min(winPlayer.data.levelIndex + upLevel, GDUtils.levelPoint.length - 1);
                    int score = winPlayer.data.levelPointView[0] + upLevel;//胜利方牌级
                    if (score>this.data.modev){
                        upLevel = score - this.data.modev;
                        score = this.data.modev;
                    }
//                    score = Math.min(score,this.data.modev);
                    if (score ==this.data.modev){
                        if (this.data.lvPlayerId==null){
                            this.data.lvPlayerId = winPlayer.getId();
                        }
                    }
                    // 设置分数等级
                    for (Struct.SettlementData settlementData : this.data.result.values()) {
                        if (settlementData != null) {
                            Player tmPlayer = this.actors.getOrDefault(settlementData.playerId, null);
                            if (tmPlayer != null) {
                                if (settlementData.playerId.equals(winPlayer.getId()) || settlementData.playerId.equals(teammateData.player.getId())) {
                                    settlementData.score = upLevel;
    //                                tmPlayer.data.levelPointView[0] = Math.min(GDUtils.levelPoint[winPlayer.data.levelIndex], this.data.modev);
                                    tmPlayer.data.levelPointView[0] = Math.min(score, this.data.modev);
                                } else {
                                    settlementData.score = 0;
    //                                tmPlayer.data.levelPointView[1] = Math.min(GDUtils.levelPoint[winPlayer.data.levelIndex], this.data.modev);
                                    tmPlayer.data.levelPointView[1] = Math.min(score, this.data.modev);
                                }
                            }
                        }
                    }
                    // 设置全局当前级牌点数
                    this.data.curLevelPoint = winPlayer.data.levelPointView[0];
                }
            } else {
                // 升级
                winPlayer.data.levelIndex = Math.min(winPlayer.data.levelIndex + upLevel, GDUtils.levelPoint.length - 1);
                int scoreLv = winPlayer.data.levelPointView[0] + upLevel;//胜利方牌级
                if (scoreLv>14){
                    scoreLv = scoreLv-13;
                    // 是否套圈
//                    if ((Integer)this.jsonData.get("ferrule") == 0){//头游搭档不为4游套圈次数加1
//                        Integer flippingType = (Integer) this.jsonData.get("flippingType");
//                        switch (flippingType) {
//                            case 1://头游三游过..
//                                if (teammateData.getRank()!=4) {
//                                    winPlayer.data.circleNum++;
//                                    teammateData.player.data.circleNum++;
//                                }else {
//                                    scoreLv = 14;//如果队友是4游，则牌级为14，不升级
//                                }
//                                break;
//                            case 2://头游过..
//                                winPlayer.data.circleNum++;
//                                teammateData.player.data.circleNum++;
//                                break;
//                        }
//                    }
                }
                // 设置分数等级
                for (Struct.SettlementData settlementData : this.data.result.values()) {
                    if (settlementData != null) {
                        Player tmPlayer = this.actors.getOrDefault(settlementData.playerId, null);
                        if (tmPlayer != null) {
                            if (settlementData.playerId.equals(winPlayer.getId()) || settlementData.playerId.equals(teammateData.player.getId())) {
                                settlementData.score = upLevel;
//                                tmPlayer.data.levelPointView[0] = GDUtils.levelPoint[winPlayer.data.levelIndex];
                                tmPlayer.data.levelPointView[0] = scoreLv;
                            } else {
                                settlementData.score = 0;
//                                tmPlayer.data.levelPointView[1] = GDUtils.levelPoint[winPlayer.data.levelIndex];
                                tmPlayer.data.levelPointView[1] = scoreLv;
                            }
                        }
                    }
                }
                // 设置全局当前级牌点数
                this.data.curLevelPoint = winPlayer.data.levelPointView[0];
            }
        }

        Log.info("升级...{}, 原级牌点数: {}, 当前级牌点数: {}", builder.toString(), oldLevelPoint, this.data.curLevelPoint);
    }


    // 升级规则 1:123级，2:124级，3:135级，4:246级
    public static int[] score(int lv){
        switch (lv) {
            case 1:
                return new int[]{1,2,3};
            case 2:
                return new int[]{1,2,4};
            case 3:
                return new int[]{1,3,5};
            case 4:
                return new int[]{2,4,6};
        }
        return new int[]{};
    }

    public static String pos(int pos) {
        switch (pos) {
            case 0:
                return "1";
            case 1:
                return "2";
            case 2:
                return "3";
            case 3:
                return "4";
        }
        return "";
    }

    public boolean checkAllOver() {
        boolean isAllOver = false;
        JSONObject jsonData = this.jsonData;
        Integer flippingType = (Integer)jsonData.get("flippingType");
        // 头游
        Struct.SettlementData winPlayerData = this.data.result.getOrDefault(1, null);
        if (winPlayerData == null || StringUtils.isEmpty(winPlayerData.playerId)) {
            return false;
        }
        Player winPlayer = this.actors.getOrDefault(winPlayerData.playerId, null);
        if (winPlayer == null) {
            return false;
        }
        // 头游搭档数据
        Struct.TeammateData teammateData = this.getTeammate(winPlayer.data.teammateId);
        int[] levelPointView = winPlayer.data.levelPointView;
//        boolean match = Arrays.stream(levelPointView).anyMatch(element -> element == this.data.modev);
        switch (flippingType) {
            case 1: // 过A结束 1.头游队友不是末游 过A结束
                if (winPlayer.data.levelPointView[0] >= this.data.modev && this.data.isOver) {
                    // 头游搭档数据
                    switch (teammateData.getRank()) {
                        case 2:
                            // 搭档2游
                            isAllOver = true;
                            break;
                        case 3:
                            // 搭档3游
                            isAllOver = true;
                            break;
                    }
                    this.setWinPlayer(winPlayer,teammateData.player);
                }
                break;
            case 2://2:有头游过A结束
                if (winPlayer.data.levelPointView[0] >= this.data.modev && this.data.isOver) {
                    isAllOver = true;
                    //设置头游和搭档胜利
                    this.setWinPlayer(winPlayer,teammateData.player);
                }
                break;
            case 3://3局不过A判负结束
                //判断打A的次数
                if (this.data.maxLvNum>=3&&!winPlayer.getId().equals(this.data.lvPlayerId)&&!teammateData.player.getId().equals(this.data.lvPlayerId)){//超过三局直接结束,并且头游或搭档不是第一次打到该牌级 则结束游戏
                    isAllOver = true;
                    this.setWinPlayer(winPlayer,teammateData.player);
                    break;
                }
                if (this.data.lvPlayerId!=null){
                    if(winPlayer.getId().equals(this.data.lvPlayerId)||teammateData.player.getId().equals(this.data.lvPlayerId)){
                        if (this.data.maxLvNum>=3){//超过3局判负
                            isAllOver = true;
                            List<Player> collect = this.actors.values().stream()
                                    .filter(player -> !winPlayer.getId().equals(player.getId()) || !teammateData.player.getId().equals(player.getId()))
                                    .collect(Collectors.toList());
                            this.setWinPlayer(collect.get(0),collect.get(1));
                            break;
                        }
                        //判断当前头游或搭档是否第一次打到该牌级,如果是则判断搭档是否为4游，不为4游直接结束
                        if (winPlayer.data.levelPointView[0] >= this.data.modev&& this.data.isOver&&teammateData.getRank()!=4) {
                            isAllOver = true;
                            this.setWinPlayer(winPlayer,teammateData.player);
                            break;
                        }else {//否则打A次数加1
                            this.data.maxLvNum++;//否则打A次数加1
                        }
                    }
                }
                break;
            case 4://3局不过A从2开始
                if (winPlayer.data.levelPointView[0] >= this.data.modev&&
                        this.data.isOver&&teammateData.getRank()!=4) { //当前头游是否打该牌级 并且搭档不为4游 则结束游戏
                    isAllOver = true;
                    this.setWinPlayer(winPlayer,teammateData.player);
                }
                //不能结束的情况，当前头游不是第一次打到该牌级并且没有满足isOver条件，则打A次数加1
                if (this.data.lvPlayerId!=null&&!this.data.isOver){//到达当前牌级点的玩家不为null
                    if(!winPlayer.getId().equals(this.data.lvPlayerId)&&!teammateData.player.getId().equals(this.data.lvPlayerId)) {
                        this.data.maxLvNum++;
                    }
                    if (this.data.maxLvNum>3){
                        //重新设置牌级为2,清空牌级次数
                        // 第一次打到级牌的玩家和搭档数据
                        Player player = this.actors.getOrDefault(this.data.lvPlayerId, null);
                        Struct.TeammateData teammate = this.getTeammate(winPlayer.data.teammateId);
                        for (Struct.SettlementData settlementData : this.data.result.values()) {
                            if (settlementData != null) {
                                Player tmPlayer = this.actors.getOrDefault(settlementData.playerId, null);
                                if (tmPlayer != null) {
                                    if (settlementData.playerId.equals(player.getId()) || settlementData.playerId.equals(teammate.player.getId())) {
                                        tmPlayer.data.levelPointView[0] = 2;
                                    } else {
                                        tmPlayer.data.levelPointView[1] = 2;
                                    }
                                }
                            }
                        }
                        this.data.maxLvNum = 0;
                        this.data.lvPlayerId = null;
                        if (winPlayer.data.levelPointView[0]>=this.data.modev){
                            this.data.lvPlayerId = winPlayer.getId();
                        }
                    }
                }
        }
        return isAllOver;
    }

    /**
     * 设置头游和搭档胜利
     */
    public void setWinPlayer(Player winPlayer, Player teammate) {
        for (Struct.SettlementData settlementData : this.data.result.values()) {
            if (settlementData != null) {
                Player tmPlayer = this.actors.getOrDefault(settlementData.playerId, null);
                if (tmPlayer != null) {
                    if (settlementData.playerId.equals(winPlayer.getId()) || settlementData.playerId.equals(teammate.getId())) {
                        settlementData.setWin(true);
                    } else {
                        settlementData.setWin(false);
                    }
                }
            }
        }
    }

    /**
     * 牌局结算
     */
    private synchronized void settlement() {
        // 升级
        this.upgrade();
        // 头游
        Struct.SettlementData winPlayerData = this.data.result.getOrDefault(1, null);
        if (winPlayerData == null || StringUtils.isEmpty(winPlayerData.playerId)) {
            return;
        }
        Player winPlayer = this.actors.getOrDefault(winPlayerData.playerId, null);
        if (winPlayer == null) {
            return;
        }

        // 记录游戏回放
        Map<String, Object> playBack = this.roomToPlayBack(this.data.currentPlayer);
        playBack.put("state", 3);
        this.playBack.put(Long.toString(System.currentTimeMillis()), data);

        //向后台发送结算信息
        GGame.getInstance().asyncExec(()->{
            this.data.result.forEach((key,value)->{
                Player tmPlayer = this.actors.getOrDefault(value.playerId, null);
                if (tmPlayer != null) {
                    try {
                        cn.hutool.json.JSONObject object = new cn.hutool.json.JSONObject();
                        ZjUserInfolog zjUserInfolog = new ZjUserInfolog();
                        zjUserInfolog.setUserId(tmPlayer.data.playerId);//用户ID
                        zjUserInfolog.setCardTableId(String.valueOf(this.data.roomId));//房间id
                        zjUserInfolog.setCardLevel(String.valueOf(this.data.curLevelPoint));//牌级
                        zjUserInfolog.setMasterScore(new BigDecimal(tmPlayer.data.masterScore));//大师分
                        int type = this.data.type;//房间类型
                        zjUserInfolog.setCompeteType(String.valueOf(type));
                        zjUserInfolog.setTurnNum(1);//轮次
                        if (type==1||type==2){
                            Contest contest = ContestService.getInstance().contestMap.getOrDefault(tmPlayer.data.contestId, null);
                            if(contest!=null){
                                zjUserInfolog.setCompeteId(String.valueOf(contest.data.contestId));//赛事id
                                zjUserInfolog.setTurnNum(contest.data.curRound);//轮次
                            }
                        }
                        zjUserInfolog.setInningNum(this.data.curRound);//局数
                        zjUserInfolog.setPosition(pos(tmPlayer.data.pos));//方位
                        zjUserInfolog.setCreateTime(new Date());
                        zjUserInfolog.setCompeteResult(tmPlayer.data.playerId.equals(winPlayerData.playerId)?"1":"2");//胜负
                        zjUserInfolog.setGameInfo(this.playBack.toString());
                        zjUserInfolog.setTableStatus(String.valueOf(this.state));
                        zjUserInfolog.setRuleInfo(this.jsonData.toString());
                        zjUserInfolog.setGameTime(this.time);
                        zjUserInfolog.setCompeteRank(key);
                        object.put("type","gameInfo");
                        object.put("data",zjUserInfolog);
                        SendGameInfoUtil.sendGameInfo(object);
                    } catch (JSONException e) {
                        Log.error("向后台发送结算信息出错...roomId={}, gameType={}, {}",
                                this.data.roomId, this.data.gameType, e);
                    }
                }
            });
        });

        // 广播结算
        for (Player tmPlayer : this.actors.values()) {
            if (tmPlayer != null) {
                // 结算信息
                Map<String, Object> data = this.roomToMessage(tmPlayer, null);
                data.put("state", 3);
                List<Map<String, Object>> otherInfo = new ArrayList<>();
                for (Player p : this.actors.values()) {
                    if (p != null && !Objects.equals(tmPlayer.getId(), p.getId())) {
                        otherInfo.add(this.playerRoomToMessage(p, true));
                    }
                }
                data.put("otherInfo", otherInfo);
                tmPlayer.receive(CommonProto.Cmd.GameStateChange_VALUE, data);
            }
        }
        RedisUtil.del("roomGambit:"+this.data.roomId);
        Log.debug("牌局结算...roomId : {}", this.data.roomId);
    }

    /**
     * 套圈结算
     */
    private synchronized void circleSettlement() {
        // 获取头游数据
        Struct.SettlementData winPlayerData = this.data.result.getOrDefault(1, null);
        if (winPlayerData == null || StringUtils.isEmpty(winPlayerData.playerId)) {
            return;
        }

        Player winPlayer = this.actors.getOrDefault(winPlayerData.playerId, null);
        if (winPlayer == null) {
            return;
        }

        // 获取头游搭档数据
        Struct.TeammateData teammateData = this.getTeammate(winPlayer.data.teammateId);
        if (teammateData == null || teammateData.player == null) {
            return;
        }

        // 计算circleA和circleB
        int[] circleCounts = calculateCircleCounts(winPlayer.getId(), teammateData.player.getId());
        int circleA = circleCounts[0];
        int circleB = circleCounts[1];

        // 确定赢家组
        boolean[] winners = determineWinners(circleA, circleB, winPlayer);

        // 设置结算数据
        setSettlementData(winners, winPlayer, teammateData);

        // 广播结算信息
        this.totalSettlement();
    }

    private int[] calculateCircleCounts(String winPlayerId, String teammatePlayerId) {
        int circleA = 0;
        int circleB = 0;

        for (Struct.SettlementData settlementData : this.data.result.values()) {
            if (settlementData == null) continue;

            Player tmPlayer = this.actors.getOrDefault(settlementData.playerId, null);
            if (tmPlayer == null) continue;

            if (settlementData.playerId.equals(winPlayerId) || settlementData.playerId.equals(teammatePlayerId)) {
                circleA = tmPlayer.data.circleNum;
            } else {
                circleB = tmPlayer.data.circleNum;
            }
        }

        return new int[]{circleA, circleB};
    }

    private boolean[] determineWinners(int circleA, int circleB, Player winPlayer) {
        if (circleA > circleB) {
            return new boolean[]{true, true, false}; // 头游和搭档赢
        } else if (circleA < circleB) {
            return new boolean[]{false, false, true}; // 对手赢
        } else { // circleA == circleB
            return winPlayer.data.levelPointView[0] > winPlayer.data.levelPointView[1]
                    ? new boolean[]{true, true, false} // 头游点数高，头游和搭档赢
                    : new boolean[]{false, false, true}; // 对手赢
        }
    }

    private void setSettlementData(boolean[] winners, Player winPlayer, Struct.TeammateData teammateData) {
        for (Struct.SettlementData settlementData : this.data.result.values()) {
            if (settlementData == null) continue;

            Player tmPlayer = this.actors.getOrDefault(settlementData.playerId, null);
            if (tmPlayer == null) continue;

            boolean isWinner =
                    (settlementData.playerId.equals(winPlayer.getId()) && winners[0]) ||
                            (settlementData.playerId.equals(teammateData.player.getId()) && winners[1]) ||
                            (!settlementData.playerId.equals(winPlayer.getId()) && !settlementData.playerId.equals(teammateData.player.getId()) && winners[2]);
            settlementData.setWin(isWinner);
        }
    }

    /**
     * 不套圈判负
     */
    private synchronized boolean noCircleSettlement() {
        boolean isAllOver = false;
        if ((Integer)this.jsonData.get("ferrule") == 0){//套圈，判断能否判负或判2
            return false;
        }
        JSONObject jsonData = this.jsonData;
        Integer flippingType = (Integer)jsonData.get("flippingType");
        // 头游
        Struct.SettlementData winPlayerData = this.data.result.getOrDefault(1, null);
        if (winPlayerData == null || StringUtils.isEmpty(winPlayerData.playerId)) {
            return false;
        }
        Player winPlayer = this.actors.getOrDefault(winPlayerData.playerId, null);
        if (winPlayer == null) {
            return false;
        }
        // 头游搭档数据
        Struct.TeammateData teammateData = this.getTeammate(winPlayer.data.teammateId);
        switch (flippingType) {
            case 3://3局不过A判负结束
                //判断打A的次数
                if (this.data.maxLvNum>=3&&!winPlayer.getId().equals(this.data.lvPlayerId)&&!teammateData.player.getId().equals(this.data.lvPlayerId)){//超过三局直接结束,并且头游或搭档不是第一次打到该牌级 则结束游戏
                    isAllOver = true;
                    this.setWinPlayer(winPlayer,teammateData.player);
                }
                if (this.data.lvPlayerId!=null){
                    if(winPlayer.getId().equals(this.data.lvPlayerId)||teammateData.player.getId().equals(this.data.lvPlayerId)){
                        //判断当前头游或搭档是否第一次打到该牌级,如果是则判断搭档是否为4游，不为4游直接结束
                        if (winPlayer.data.levelPointView[0] >= this.data.modev&& this.data.isOver&&teammateData.getRank()!=4) {
                            isAllOver = true;
                            this.setWinPlayer(winPlayer,teammateData.player);
                        }else {//否则打A次数加1
                            this.data.maxLvNum++;//否则打A次数加1
                        }
                    }else if (this.data.maxLvNum>=3){//超过3局判负
                        isAllOver = true;
                        List<Player> collect = this.actors.values().stream()
                                .filter(player -> !winPlayer.getId().equals(player.getId()) || !teammateData.player.getId().equals(player.getId()))
                                .collect(Collectors.toList());
                        this.setWinPlayer(collect.get(0),collect.get(1));
                    }else {
                        //如果当前头游或搭档不是第一次打到该牌级，则打A次数加1
                        this.data.maxLvNum++;//否则打A次数加1
                    }
                }
                break;
            case 4://3局不过A从2开始
                if (winPlayer.data.levelPointView[0] >= this.data.modev&&
                        this.data.isOver&&teammateData.getRank()!=4) { //当前头游是否打该牌级 并且搭档不为4游 则结束游戏
                    isAllOver = true;
                    this.setWinPlayer(winPlayer,teammateData.player);
                }
                //不能结束的情况，当前头游不是第一次打到该牌级并且没有满足isOver条件，则打A次数加1
                if (this.data.lvPlayerId!=null&&!this.data.isOver){//到达当前牌级点的玩家不为null
                    if(!winPlayer.getId().equals(this.data.lvPlayerId)&&!teammateData.player.getId().equals(this.data.lvPlayerId)) {
                        this.data.maxLvNum++;
                    }
                    if (this.data.maxLvNum>3){
                        //重新设置牌级为2,清空牌级次数
                        // 第一次打到级牌的玩家和搭档数据
                        Player player = this.actors.getOrDefault(this.data.lvPlayerId, null);
                        Struct.TeammateData teammate = this.getTeammate(winPlayer.data.teammateId);
                        for (Struct.SettlementData settlementData : this.data.result.values()) {
                            if (settlementData != null) {
                                Player tmPlayer = this.actors.getOrDefault(settlementData.playerId, null);
                                if (tmPlayer != null) {
                                    if (settlementData.playerId.equals(player.getId()) || settlementData.playerId.equals(teammate.player.getId())) {
                                        tmPlayer.data.levelPointView[0] = 2;
                                    } else {
                                        tmPlayer.data.levelPointView[1] = 2;
                                    }
                                }
                            }
                        }
                        this.data.maxLvNum = 0;
                        this.data.lvPlayerId = null;
                        if (winPlayer.data.levelPointView[0]>=this.data.modev){
                            this.data.lvPlayerId = winPlayer.getId();
                        }
                    }
                }
        }
        return isAllOver;
    }

    /**
     * 游戏结束
     */
    private synchronized void gameOver() {
        this.state = 3;
        this.time = 0;
        this.flag = false;
        Log.debug("牌局结束...roomId : {}", this.data.roomId);

//        Log.info("回放数据 : ", this.playBack.toString());
    }

    /**
     * 下一局重置房间数据
     */
    @Override
    public void resetNextRoom() {
        Log.debug("下一局重置玩家数据...");
        for (Player player : this.data.playerMap) {
            if (player != null) {
                player.data.state = 0;
                player.data.remainCards = new int[0];
                player.data.discardList = new int[0];
                player.data.disCardType = null;
                player.data.tributeCard = 0;
                player.data.tributeGetCard = 0;
                player.data.isChangePos = false;
            }
        }

        Log.debug("下一局重置桌子数据...");
        this.data.currentPlayer = null;
        this.data.otherOpPlayerList.clear();
        this.data.curDisCardList = new int[]{};
        this.data.curDisCardType = null;
        this.data.disCardPlayer = null;
        this.data.passList.clear();
        this.data.tributeState = 0;
        this.data.tributeCardMap.clear();
        this.data.cardList.clear();
        this.data.isOver = false;
        // 根据配置清理参数
        if (!this.data.isTribute) {
//            this.data.result.clear();
            this.data.tributeList.clear();
            this.data.towKingList.clear();
        }

        // 初始化牌桌
        this.initCardList();
    }

    /**
     * 清理桌子信息
     */
    @Override
    public void resetRoom() {
        Log.debug("清理房间玩家数据...");
        if (!this.actors.isEmpty()) {
            for (Player player : this.actors.values()) {
                if (player != null) {
                    player.resetRoom();
                }
            }
            this.actors.clear();
        }

        Log.debug("清理房间数据...");
        this.jsonData.clear();
        this.flag = false;
        this.time = 0;
        this.state = 0;
        this.lock = new ReentrantLock();
        this.changePosMap.clear();
        this.playBack.clear();

        this.data.init();
    }

    /**
     * 房间信息转消息
     *
     * @param player 本人
     * @param opPlayer 当前操作玩家
     * @return
     */
    @Override
    public Map<String, Object> roomToMessage(Player player, Player opPlayer) {
        Map<String, Object> data = new HashMap<>();
        data.put("roomId", this.data.roomId);
        data.put("type", this.data.type);
        data.put("gameType", this.data.gameType);
        data.put("ownerId", this.data.ownerId);
        data.put("state", this.state);
        data.put("time", this.time);

        data.put("mode", this.data.mode);
        data.put("modev", this.data.modev);
        data.put("maxRound", this.data.maxRound);
        data.put("maxTime", this.data.maxTime);

        data.put("curRound", this.data.curRound);
        data.put("curTime", this.data.curTime);
        data.put("zhuangPos", this.data.zhuangPos);
        // 当前级牌点数
        data.put("curLevelPoint", this.data.curLevelPoint);
        // 是否有进还贡
        data.put("isTribute", this.data.isTribute);
        data.put("opId", opPlayer != null ? opPlayer.getId() : "");
        data.put("opPos", opPlayer != null ? opPlayer.data.pos : -1);
        data.put("curDiscardList", this.data.curDisCardList);
        data.put("curDisCardType", this.data.curDisCardType);
        data.put("curDiscardPlayer", this.data.disCardPlayer != null ? this.data.disCardPlayer.getId() : "");
        data.put("curDiscardSex", this.data.disCardPlayer != null ? this.data.disCardPlayer.data.sex : 0);
        data.put("curPassList", this.data.passList);
        data.put("roomIp",this.data.roomIp);
        data.put("myInfo", this.playerRoomToMessage(player, true));
        List<Map<String, Object>> otherInfo = new ArrayList<>();
        for (Player tmPlayer : this.actors.values()) {
            if (tmPlayer != null && !Objects.equals(tmPlayer.getId(), player.getId())) {
                otherInfo.add(this.playerRoomToMessage(tmPlayer, false));
            }
        }
        data.put("otherInfo", otherInfo);
        // 增加结算玩家信息
        if (!this.data.result.isEmpty()) {
            Map<Integer, Object> settlementInfo = new HashMap<>();
            for (Map.Entry<Integer, Struct.SettlementData> entry : this.data.result.entrySet()) {
                Map<String, Object> settlementData = new HashMap<>();
                settlementData.put("playerId", entry.getValue().playerId);
                settlementData.put("score", entry.getValue().score);
                settlementInfo.put(entry.getKey(), settlementData);
            }
            data.put("settlementInfo", settlementInfo);
        }
        // todo 测试代码
        data.put("房间规则", this.jsonData);
        return data;
    }

    /**
     * 玩家信息转消息
     *
     * @param flag 是否显示手牌
     * @return
     */
    public Map<String, Object> playerRoomToMessage(Player player, boolean flag) {
        Map<String, Object> data = new HashMap<>();
        data.put("id", player.getId());
        data.put("name", player.data.name);
        data.put("avatar", "https://img95.699pic.com/element/40109/0194.png_300.png");
        data.put("sex", player.data.sex);
        data.put("pos", player.data.pos);
        data.put("state", player.data.state);
        data.put("isAuto", player.data.isAuto);
        // 级数显示 [我方级数, 对方级数]
        data.put("levelPointView", player.data.levelPointView);
        // 队友Id
        data.put("teammateId", player.data.teammateId);
        //最高分值
        Map<Integer, Integer> bestScore = player.data.bestScore;
        if (bestScore.isEmpty()||bestScore==null||bestScore.size()<=0) {
            bestScore.put(0, 0);
        }
        data.put("bestScore", bestScore);
        // 玩家牌数据
        Map<String, Object> cardInfo = new HashMap<>();
        cardInfo.put("remainCards", flag? player.data.remainCards : "");
        cardInfo.put("remainCardsNum", player.data.remainCards.length);
        cardInfo.put("discardList", player.data.discardList);
        cardInfo.put("disCardType", player.data.disCardType);
        cardInfo.put("tributeGetCard", player.data.tributeGetCard);
        data.put("cardInfo", cardInfo);
        return data;
    }

    /**
     * 房间信息转游戏回放
     *
     * @param opPlayer 当前操作玩家
     * @return
     */
    public Map<String, Object> roomToPlayBack(Player opPlayer) {
        Map<String, Object> data = new HashMap<>();
        data.put("roomId", this.data.roomId);
        data.put("type", this.data.type);
        data.put("gameType", this.data.gameType);
        data.put("ownerId", this.data.ownerId);
        data.put("state", this.state);
        data.put("time", this.time);

        data.put("ip", this.ip);

        data.put("maxRound", this.data.maxRound);
        data.put("curRound", this.data.curRound);
        data.put("zhuangPos", this.data.zhuangPos);
        // 当前级牌点数
        data.put("curLevelPoint", this.data.curLevelPoint);
        // 是否有进还贡
        data.put("isTribute", this.data.isTribute);
        data.put("opId", opPlayer != null ? opPlayer.getId() : "");
        data.put("opPos", opPlayer != null ? opPlayer.data.pos : -1);
        data.put("curDiscardList", this.data.curDisCardList);
        data.put("curDisCardType", this.data.curDisCardType);
        data.put("curDiscardPlayer", this.data.disCardPlayer != null ? this.data.disCardPlayer.getId() : "");
        data.put("curDiscardSex", this.data.disCardPlayer != null ? this.data.disCardPlayer.data.sex : 0);
        data.put("curPassList", this.data.passList.toArray());
        // 玩家信息
        if (!this.actors.isEmpty()) {
            List<Map<String, Object>> playerInfo = new ArrayList<>();
            for (Player tmPlayer : this.actors.values()) {
                if (tmPlayer != null) {
                    playerInfo.add(playerRoomToPlayBack(tmPlayer));
                }
            }
            data.put("playerInfo", playerInfo);
        }
        // 增加结算玩家信息
        if (!this.data.result.isEmpty()) {
            Map<String, Object> settlementInfo = new HashMap<>();
            for (Map.Entry<Integer, Struct.SettlementData> entry : this.data.result.entrySet()) {
                Map<String, Object> settlementData = new HashMap<>();
                settlementData.put("playerId", entry.getValue().playerId);
                settlementInfo.put(entry.getKey().toString(), settlementData);
            }
            data.put("settlementInfo", settlementInfo);
        }
        return data;
    }

    /**
     * 玩家信息转消息
     *
     * @param player 玩家
     * @return
     */
    public Map<String, Object> playerRoomToPlayBack(Player player) {
        Map<String, Object> data = new HashMap<>();
        data.put("id", player.getId());
        data.put("name", player.data.name);
        data.put("avatar", "https://img95.699pic.com/element/40109/0194.png_300.png");
        data.put("sex", player.data.sex);
        data.put("pos", player.data.pos);
        data.put("state", player.data.state);
        data.put("isAuto", player.data.isAuto);
        // 级数显示 [我方级数, 对方级数]
        data.put("levelPointView", player.data.levelPointView);
        // 队友Id
        data.put("teammateId", player.data.teammateId);

        // 玩家牌数据
        Map<String, Object> cardInfo = new HashMap<>();
        cardInfo.put("remainCards", player.data.remainCards);
        cardInfo.put("remainCardsNum", player.data.remainCards.length);
        cardInfo.put("discardList", player.data.discardList);
        cardInfo.put("disCardType", player.data.disCardType);
        cardInfo.put("tributeGetCard", player.data.tributeGetCard);
        data.put("cardInfo", cardInfo);
        return data;
    }
}
