package com.incubator.game.room;

import cn.hutool.json.JSONException;
import com.alibaba.fastjson2.JSONObject;
import com.incubator.common.log4j.Log;
import com.incubator.core.room.GRoomInterface;
import com.incubator.game.constant.RedisKeyConstant;
import com.incubator.game.contest.Contest;
import com.incubator.game.contest.ContestService;
import com.incubator.game.data.entity.ZjUserInfolog;
import com.incubator.game.data.jedis.RedisUtil;
import com.incubator.game.data.po.RoomPO;
import com.incubator.game.player.Player;
import com.incubator.game.util.GDUtils;
import com.incubator.game.util.SendGameInfoUtil;

import java.math.BigDecimal;
import java.util.Date;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantLock;

/**
 * 房间实体基类
 */
public class Room implements GRoomInterface {

    /** 房间内数据 **/
    public RoomPO data;
    /** 房间角色 [key:玩家id, value:玩家对象]**/
    public Map<String, Player> actors = new ConcurrentHashMap<>();

    /** 房间规则数据 **/
    public JSONObject jsonData;

    /** 激活标记 **/
    public boolean isActive;
    /** 检测标记 **/
    public boolean flag;
    /** 时间秒数 **/
    public int time;
    /** 房间状态(0等待 1开局 2进行中 3已结束) **/
    public int state;
    /** 并发锁 **/
    public Lock lock = new ReentrantLock();
    /** 换位缓存 [key:同意的玩家, value:发起的玩家]**/
    public Map<String, String> changePosMap = new ConcurrentHashMap<>();

    /** 回放数据 **/
    public JSONObject playBack = new JSONObject();

    /** 房间任务主线程 **/
    private ScheduledExecutorService scheduler;

    public Room() {
        this.scheduler = Executors.newSingleThreadScheduledExecutor();
    }

    /** 初始化 **/
    public void init(Player player, JSONObject jsonData) {}

    /** 加入房间 **/
    public void joinRoom(Player player) {}

    /** 离开房间 **/
    public void leaveRoom(Player player) {}

    /**
     * 开启房间主线程任务
     */
    public void startRoomTask() {
        this.isActive = true;
        if (scheduler == null) {
            this.scheduler = Executors.newSingleThreadScheduledExecutor();
        }
        this.scheduler.scheduleAtFixedRate(() -> {
            if (this.isActive) {
                this.doUpdate();
            }
        }, 0, 1, TimeUnit.SECONDS);
    }

    public void initCardList() {}

    /**
     * 房间主线程逻辑
     */
    public void doUpdate() {}

    public void doSendCard() {}

    public void doStart() {}

    /** 玩家换座位 **/
    public void switchSeat(Player player, int targetSeat) {}

    public void playerDisCard(Player player, int[] disCardList, GDUtils.CardType disCardType) {}

    public void playerPass(Player player, boolean isAuto) {}

    public boolean checkBackTributeStart() {
        return false;
    }

    public void resetNextRoom() {}

    /**
     * 房间重置逻辑
     */
    public void resetRoom() {}

    /**
     * 销毁房间，释放资源
     */
    public void destroy() {
        Log.info("Room {} has been destroyed.", this.data.roomId);
        this.isActive = false;
        try {
            cn.hutool.json.JSONObject object = new cn.hutool.json.JSONObject();
            object.put("type","gameOver");
            SendGameInfoUtil.sendGameInfo(object);
        } catch (JSONException e) {
            Log.error("向后台发送结算信息出错...{}", e);
        }
        this.scheduler.shutdownNow();
        this.scheduler = null;
        // 从缓存移除
        RoomService.getInstance().roomMap.remove(this.data.roomId);
        RedisUtil.del("roomGambit:"+this.data.roomId);
        RedisUtil.delRoomToRedis(RedisKeyConstant.WK_ROOM_MAP, String.valueOf(this.data.roomId));
        //RedisUtil.removeKeyFromRoomMap(RedisKeyConstant.WK_ROOM_MAP, this.data.roomId);
        // 重置房间数据
        this.resetRoom();
        // 归还房间对象
        RoomService.getInstance().roomPool.releaseRoom(this);
    }

    public Map<String, Object> roomToMessage(Player player, Player opPlayer) {
        return null;
    }

    public Map<String, Object> playerRoomToMessage(Player player, boolean flag) {
        return null;
    }

    public String ip;
    public String port;
    public void setIp(String s) {
        this.ip = s;
    }

    public void setPort(String s) {
        this.port = s;
    }


}
