package com.incubator.game.room;

import cn.hutool.json.JSONException;
import com.alibaba.fastjson2.JSONObject;
import com.incubator.common.log4j.Log;
import com.incubator.common.util.RandomUtil;
import com.incubator.game.GGame;
import com.incubator.game.constant.RedisKeyConstant;
import com.incubator.game.constant.RoomStatus;
import com.incubator.game.contest.Contest;
import com.incubator.game.contest.ContestService;
import com.incubator.game.data.entity.ZjUserInfolog;
import com.incubator.game.data.jedis.RedisUtil;
import com.incubator.game.data.po.RoomPO;
import com.incubator.game.player.Player;
import com.incubator.game.util.*;
import com.incubator.message.proto.CommonProto;
import org.apache.commons.lang3.StringUtils;

import java.math.BigDecimal;
import java.util.*;
import java.util.concurrent.locks.ReentrantLock;

/**
 * 转蛋房间
 */
public class ZDRoom extends Room {

    public ZDRoom() {}

    @Override
    public void init(Player player, JSONObject jsonData) {
        this.jsonData = jsonData;
        // todo 默认配置
        this.data = new RoomPO();
        this.data.roomId = jsonData.getLong("roomId");
        this.data.type = jsonData.getInteger("type");
        this.data.gameType = jsonData.getInteger("gameType");
        this.data.ownerId = player != null? player.getId() : "";
        this.data.maxNum = 4;
        this.data.mode = jsonData.getInteger("mode");
        this.data.modev = jsonData.getInteger("modev");

        // 按局数
        if (this.data.mode == 2) {
            this.data.maxRound = jsonData.getInteger("modev");
        }

        // 按时间
        if (this.data.mode == 3) {
            this.data.curTime = System.currentTimeMillis();
            long time = jsonData.getInteger("modev");
            this.data.maxTime = this.data.curTime + (time * 60L * 1000L);
            this.data.maxRound = 9999;
        }

        this.data.playerMap = new Player[this.data.maxNum];
        // 是否有进还贡规则
        this.data.isTribute = false;
        // 初始化牌库
        this.initCardList();
        // 初始化比赛
        this.data.contestId = this.data.type == 1? jsonData.getLong("contestId") : 0;
    }

    /**
     * 玩家加入房间
     */
    @Override
    public synchronized void joinRoom(Player player) {
        if (player == null) {
            return;
        }
        lock.lock();
        try {
            if (this.actors.size() >= this.data.maxNum) {
                return; // 房间已满
            }
            // 重置玩家房间数据
            player.resetRoom();
            // 玩家绑定房间
            player.data.roomId = this.data.roomId;

            // 加入房间角色缓存
            if (!this.actors.containsKey(player.getId())) {
                this.actors.put(player.getId(), player);
            }
            // 设置座位号
            for (int i = 0; i < this.data.playerMap.length; i++) {
                Player tmPlayer = this.data.playerMap[i];
                if (tmPlayer == null) {
                    player.data.pos = i;
                    this.data.playerMap[i] = player;
                    break;
                }
            }

            // 推送房间其他3个玩家
            for (Player tmPlayer : this.actors.values()) {
                if (tmPlayer != null && !Objects.equals(tmPlayer.getId(), player.getId())) {
                    Map<String, Object> data = this.roomToMessage(tmPlayer, null);
                    tmPlayer.receive(CommonProto.Cmd.RoomPlayerJoin_VALUE, data);
                }
            }

            // 有人就开启定时器
            if (!this.actors.isEmpty())  {
                this.startRoomTask();
            }
        } finally {
            lock.unlock();
        }
    }

    /** 玩家离开房间 **/
    @Override
    public synchronized void leaveRoom(Player player) {
        if (player == null) {
            return;
        }
        lock.lock();
        try {
            // 清理房间数据
            this.actors.remove(player.getId());
            RoomStatus roomToRedis = RedisUtil.getRoomToRedis(RedisKeyConstant.WK_ROOM_MAP, String.valueOf(this.data.roomId));
            if (roomToRedis != null){
                List<String> players = roomToRedis.getPlayers();
                players.remove(player.getId());
                roomToRedis.setPlayers(players);
            }
            this.data.playerMap[player.data.pos] = null;
            // todo 房主退出
            if (player.getId().equals(this.data.ownerId)) {
//                Player nextPlayer = this.getNextPlayer(player);
//                if (nextPlayer != null) {
//                    this.data.ownerId = nextPlayer.getId();
//                }
            }
            // 重置玩家房间数据
            player.resetRoom();

            // 推送房间其他玩家
            for (Player tmPlayer : this.actors.values()) {
                if (tmPlayer != null && !Objects.equals(tmPlayer.getId(), player.getId())) {
                    Map<String, Object> data = roomToMessage(tmPlayer, null);
                    tmPlayer.receive(CommonProto.Cmd.RoomPlayerLeave_VALUE, data);
                }
            }
        } finally {
            lock.unlock();
        }

        // 广播
        if (this.state == 2){
            for (Player tmPlayer : this.actors.values()) {
                if (tmPlayer != null) {
                    tmPlayer.receive(CommonProto.Cmd.DisbandRoom_VALUE, new HashMap<>());
                }
            }
            this.destroy();
        }
    }

    /**
     * 初始化牌库
     */
    @Override
    public void initCardList() {
        for (int i = 0; i < 2; i++) {
            this.data.cardList.addAll(Arrays.asList(GDUtils.values));
            Collections.shuffle(this.data.cardList);
        }
    }

    /**
     * 主线程逻辑
     */
    @Override
    public void doUpdate() {
        synchronized (this) {
//            Log.debug("检测: 经典掼蛋房间主线程执行...");

            // 房间没人直接解散
            if (this.actors.isEmpty()) {
                this.destroy();
                return;
            }

            switch (this.state) {
                case 0:
                    // 等待状态
                    if (this.checkReadyStart()) {
                        this.state = 1;
                        this.flag = false;
                        this.time = 0;
                    }

                    // 120秒未开始直接解散
//                    if (this.time >= 120 && this.type != 1) {
//                        // 广播
//                        for (Player tmPlayer : this.actors.values()) {
//                            if (tmPlayer != null) {
//                                tmPlayer.receive(CommonProto.Cmd.DisbandRoom_VALUE, new HashMap<>());
//                            }
//                        }
//                        this.destroy();
//                    }

                    break;
                case 1:
                    //锁
                    RedisUtil.set("roomGambit:"+this.data.roomId, "0");

                    // 开局
                    if (!this.flag) {
                        //开始
                        this.doStart();
                        Log.debug("检测: 房间号:{}, 房间人数:{}, 当前局数:{}, 状态:开局...VS动画...", this.data.roomId, this.actors.size(), this.data.curRound);
                    }
                    break;
                case 2:
                    // 进行中
                    if (!this.flag) {
                        // 检查进贡是否完成
                        if (this.data.tributeState == 1 && this.checkTributeStart()) {
                            // 切换还贡状态
                            this.doChangeBackTribute();
                            this.time = 0;
                        }

                        // 检查还贡是否完成
                        if (this.data.tributeState == 2 && this.checkBackTributeStart()) {
                            // 进还贡换牌
                            this.doChangeTributeCard();
                            this.flag = true;
                            this.time = 0;
                        }

                        // 80秒强制操作进还贡
//                        if (this.time >= 80) {
//                            this.doTribute();
//                            this.flag = true;
//                            this.time = 0;
//                        }
                    } else {
                        // 20秒未操作,强制过
//                        if (this.time >= 20) {
//                            Log.debug("强制操作过...");
//                            this.playerPass(this.data.currentPlayer, true);
//                            this.flag = true;
//                            this.time = 0;
//                        }
                    }
                    break;
                case 3:
                    // 结束
                    if (!this.flag) {
                        this.flag = true;
                        this.time = 0;
                        Log.debug("游戏结束: 房间号 : {}, 房间人数 : {}, 当前局数 : {}", this.data.roomId, this.actors.size(), this.data.curRound);
                    } else {
                        // 根据类型判断结束走向
                        switch (this.data.mode) {
                            case 1:
                                // 连续打
                                this.resetNextRoom();
                                this.state = 0;
                                this.flag = false;
                                this.time = 0;
                                break;
                            case 2:
                                // 按局数
                                // 小于最大局数,切等待状态
                                if (this.data.curRound < this.data.maxRound) {
                                    this.resetNextRoom();
                                    this.state = 0;
                                    this.flag = false;
                                    this.time = 0;
                                } else {
                                    //判断锁是否存在 //等待消失
                                    try {
                                        Thread.sleep(1000);
                                    } catch (InterruptedException e) {
                                        e.printStackTrace();
                                    }
                                    this.totalSettlement();
                                    //判断锁是否存在 //等待消失
                                    try {
                                        Thread.sleep(1000);
                                    } catch (InterruptedException e) {
                                        e.printStackTrace();
                                    }

                                    // 广播
                                    for (Player tmPlayer : this.actors.values()) {
                                        if (tmPlayer != null) {
                                            tmPlayer.receive(CommonProto.Cmd.DisbandRoom_VALUE, new HashMap<>());
                                        }
                                    }
                                    if (this.data.type != 1){
                                        this.destroy();
                                    }
                                    this.isActive = false;
                                }
                                break;
                            case 3:
                                // 按时间
                                // 小于最大时间,切等待状态
                                if (System.currentTimeMillis() < this.data.maxTime) {
                                    this.resetNextRoom();
                                    this.state = 0;
                                    this.flag = false;
                                    this.time = 0;
                                } else {
                                    try {
                                        Thread.sleep(1000);
                                    } catch (InterruptedException e) {
                                        e.printStackTrace();
                                    }
                                    this.totalSettlement();
                                    //判断锁是否存在 //等待消失

                                    try {
                                        Thread.sleep(1000);
                                    } catch (InterruptedException e) {
                                        e.printStackTrace();
                                    }

                                    // 广播
                                    for (Player tmPlayer : this.actors.values()) {
                                        if (tmPlayer != null) {
                                            tmPlayer.receive(CommonProto.Cmd.DisbandRoom_VALUE, new HashMap<>());
                                        }
                                    }
                                    if (this.data.type != 1){
                                        this.destroy();
                                    }
                                }
                                break;
                            default:
                                break;
                        }

                        // 120秒未开始直接解散
                        if (this.time >= 120&&this.data.type!=1) {
                            // 广播
                            for (Player tmPlayer : this.actors.values()) {
                                if (tmPlayer != null) {
                                    tmPlayer.receive(CommonProto.Cmd.DisbandRoom_VALUE, new HashMap<>());
                                }
                            }
                            this.destroy();
                        }
                    }
                    break;

            }

            this.time += 1;
        }
    }


    /**
     * 总结算
     */
    public void totalSettlement() {
        // 广播结算信息
        for (Player tmPlayer : this.actors.values()) {
            if (tmPlayer != null) {
                Map<String, Object> data = this.roomToMessage(tmPlayer, null);
                // 增加结算玩家信息
                if (!this.data.result.isEmpty()) {
                    Map<Integer, Object> settlementInfo = new HashMap<>();
                    for (Map.Entry<Integer, Struct.SettlementData> entry : this.data.result.entrySet()) {
                        Map<String, Object> settlementData = new HashMap<>();
                        settlementData.put("playerId", entry.getValue().playerId);
                        settlementData.put("score", entry.getValue().score);
                        settlementData.put("levelPoint", this.actors.getOrDefault(entry.getValue().playerId,null).data.levelPointView[0]);
                        settlementInfo.put(entry.getKey(), settlementData);
                    }
                    data.put("settlementInfo", settlementInfo);
                }
                tmPlayer.receive(CommonProto.Cmd.SummaryCalculation_VALUE, data);
            }
        }
    }

    /**
     * 检测是否可以开始 3秒倒计时开始
     *
     * @return
     */
    public boolean checkReadyStart() {
        if (!this.isStart() || this.state != 0) {
            return false;
        }

        // 广播3秒倒计时
        for (Player tmPlayer : this.actors.values()) {
            if (tmPlayer != null) {
                Map<String, Object> data = this.roomToMessage(tmPlayer, null);
                data.put("state", 1);
                data.put("time", 0);
//                data.put("time", 3);
                tmPlayer.receive(CommonProto.Cmd.GameStateChange_VALUE, data);
            }
        }

        return true;
    }

    /**
     * 是否玩家都已准备,可开始
     *
     * @return
     */
    public boolean isStart() {
        int cnt = 0;
        for (Player player : this.actors.values()) {
            if (player != null && player.data.state == 1) {
                cnt += 1;
            }
        }
        return cnt >= this.data.maxNum;
    }

    /**
     * 给玩家发牌，每人27张
     */
    @Override
    public synchronized void doSendCard() {
        for (Player player : this.actors.values()) {
            if (player != null) {
                // 设置用户状态
                player.data.state = 2;
                // 设置队友位置
                if (this.data.maxNum >= 4) {
                    Integer[] nextPos = this.getNextPos(player.data.pos);
                    Player tmpPlayer = this.data.playerMap[nextPos[1]];
                    if (tmpPlayer != null) {
                        player.data.teammateId = tmpPlayer.getId();
                    }
                }
                // 发牌
                for (int i = 0; i < 27; i++) {
                    int card = this.data.cardList.get(0);
                    // 设置当前赖子
                    if (GDUtils.isWildCard(card, this.data.curLevelPoint)) {
                        this.data.wildCard = card;
                    }
                    player.data.remainCards = CardUtils.add(player.data.remainCards, card);
                    this.data.cardList.remove(0);
                }
                // todo 测试代码
//                if (this.data.wildCard != 0) {
//                    player.data.remainCards = CardUtils.add(player.data.remainCards, this.data.wildCard);
//                    player.data.remainCards = CardUtils.add(player.data.remainCards, this.data.wildCard);
//                    player.data.remainCards = CardUtils.add(player.data.remainCards, this.data.wildCard);
//                }
            }
        }


        // 随机搭档
        if (this.jsonData.getInteger("team") == 1) {
            // 第一把之后才换位置
//            if (this.data.curRound == 1) {
//                return;
//            }
            List<String> teammate = new ArrayList<>();
            for (Player tmPlayer : this.actors.values()) {
                if (tmPlayer != null && CardUtils.contains(tmPlayer.data.remainCards, this.data.showCard)) {
                    // 设置翻牌队友
                    if (!teammate.contains(tmPlayer.getId())) {
                        teammate.add(tmPlayer.getId());
                    }
                }
            }
            if (teammate.size() == 2) {
                List<String> other = new ArrayList<>();
                for (Player tmPlayer : this.actors.values()) {
                    if (tmPlayer != null && !teammate.contains(tmPlayer.getId())) {
                        other.add(tmPlayer.getId());
                    }
                }
                if (other.size() == 2) {
                    Player p1 = this.actors.getOrDefault(teammate.get(0), null);
                    Player p2 = this.actors.getOrDefault(teammate.get(1), null);
                    if (p1 != null && p2 != null) {
                        p1.data.pos = 0;
                        p2.data.pos = 2;
                        p1.data.teammateId = p2.getId();
                        p2.data.teammateId = p1.getId();
                        // 设置座位
                        this.data.playerMap[0] = p1;
                        this.data.playerMap[2] = p2;
                    }
                    Player p3 = this.actors.getOrDefault(other.get(0), null);
                    Player p4 = this.actors.getOrDefault(other.get(1), null);
                    if (p3 != null && p4 != null) {
                        p3.data.pos = 1;
                        p4.data.pos = 3;
                        p3.data.teammateId = p4.getId();
                        p4.data.teammateId = p3.getId();
                        // 设置座位
                        this.data.playerMap[1] = p3;
                        this.data.playerMap[3] = p4;
                    }
                    this.data.zhuangPos = 0;
                }
            }
        }

        // 均匀分配
        if (this.jsonData.getInteger("team") == 2) {
            // 第一把之后才换位置
            if (this.data.curRound == 1) {
                return;
            }
            // 按局数 并且 局数必须是3的倍数
            if (this.data.mode != 2 || this.data.maxRound % 3 != 0) {
                return;
            }
            // 0号玩家不变
//            Player fixedPlayer = this.data.playerMap[0];
            // 暂存 1号位置的玩家
            Player temp = this.data.playerMap[1];

            // 将 2号和 3号玩家向前移
            this.data.playerMap[1] = this.data.playerMap[2];
            this.data.playerMap[2] = this.data.playerMap[3];
            // 将原 1号玩家移动到 3号位置
            this.data.playerMap[3] = temp;
            // 保持 0号位置玩家不变
//            this.data.playerMap[0] = fixedPlayer;

            // 重置一下玩家pos
            for (int i = 0; i < this.data.playerMap.length; i++) {
                Player tmPlayer = this.data.playerMap[i];
                if (tmPlayer != null) {
                    tmPlayer.data.pos = i;
                }
            }

            // 设置队友
            Player p0 = this.data.playerMap[0];
            Player p2 = this.data.playerMap[2];
            if (p0 != null && p2 != null) {
                p0.data.teammateId = p2.getId();
                p2.data.teammateId = p0.getId();
            }
            Player p1 = this.data.playerMap[1];
            Player p3 = this.data.playerMap[3];
            if (p1 != null && p3 != null) {
                p1.data.teammateId = p3.getId();
                p3.data.teammateId = p1.getId();
            }
        }
    }

    /**
     * 游戏开局
     */
    @Override
    public synchronized void doStart() {
        lock.lock();
        try {
            this.data.curRound += 1;
            // 随机定庄
            this.data.zhuangPos = RandomUtil.getIndex(this.data.maxNum);
            // 随机打几
            if (this.jsonData.getInteger("tc") == 0) {
                this.data.curLevelIndex = RandomUtil.getIndex(GDUtils.randomTeam.length);
                // 随机队友-系统翻牌
                this.data.showCard = GDUtils.randomTeam[this.data.curLevelIndex];
            }
            // 固定打2
            if (this.jsonData.getInteger("tc") == 1) {
                this.data.curLevelIndex = 12;
                // 随机队友-系统翻牌
                this.data.showCard = GDUtils.randomTeam[this.data.curLevelIndex];
            }
            // 级牌下标
            int index = (this.data.curLevelIndex + 1) % GDUtils.randomTeam.length;
            this.data.curLevelPoint = (GDUtils.randomTeam[index] & 0xF);
            // 发牌
            this.doSendCard();
            // 进还贡数据
            Map<String, Object> tributeInfo = null;
            if (this.data.isTribute) {
                tributeInfo = new HashMap<>();
                this.checkTributeInfo();
                // 定庄
                if (this.isTwoKing()) {
                    // 定庄 头游先出
                    Struct.SettlementData player1Data = this.data.result.getOrDefault(1, null);
                    if (player1Data == null || StringUtils.isEmpty(player1Data.playerId)) {
                        return;
                    }
                    Player p1 = this.actors.getOrDefault(player1Data.playerId, null);
                    if (p1 != null) {
                        this.data.zhuangPos = p1.data.pos;
                    }
                    this.data.result.clear();
                }
                if (this.data.towKingList.size() >= 2) {
                    this.data.tributeState = 3;
                } else {
                    if (!this.data.tributeList.isEmpty() && !this.isTwoKing() && this.data.tributeState == 0) {
                        this.data.tributeState = 1;
                    }
                }
                tributeInfo.put("tributeState", this.data.tributeState);
                tributeInfo.put("tributeList", this.data.tributeList);
                tributeInfo.put("twoKingInfo", this.data.towKingList);
                tributeInfo.put("tributeCardMap", this.data.tributeCardMap);
            }
            // 设置当前操作玩家
            this.data.currentPlayer = this.data.playerMap[this.data.zhuangPos];
            if (this.jsonData.getInteger("team") == 2&&this.data.result.size()>0){
                Struct.SettlementData orDefault = this.data.result.getOrDefault(4, null);
                String playerId = orDefault.getPlayerId();
                this.data.currentPlayer = this.actors.getOrDefault(playerId, null);
            }
            this.data.result.clear();//清理排名信息
            // 广播
            for (Player tmPlayer : this.actors.values()) {
                if (tmPlayer != null) {
                    Map<String, Object> data = this.roomToMessage(tmPlayer, this.data.currentPlayer);
                    data.put("zhuangPos", this.data.zhuangPos);
                    if (this.data.isTribute) {
                        data.put("zhuangPos", !this.data.tributeList.isEmpty() && !this.isTwoKing()? -1 : this.data.zhuangPos);
                    }
                    data.put("tributeInfo", tributeInfo);
                    data.put("state", 2);
                    data.put("time", !this.data.tributeList.isEmpty()? 40 : 20);
                    Log.debug("玩家 : {}, 手牌 : {}", tmPlayer.data.name, tmPlayer.data.remainCards);
                    tmPlayer.receive(CommonProto.Cmd.GameStateChange_VALUE, data);
                }
            }

            // 记录游戏回放
            Map<String, Object> data = this.roomToPlayBack(this.data.currentPlayer);
            this.playBack.put(Long.toString(System.currentTimeMillis()), data);

            // 是否有进还贡数据 或 抗贡
            if (this.data.isTribute && !this.data.tributeList.isEmpty() && !this.isTwoKing()) {
                this.state = 2;
                this.flag = false;
                this.time = 0;
            } else {
                this.state = 2;
                this.flag = true;
                this.time = 0;
            }
        } catch (Exception e) {
            Log.error("===游戏开局出错...", e);
        } finally {
            lock.unlock();
        }
    }

    /**
     * 开局检测是否进贡、还贡
     *
     * @return
     */
    public synchronized void checkTributeInfo() {
        // 头游
        Struct.SettlementData player1Data = this.data.result.getOrDefault(1, null);
        if (player1Data == null || StringUtils.isEmpty(player1Data.playerId)) {
            return;
        }
        Player p1 = this.actors.getOrDefault(player1Data.playerId, null);
        if (p1 == null) {
            return;
        }
        // 末游
        Player p4 = null;

        // 是否双贡
        boolean isDoubleTribute = false;
        // 头游搭档数据
        Struct.TeammateData teammateData = this.getTeammate(p1.data.teammateId);
        if (teammateData.getRank() == 2) {
            // 进贡, 双下
            isDoubleTribute = true;
        }
        // 进还贡
        Struct.SettlementData p4Data = this.data.result.getOrDefault(4, null);
        if (p4Data == null || StringUtils.isEmpty(p4Data.playerId)) {
            return;
        }
        p4 = this.actors.getOrDefault(p4Data.playerId, null);
        if (p4 != null) {
            this.data.tributeList.add(new String[]{p4.getId() , p1.getId()});
        }
        // 双贡
        if (isDoubleTribute) {
            Struct.SettlementData p3Data = this.data.result.getOrDefault(3, null);
            if (p3Data == null || StringUtils.isEmpty(p3Data.playerId)) {
                return;
            }
            Player p3 = this.actors.getOrDefault(p3Data.playerId, null);
            if (p3 != null) {
                this.data.tributeList.add(new String[]{p3.getId(), teammateData.player.getId()});
            }
        }

        // 双贡情况
        if (this.data.tributeList.size() >= 2) {
            // 1游[0:进贡者, 1:还贡者]
            String[] tribute1 = this.data.tributeList.get(0);
            // 2游[0:进贡者, 1:还贡者]
            String[] tribute2 = this.data.tributeList.get(1);

            // 1游
            Player player1 = this.actors.getOrDefault(tribute1[1], null);
            // 1游进贡者
            Player tributePlayer1 = this.actors.getOrDefault(tribute1[0], null);
            // 2游
            Player player2 = this.actors.getOrDefault(tribute2[1], null);
            // 2游进贡者
            Player tributePlayer2 = this.actors.getOrDefault(tribute2[0], null);

            if (player1 == null || tributePlayer1 == null || player2 == null || tributePlayer2 == null) {
                return;
            }
            // 1游进贡者最大的手牌
            int player1MaxCard = GDUtils.getTributeMaxCard(tributePlayer1.data.remainCards, this.data.curLevelPoint);
            // 2游进贡者最大手牌
            int player2MaxCard = GDUtils.getTributeMaxCard(tributePlayer2.data.remainCards, this.data.curLevelPoint);

            // 1游取牌点大的进贡者
            if (player2MaxCard > player1MaxCard) {
                tribute1[0] = tributePlayer2.getId();
                tribute2[0] = tributePlayer1.getId();
                // 设置庄家
                this.data.zhuangPos = tributePlayer2.data.pos;
            } else if (player2MaxCard == player1MaxCard) {
                // 牌点相同按顺时针进贡
                // 设置1游进贡者为1游下家
                Player nextPlayer = this.getNextPlayer(player1);
                if (nextPlayer != null) {
                    tribute1[0] = nextPlayer.getId();
                }
                //设置2游进贡者为2游下家
                nextPlayer = this.getNextPlayer(player2);
                if (nextPlayer != null) {
                    tribute2[0] = nextPlayer.getId();
                }
                // 设置庄家
                this.data.zhuangPos = tributePlayer2.data.pos;
            } else {
                // 设置庄家
                this.data.zhuangPos = tributePlayer1.data.pos;
            }
        }

        // 单贡情况
        if (this.data.tributeList.size() == 1) {
            // 1游[0:进贡者, 1:还贡者]
            String[] tribute1 = this.data.tributeList.get(0);
            // 1游
            Player player1 = this.actors.getOrDefault(tribute1[1], null);
            // 1游进贡者
            Player tributePlayer1 = this.actors.getOrDefault(tribute1[0], null);
            if (player1 == null || tributePlayer1 == null) {
                return;
            }
            // 设置庄家
            this.data.zhuangPos = tributePlayer1.data.pos;
        }

        // 抗贡
        for (String[] tributeId : this.data.tributeList) {
            // 进贡者
            Player tributePlayer = this.actors.getOrDefault(tributeId[0], null);
            if (tributePlayer != null) {
                // 统计手牌数量
                Map<Integer, Integer> cardMap = CardUtils.getCardMap(tributePlayer.data.remainCards);
                if (!cardMap.isEmpty()) {
                    int num = cardMap.getOrDefault(GDUtils.BIGKING, 0);
                    if (num > 0 && !this.data.towKingList.contains(tributeId[0])) {
                        // 记录信息
                        this.data.towKingList.add(tributeId[0]);
                    }
                }
            }
        }

        if (this.data.towKingList.size() < 2) {
            this.data.towKingList.clear();
        }
    }

    /**
     * 检测双王抗贡
     *
     * @return
     */
    public boolean isTwoKing() {
        return this.data.towKingList.size() >= 2;
    }

    /**
     * 检测进贡是否完成
     *
     * @return
     */
    public synchronized boolean checkTributeStart() {
//        Log.debug("检测进贡是否完成...");
        int cnt = 0;
        for (String[] playerIds : this.data.tributeList) {
            if (!StringUtils.isEmpty(playerIds[0]) && !StringUtils.isEmpty(playerIds[1])) {
                // 进贡者
                Player tributePlayer = this.actors.getOrDefault(playerIds[0], null);
                if (tributePlayer != null && tributePlayer.data.tributeCard > 0) {
                    cnt += 1;
                }
            }
        }
        return cnt == this.data.tributeList.size();
    }

    /**
     * 检测还贡是否完成
     *
     * @return
     */
    @Override
    public synchronized boolean checkBackTributeStart() {
//        Log.debug("检测还贡是否完成...");
        int cnt = 0;
        for (String[] playerIds : this.data.tributeList) {
            if (!StringUtils.isEmpty(playerIds[0]) && !StringUtils.isEmpty(playerIds[1])) {
                // 还贡者
                Player backPlayer = this.actors.getOrDefault(playerIds[1], null);
                if (backPlayer != null && backPlayer.data.tributeCard > 0) {
                    cnt += 1;
                }
            }
        }
        return cnt == this.data.tributeList.size();
    }

    /**
     * 切换还贡状态
     */
    public synchronized void doChangeBackTribute() {
        this.data.tributeState = 2;
        // 进还贡数据
        Map<String, Object> tributeInfo = new HashMap<>();
        tributeInfo.put("tributeState", this.data.tributeState);
        tributeInfo.put("tributeList", this.data.tributeList);
        tributeInfo.put("twoKingInfo", this.data.towKingList);
        tributeInfo.put("tributeCardMap", this.data.tributeCardMap);

        // 广播
        for (Player tmPlayer : this.actors.values()) {
            if (tmPlayer != null) {
                Map<String, Object> data = this.roomToMessage(tmPlayer, this.data.currentPlayer);
                data.put("zhuangPos", !this.data.tributeList.isEmpty() && !this.isTwoKing()? -1 : this.data.zhuangPos);
                data.put("tributeInfo", tributeInfo);
                data.put("state", 2);
                data.put("time", !this.data.tributeList.isEmpty()? 40 : 20);
                Log.debug("玩家 : {}, 手牌 : {}", tmPlayer.data.name, tmPlayer.data.remainCards);
                tmPlayer.receive(CommonProto.Cmd.GameStateChange_VALUE, data);
            }
        }
    }

    /**
     * 强制进贡、还贡
     */
    public synchronized void doTribute() {
        Log.debug("强制进贡、还贡...");
        // 强制选牌
        for (String[] playerIds : this.data.tributeList) {
            if (!StringUtils.isEmpty(playerIds[0]) && !StringUtils.isEmpty(playerIds[1])) {
                // 进贡者
                Player tributePlayer = this.actors.getOrDefault(playerIds[0], null);
                // 还贡者
                Player backPlayer = this.actors.getOrDefault(playerIds[1], null);
                if (tributePlayer == null || backPlayer == null) {
                    continue;
                }
                // 进贡者选牌
                int maxCard = GDUtils.getTributeMaxCard(tributePlayer.data.remainCards, this.data.curLevelPoint);
                this.data.tributeCardMap.put(tributePlayer.getId(), maxCard);
                // 还贡者选牌
                int tributeCard = 0;
                for (Integer card : backPlayer.data.remainCards) {
                    byte low = (byte) (card & 0xf);
                    // 过滤级牌
                    if (low == this.data.curLevelPoint) {
                        continue;
                    }
                    if (low <= 10) {
                        tributeCard = card;
                        break;
                    }
                }
                this.data.tributeCardMap.put(backPlayer.getId(), tributeCard);
                Log.debug("强制进还贡...进贡者 : {}, 选牌 : {}, 还贡者 : {}, 选牌 : {}",
                        tributePlayer.data.name, maxCard, backPlayer.data.name, tributeCard);
            }
        }
        // 换牌
        this.doChangeTributeCard();
    }

    /**
     * 进还贡换牌
     */
    public synchronized void doChangeTributeCard() {
        if (this.data.tributeList.isEmpty()) {
            return;
        }
        Log.debug("进还贡换牌...");
        int tributeCard = 0;
        int backCard = 0;
        for (String[] playerIds : this.data.tributeList) {
            if (!StringUtils.isEmpty(playerIds[0]) && !StringUtils.isEmpty(playerIds[1])) {
                // 进贡者
                Player tributePlayer = this.actors.getOrDefault(playerIds[0], null);
                // 还贡者
                Player backPlayer = this.actors.getOrDefault(playerIds[1], null);
                if (tributePlayer == null || backPlayer == null) {
                    continue;
                }
                // 移除进贡者手牌
                tributeCard = this.data.tributeCardMap.getOrDefault(tributePlayer.getId(), 0);
                if (tributeCard > 0) {
                    // 还贡者添加牌
                    backPlayer.data.remainCards = CardUtils.add(backPlayer.data.remainCards, tributeCard);
                    backPlayer.data.tributeGetCard = tributeCard;
                }

                // 移除还贡者手牌
                backCard = this.data.tributeCardMap.getOrDefault(backPlayer.getId(), 0);
                if (backCard > 0) {
                    // 进贡者添加牌
                    tributePlayer.data.remainCards = CardUtils.add(tributePlayer.data.remainCards, backCard);
                    tributePlayer.data.tributeGetCard = backCard;
                }
                // 更新数据
                this.actors.put(tributePlayer.getId(), tributePlayer);
                this.actors.put(backPlayer.getId(), backPlayer);
            }
        }
        // 清理数据
        this.data.tributeState = 0;
        this.data.result.clear();
        this.data.tributeList.clear();
        // 进还贡数据
        Map<String, Object> tributeInfo = new HashMap<>();
        tributeInfo.put("tributeList", this.data.tributeList);
        tributeInfo.put("twoKingInfo", this.data.towKingList);
        tributeInfo.put("tributeState", this.data.tributeState);
        // 进还贡的选牌map
        tributeInfo.put("tributeCardMap", this.data.tributeCardMap);
        // 广播
        for (Player tmPlayer : this.actors.values()) {
            if (tmPlayer != null) {
                Map<String, Object> data = this.roomToMessage(tmPlayer, this.data.currentPlayer);
                data.put("tributeInfo", tributeInfo);
                data.put("state", 2);
                data.put("time", 20);
                Log.debug("进还贡换牌...玩家 : {}, 手牌 : {}", tmPlayer.data.name, tmPlayer.data.remainCards);
                tmPlayer.receive(CommonProto.Cmd.GameStateChange_VALUE, data);
            }
        }
    }

    /**
     * 根据当前玩家位置, 获取下一个玩家位置
     *
     * @param pos 当前操作玩家的位置
     * @return
     */
    public Integer[] getNextPos(int pos) {
        Integer[] nextpos = null;
        switch (pos) {
            case 3:
                nextpos = new Integer[] { 0, 1, 2 };
                break;
            case 2:
                nextpos = new Integer[] { 3, 0, 1 };
                break;
            case 1:
                nextpos = new Integer[] { 2, 3, 0 };
                break;
            case 0:
                nextpos = new Integer[] { 1, 2, 3 };
                break;
        }
        return nextpos;
    }

    /**
     * 根据当前玩家位置, 获取下一个可操作玩家
     *
     * @param player 当前操作玩家
     * @return
     */
    public Player getNextPlayer(Player player) {
        Player nextPlayer = null;
        Integer[] nextPos = this.getNextPos(player.data.pos);
        for (Integer index : nextPos) {
            Player temPlayer = this.data.playerMap[index];
            if (temPlayer.data.state == 2 && !temPlayer.getId().equals(player.getId())) {
                nextPlayer = temPlayer;
                break;
            }
        }
        return nextPlayer;
    }

    /**
     * 玩家换座位
     *
     * @param player
     * @param targetSeat
     */
    @Override
    public synchronized void switchSeat(Player player, int targetSeat) {
        lock.lock();
        try {
            if (targetSeat < 0 || targetSeat >= this.data.maxNum) {
                return; // 目标座位无效
            }
            // 找到玩家当前座位
            int currentSeat = -1;
            for (int i = 0; i < this.data.maxNum; i++) {
                Player tmPlayer = this.data.playerMap[i];
                if (tmPlayer != null && tmPlayer.getId().equals(player.getId())) {
                    currentSeat = i;
                    break;
                }
            }
            if (currentSeat == -1) {
                return; // 玩家不在座位上
            }

            // 检查目标座位
            Player targetPlayer = this.data.playerMap[targetSeat];
            if (targetPlayer == null) {
                player.data.pos = targetSeat;
                // 如果目标座位为空，直接换过去
                this.data.playerMap[targetSeat] = player;
                this.data.playerMap[currentSeat] = null;
                Log.debug("roomId: " + this.data.roomId + ", " + player.data.name + " 换到座位号: " + targetSeat);
            }
            // 如果目标座位已被占用，与目标玩家交换位置
            if (targetPlayer != null && targetPlayer.data.isChangePos) {
                player.data.pos = targetSeat;
                targetPlayer.data.pos = currentSeat;
                this.data.playerMap[targetSeat] = player;
                this.data.playerMap[currentSeat] = targetPlayer;
                Log.debug("roomId: " + this.data.roomId + ", " + player.data.name + " 与 " + targetPlayer.data.name + " 交换了座位");
            }
        } finally {
            lock.unlock();
        }
    }

    /**
     * 玩家出牌
     *
     * @param player    出牌玩家
     * @param disCardList   打出的牌
     * @param disCardType   打出的牌型
     */
    @Override
    public synchronized void playerDisCard(Player player, int[] disCardList, GDUtils.CardType disCardType) {
        // 是否自己回合
        if (!this.data.currentPlayer.getId().equals(player.getId())) {
            return;
        }
        // 出的牌是否都在手上
        if (!CardUtils.contains(player.data.remainCards, disCardList)) {
            return;
        }
        Log.info("出牌...玩家ID:{}, 位置:{}, 出牌牌型:{}, 出牌:{}, 手牌大小:{}, 手牌信息: {}",
                player.getId(), player.data.pos, disCardType, Arrays.toString(disCardList),
                player.data.remainCards.length, Arrays.toString(player.data.remainCards));
        // 从手牌中移除
        for (Integer card : disCardList) {
            player.data.remainCards = CardUtils.remove(player.data.remainCards, card);
        }
        // 设置出牌堆
        this.data.curDisCardList = disCardList;
        this.data.curDisCardType = disCardType;
        // 炸弹计算倍数
        if (GDUtils.isBombCalculateMultiples(disCardType)) {
            this.data.bombNum += 1;
        }
        // 设置当前出牌的玩家
        this.data.disCardPlayer = player;

        // 清空过玩家集合
        this.data.passList.removeIf(id -> player.getId().equals(id));

        // 设置接风玩家
        this.data.jieFengPlayer = null;
        // 设置下一个操作玩家
        this.data.currentPlayer = this.getNextPlayer(player);
        // 出完牌设置结算数据
        if (player.data.remainCards.length == 0) {
            player.data.state = 3;
            this.data.result.put(this.data.result.size() + 1, new Struct.SettlementData(player.getId(), 0));
        }
        // 设置其他可操作玩家集合
        this.data.otherOpPlayerList.clear();
        for (Player tmPlayer : this.actors.values()) {
            if (tmPlayer != null && !tmPlayer.getId().equals(player.getId())) {
                if (tmPlayer.data.state == 2) {
                    if (!this.data.otherOpPlayerList.contains(tmPlayer.getId())) {
                        this.data.otherOpPlayerList.add(tmPlayer.getId());
                    }
                }
            }
        }

        // 广播其他3人
        for (Player tmPlayer : this.actors.values()) {
            if (tmPlayer != null && !Objects.equals(tmPlayer.getId(), player.getId())) {
                Map<String, Object> data = this.roomToMessage(tmPlayer, this.data.currentPlayer);
                data.put("time", 20);
                tmPlayer.receive(CommonProto.Cmd.OtherPlayCards_VALUE, data);
            }
        }

        boolean isGameOver = false;

        // 4人房2人一队手牌出完,直接结算
        if (this.data.result.size() == 2) {
            // 1游
            Struct.SettlementData player1Data = this.data.result.get(1);
            if (player1Data == null || StringUtils.isEmpty(player1Data.playerId)) {
                return;
            }
            Player player1 = this.actors.getOrDefault(player1Data.playerId, null);
            if (player1 == null) {
                return;
            }

            // 2游
            Struct.SettlementData player2Data = this.data.result.get(2);
            if (player2Data == null || StringUtils.isEmpty(player2Data.playerId)) {
                return;
            }
            Player player2 = this.actors.getOrDefault(player2Data.playerId, null);
            if (player2 == null) {
                return;
            }

            // 1游2游是否一队
            if (player1.data.teammateId.equals(player2.getId())) {
                // 已结算的用户id
                List<String> ids = new ArrayList<>();
                for (Struct.SettlementData settlementData : this.data.result.values()) {
                    if (settlementData != null && !StringUtils.isEmpty(settlementData.playerId)) {
                        ids.add(settlementData.playerId);
                    }
                }
                // 补充3游4游玩家
                for (Player tmPlayer : this.data.playerMap) {
                    if (tmPlayer != null) {
                        tmPlayer.data.state = 3;
                        // 补充最后一个玩家
                        if (!ids.contains(tmPlayer.getId())) {
                            this.data.result.put(this.data.result.size() + 1, new Struct.SettlementData(tmPlayer.getId(), 0));
                        }
                    }
                }
                isGameOver = true;
            }
        }

        // 4人房间3人手牌出完，直接结算
        if (this.data.result.size() >= this.data.maxNum - 1) {
            // 已结算的用户id
            List<String> ids = new ArrayList<>();
            for (Struct.SettlementData settlementData : this.data.result.values()) {
                if (settlementData != null && !StringUtils.isEmpty(settlementData.playerId)) {
                    ids.add(settlementData.playerId);
                }
            }
            for (Player tmPlayer : this.data.playerMap) {
                if (tmPlayer != null) {
                    tmPlayer.data.state = 3;
                    // 补充最后一个玩家
                    if (!ids.contains(tmPlayer.getId())) {
                        this.data.result.put(this.data.result.size() + 1, new Struct.SettlementData(tmPlayer.getId(), 0));
                    }
                }
            }

            isGameOver = true;
        }

        // 记录游戏回放
        Map<String, Object> data = this.roomToPlayBack(this.data.currentPlayer);
        this.playBack.put(Long.toString(System.currentTimeMillis()), data);

        if (isGameOver) {
            // 结算
            try {
                Thread.sleep(2000);
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
            this.settlement();
            // 结束
            this.gameOver();
        }
    }

    /**
     * 玩家过
     *
     * @param player    点击过玩家
     * @param isAuto    是否自动操作过
     */
    @Override
    public synchronized void playerPass(Player player, boolean isAuto) {
        // 当前可操作玩家是否自己
        if (!this.data.currentPlayer.getId().equals(player.getId())) {
            return;
        }
        Log.info("过...玩家ID:{}, 位置:{}, 手牌大小:{}, 手牌信息: {}", player.getId(), player.data.pos, player.data.remainCards.length, Arrays.toString(player.data.remainCards));

        boolean isClear = false;

        // 清掉自己出牌堆
        player.data.discardList = new int[0];

        // 添加过玩家列表
        if (!this.data.passList.contains(player.getId())) {
            this.data.passList.add(player.getId());
        }
        // 设置其他可操作玩家集合
        this.data.otherOpPlayerList.removeIf(id -> player.getId().equals(id));

        // 4人房3人不出，清空出牌堆
        if (this.data.otherOpPlayerList.isEmpty()) {
            // 清空出牌堆
            this.data.curDisCardList = new int[0];
            this.data.curDisCardType = null;

            for (Player p : this.actors.values()) {
                if (p != null && p.data.discardList.length > 0) {
                    p.data.discardList = new int[0];
                }
            }
            // 所有玩家都点过
            if (!this.data.passList.isEmpty()) {
                // 清空过玩家集合
                this.data.passList.clear();
                // 设置接风标识
                if (this.data.maxNum == 4 && this.data.disCardPlayer.data.remainCards.length == 0) {
                    this.data.jieFengPlayer = this.actors.getOrDefault(this.data.disCardPlayer.data.teammateId, null);
                }
            }
            isClear = true;
        }

        // 判断接风
        if (isClear && this.data.jieFengPlayer != null) {
            this.data.currentPlayer = this.actors.getOrDefault(this.data.jieFengPlayer.getId(), null);
            this.data.disCardPlayer = null;
        } else {
            // 设置下一个操作玩家
            this.data.currentPlayer = this.getNextPlayer(player);
        }

        // 当前过玩家的下家清掉出牌堆
        Integer[] nextPos = this.getNextPos(player.data.pos);
        Player nextPlayer = this.data.playerMap[nextPos[0]];
        if (nextPlayer != null && nextPlayer.data.discardList.length > 0) {
            nextPlayer.data.discardList = new int[0];
        }

        // 广播其他3人
        for (Player tmPlayer : this.actors.values()) {
            if (tmPlayer != null && !Objects.equals(tmPlayer.getId(), player.getId())) {
                Map<String, Object> data = this.roomToMessage(tmPlayer, this.data.currentPlayer);
                // 接风玩家
                data.put("jieFengPlayer", isClear && this.data.jieFengPlayer != null? this.data.jieFengPlayer.getId() : "");
                data.put("time", 20);
                tmPlayer.receive(CommonProto.Cmd.OtherPlayCards_VALUE, data);
            }
        }

        // 记录游戏回放
        Map<String, Object> data = this.roomToPlayBack(this.data.currentPlayer);
        this.playBack.put(Long.toString(System.currentTimeMillis()), data);
    }

    /**
     * 获取队友
     *
     * @param playerId
     * @return
     */
    public Struct.TeammateData getTeammate(String playerId) {
        Struct.TeammateData data = new Struct.TeammateData();
        for (Map.Entry<Integer, Struct. SettlementData> entry : this.data.result.entrySet()) {
            if (entry.getValue() != null && !StringUtils.isEmpty(entry.getValue().playerId)) {
                Player tmPlayer = this.actors.getOrDefault(entry.getValue().playerId, null);
                if (tmPlayer != null && tmPlayer.getId().equals(playerId)) {
                    data.player = tmPlayer;
                    data.rank = entry.getKey();
                }
            }
        }
        return data;
    }

    /**
     * 牌局结算
     */
    private synchronized void settlement() {
        StringBuilder builder = new StringBuilder();

        // 头游
        Struct.SettlementData winPlayerData = this.data.result.getOrDefault(1, null);
        if (winPlayerData == null || StringUtils.isEmpty(winPlayerData.playerId)) {
            return;
        }
        Player winPlayer = this.actors.getOrDefault(winPlayerData.playerId, null);
        if (winPlayer == null) {
            return;
        }

        // 分
        int scores = 0;
        // 头游搭档数据
        Struct.TeammateData teammateData = this.getTeammate(winPlayer.data.teammateId);
        switch (teammateData.getRank()) {
            case 2:
                // 搭档2游, +3
                if (this.jsonData.getInteger("pv") == 0) {
                    scores = 3;
                } else {
                    scores = 4;
                }
                builder.append("1游玩家: ").append(winPlayer.data.name)
                        .append(", 搭档玩家: ").append(teammateData.player.data.name)
                        .append(" 2游, +").append(scores);
                break;
            case 3:
                // 搭档3游, +2
                scores = 2;
                builder.append("1游玩家: ").append(winPlayer.data.name)
                        .append(", 搭档玩家: ").append(teammateData.player.data.name)
                        .append(" 3游, +").append(scores);
                break;
            case 4:
                // 搭档4游, +1
                scores = 1;
                builder.append("1游玩家: ").append(winPlayer.data.name)
                        .append(", 搭档玩家: ").append(teammateData.player.data.name)
                        .append(" 4游, +").append(scores);
                break;
        }
        // 翻倍 0不翻倍 1加倍 2翻倍
        if (this.jsonData.getInteger("dv") == 1) {
            // 加倍
            for (int i = 0; i < this.data.bombNum; i++) {
                scores += scores;
            }
//                scores += Math.max(this.data.multiple, 0);
        }
        if (this.jsonData.getInteger("dv") == 2) {
            // 翻倍
            for (int i = 0; i < this.data.bombNum; i++) {
                scores *= 2;
            }
//                scores *= (this.data.multiple > 0? this.data.multiple : 1);
        }
        for (Struct.SettlementData settlementData : this.data.result.values()) {
            if (settlementData != null) {
                Player tmPlayer = this.actors.getOrDefault(settlementData.playerId, null);
                if (tmPlayer != null) {
                    if (settlementData.playerId.equals(winPlayer.getId())
                            || settlementData.playerId.equals(teammateData.player.getId())) {
                        settlementData.score = scores;
                        tmPlayer.data.accumulatedPoints += scores;
                    } else {
                        settlementData.score = -scores;
                        tmPlayer.data.accumulatedPoints -= scores;
                    }
                }
            }
        }

        // 广播结算
        for (Player tmPlayer : this.actors.values()) {
            if (tmPlayer != null) {
                // 结算信息
                Map<String, Object> data = this.roomToMessage(tmPlayer, null);
                data.put("state", 3);
                tmPlayer.receive(CommonProto.Cmd.GameStateChange_VALUE, data);
            }
        }
        RedisUtil.del("roomGambit:"+this.data.roomId);
        Log.info("牌局结算...roomId={}, {}", this.data.roomId, builder.toString());

        // 记录游戏回放
        Map<String, Object> data = this.roomToPlayBack(this.data.currentPlayer);
        data.put("state", 3);
        this.playBack.put(Long.toString(System.currentTimeMillis()), data);

        // 向后台发送结算信息
        GGame.getInstance().asyncExec(()->{
            this.data.result.forEach((key,value)->{
                if (value != null) {
                    Player tmPlayer = this.actors.getOrDefault(value.playerId, null);
                    if (tmPlayer != null) {
                        try {
                            cn.hutool.json.JSONObject object = new cn.hutool.json.JSONObject();
                            ZjUserInfolog zjUserInfolog = new ZjUserInfolog();
                            zjUserInfolog.setUserId(tmPlayer.data.playerId);//用户ID
                            zjUserInfolog.setCardTableId(String.valueOf(this.data.roomId));//房间id
                            zjUserInfolog.setCardLevel(String.valueOf(this.data.curLevelPoint));//牌级
                            zjUserInfolog.setMasterScore(new BigDecimal(tmPlayer.data.masterScore));//大师分
                            zjUserInfolog.setPoints(value.score);//分数
                            int type = this.data.type;//房间类型
                            zjUserInfolog.setCompeteType(String.valueOf(type));
                            zjUserInfolog.setTurnNum(1);//轮次
                            if (type==1){
                                Contest contest = ContestService.getInstance().contestMap.getOrDefault(tmPlayer.data.contestId, null);
                                if(contest!=null){
                                    zjUserInfolog.setCompeteId(String.valueOf(contest.data.contestId));//赛事id
                                    zjUserInfolog.setTurnNum(contest.data.curRound);//轮次
                                }
                            }
                            zjUserInfolog.setInningNum(this.data.curRound);
                            zjUserInfolog.setPosition(pos(tmPlayer.data.pos));
                            zjUserInfolog.setCreateTime(new Date());
                            zjUserInfolog.setCompeteResult(tmPlayer.data.playerId.equals(winPlayerData.playerId)?"1":"2");
                            zjUserInfolog.setGameInfo(this.playBack.toString());
                            zjUserInfolog.setTableStatus(String.valueOf(this.state));
                            zjUserInfolog.setRuleInfo(this.jsonData.toString());
                            zjUserInfolog.setGameTime(this.time);
                            object.put("type","gameInfo");
                            object.put("data",zjUserInfolog);
                            SendGameInfoUtil.sendGameInfo(object);
                        } catch (JSONException e) {
                            Log.error("向后台发送结算信息出错...roomId={}, gameType={}, {}",
                                    this.data.roomId, this.data.gameType, e);
                        }
                    }
                }
            });
        });

    }

    public static String pos(int pos) {
        switch (pos) {
            case 0:
                return "1";
            case 1:
                return "2";
            case 2:
                return "3";
            case 3:
                return "4";
        }
        return "";
    }

    /**
     * 游戏结束
     */
    private synchronized void gameOver() {
        this.state = 3;
        this.time = 0;
        this.flag = false;

//        Log.info("回放数据 : ", this.playBack.toString());
    }

    /**
     * 下一局重置房间数据
     */
    @Override
    public void resetNextRoom() {
        Log.debug("下一局重置玩家数据...");
        for (Player player : this.data.playerMap) {
            if (player != null) {
                player.data.state = 0;
                player.data.remainCards = new int[0];
                player.data.discardList = new int[0];
                player.data.disCardType = null;
                player.data.tributeCard = 0;
                player.data.tributeGetCard = 0;
                player.data.isChangePos = false;
            }
        }

        Log.debug("下一局重置桌子数据...");
        this.data.currentPlayer = null;
        this.data.otherOpPlayerList.clear();
        this.data.curDisCardList = new int[]{};
        this.data.curDisCardType = null;
        this.data.disCardPlayer = null;
        this.data.passList.clear();
        this.data.jieFengPlayer = null;
        this.data.bombNum = 0;
        this.data.tributeState = 0;
        this.data.tributeCardMap.clear();
        this.data.cardList.clear();
        // 根据配置清理参数
        if (!this.data.isTribute) {
//            if (this.data.type !=1){
//                this.data.result.clear();
//            }
            this.data.tributeList.clear();
            this.data.towKingList.clear();
        }

        // 初始化牌桌
        this.initCardList();
    }

    /**
     * 清理桌子信息
     */
    @Override
    public void resetRoom() {
        Log.debug("清理房间玩家数据...");
        if (!this.actors.isEmpty()) {
            for (Player player : this.actors.values()) {
                if (player != null) {
                    player.resetRoom();
                }
            }
            this.actors.clear();
        }

        Log.debug("清理房间数据...");
        this.jsonData.clear();
        this.flag = false;
        this.time = 0;
        this.state = 0;
        this.lock = new ReentrantLock();
        this.changePosMap.clear();
        this.playBack.clear();

        this.data.init();
    }

    /**
     * 房间信息转消息
     *
     * @param player 本人
     * @param opPlayer 当前操作玩家
     * @return
     */
    @Override
    public Map<String, Object> roomToMessage(Player player, Player opPlayer) {
        Map<String, Object> data = new HashMap<>();
        data.put("roomId", this.data.roomId);
        data.put("type", this.data.type);
        data.put("gameType", this.data.gameType);
        data.put("ownerId", this.data.ownerId);
        data.put("state", this.state);
        data.put("time", this.time);
        data.put("mode", this.data.mode);
        data.put("modev", this.data.modev);
        data.put("maxRound", this.data.maxRound);
        data.put("maxTime", this.data.maxTime);

        data.put("curRound", this.data.curRound);
        data.put("curTime", this.data.curTime);
        data.put("zhuangPos", this.data.zhuangPos);
        data.put("showCard", this.data.showCard);
        // 当前级牌点数
        data.put("curLevelPoint", this.data.curLevelPoint);
        // 是否有进还贡
        data.put("isTribute", this.data.isTribute);
        data.put("opId", opPlayer != null ? opPlayer.getId() : "");
        data.put("opPos", opPlayer != null ? opPlayer.data.pos : -1);
        data.put("curDiscardList", this.data.curDisCardList);
        data.put("curDisCardType", this.data.curDisCardType);
        data.put("curDiscardPlayer", this.data.disCardPlayer != null ? this.data.disCardPlayer.getId() : "");
        data.put("curDiscardSex", this.data.disCardPlayer != null ? this.data.disCardPlayer.data.sex : 0);
        data.put("curPassList", this.data.passList);
        data.put("roomIp",this.data.roomIp);
        data.put("myInfo", this.playerRoomToMessage(player, true));
        List<Map<String, Object>> otherInfo = new ArrayList<>();
        for (Player tmPlayer : this.actors.values()) {
            if (tmPlayer != null && !Objects.equals(tmPlayer.getId(), player.getId())) {
                otherInfo.add(this.playerRoomToMessage(tmPlayer, false));
            }
        }
        data.put("otherInfo", otherInfo);
        // 增加结算玩家信息
        if (!this.data.result.isEmpty()) {
            Map<Integer, Object> settlementInfo = new HashMap<>();
            for (Map.Entry<Integer, Struct.SettlementData> entry : this.data.result.entrySet()) {
                Map<String, Object> settlementData = new HashMap<>();
                settlementData.put("playerId", entry.getValue().playerId);
                settlementData.put("score", entry.getValue().score);
                settlementInfo.put(entry.getKey(), settlementData);
            }
            data.put("settlementInfo", settlementInfo);
        }
        // todo 测试代码
        data.put("房间规则", this.jsonData);
        return data;
    }

    /**
     * 玩家信息转消息
     *
     * @param flag 是否显示手牌
     * @return
     */
    public Map<String, Object> playerRoomToMessage(Player player, boolean flag) {
        Map<String, Object> data = new HashMap<>();
        data.put("id", player.getId());
        data.put("name", player.data.name);
        data.put("avatar", "https://img95.699pic.com/element/40109/0194.png_300.png");
        data.put("sex", player.data.sex);
        data.put("pos", player.data.pos);
        data.put("state", player.data.state);
        data.put("isAuto", player.data.isAuto);
        // 转蛋积分-累计
        data.put("accumulatedPoints", player.data.accumulatedPoints);
        // 队友Id
        data.put("teammateId", player.data.teammateId);

        // 玩家牌数据
        Map<String, Object> cardInfo = new HashMap<>();
        cardInfo.put("remainCards", flag? player.data.remainCards : "");
        cardInfo.put("remainCardsNum", player.data.remainCards.length);
        cardInfo.put("discardList", player.data.discardList);
        cardInfo.put("disCardType", player.data.disCardType);
        cardInfo.put("tributeGetCard", player.data.tributeGetCard);
        data.put("cardInfo", cardInfo);
        return data;
    }

    /**
     * 房间信息转游戏回放
     *
     * @param opPlayer 当前操作玩家
     * @return
     */
    public Map<String, Object> roomToPlayBack(Player opPlayer) {
        Map<String, Object> data = new HashMap<>();
        data.put("roomId", this.data.roomId);
        data.put("type", this.data.type);
        data.put("gameType", this.data.gameType);
        data.put("ownerId", this.data.ownerId);
        data.put("state", this.state);
        data.put("time", this.time);

        data.put("maxRound", this.data.maxRound);
        data.put("curRound", this.data.curRound);
        data.put("zhuangPos", this.data.zhuangPos);
        data.put("showCard", this.data.showCard);
        // 当前级牌点数
        data.put("curLevelPoint", this.data.curLevelPoint);
        // 是否有进还贡
        data.put("isTribute", this.data.isTribute);
        data.put("opId", opPlayer != null ? opPlayer.getId() : "");
        data.put("opPos", opPlayer != null ? opPlayer.data.pos : -1);
        data.put("curDiscardList", this.data.curDisCardList);
        data.put("curDisCardType", this.data.curDisCardType);
        data.put("curDiscardPlayer", this.data.disCardPlayer != null ? this.data.disCardPlayer.getId() : "");
        data.put("curDiscardSex", this.data.disCardPlayer != null ? this.data.disCardPlayer.data.sex : 0);
        data.put("curPassList", this.data.passList.toArray());
        // 玩家信息
        if (!this.actors.isEmpty()) {
            List<Map<String, Object>> playerInfo = new ArrayList<>();
            for (Player tmPlayer : this.actors.values()) {
                if (tmPlayer != null) {
                    playerInfo.add(playerRoomToPlayBack(tmPlayer));
                }
            }
            data.put("playerInfo", playerInfo);
        }
        // 增加结算玩家信息
        if (!this.data.result.isEmpty()) {
            Map<String, Object> settlementInfo = new HashMap<>();
            for (Map.Entry<Integer, Struct.SettlementData> entry : this.data.result.entrySet()) {
                Map<String, Object> settlementData = new HashMap<>();
                settlementData.put("playerId", entry.getValue().playerId);
                settlementInfo.put(entry.getKey().toString(), settlementData);
            }
            data.put("settlementInfo", settlementInfo);
        }
        return data;
    }

    /**
     * 玩家信息转消息
     *
     * @param player 玩家
     * @return
     */
    public Map<String, Object> playerRoomToPlayBack(Player player) {
        Map<String, Object> data = new HashMap<>();
        data.put("id", player.getId());
        data.put("name", player.data.name);
        data.put("avatar", "https://img95.699pic.com/element/40109/0194.png_300.png");
        data.put("sex", player.data.sex);
        data.put("pos", player.data.pos);
        data.put("state", player.data.state);
        data.put("isAuto", player.data.isAuto);
        // 级数显示 [我方级数, 对方级数]
        data.put("levelPointView", player.data.levelPointView);
        // 队友Id
        data.put("teammateId", player.data.teammateId);

        // 玩家牌数据
        Map<String, Object> cardInfo = new HashMap<>();
        cardInfo.put("remainCards", player.data.remainCards);
        cardInfo.put("remainCardsNum", player.data.remainCards.length);
        cardInfo.put("discardList", player.data.discardList);
        cardInfo.put("disCardType", player.data.disCardType);
        cardInfo.put("tributeGetCard", player.data.tributeGetCard);
        data.put("cardInfo", cardInfo);
        return data;
    }
}
