package com.incubator.game.util;

import java.util.*;

/**
 * 棋牌相关工具类
 * 
 * @author Administrator
 *
 */
public final class CardUtils {

	/**
	 * 删除数组中的一个指定元素
	 *
	 * @param target 原始数组
	 * @param v 要删除的元素
	 * @return 新数组（删除元素后的数组）
	 */
	public static int[] remove(int[] target, int v) {
		int len = target.length;
		if (len <= 0) {
			return target;
		}
		int[] b = new int[len];
		System.arraycopy(target, 0, b, 0, len);
		Arrays.sort(b);
		int c = Arrays.binarySearch(b, v);
		if (c < 0) {
			return target;
		}

		int[] dest = new int[len - 1];
		for (int i = 0; i < len; i++) {
			if (target[i] == v) {
				if (i > 0) {
					System.arraycopy(target, 0, dest, 0, i);
				}
				if (i != len - 1) {
					System.arraycopy(target, i + 1, dest, i, len - i - 1);
				}
				break;
			}
		}
		return dest;
	}


	/***
	 * 删除数组中的元素
	 *
	 * @param targer 数据源
	 * @param v 删除的元素
	 * @return 新的数据源
	 */
	public static Long[] remove(Long[] targer, Long v) {
		int len = targer.length;
		if (len <= 0) {
			return targer;
		}
		Long[] b = new Long[len];
		System.arraycopy(targer, 0, b, 0, len);
		Arrays.sort(b);
		int c = Arrays.binarySearch(b, v);
		if (c < 0) {
			return targer;
		}

		Long[] dest = new Long[len - 1];
		for (int i = 0; i < len; i++) {
			if (targer[i].equals(v)) {
				if (i > 0) {
					System.arraycopy(targer, 0, dest, 0, i);
				}
				if (i != len - 1) {
					System.arraycopy(targer, i + 1, dest, i, len - i - 1);
				}
				break;
			}
		}
		return dest;
	}

	/**
	 * 向数组添加元素
	 * 
	 * @param src 数据源
	 * @param v　增加的元素
	 * @return 新的数据源
	 */
	public static int[] add(int[] src, int v) {
		int[] copy = Arrays.copyOf(src, src.length + 1); // 扩展数组
		copy[src.length] = v; // 在最后位置添加 v
		return copy;
	}

	/**
	 * 向数组前面添加元素
	 * 
	 * @param src 数据源
	 * @param v 增加的元素
	 * @return 新的数据源
	 */
	public static Integer[] forwardAdd(Integer[] src, Integer v) {
		List<Integer> list = new ArrayList<Integer>();
        Collections.addAll(list, src);
		Integer lastCard = list.get(list.size() - 1);
		list.remove(lastCard);
		list.add(v);
		list.add(lastCard);
		Integer[] copy = new Integer[list.size()];
		for (int i = 0; i < list.size(); i++) {
			copy[i] = list.get(i);
		}
		return copy;
	}

	/**
	 * 向数组添加元素
	 * 
	 * @param src 数据源
     * @param v 增加的元素
     * @return 新的数据源
	 */
	public static Long[] add(Long[] src, Long v) {
		if (src == null) {
			src = new Long[] {};
		}
		int srcLen = src.length;
		Long[] dest = new Long[srcLen + 1];
		System.arraycopy(src, 0, dest, 0, srcLen);
		dest[srcLen] = v;
		return dest;
	}

	/**
	 * 向数组添加数组
	 * 
	 * @param src 数据源
	 * @param a 增加的数组
	 * @return 新的数据源
	 */
	public static Integer[] add(Integer[] src, Integer[] a) {
		if (src == null) {
			src = new Integer[] {};
		}
		if (a == null) {
			a = new Integer[] {};
		}
		Integer[] dest = new Integer[src.length + a.length];
		System.arraycopy(src, 0, dest, 0, src.length);
		System.arraycopy(a, 0, dest, src.length, a.length);
		return dest;
	}

	/**
	 * 向list添加数组
	 * 
	 * @param src 数据源list
	 * @param tar 增加的数组
	 * @return 新的数据源list
	 */
	public static List<Integer> add(List<Integer> src, int[] tar) {
		for (int t : tar)
			src.add(t);
		return src;
	}

	/**
	 * 获取数组的最大值
	 * 
	 * @param src
	 * @return
	 */
	public static Integer getMaxNum(Integer[] src) {
		Integer max = 0;
        for (Integer aSrc : src) {
            if (max < aSrc) {
                max = aSrc;
            }
        }
		return max;
	}

	/**
	 * 获取数组中某个元素的数量
	 * 
	 * @param target
	 * @param card
	 * @return
	 */
	public static int countCardNum(Integer[] target, Integer card) {
		int i = 0;
		for (Integer tar : target) {
			if (tar.equals(card))
				i++;
		}
		return i;
	}

	/**
	 * 判断某一元素是否在数组里
	 * 
	 * @param arr
	 * @param v
	 * @return
	 */
	public static boolean isContains(Integer[] arr, Integer v) {
		if (arr == null) {
			return false;
		} else {
			for (Integer i : arr) {
				if (i.equals(v)) {
					return true;
				}
			}
		}
		return false;
	}

	/**
	 * 获取去重后的数组
	 * 
	 * @param a
	 * @return
	 */
	public static int[] dupRemove(int[] a) {
		Set<Integer> set = new HashSet<Integer>();
		for (int anA : a) {
			set.add(anA);
		}
		List<Integer> list = new ArrayList<Integer>(set);
		int[] arrayResult = new int[list.size()];
		for (int i = 0; i < list.size(); i++) {
			arrayResult[i] = list.get(i);
		}
		return arrayResult;
	}

	/**
	 * 判断数组是否包含元素
	 * 
	 * @param src
	 * @param v
	 * @return
	 */
	public static boolean contains(int[] src, int v) {
		if (src != null) {
			for (int i : src) {
				if (i == v) {
					return true;
				}
			}
		}
		return false;
	}

	/**
	 * 判断数组是否包含元素
	 *
	 * @param src
	 * @param v
	 * @return
	 */
	public static boolean contains(Integer[] src, Integer v) {
		if (src != null) {
			for (Integer i : src) {
				if (i.equals(v)) {
					return true;
				}
			}
		}
		return false;
	}

	/**
	 * 判断数组是否包含元素
	 *
	 * @param src
	 * @param v
	 * @return
	 */
	public static boolean contains(int[] src, int[] v) {
		if (src != null) {
			int cnt = 0;
			for (int _v : v) {
				for (int i : src) {
					if (i == _v) {
						cnt += 1;
					}
				}
			}
			return cnt >= v.length;
		}
		return false;
	}

	/**
	 * 判断数组是否包含元素
	 * 
	 * @param src
	 * @param v
	 * @return
	 */
	public static boolean contains(List<Long> src, Long v) {
		if (src != null) {
			for (Long i : src) {
				if (i.equals(v)) {
					return true;
				}
			}
		}
		return false;
	}

	/**
	 * 判断数组是否包含元素
	 * 
	 * @param src
	 * @param v
	 * @return
	 */
	public static boolean contains(Long[] src, Long v) {
		if (src != null) {
			for (Long i : src) {
				if (i.equals(v)) {
					return true;
				}
			}
		}
		return false;
	}

	/**
	 * list转byte[]
	 */
	public static int[] asArray(List<Integer> cards) {
		if (cards != null) {
			int[] arr = new int[cards.size()];
			for (int i = 0; i < cards.size(); i++) {
				arr[i] = cards.get(i);
			}
			return arr;
		}
		return null;
	}

	public static String decimalToHex(int decimal) {
		String hex = "";
		while (decimal != 0) {
			int hexValue = decimal % 16;
			hex = toHexChar(hexValue) + hex;
			decimal = decimal / 16;
		}
		return "0x" + hex;
	}

	/**
	 * 将0~15的十进制数转换成0~F的十六进制数
	 *
	 * @param hexValue
	 * @return
	 */
	public static char toHexChar(int hexValue) {
		if (hexValue <= 9 && hexValue >= 0)
			return (char) (hexValue + '0');
		else
			return (char) (hexValue - 10 + 'A');
	}

	/**
	 * 统计手牌数量
	 * 
	 * @param cards
	 * @return
	 */
	public static Map<Integer, Integer> getCardMap(int[] cards) {
		// 排除庄家开局摸的牌
//		if (cards.length == 14) {
//			cards[13] = 0;
//		}
		// 统计手牌中相同牌值的牌
		Map<Integer, Integer> map = new HashMap<>();
		for (Integer card : cards) {
			if (card != 0) {
				if (map.containsKey(card)) {
					map.put(card, map.get(card) + 1);
				} else {
					map.put(card, 1);
				}
			}
		}
		return map;
	}

	/**
	 * 从大到小排序
	 * 
	 * @param cards
	 * @return
	 */
	public static List<Integer> sortCards(int[] cards) {
		List<Integer> copy = new ArrayList<>();
		for (Integer b : cards) {
			copy.add(b);
		}
		// 排序后的list
		List<Integer> sort = new ArrayList<>();
		int copySize = copy.size();
		for (int j = copySize; j > 0; j--) {
			int maxCard = getMaxCard(cards);
			sort.add(maxCard);
			if (sort.contains(maxCard)) {
                copy.removeIf(card -> card == maxCard);
			}
		}
		return sort;
	}

	/**
	 * 获取最大的牌
	 * 
	 * @param cards
	 * @return
	 */
	public static int getMaxCard(int[] cards) {
		int maxCards = 0;
		for (Integer card : cards) {
			byte low = (byte) (card & 0xf);
			byte high = (byte) (card >> 4);
			if (low > (maxCards & 0xf)) {
				maxCards = card;
			} else if (low == (maxCards & 0xf)) {
				if (high > (maxCards >> 4)) {
					maxCards = card;
				}
			}
		}
		return maxCards;
	}

	/**
	 * 打印手牌
	 *
	 * @param args	玩家手牌
	 * @param curLevelPoint	当前级牌点数
	 * @return
	 */
	public static String printCards(int[] args, int curLevelPoint) {
		List<String> desc = new ArrayList<>();
		for (Integer card : args) {
			if (card == 239) {
				desc.add("小王");
			} else if (card == 255) {
				desc.add("大王");
			} else {
				byte high = (byte) (card >> 4);
				byte low = (byte) (card & 0xf);
				String cardLow = "";
				switch (low){
					case 11:
						cardLow = "J";
						break;
					case 12:
						cardLow = "Q";
						break;
					case 13:
						cardLow = "K";
						break;
					case 14:
						cardLow = "A";
						break;
					case 15:
						switch (curLevelPoint) {
							case 2:
								cardLow = "2";
								break;
							case 3:
								cardLow = "3";
								break;
							case 4:
								cardLow = "4";
								break;
							case 5:
								cardLow = "5";
								break;
							case 6:
								cardLow = "6";
								break;
							case 7:
								cardLow = "7";
								break;
							case 8:
								cardLow = "8";
								break;
							case 9:
								cardLow = "9";
								break;
							case 10:
								cardLow = "10";
								break;
							case 11:
								cardLow = "J";
								break;
							case 12:
								cardLow = "Q";
								break;
							case 13:
								cardLow = "K";
								break;
							case 14:
								cardLow = "A";
								break;
						}
						break;
				}

				switch (high) {
					case 0:
						desc.add("方片" + (cardLow.isEmpty() ? low : cardLow));
						break;
					case 1:
						desc.add("梅花" + (cardLow.isEmpty() ? low : cardLow));
						break;
					case 2:
						desc.add("红桃" + (cardLow.isEmpty() ? low : cardLow));
						break;
					case 3:
						desc.add("黑桃" + (cardLow.isEmpty() ? low : cardLow));
						break;
				}
			}
		}
		return Arrays.toString(desc.toArray());
	}
}
