package com.incubator.game.util;

import java.util.*;
import java.util.stream.Collectors;

/**
 * 掼蛋 工具类
 */
public final class GDUtils {

    /**
     * 掼蛋对应的数值
     * 0x02-0x0e 方片（♦️） 2(2) 3(3) 4(4) 5(5) 6(6) 7(7) 8(8) 9(9) 10(10) J(11) Q(12) K(13) A(14)
     * 0x12-0x1e 梅花（♣️） 2(18) 3(19) 4(20) 5(21) 6(22) 7(23) 8(24) 9(25) 10(26) J(27) Q(28) K(29) A(30)
     * 0x22-0x2e 红桃（♥️） 2(34) 3(35) 4(36) 5(37) 6(38) 7(39) 8(40) 9(41) 10(42) J(43) Q(44) K(45) A(46)
     * 0x32-0x3e 黑桃（♠️） 2(50) 3(51) 4(52) 5(53) 6(54) 7(55) 8(56) 9(57) 10(58) J(59) Q(60) K(61) A(62)
     * 0xef      小王(239)
     * 0xff      大王(255)
     */
    public static Integer[] values = {
            0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08, 0x09, 0x0a, 0x0b, 0x0c, 0x0d, 0x0e,
            0x12, 0x13, 0x14, 0x15, 0x16, 0x17, 0x18, 0x19, 0x1a, 0x1b, 0x1c, 0x1d, 0x1e,
            0x22, 0x23, 0x24, 0x25, 0x26, 0x27, 0x28, 0x29, 0x2a, 0x2b, 0x2c, 0x2d, 0x2e,
            0x32, 0x33, 0x34, 0x35, 0x36, 0x37, 0x38, 0x39, 0x3a, 0x3b, 0x3c, 0x3d, 0x3e,
            0xef,
            0xff
    };

    /** 大王数值 **/
    public static int BIGKING = 0xff;

    /** 随机队友-系统翻牌 2、3、4、5、6、7、8、9、10、J、Q、K、A **/
    public static Integer[] randomTeam = {
            0x22, 0x23, 0x24, 0x25, 0x26, 0x27, 0x28, 0x29, 0x2a, 0x2b, 0x2c, 0x2d, 0x2e
    };

    /** 级数配置 2、3、4、5、6、7、8、9、10、J、Q、K、A **/
    public static int[] levelPoint = {
            2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14
    };

    // 牌型枚举
    public enum CardType {
        SINGLE,              // 单张
        PAIR,                // 对子
        PAIR_SEQUENCE,       // 3连对(3个连续对子)
        TRIPLE,              // 三不带
        TRIPLE_WITH_TWO,     // 三带二
        TRIPLE_SEQUENCE,     // 三顺(二个连续三张)
        STRAIGHT,            // 顺子(5张牌)
        BOMB,                // 通用炸弹
        FOUR_BOMB,           // 四炸弹
        FIVE_BOMB,           // 五炸弹
        SIX_BOMB,            // 六炸弹
        SEVEN_BOMB,          // 七炸弹
        EIGHT_BOMB,          // 八炸弹
        FOUR_KINGS,          // 四王炸
        ROYAL_FLUSH,         // 同花顺（可选）
        INVALID              // 无效牌型
    }

    /**
     * 是否是炸弹
     *
     * @param type
     * @return
     */
    public static boolean isBomb(CardType type) {
        return type == CardType.BOMB
                || type == CardType.FOUR_BOMB
                || type == CardType.FIVE_BOMB
                || type == CardType.SIX_BOMB
                || type == CardType.SEVEN_BOMB
                || type == CardType.EIGHT_BOMB
                || type == CardType.FOUR_KINGS
                || type == CardType.ROYAL_FLUSH;
    }

    /**
     * 炸弹是否计算倍数
     *
     * @param type
     * @return
     */
    public static boolean isBombCalculateMultiples(CardType type) {
        return type == CardType.SIX_BOMB
                || type == CardType.SEVEN_BOMB
                || type == CardType.EIGHT_BOMB
                || type == CardType.FOUR_KINGS;
    }

    /**
     * 进还贡时，获取手牌最大的牌（过滤红星级牌）
     *
     * @param cards 玩家手牌
     * @param curLevelPoint 当前级牌点数
     * @return
     */
    public static int getTributeMaxCard(int[] cards, int curLevelPoint) {
        int maxCards = 0;
        for (Integer card : cards) {
            byte low = (byte) getCardPoint(card, curLevelPoint);
            byte high = (byte) (card >> 4);
            // 过滤红星级牌
            if (GDUtils.isWildCard(card, curLevelPoint)) {
                continue;
            }
            if (low > (byte) (maxCards & 0xf)) {
                maxCards = card;
            } else if (low == (byte) (maxCards & 0xf)) {
                if (high > (byte) (maxCards >> 4)) {
                    maxCards = card;
                }
            }
        }
        return maxCards;
    }

    /**
     * 随机队友-亮牌
     *
     * @return
     */
    public static int randomTeamShowCards() {
        List<Integer> randomTeam = new ArrayList<>(Arrays.asList(GDUtils.randomTeam));
        Collections.shuffle(randomTeam);
        return randomTeam.get(0);
    }

    /**
     * 是否为赖子 红星级牌
     *
     * @param card
     * @param curLevelPoint
     * @return
     */
    public static boolean isWildCard(int card, int curLevelPoint) {
        byte low = (byte) (card & 0xF);
        byte high = (byte) (card >> 4);
        // 红心级牌
        return high == (byte) 2 && low == (byte) curLevelPoint;
    }

    /**
     * 判断玩家出的牌是否可以压过上一手牌
     *
     * @param previousCards 上一手牌
     * @param previousCardsType 上一手牌的牌型
     * @param disCardsList 当前玩家出的牌型
     * @param disCardsType 当前玩家出的牌型
     * @param currentLevel 当前级数
     * @param wildCard 赖子牌
     * @return 是否可以出牌
     */
    public static boolean canDisCard(int[] previousCards, CardType previousCardsType,
                                     int[] disCardsList, GDUtils.CardType disCardsType,
                                     int currentLevel,
                                     int wildCard) {
        // 0.桌上无牌直接出
        if (previousCards == null || previousCardsType == null) {
            return true;
        }

        // 1.牌型不一样,判断手牌是否是炸弹
        if (disCardsType != previousCardsType && !GDUtils.isBomb(disCardsType)) {
            return false;
        }

        // 获取权重值
        int previousWeight = GDUtils.getCardTypeWeight(previousCardsType);
        int currentWeight = GDUtils.getCardTypeWeight(disCardsType);

        // 权重不同，按权重大小比较
        if (currentWeight > previousWeight) {
            return true;
        }

        // 权重相同，根据牌点大小进行比较（仅适用于同类型的牌型，如炸弹或顺子）
        if (currentWeight == previousWeight) {
            return GDUtils.compareCardStrength(previousCards, disCardsList, currentLevel, disCardsType, wildCard) > 0;
        }

        return false; // 其他情况不能出牌
    }

    /**
     * 比较两组牌的大小（仅在牌型相同时调用）
     *
     * @param previousCards 上一手牌
     * @param disCardList 当前玩家出的牌
     * @param curLevel 当前级数
     * @param disCardsType 出牌牌型
     * @param wildCard 赖子牌
     * @return 正数表示当前牌较大，负数表示上一手牌较大，0表示相等
     */
    public static int compareCardStrength(int[] previousCards, int[] disCardList, int curLevel, CardType disCardsType, int wildCard) {
        List<Integer> previousPoints =null;
        List<Integer> currentPoints = null;
        if (disCardsType.equals(CardType.SINGLE)||disCardsType.equals(CardType.PAIR)){
            previousPoints = Arrays.stream(previousCards)
                    .map(card -> GDUtils.getCardPointWildCard(card, wildCard, curLevel,1))
                    .sorted()
                    .boxed()
                    .collect(Collectors.toList());
            currentPoints = Arrays.stream(disCardList)
                    .map(card -> GDUtils.getCardPointWildCard(card, wildCard, curLevel,1))
                    .sorted()
                    .boxed()
                    .collect(Collectors.toList());
        }else{
            previousPoints = Arrays.stream(previousCards)
                    .map(card -> GDUtils.getCardPointWildCard(card, wildCard, curLevel,2))
                    .sorted()
                    .boxed()
                    .collect(Collectors.toList());
            currentPoints = Arrays.stream(disCardList)
                    .map(card -> GDUtils.getCardPointWildCard(card, wildCard, curLevel,2))
                    .sorted()
                    .boxed()
                    .collect(Collectors.toList());
        }


        int wildCardCountCurrent = countWildCards(currentPoints, -1);
        int wildCardCountPrevious = countWildCards(previousPoints, -1);

        switch (disCardsType) {
            case SINGLE:
                // 单张直接比较最大点数
                return compareSingle(previousPoints, currentPoints, curLevel);
            case PAIR:
                // 对子：只比较两张部分
                return comparePair(previousPoints, currentPoints, curLevel, wildCardCountCurrent, wildCardCountPrevious);
            case PAIR_SEQUENCE:
                // 3连对
                return comparePairSequence(previousPoints, currentPoints, curLevel, wildCardCountCurrent, wildCardCountPrevious);
            case TRIPLE:
                // 三不带：考虑赖子的补充逻辑
                return compareTriple(previousPoints, currentPoints, curLevel, wildCardCountCurrent, wildCardCountPrevious);
            case TRIPLE_WITH_TWO:
                // 三带二：三张部分优先考虑赖子牌的补充
                return compareTripleWithTwo(previousPoints, currentPoints, curLevel, wildCardCountCurrent, wildCardCountPrevious);
            case TRIPLE_SEQUENCE:
                // 三顺(二个连续三张)
                return compareThreeConsecutive(previousPoints, currentPoints, curLevel, wildCardCountCurrent, wildCardCountPrevious);
            case ROYAL_FLUSH:
                return compareRoyalFlush(previousPoints, currentPoints, curLevel, wildCardCountCurrent, wildCardCountPrevious);
                case STRAIGHT:
                // 顺子：赖子牌可以补齐顺子
                return compareStraight(previousPoints, currentPoints, curLevel, wildCardCountCurrent, wildCardCountPrevious);
            case BOMB:
                // 炸弹：赖子牌可以增强炸弹的级别
                return compareBomb(previousPoints, currentPoints,curLevel, wildCardCountCurrent, wildCardCountPrevious);
            default:
                if (disCardsType.equals(CardType.FIVE_BOMB)||
                    disCardsType.equals(CardType.SIX_BOMB)||
                    disCardsType.equals(CardType.SEVEN_BOMB)||
                    disCardsType.equals(CardType.EIGHT_BOMB)
                ){
                    return compareBomb(previousPoints, currentPoints,curLevel, wildCardCountCurrent, wildCardCountPrevious);
                }
                // 默认按最大点数比较
                return Integer.compare(
                        currentPoints.get(currentPoints.size() - 1),
                        previousPoints.get(previousPoints.size() - 1)
                );
        }
    }

    /**
     * 统计赖子牌数量
     */
    private static int countWildCards(List<Integer> points, int wildCard) {
        return (int) points.stream().filter(card -> card == wildCard).count();
    }


    /**
     * 替换赖子牌为当前手牌中的任意其他牌（排除大王和小王）
     *
     * @param points 当前手牌的点数列表
     * @param wildCardCount 当前赖子牌的数量
     * @param curLevel 当前级数（主牌）
     * @param requiredCount 需要的牌型数量（例如三张牌需要 3）
     * @return 调整后的点数列表
     */
  /*  private static List<Integer> replaceWildCardsWithTrump(List<Integer> points, int wildCardCount, int curLevel, int requiredCount) {
        List<Integer> adjustedPoints = new ArrayList<>(points);
        adjustedPoints.removeIf(p -> p == -1); // 移除赖子标识

        // 找到当前手牌中排除大王和小王的最小点数
        List<Integer> availablePoints = adjustedPoints.stream()
                .filter(p -> p != 16 && p != 17)  // 排除大王（17）和小王（16）
                .distinct()  // 确保不重复
                .sorted()
                .collect(Collectors.toList());

        // 如果有可用的牌点，则选择最小的点数作为赖子的替换点
        int replacementPoint = availablePoints.isEmpty() ? curLevel : availablePoints.get(0); // 默认选择最小点数，如果没有则选择当前级牌

        // 替换赖子牌为选定的点数
        while (wildCardCount > 0 && adjustedPoints.size() < requiredCount) {
            adjustedPoints.add(replacementPoint); // 使用替换点数填充赖子
            wildCardCount--;
        }

        // 如果赖子牌数量仍然大于0，但已经满足requiredCount，将剩余的赖子牌添加为当前级牌
        while (wildCardCount > 0) {
            adjustedPoints.add(curLevel);
            wildCardCount--;
        }

        adjustedPoints.sort(Collections.reverseOrder()); // 按照点数降序排序
        return adjustedPoints;
    }*/

    /**
     * 比较单张牌的大小
     *
     * @param previousPoints 上一手牌的点数列表
     * @param currentPoints 当前手牌的点数列表
     * @param curLevel 当前级数（主牌）
     * @return 正数表示当前牌较大，负数表示上一手牌较大，0表示相等
     */
    private static int compareSingle(List<Integer> previousPoints, List<Integer> currentPoints, int curLevel) {
        // 大王和小王的特殊点数
        final int BIG_JOKER = 17; // 大王
        final int SMALL_JOKER = 16; // 小王

        // 获取上一手牌和当前手牌的最大点数
        int maxPrevious = previousPoints.stream()
                .mapToInt(p -> p == -1 ? curLevel : p) // 将赖子牌视为主牌点数
                .max()
                .orElse(-1);

        int maxCurrent = currentPoints.stream()
                .mapToInt(p -> p == -1 ? curLevel : p) // 将赖子牌视为主牌点数
                .max()
                .orElse(-1);

        // 判断是否有大王或小王
        boolean previousHasBigJoker = previousPoints.contains(BIG_JOKER);
        boolean previousHasSmallJoker = previousPoints.contains(SMALL_JOKER);
        boolean currentHasBigJoker = currentPoints.contains(BIG_JOKER);
        boolean currentHasSmallJoker = currentPoints.contains(SMALL_JOKER);

        // 优先级比较：大王 > 小王 > 主牌
        if (currentHasBigJoker && !previousHasBigJoker) {
            return 1; // 当前牌含大王，优先级更高
        }
        if (!currentHasBigJoker && previousHasBigJoker) {
            return -1; // 上一手牌含大王，优先级更高
        }
        if (currentHasSmallJoker && !previousHasSmallJoker) {
            return 1; // 当前牌含小王，优先级更高
        }
        if (!currentHasSmallJoker && previousHasSmallJoker) {
            return -1; // 上一手牌含小王，优先级更高
        }

        // 主牌优先级比较
        boolean previousIsTrump = (maxPrevious == curLevel);
        boolean currentIsTrump = (maxCurrent == curLevel);

        if (currentIsTrump && !previousIsTrump) {
            return 1; // 当前牌是主牌，优先级更高
        }
        if (!currentIsTrump && previousIsTrump) {
            return -1; // 上一手牌是主牌，优先级更高
        }

        // 默认比较点数
        return Integer.compare(maxCurrent, maxPrevious);
    }

    /**
     * 比较三连对牌型的大小（考虑赖子牌）
     *
     * @param previousPoints 上一手牌的点数列表
     * @param currentPoints 当前手牌的点数列表
     * @param curLevel 当前级数（主牌）
     * @param wildCardCountCurrent 当前手牌的赖子数量
     * @param wildCardCountPrevious 上一手牌的赖子数量
     * @return 正数表示当前牌较大，负数表示上一手牌较大，0表示相等
     */
    private static int comparePairSequence(List<Integer> previousPoints, List<Integer> currentPoints, int curLevel, int wildCardCountCurrent, int wildCardCountPrevious) {
        // 替换赖子牌为当前级牌点数，形成有效的对子序列
        List<Integer> adjustedPrevious = replaceWildCardsPairSequence(previousPoints, wildCardCountPrevious, curLevel);
        List<Integer> adjustedCurrent = replaceWildCardsPairSequence(currentPoints, wildCardCountCurrent, curLevel);
        // 使用 Collections.sort 进行降序排序
        Collections.sort(adjustedPrevious, Collections.reverseOrder());
        // 使用 Collections.sort 进行降序排序
        Collections.sort(adjustedCurrent, Collections.reverseOrder());
        // 获取三连对的最大点数
        int maxPrevious = getMaxConsecutiveValue(adjustedPrevious);
        int maxCurrent = getMaxConsecutiveValue(adjustedCurrent);

        // 级牌优先级比较
//        boolean previousIsTrump = (maxPrevious == curLevel);
//        boolean currentIsTrump = (maxCurrent == curLevel);
//
//        if (currentIsTrump && !previousIsTrump) return 1; // 当前牌是级牌三连对
//        if (!currentIsTrump && previousIsTrump) return -1; // 上一手牌是级牌三连对

        // 默认比较最大点数
        return Integer.compare(maxCurrent, maxPrevious);
    }

    /**
     * 比较对子牌的大小
     *
     * @param previousPoints 上一手牌的点数列表
     * @param currentPoints 当前手牌的点数列表
     * @param curLevel 当前级数（主牌）
     * @param wildCardCountCurrent 当前手牌的赖子数量
     * @param wildCardCountPrevious 上一手牌的赖子数量
     * @return 正数表示当前牌较大，负数表示上一手牌较大，0表示相等
     */
    private static int comparePair(List<Integer> previousPoints, List<Integer> currentPoints, int curLevel, int wildCardCountCurrent, int wildCardCountPrevious) {
        // 特殊点数处理：大王和小王
        final int BIG_JOKER = 17; // 大王
        final int SMALL_JOKER = 16; // 小王

        // 检查上一手牌和当前手牌是否包含大王或小王
        boolean previousHasBigJoker = previousPoints.contains(BIG_JOKER);
        boolean previousHasSmallJoker = previousPoints.contains(SMALL_JOKER);
        boolean currentHasBigJoker = currentPoints.contains(BIG_JOKER);
        boolean currentHasSmallJoker = currentPoints.contains(SMALL_JOKER);

        // 优先比较大王和小王
        if (currentHasBigJoker && !previousHasBigJoker) return 1;
        if (!currentHasBigJoker && previousHasBigJoker) return -1;
        if (currentHasSmallJoker && !previousHasSmallJoker) return 1;
        if (!currentHasSmallJoker && previousHasSmallJoker) return -1;

        // 补充赖子牌以形成对子
        List<Integer> adjustedPrevious = replaceWildCardsWithTrump(previousPoints, wildCardCountPrevious, curLevel, 2);
        List<Integer> adjustedCurrent = replaceWildCardsWithTrump(currentPoints, wildCardCountCurrent, curLevel, 2);

        // 获取对子的最大点数
        int maxPreviousPair = getPairValue(adjustedPrevious);
        int maxCurrentPair = getPairValue(adjustedCurrent);

        // 级牌优先级比较
        boolean previousIsTrumpPair = (maxPreviousPair == curLevel);
        boolean currentIsTrumpPair = (maxCurrentPair == curLevel);

        if (currentIsTrumpPair && !previousIsTrumpPair) {
            return 1; // 当前牌是主牌对子，优先级更高
        }
        if (!currentIsTrumpPair && previousIsTrumpPair) {
            return -1; // 上一手牌是主牌对子，优先级更高
        }

        // 默认比较对子点数
        return Integer.compare(maxCurrentPair, maxPreviousPair);
    }

    /**
     * 获取对子的最大点数（假设列表已经是对子形式或调整后的形式）
     *
     * @param points 调整后的点数列表
     * @return 对子的最大点数
     */
    private static int getPairValue(List<Integer> points) {
        // 假设points已经通过replaceWildCardsWithTrump调整为至少有两个元素
        // 并且已经排序，我们直接返回第二个元素（即较大的那个）
        // 因为对子中两个点数应该相同，返回任意一个即可（这里选择较大的，即第二个）
        // 如果不是严格对子（比如通过赖子补充），返回最大点数作为代表
        if (points.size() >= 2) {
            // 如果存在多个相同的最大点数，返回其中一个即可
            // 这里简单返回最大的那个，因为已经排序
            return points.get(points.size() - 1);
        }
        // 理论上不应该发生，因为replaceWildCardsWithTrump应保证至少有两个元素
        throw new IllegalArgumentException("点数列表应至少包含两个元素以形成对子");
    }

    /**
     * 比较三不带牌型的大小
     *
     * @param previousPoints 上一手牌的点数列表
     * @param currentPoints 当前手牌的点数列表
     * @param curLevel 当前级数（主牌）
     * @param wildCardCountCurrent 当前手牌的赖子数量
     * @param wildCardCountPrevious 上一手牌的赖子数量
     * @return 正数表示当前牌较大，负数表示上一手牌较大，0表示相等
     */
    private static int compareTriple(List<Integer> previousPoints, List<Integer> currentPoints, int curLevel, int wildCardCountCurrent, int wildCardCountPrevious) {
        // 替换赖子牌为当前级牌或其他牌点
        List<Integer> adjustedPrevious = replaceWildCardsWithTrump(previousPoints, wildCardCountPrevious, curLevel, 3);
        List<Integer> adjustedCurrent = replaceWildCardsWithTrump(currentPoints, wildCardCountCurrent, curLevel, 3);

        // 获取三张牌的最大点数
        int maxPreviousTriple = getMaxFrequencyCard(adjustedPrevious, 3);
        int maxCurrentTriple = getMaxFrequencyCard(adjustedCurrent, 3);

        // 级牌优先级比较
        boolean previousIsTrumpTriple = (maxPreviousTriple == curLevel);
        boolean currentIsTrumpTriple = (maxCurrentTriple == curLevel);

        if (currentIsTrumpTriple && !previousIsTrumpTriple) {
            return 1; // 当前牌是主牌三张，优先级更高
        }
        if (!currentIsTrumpTriple && previousIsTrumpTriple) {
            return -1; // 上一手牌是主牌三张，优先级更高
        }

        // 默认比较三张牌点数
        return Integer.compare(maxCurrentTriple, maxPreviousTriple);
    }

    /**
     * 比较三带二牌型的大小（仅比较三张牌的部分，忽略两张牌）
     *
     * @param previousPoints 上一手牌的点数列表
     * @param currentPoints 当前手牌的点数列表
     * @param curLevel 当前级数（主牌）
     * @param wildCardCountCurrent 当前手牌的赖子数量
     * @param wildCardCountPrevious 上一手牌的赖子数量
     * @return 正数表示当前牌较大，负数表示上一手牌较大，0表示相等
     */
    private static int compareTripleWithTwo(List<Integer> previousPoints, List<Integer> currentPoints, int curLevel, int wildCardCountCurrent, int wildCardCountPrevious) {
        // 替换赖子牌为当前级牌点数，形成有效的三带二牌型
        List<Integer> adjustedPrevious = replaceWildCardsTwoInThree(previousPoints, curLevel);
        List<Integer> adjustedCurrent = replaceWildCardsTwoInThree(currentPoints,curLevel);

        // 获取三张牌的最大点数
        int maxPreviousTriple = getMaxFrequencyCard(adjustedPrevious, 3);
        int maxCurrentTriple = getMaxFrequencyCard(adjustedCurrent, 3);

        // 主牌优先级比较
        boolean previousIsTrumpTriple = (maxPreviousTriple == curLevel);
        boolean currentIsTrumpTriple = (maxCurrentTriple == curLevel);

        if (currentIsTrumpTriple && !previousIsTrumpTriple) {
            return 1; // 当前牌是主牌三张，优先级更高
        }
        if (!currentIsTrumpTriple && previousIsTrumpTriple) {
            return -1; // 上一手牌是主牌三张，优先级更高
        }

        // 如果三张牌的点数相同，比较两张牌的点数
        if (maxCurrentTriple == maxPreviousTriple) {
            // 移除三张牌的部分，剩下两张牌的部分
            adjustedPrevious.removeIf(card -> card == maxPreviousTriple);
            adjustedCurrent.removeIf(card -> card == maxCurrentTriple);

            // 获取两张牌的最大点数
            int maxPreviousPair = getMaxFrequencyCard(adjustedPrevious, 2);
            int maxCurrentPair = getMaxFrequencyCard(adjustedCurrent, 2);

            // 主牌优先级比较
            boolean previousIsTrumpPair = (maxPreviousPair == curLevel);
            boolean currentIsTrumpPair = (maxCurrentPair == curLevel);

            if (currentIsTrumpPair && !previousIsTrumpPair) {
                return 1; // 当前牌是主牌两张，优先级更高
            }
            if (!currentIsTrumpPair && previousIsTrumpPair) {
                return -1; // 上一手牌是主牌两张，优先级更高
            }

            // 默认比较两张牌的点数
            return Integer.compare(maxCurrentPair, maxPreviousPair);
        }

        // 默认比较三张牌的点数
        return Integer.compare(maxCurrentTriple, maxPreviousTriple);
    }

    /**
     * 比较两组三张连续牌型的大小，支持赖子牌的处理
     *
     * @param previousPoints 上一手牌的点数列表（三张连续牌）
     * @param currentPoints 当前手牌的点数列表（三张连续牌）
     * @param curLevel 当前级数（主牌）
     * @param wildCardCountCurrent 当前手牌的赖子数量
     * @param wildCardCountPrevious 上一手牌的赖子数量
     * @return 正数表示当前牌较大，负数表示上一手牌较大，0表示相等
     */
    private static int compareThreeConsecutive(List<Integer> previousPoints, List<Integer> currentPoints, int curLevel, int wildCardCountCurrent, int wildCardCountPrevious) {
        // 替换赖子牌为当前级牌或其他牌点
        List<Integer> adjustedPrevious = replaceWildCardsSteelPlates(previousPoints, wildCardCountPrevious, curLevel, 6);
        List<Integer> adjustedCurrent = replaceWildCardsSteelPlates(currentPoints, wildCardCountCurrent, curLevel, 6);

        // 获取三顺的最大点数
        int maxPrevious = getMaxConsecutiveValue(adjustedPrevious);
        int maxCurrent = getMaxConsecutiveValue(adjustedCurrent);

        // 级牌优先级比较 三顺不用比较主牌 因为三顺是连续的 所以直接比较最大值即可
//        boolean previousIsTrump = (maxPrevious == curLevel);
//        boolean currentIsTrump = (maxCurrent == curLevel);
//
//        if (currentIsTrump && !previousIsTrump) return 1; // 当前牌是级牌三顺
//        if (!currentIsTrump && previousIsTrump) return -1; // 上一手牌是级牌三顺

        // 默认比较最大点数
        return Integer.compare(maxCurrent, maxPrevious);
    }

    /**
     * 计算手牌的强度（考虑赖子牌替换后的牌型）
     * @param points 调整后的点数列表
     * @param wildCardCount 赖子牌数量
     * @return 该手牌的强度值
     */
    private static int calculateHandStrength(List<Integer> points, int wildCardCount) {
        // 如果有赖子牌，根据赖子的数量来决定牌型的强度
        if (wildCardCount == 0) {
            // 没有赖子牌，则返回手牌的最大点数
            return points.get(points.size() - 1);
        }

        // 赖子牌数量 > 0 时，尝试使用赖子提升最大点数
        int adjustedStrength = points.get(points.size() - 1); // 先按最大牌计算
        // 如果赖子数量比较多，可能有机会改变牌型的强度，考虑赖子的替换效果
        if (wildCardCount > 1) {
            adjustedStrength += 1;  // 假设通过赖子牌，我们可以增加一张更高的牌
        }
        return adjustedStrength;
    }

    /**
     * 比较两手牌的点数
     * @param previousPoints 上一手牌的点数列表
     * @param currentPoints 当前手牌的点数列表
     * @return 正数表示当前牌较大，负数表示上一手牌较大，0表示相等
     */
    private static int compareByPoints(List<Integer> previousPoints, List<Integer> currentPoints) {
        // 先比较最大点数
        int maxPrevious = getMaxConsecutiveValue(previousPoints);
        int maxCurrent = getMaxConsecutiveValue(currentPoints);

        // 如果当前牌的最大点数较大，返回1
        if (maxCurrent > maxPrevious) {
            return 1;
        }
        // 如果上一手牌的最大点数较大，返回-1
        if (maxCurrent < maxPrevious) {
            return -1;
        }

        // 如果最大点数相同，比较最小点数
        int minPrevious = getMinConsecutiveValue(previousPoints);
        int minCurrent = getMinConsecutiveValue(currentPoints);

        return Integer.compare(minCurrent, minPrevious);
    }

    /**
     * 获取连续三张牌的最大点数（即三张中的最大牌）
     *
     * @param points 调整后的点数列表
     * @return 三张牌中的最大点数
     */
    private static int getMaxConsecutiveValue(List<Integer> points) {
        return points.get(points.size() - 1); // 假设牌已经是顺序排列
    }

    /**
     * 获取连续三张牌的最小点数（即三张中的最小牌）
     *
     * @param points 调整后的点数列表
     * @return 三张牌中的最小点数
     */
    private static int getMinConsecutiveValue(List<Integer> points) {
        return points.get(0); // 假设牌已经是顺序排列
    }

    /**
     * 比较顺子牌型的大小（考虑赖子牌的补充）
     *
     * @param previousPoints 上一手牌的点数列表
     * @param currentPoints 当前手牌的点数列表
     * @param curLevel 当前级数（主牌）
     * @param wildCardCountCurrent 当前手牌的赖子数量
     * @param wildCardCountPrevious 上一手牌的赖子数量
     * @return 正数表示当前牌较大，负数表示上一手牌较大，0表示相等
     */
    private static int compareStraight(List<Integer> previousPoints, List<Integer> currentPoints, int curLevel, int wildCardCountCurrent, int wildCardCountPrevious) {
        // 替换赖子牌为当前级牌或其他牌点
        List<Integer> adjustedPrevious = replaceWildCardsWithTrump(previousPoints, wildCardCountPrevious, curLevel, 5);
        List<Integer> adjustedCurrent = replaceWildCardsWithTrump(currentPoints, wildCardCountCurrent, curLevel, 5);

        // 判断是否为循环顺子（A-2-3-4-5）
        boolean isPreviousCycle = isCycleStraight(adjustedPrevious);
        boolean isCurrentCycle = isCycleStraight(adjustedCurrent);

        // 如果上一手牌是循环顺子，当前牌不是循环顺子，则当前牌更大
        if (isPreviousCycle && !isCurrentCycle) {
            return 1;
        }
        // 如果当前牌是循环顺子，上一手牌不是循环顺子，则上一手牌更大
        if (isCurrentCycle && !isPreviousCycle) {
            return -1;
        }

        // 获取每手牌的最小点数
        int minPreviousStraight = getMinStraightValue(adjustedPrevious);
        int minCurrentStraight = getMinStraightValue(adjustedCurrent);

        // 如果当前顺子的最小点数较大，返回1
        if (minCurrentStraight > minPreviousStraight) {
            return 1;
        }
        // 如果上一手牌的最小点数较大，返回-1
        if (minCurrentStraight < minPreviousStraight) {
            return -1;
        }

        // 如果顺子的最小点数相同，比较顺子的最大点数
        int maxPreviousStraight = getMaxStraightValue(adjustedPrevious);
        int maxCurrentStraight = getMaxStraightValue(adjustedCurrent);

        // 如果当前顺子的最大点数较大，返回1
        if (maxCurrentStraight > maxPreviousStraight) {
            return 1;
        }
        // 如果上一手牌的最大点数较大，返回-1
        if (maxCurrentStraight < maxPreviousStraight) {
            return -1;
        }

        // 如果顺子的最小点数和最大点数都相同，返回0
        return 0;
    }
    /**
     * 判断是否为循环顺子（A-2-3-4-5）
     */
    private static boolean isCycleStraight(List<Integer> points) {
        // 判断是否包含 A、2、3、4、5
        return points.contains(14) && points.contains(2) && points.contains(3) && points.contains(4) && points.contains(5);
    }
 /*   *//**
     * 获取顺子的最大点数（即顺子中的最大牌）
     *
     * @param points 调整后的点数列表
     * @return 顺子的最大点数
     *//*
    private static int getMaxStraightValue(List<Integer> points) {
        // 顺子应该由连续的五张牌组成，获取顺子的最大点数
        return points.get(points.size() - 1);
    }

    *//**
     * 获取顺子的最小点数（即顺子中的最小牌）
     *
     * @param points 调整后的点数列表
     * @return 顺子的最小点数
     *//*
    private static int getMinStraightValue(List<Integer> points) {
        // 顺子应该由连续的五张牌组成，获取顺子的最小点数
        return points.get(0);
    }*/

    /**
     * 比较炸弹牌型的大小（考虑赖子牌的增强）
     *
     * @param previousPoints 上一手牌的点数列表
     * @param currentPoints 当前手牌的点数列表
     * @param wildCardCountCurrent 当前手牌中的赖子数量
     * @param wildCardCountPrevious 上一手牌中的赖子数量
     * @return 正数表示当前牌较大，负数表示上一手牌较大，0表示相等
     */
    private static int compareBomb(List<Integer> previousPoints, List<Integer> currentPoints,int curLevel, int wildCardCountCurrent, int wildCardCountPrevious) {
        // 获取上一手牌和当前手牌中的最大炸弹点数
//        int maxPreviousBomb = getMaxFrequencyCard(previousPoints, 4);
//        int maxCurrentBomb = getMaxFrequencyCard(currentPoints, 4);
        int maxPreviousBomb = Collections.max(previousPoints);
        int maxCurrentBomb = Collections.max(currentPoints);
        //优先比较级牌
        if (maxCurrentBomb == curLevel) {
            return 1;
        }

        // 如果当前手牌的炸弹点数较大，返回1
        if (maxCurrentBomb > maxPreviousBomb) {
            return 1;
        }
        // 如果上一手牌的炸弹点数较大，返回-1
        if (maxCurrentBomb < maxPreviousBomb) {
            return -1;
        }

        // 如果炸弹点数相同，考虑赖子的数量，赖子越多，炸弹越强
        if (wildCardCountCurrent > wildCardCountPrevious) {
            return 1;  // 当前手牌有更多赖子，炸弹更强
        }
        if (wildCardCountCurrent < wildCardCountPrevious) {
            return -1; // 上一手牌有更多赖子，炸弹更强
        }

        // 如果炸弹点数和赖子数量都相同，则返回平局
        return 0;
    }

    /**
     * 获取指定频率的最大点数
     *
     * @param points 点数列表
     * @param frequency 目标频率（如 3 表示三张）
     * @return 满足频率的最大点数
     */
    private static int getMaxFrequencyCard(List<Integer> points, int frequency) {
        return points.stream()
                .collect(Collectors.groupingBy(p -> p, Collectors.counting()))
                .entrySet().stream()
                .filter(entry -> entry.getValue() >= frequency)
                .mapToInt(Map.Entry::getKey)
                .max()
                .orElse(-1); // 如果没有满足条件的点数，返回 -1
    }

    /**
     * 获取牌点
     * @param card 牌的值
     * @return 牌点
     */
    public static int getCardPoint(int card) {
        if (card == 0xEF) return 16; // 小王
        if (card == 0xFF) return 17; // 大王
        return card & 0xF;
    }

    /**
     * 获取牌点，考虑级牌影响
     * @param card 牌的值
     * @param curLevel 当前级数
     * @return 牌点
     */
    public static int getCardPoint(int card, int curLevel) {
        if (card == 0xEF) return 16; // 小王
        if (card == 0xFF) return 17; // 大王

        int point = card & 0xF; // 其他牌点
        if (point == curLevel) {
            return 15; // 级牌
        }
        return point;
    }

    /**
     * 获取单张牌的点数
     *
     * @param card 当前牌的值
     * @param wildCard 赖子牌的值
     * @param curLevel 当前级数
     * @return 返回牌的点数
     */
    public static int getCardPointWildCard(int card, int wildCard, int curLevel,int type) {
        if (card == wildCard) {
            // 如果是赖子牌，默认返回一个通用标识值（如 -1），后续处理时动态替换
            return -1;
        }
        // 计算牌的点数
        if(type ==1){
            return getCardPoint(card, curLevel);
        }
        return getCardPoint(card);
    }


    /**
     * 获取牌型的权重值
     * @param cardType 牌型
     * @return 权重值
     */
    public static int getCardTypeWeight(CardType cardType) {
        switch (cardType) {
            case FOUR_KINGS: return 7; // 4个王 > ...
            case EIGHT_BOMB: return 6; // 8炸弹 > ...
            case SEVEN_BOMB: return 5; // 7炸弹 > ...
            case SIX_BOMB: return 4;   // 6炸弹 > ...
            case ROYAL_FLUSH: return 3;// 同花顺 > ...
            case FIVE_BOMB: return 2;  // 5炸弹 > ...
            case FOUR_BOMB: return 1;  // 4炸弹 > ...
            default: return 0; // 其他牌型
        }
    }

    /**
     * 判断牌型
     * @param disCardList 玩家出的牌（int数组）
     * @param wildCard 赖子牌点数
     * @return 牌型枚举值
     */
    public static CardType determineCardType(int[] disCardList, int wildCard,CardType previousCardsType) {
        if (disCardList == null || disCardList.length == 0) {
            return CardType.INVALID;
        }

        // 提取牌点，处理赖子（wildCard）
        List<Integer> points = new ArrayList<>();
        int wildCount = 0; // 记录赖子数量
        for (int card : disCardList) {
            if (card == wildCard) {
                wildCount++;
            } else {
                points.add(GDUtils.getCardPoint(card));
            }
        }

        // 统计牌点频率
        Map<Integer, Long> pointFrequency = points.stream()
                .collect(Collectors.groupingBy(p -> p, Collectors.counting()));

        // 排序后的牌点集合
        List<Integer> sortedPoints = points.stream()
                .distinct()
                .sorted()
                .collect(Collectors.toList());

        // 判断不同牌型
        int cardCount = disCardList.length;
        int uniqueCount = pointFrequency.size();

        // 1. 四王炸（大小王各两张）
        if (cardCount == 4 && pointFrequency.size() == 2 && points.contains(16) && points.contains(17)
                && pointFrequency.getOrDefault(16, 0L) + wildCount >= 2
                && pointFrequency.getOrDefault(17, 0L) + wildCount >= 2) {
            return CardType.FOUR_KINGS;
        }

        // 2. 特定炸弹类型（四炸弹到八炸弹）
        for (int i = 4; i <= 8; i++) {
            if (cardCount == i && uniqueCount == 1) {
                long count = pointFrequency.values().iterator().next();
                if (count + wildCount == i) {
                    switch (i) {
                        case 4: return CardType.FOUR_BOMB;
                        case 5: return CardType.FIVE_BOMB;
                        case 6: return CardType.SIX_BOMB;
                        case 7: return CardType.SEVEN_BOMB;
                        case 8: return CardType.EIGHT_BOMB;
                    }
                }
            }
        }

        // 3. 通用炸弹（四张及以上相同牌点）
        if (uniqueCount == 1 && pointFrequency.values().iterator().next() + wildCount >= 4) {
            return CardType.BOMB;
        }

        // 4. 同花顺
        if (cardCount == 5 && isRoyalFlush(disCardList, wildCard)) {
            return CardType.ROYAL_FLUSH;
        }

        // 5. 顺子（赖子可补齐缺失点数）
        if (cardCount == 5 && uniqueCount + wildCount == 5 && isStraightWithWild(sortedPoints, wildCount)) {
            return CardType.STRAIGHT;
        }

        // 6. 三顺（赖子可补齐缺失三张）
        if (cardCount == 6 && cardCount % 3 == 0 && uniqueCount + wildCount >= cardCount / 3
                && isTripleSequenceWithWild(pointFrequency, wildCount)) {
            if (previousCardsType!=null&&previousCardsType.equals(CardType.TRIPLE_SEQUENCE)){
                return CardType.TRIPLE_SEQUENCE;
            }
            if (previousCardsType==null){
                return CardType.TRIPLE_SEQUENCE;
            }
        }

        // 7. 连对（赖子可补齐缺失对数）
        if (cardCount >= 6 && cardCount % 2 == 0 && uniqueCount + wildCount >= cardCount / 2
                && isPairSequenceWithWild(pointFrequency, wildCount)) {
            if (previousCardsType!=null&&previousCardsType.equals(CardType.PAIR_SEQUENCE)){
                return CardType.PAIR_SEQUENCE;
            }
            if (previousCardsType==null){
                return CardType.PAIR_SEQUENCE;
            }
        }

        // 8. 三带二
        if (cardCount == 5 && uniqueCount <= 3 && isTripleWithTwo(pointFrequency, wildCount)) {
            return CardType.TRIPLE_WITH_TWO;
        }

        // 9. 三不带
        if (cardCount == 3 && uniqueCount == 1) {
            return CardType.TRIPLE;
        }

        // 10. 对子
        if (cardCount == 2 && (uniqueCount == 1 || wildCount == 1 || (uniqueCount == 0 && wildCount == 2))) { // wildCount == 1 to form a pair
            return CardType.PAIR;
        }

        // 11. 单张
        if (cardCount == 1) {
            return CardType.SINGLE;
        }

        return CardType.INVALID; // 非法牌型
    }

    // 判断是否为三带二牌型
    private static boolean isTripleWithTwo(Map<Integer, Long> pointFrequency, int wildCount) {
        // 统计每种点数的牌的数量
        List<Integer> points = new ArrayList<>(pointFrequency.keySet());
        points.sort(Comparator.naturalOrder()); // 按点数升序排序

        // 初始化三张和两张的需求
        boolean hasTriple = false;
        boolean hasPair = false;
        boolean hasTwoSingles = false;

        // 剩余赖子牌数量
        int remainingWilds = wildCount;

        // 检查是否能构成三张
        for (Integer point : points) {
            long count = pointFrequency.get(point);
            if (count >= 3) {
                hasTriple = true;
                break;
            } else if (count > 0) {
                int neededWilds = 3 - (int) count;
                if (remainingWilds >= neededWilds) {
                    remainingWilds -= neededWilds;
                    hasTriple = true;
                    break;
                }
            }
        }

        // 如果无法构成三张，直接返回 false
        if (!hasTriple) {
            return false;
        }

        // 检查是否能构成两张（不允许与三张相同点数）
        for (Integer point : points) {
            long count = pointFrequency.get(point);
            if (count >= 2) {
                hasPair = true;
                break;
            } else if (count > 0) {
                int neededWilds = 2 - (int) count;
                if (remainingWilds >= neededWilds) {
                    remainingWilds -= neededWilds;
                    hasPair = true;
                    break;
                }
            }
        }

        // 返回是否能同时构成三张和两张
        return hasTriple && hasPair;
    }
    /**
     * 比较同花顺子牌型的大小（考虑赖子牌的补充）
     *
     * @param previousPoints 上一手牌的点数列表
     * @param currentPoints 当前手牌的点数列表
     * @param curLevel 当前级数（主牌）
     * @param wildCardCountCurrent 当前手牌的赖子数量
     * @param wildCardCountPrevious 上一手牌的赖子数量
     * @return 正数表示当前牌较大，负数表示上一手牌较大，0表示相等
     */
    private static int compareRoyalFlush(List<Integer> previousPoints, List<Integer> currentPoints, int curLevel, int wildCardCountCurrent, int wildCardCountPrevious) {
        // 替换赖子牌为当前级牌点数，形成有效的同花顺
        List<Integer> adjustedPrevious = replaceWildCardsWithTrump(previousPoints, wildCardCountPrevious, curLevel, 5);
        List<Integer> adjustedCurrent = replaceWildCardsWithTrump(currentPoints, wildCardCountCurrent, curLevel, 5);

        // 获取每手牌的最小点数
        int minPreviousStraight = getMinStraightValue(adjustedPrevious);
        int minCurrentStraight = getMinStraightValue(adjustedCurrent);


        // 判断是否为循环顺子（A-2-3-4-5）
        boolean isPreviousCycle = isCycleStraight(adjustedPrevious);
        boolean isCurrentCycle = isCycleStraight(adjustedCurrent);

        // 如果上一手牌是循环顺子，当前牌不是循环顺子，则当前牌更大
        if (isPreviousCycle && !isCurrentCycle) {
            return 1;
        }
        // 如果当前牌是循环顺子，上一手牌不是循环顺子，则上一手牌更大
        if (isCurrentCycle && !isPreviousCycle) {
            return -1;
        }

        // 如果当前顺子的最小点数较大，返回1
        if (minCurrentStraight > minPreviousStraight) {
            return 1;
        }
        // 如果上一手牌的最小点数较大，返回-1
        if (minCurrentStraight < minPreviousStraight) {
            return -1;
        }

        // 如果顺子的最小点数相同，比较顺子的最大点数
        int maxPreviousStraight = getMaxStraightValue(adjustedPrevious);
        int maxCurrentStraight = getMaxStraightValue(adjustedCurrent);

        // 如果当前顺子的最大点数较大，返回1
        if (maxCurrentStraight > maxPreviousStraight) {
            return 1;
        }
        // 如果上一手牌的最大点数较大，返回-1
        if (maxCurrentStraight < maxPreviousStraight) {
            return -1;
        }

        // 如果顺子的最小点数和最大点数都相同，返回0
        return 0;
    }

    /**
     * 替换赖子牌为当前手牌中的其他牌，以形成有效的牌型
     *
     * @param points 当前手牌的点数列表
     * @param wildCardCount 赖子牌的数量
     * @param curLevel 当前级数（主牌）
     * @param requiredCount 需要的牌型数量（当前为三带二为5）
     * @return 调整后的点数列表
     */
    private static List<Integer> replaceWildCardsWithTrump(List<Integer> points, int wildCardCount, int curLevel, int requiredCount) {
        List<Integer> adjustedPoints = new ArrayList<>(points);
        adjustedPoints.removeIf(p -> p == -1); // 移除赖子标识
        // 判断是否存在14且存在2-5之间的值
        boolean shouldReplace = adjustedPoints.stream()
                .anyMatch(value -> value == 14) &&
                adjustedPoints.stream()
                        .anyMatch(value -> value >= 2 && value <= 5);

        if (shouldReplace) {
            // 替换所有的14为1
            adjustedPoints.replaceAll(value -> value == 14 ? 1 : value);
        }
        // 找到当前手牌中排除大王和小王的最小点数
        List<Integer> availablePoints = adjustedPoints.stream()
                .filter(p -> p != 16 && p != 17)  // 排除大王（17）和小王（16）
                .distinct()  // 确保不重复
                .sorted()
                .collect(Collectors.toList());

        // 如果有可用的牌点，则选择最小的点数作为赖子的替换点
        int replacementPoint = availablePoints.isEmpty() ? curLevel : availablePoints.get(0); // 默认选择最小点数，如果没有则选择当前级牌

        // 替换赖子牌为选定的点数
        while (wildCardCount > 0 && adjustedPoints.size() < requiredCount) {
            adjustedPoints.add(replacementPoint); // 使用替换点数填充赖子
            wildCardCount--;
        }

        // 如果赖子牌数量仍然大于0，但已经满足requiredCount，将剩余的赖子牌添加为当前级牌
        while (wildCardCount > 0) {
            adjustedPoints.add(curLevel);
            wildCardCount--;
        }

        adjustedPoints.sort(Collections.reverseOrder()); // 按照点数降序排序
        return adjustedPoints;
    }

    /**
     * 连对牌值
     * @param points 牌值点列表
     * @param wildCardCount 癞子牌的数量
     * @param curLevel
     * @return
     */
    private static List<Integer> replaceWildCardsPairSequence(List<Integer> points, int wildCardCount, int curLevel) {
        points = new ArrayList<>(points);
        wildCardCount = Collections.frequency(points, -1);

        // 统计每个点数的牌的数量
        Map<Integer, Integer> cardCountMap = new HashMap<>();
        //构建结果列表
        List<Integer> result = new ArrayList<>();
        for (int point : points) {
            if (point!= -1) {
                cardCountMap.put(point, cardCountMap.getOrDefault(point, 0) + 1);
            }
        }
        // 如果没有癞子且已经是完整的三连对，直接返回
        if (wildCardCount == 0) {
            List<Integer> sortedPoints = points.stream()
                    .sorted(Comparator.reverseOrder())
                    .collect(Collectors.toList());
            Integer firstMax = sortedPoints.get(0);
            Integer secondMax = sortedPoints.get(1);
            if (firstMax == 14 && secondMax != 13) {
                // 1. 移除所有的 14
                sortedPoints.removeIf(num -> num == 14);
                // 2. 添加两个 1
                sortedPoints.add(1);
                sortedPoints.add(1);
            }
            return sortedPoints;
        }
        // 特殊处理：仅在特定条件下将三个14转换为1
        if (cardCountMap.containsKey(14)) {
            List<Integer> sortedKeys = new ArrayList<>(cardCountMap.keySet());
            Collections.sort(sortedKeys, Collections.reverseOrder());

            int firstMax = sortedKeys.get(0);
            int secondMax = sortedKeys.size() > 1 ? sortedKeys.get(1) : -1;

            // 检查是否有三个14，且第二大点数小于14
            if (firstMax == 14 && secondMax != 13) {
                cardCountMap.remove(14); // 移除三个14
                cardCountMap.put(1, cardCountMap.getOrDefault(1, 0) + 3); // 添加三个1
            }
        }
        // 处理癞子补充缺牌逻辑
        if (wildCardCount ==1) { //如果癞子数量为1，则替换单张牌
            Optional<Integer> first = cardCountMap.entrySet()
                    .stream()
                    .filter(entry -> entry.getValue() == 1)
                    .map(Map.Entry::getKey)
                    .findFirst();
            if (first.isPresent()) {
                Integer integer = first.get();
                cardCountMap.put(integer, 2);
            }
        }else if(wildCardCount ==2){
            //查询key值数量
            List<Integer> sortedKeys = new ArrayList<>(cardCountMap.keySet());
            if (sortedKeys.size()<3){ //小于3则为特殊牌型 2233-1-1 直接替换
                Integer maxValue = Collections.max(sortedKeys);//14
                Integer minValue = Collections.min(sortedKeys);//13 QKKAA
                if (maxValue-minValue==1){
                    cardCountMap.put(maxValue+1>14?minValue-1:maxValue+1, 2);
                }else {
                    cardCountMap.put(maxValue-1, 2);
                }
            }else {
                //如果等于3则为普通牌型 223-1-1 直接替换
                List<Integer> collect = cardCountMap.entrySet()
                        .stream()
                        .filter(entry -> entry.getValue() == 1)
                        .map(Map.Entry::getKey)
                        .collect(Collectors.toList());
                cardCountMap.put(collect.get(0), 2);
                cardCountMap.put(collect.get(1), 2);
            }
        }
        while (result.size() < 6 && !cardCountMap.isEmpty()) {
            boolean found = false;
            for (Map.Entry<Integer, Integer> entry : cardCountMap.entrySet()) {
                int point = entry.getKey();
                int count = entry.getValue();

                if (count >= 2) {
                    result.add(point);
                    result.add(point);
                    cardCountMap.put(point, count - 2);
                    found = true;
                    break;
                }
            }

            if (!found) {
                break;
            }
        }

        return result;
    }

    /**
     * 钢板牌值
     * @param points
     * @param wildCardCount
     * @param curLevel
     * @param requiredCount
     * @return
     */
    private static List<Integer> replaceWildCardsSteelPlates(List<Integer> points, int wildCardCount, int curLevel, int requiredCount){
        points = new ArrayList<>(points);
        // 计算癞子牌的数量
        wildCardCount = Collections.frequency(points, -1);

        // 统计每个点数的牌的数量
        Map<Integer, Integer> cardCountMap = new HashMap<>();
        for (int point : points) {
            cardCountMap.put(point, cardCountMap.getOrDefault(point, 0) + 1);
        }

        if (wildCardCount == 0) {
            // 特殊处理：仅在特定条件下将三个14转换为1
            if (cardCountMap.containsKey(14)) {
                List<Integer> sortedKeys = new ArrayList<>(cardCountMap.keySet());
                Collections.sort(sortedKeys, Collections.reverseOrder());

                int firstMax = sortedKeys.get(0);
                int secondMax = sortedKeys.size() > 1 ? sortedKeys.get(1) : -1;

                // 检查是否有三个14，且第二大点数小于14
                if (firstMax == 14 && cardCountMap.get(14) == 3 && secondMax < 14 && secondMax != 13) {
                    cardCountMap.put(14, 0); // 移除三个14
                    cardCountMap.put(1, cardCountMap.getOrDefault(1, 0) + 3); // 添加三个1
                }
            }
            // 构建结果列表
            List<Integer> result = new ArrayList<>();
            while (result.size() < 6 && !cardCountMap.isEmpty()) {
                boolean found = false;
                for (Map.Entry<Integer, Integer> entry : cardCountMap.entrySet()) {
                    int point = entry.getKey();
                    int count = entry.getValue();

                    if (count >= 3) {
                        result.add(point);
                        result.add(point);
                        result.add(point);
                        cardCountMap.put(point, count - 3);
                        found = true;
                        break;
                    }
                }

                if (!found) {
                    break;
                }
            }

            return result;
        }

        // 处理癞子替换
        if (wildCardCount > 0) {
            List<Integer> sortedKeys = new ArrayList<>(cardCountMap.keySet());
            Collections.sort(sortedKeys);

            for (int i = sortedKeys.size() - 1; i >= 0 && wildCardCount > 0; i--) {
                int point = sortedKeys.get(i);
                while (cardCountMap.get(point) < 3 && wildCardCount > 0) {
                    cardCountMap.put(point, cardCountMap.get(point) + 1);
                    wildCardCount--;
                }
            }
        }
        // 特殊处理：仅在特定条件下将三个14转换为1
        if (cardCountMap.containsKey(14)) {
            List<Integer> sortedKeys = new ArrayList<>(cardCountMap.keySet());
            Collections.sort(sortedKeys, Collections.reverseOrder());

            int firstMax = sortedKeys.get(0);
            int secondMax = sortedKeys.size() > 1 ? sortedKeys.get(1) : -1;

            // 检查是否有三个14，且第二大点数小于14
            if (firstMax == 14 && cardCountMap.get(14) == 3 && secondMax < 14 && secondMax != 13) {
                cardCountMap.put(14, 0); // 移除三个14
                cardCountMap.put(1, cardCountMap.getOrDefault(1, 0) + 3); // 添加三个1
            }
        }


        // 构建结果列表
        List<Integer> result = new ArrayList<>();
        while (result.size() < 6 && !cardCountMap.isEmpty()) {
            boolean found = false;
            for (Map.Entry<Integer, Integer> entry : cardCountMap.entrySet()) {
                int point = entry.getKey();
                int count = entry.getValue();

                if (count >= 3) {
                    result.add(point);
                    result.add(point);
                    result.add(point);
                    cardCountMap.put(point, count - 3);
                    found = true;
                    break;
                }
            }

            if (!found) {
                break;
            }
        }

        return result;
    }


    /**
     * 三带二
     * @param points
     * @param curLevel
     * @return
     */
    private static List<Integer> replaceWildCardsTwoInThree(List<Integer> points, int curLevel) {
        List<Integer> mutablePoints = new ArrayList<>(points);
        // 计算癞子牌的数量
        long wildCardCount = mutablePoints.stream().filter(p -> p == -1).count();

        if (wildCardCount == 0) {
            return new ArrayList<>(mutablePoints);
        }

        // 移除所有癞子牌
        mutablePoints.removeIf(p -> p == -1);

        // 统计每个点数的牌的数量
        Map<Integer, Integer> cardCountMap = mutablePoints.stream()
                .collect(Collectors.toMap(p -> p, p -> 1, Integer::sum));

        // 替换癞子牌
        while (mutablePoints.size() < 5) {
            if (wildCardCount == 1) {
                replaceSingleTrump(mutablePoints, cardCountMap, curLevel, wildCardCount);
            } else if (wildCardCount == 2) {
                replaceDoubleTrump(mutablePoints, cardCountMap, curLevel, wildCardCount);
            }
        }

        // 确保最终牌的数量不超过5张
        return mutablePoints.subList(0, Math.min(mutablePoints.size(), 5));
    }

    private static void replaceSingleTrump(List<Integer> points, Map<Integer, Integer> cardCountMap, int curLevel, long wildCardCount) {
        // 找单张并替换
        for (Map.Entry<Integer, Integer> entry : cardCountMap.entrySet()) {
            if (entry.getValue() == 1) {
                points.add(entry.getKey());
                wildCardCount--;
                if (wildCardCount == 0 || points.size() >= 5) break;
            }
        }
        // 如果还有剩余癞子且未达到5张，尝试补充级牌或其他最大牌
        if (wildCardCount > 0 && points.size() < 5) {
            Integer cardToAdd = cardCountMap.containsKey(curLevel) ? curLevel : Collections.max(cardCountMap.keySet());
            // 检查 cardToAdd 是否为 16 或 17
            if (cardToAdd == 16 || cardToAdd == 17) {
                // 获取所有键
                Set<Integer> keys = cardCountMap.keySet();
                // 移除当前的 cardToAdd
                keys.remove(cardToAdd);
                // 剩下的键就是另一个值
                cardToAdd = keys.iterator().next();
            }
            // 添加到 points
            points.add(cardToAdd);
        }
    }

    private static void replaceDoubleTrump(List<Integer> points, Map<Integer, Integer> cardCountMap, int curLevel, long wildCardCount) {
        // 优先补充级牌到3张
        if (cardCountMap.containsKey(curLevel)) {
            int count = cardCountMap.get(curLevel);
            while (count < 3 && wildCardCount > 0 && points.size() < 5) {
                points.add(curLevel);
                count++;
                wildCardCount--;
            }
        }
        // 如果还有剩余癞子且未达到5张，补充其他最大牌到3张
        if (wildCardCount > 0 && points.size() < 5) {
            Optional<Map.Entry<Integer, Integer>> maxEntry = cardCountMap.entrySet().stream()
                    .filter(e -> e.getKey() != curLevel)
                    .max(Map.Entry.comparingByKey());
            maxEntry.ifPresent(entry -> {
                int count = entry.getValue();
                while (count < 3 && points.size() < 5&& entry.getKey()!=16&&entry.getKey()!=17) {
                    points.add(entry.getKey());
                    count++;
                }
            });
        }
        // 如果还有剩余癞子且未达到5张，添加任意一张非级牌的最大牌
        if (wildCardCount > 0 && points.size() < 5) {
            List<Integer> filteredKeys = cardCountMap.keySet().stream()
                    .filter(k -> k != curLevel&& k!=16&&k!=17)
                    .collect(Collectors.toList());
            if (!filteredKeys.isEmpty()) {
                int nextCard = Collections.max(filteredKeys);
                points.add(nextCard);
            }
        }
    }

    /**
     * 获取顺子的最小点数（即顺子中的最小牌）
     *
     * @param points 调整后的点数列表
     * @return 顺子的最小点数
     */
    private static int getMinStraightValue(List<Integer> points) {
        // 顺子应该由连续的五张牌组成，获取顺子的最小点数
        return points.stream().min(Integer::compare).orElse(-1);
    }

    /**
     * 获取顺子的最大点数（即顺子中的最大牌）
     *
     * @param points 调整后的点数列表
     * @return 顺子的最大点数
     */
    private static int getMaxStraightValue(List<Integer> points) {
        // 顺子应该由连续的五张牌组成，获取顺子的最大点数
        return points.stream().max(Integer::compare).orElse(-1);
    }
    // 判断是否为顺子（赖子补齐）
    public static boolean isStraightWithWild(List<Integer> points, int wildCount) {
        //处理特殊A值
        boolean hasLowSequence = false;
        for (int point : points) {
            if (point >= 2 && point <= 5) {
                hasLowSequence = true;
                break;
            }
        }
        if (hasLowSequence) {
            for (int i = 0; i < points.size(); i++) {
                if (points.get(i) == 14) {
                    points.set(i, 1);
                }
            }
        }
        // 将牌点排序
        List<Integer> nonWildPoints = new ArrayList<>(points);
        nonWildPoints.removeIf(p -> p == -1);
        // 排序
        Collections.sort(nonWildPoints);
        // 检查是否已经连续
        for (int i = 0; i < nonWildPoints.size() - 1; i++) {
            if (nonWildPoints.size()>=5){
                break;
            }
            if (nonWildPoints.get(i) + 1 != nonWildPoints.get(i + 1)) {
                nonWildPoints.add(nonWildPoints.get(i) + 1);
                wildCount--;
                // 排序
                Collections.sort(nonWildPoints);
            }
        }
        Set<Integer> set = new HashSet<>(nonWildPoints);
        if (set.size() != nonWildPoints.size()) {
            return false; // 存在重复元素
        }
        int min = Collections.min(set);
        int max = Collections.max(set);
        // 检查是否所有从min到max的数都在集合中
        return set.size() == (max - min + 1);
    }


    // 判断是否为连对（赖子补齐）
//    private static boolean isPairSequenceWithWild(Map<Integer, Long> pointFrequency, int wildCount) {
//        List<Integer> sortedKeys = pointFrequency.keySet().stream().sorted().collect(Collectors.toList());
//        // 特定牌型判断：A-A-2-2-3-3 是最小的三连对
//        if (sortedKeys.contains(14) && sortedKeys.contains(2) && sortedKeys.contains(3)) {
//            return true; // A-A-2-2-3-3 是最小的三连对
//        }
//        // 特定牌型判断：Q-Q-K-K-A-A 是最大的三连对
//        if (sortedKeys.contains(12) && sortedKeys.contains(13) && sortedKeys.contains(14)) {
//            return true; // Q-Q-K-K-A-A 是最大的三连对
//        }
//        int gaps = 0;
//        for (int i = 0; i < sortedKeys.size() - 1; i++) {
//            if (sortedKeys.get(i + 1) - sortedKeys.get(i) != 1) {
//                return false; // 非连续
//            }
//            gaps += (int) (2 - pointFrequency.get(sortedKeys.get(i))); // 计算每个点数对缺失的牌
//        }
//        gaps += (int) (2 - pointFrequency.get(sortedKeys.get(sortedKeys.size() - 1)));
//        return gaps <= wildCount;
//    }
    private static boolean isPairSequenceWithWild(Map<Integer, Long> pointFrequency, int wildCount) {
        // 提取并排序所有牌点
        List<Integer> keys = new ArrayList<>(pointFrequency.keySet());
        Collections.sort(keys);

        // 处理A的特殊情况
        for (int i = 0; i < keys.size(); i++) {
            if (keys.get(i) == 14) { // 找到A（视为14）
                if (i > 0) {
                    int prevKey = keys.get(i - 1);
                    if (prevKey == 2 || prevKey == 3) {
                        // 将A视为1，并更新 pointFrequency
                        pointFrequency.put(1, pointFrequency.getOrDefault(1, 0L) + pointFrequency.get(14));
                        keys.add(1);
                        keys.removeIf(key -> key == 14);
                        pointFrequency.remove(14);
                    }
                }
            }
        }

        List<Integer> sortedKeys = new ArrayList<>(keys);
        Collections.sort(sortedKeys);

        // 使用癞子填补空缺
        int i = 0;
        if (wildCount == 0){
            int first = sortedKeys.get(i);
            int second = sortedKeys.get(i + 1);
            int third = sortedKeys.get(i + 2);

            if (second - first == 1 && third - second == 1) {
                // 已经是连续的，检查数量
                if (pointFrequency.get(first) >= 2 &&
                        pointFrequency.get(second) >= 2 &&
                        pointFrequency.get(third) >= 2) {
                    return true;
                }
            }
        }else if (wildCount == 1){
            //判断key值数量
            if (sortedKeys.size()<3){
                return false;
            }else {
                int first = sortedKeys.get(i);
                int second = sortedKeys.get(i + 1);
                int third = sortedKeys.get(i + 2);

                if (second - first == 1 && third - second == 1) {
                    while (wildCount>0){
                        if(pointFrequency.get(first) < 2){
                            long l = pointFrequency.get(first) + 1;
                            pointFrequency.put(first, l);
                            wildCount--;
                            continue;
                        }
                        if(pointFrequency.get(second) < 2){
                            long l = pointFrequency.get(second) + 1;
                            pointFrequency.put(second, l);
                            wildCount--;
                            continue;
                        }
                        if(pointFrequency.get(third) < 2){
                            long l = pointFrequency.get(third) + 1;
                            pointFrequency.put(third, l);
                            wildCount--;
                        }
                    }
                    // 已经是连续的，检查数量
                    if (pointFrequency.get(first) >= 2 &&
                            pointFrequency.get(second) >= 2 &&
                            pointFrequency.get(third) >= 2) {
                        return true;
                    }
                }
            }
        }else if (wildCount == 2){
            if (sortedKeys.size()<3){
                int first = sortedKeys.get(i);
                int second = sortedKeys.get(i + 1);
                if (second - first == 2||second - first == 1){
                    if (pointFrequency.get(first) >= 2 &&
                            pointFrequency.get(second) >= 2){
                        return true;
                    }
                }
            }else {
                int first = sortedKeys.get(i);
                int second = sortedKeys.get(i + 1);
                int third = sortedKeys.get(i + 2);

                if (second - first == 1 && third - second == 1) {
                    while (wildCount>0){
                        if(pointFrequency.get(first) < 2){
                            long l = pointFrequency.get(first) + 1;
                            pointFrequency.put(first, l);
                            wildCount--;
                            continue;
                        }
                        if(pointFrequency.get(second) < 2){
                            long l = pointFrequency.get(second) + 1;
                            pointFrequency.put(second, l);
                            wildCount--;
                            continue;
                        }
                        if(pointFrequency.get(third) < 2){
                            long l = pointFrequency.get(third) + 1;
                            pointFrequency.put(third, l);
                            wildCount--;
                        }
                    }
                    // 已经是连续的，检查数量
                    if (pointFrequency.get(first) >= 2 &&
                            pointFrequency.get(second) >= 2 &&
                            pointFrequency.get(third) >= 2) {
                        return true;
                    }
                }
            }
        }
        return false;
    }

    // 判断是否为三顺（赖子补齐）
    private static boolean isTripleSequenceWithWild(Map<Integer, Long> pointFrequency, int wildCount) {
        if (pointFrequency.size()>=3){
            return false;
        }

        List<Integer> triples = pointFrequency.keySet().stream()
                .filter(key -> pointFrequency.get(key) >= 3||pointFrequency.get(key)+wildCount>=3)
                .sorted()
                .collect(Collectors.toList());

        // 如果没有符合条件的三张连续牌，直接返回 false
        if (triples.isEmpty()) {
            return false;
        }
        // 排除特例：A-A-A-2-2-2 是有效的最小三顺
        if (triples.contains(14) && triples.contains(2)) { // A 和 2 的点数
            return true; // A-A-A-2-2-2 不是有效的三顺
        }
        // 特例：K-K-K-A-A-A 是有效的最大三顺
        if (triples.contains(13) && triples.contains(14)) { // K 和 A 的点数
            return true; // K-K-K-A-A-A 是最大的三顺
        }

        int gaps = 0;
        for (int i = 0; i < triples.size() - 1; i++) {
            // 允许跨界的情况（K-A、A-2）
            if (triples.get(i) == 14 && triples.get(i + 1) == 2) {
                continue;
            }
            if (triples.get(i + 1) - triples.get(i) != 1) {
                return false;
            }
            gaps += (int) (3 - pointFrequency.get(triples.get(i)));
        }
        // 最后一个元素也需要处理
        gaps += (int) (3 - pointFrequency.get(triples.get(triples.size() - 1)));
        return gaps <= wildCount;
    }

    /**
     * 判断是否为同花顺（可选规则）
     *
     * @param cards 牌的数组
     * @param laiziCardPoints 赖子牌点
     * @return true 如果是同花顺，false 否则
     */
    private static boolean isRoyalFlush(int[] cards, int laiziCardPoints) {
        // 提取牌的花色和点数
        List<Integer> suits = new ArrayList<>();
        List<Integer> points = new ArrayList<>();
        int wildCount = 0;
        for (int card : cards) {
            if (card == laiziCardPoints) { // 赖子牌
                wildCount++;
                continue;
            }
            suits.add((card & 0xF0) >> 4); // 提取花色
            points.add(card & 0x0F); // 提取点数
        }

        // 检查花色是否相同
        if (suits.stream().distinct().count() > 1) {
            return false; // 花色不相同
        }
        // 特殊情况处理：A-2-3-4-5 和 10-J-Q-K-A
        if (points.contains(14) && points.contains(2)&& points.contains(3) && points.contains(4) && points.contains(5)) {
            return true; // A-2-3-4-5
        }
        // 检查牌点是否连续
        Collections.sort(points);
        int gaps = 0;
        for (int i = 0; i < points.size() - 1; i++) {
            gaps += points.get(i + 1) - points.get(i) - 1;
        }

        if (points.contains(10) && points.contains(11) && points.contains(12) && points.contains(13) && points.contains(14)) {
            return true;
        }
        // 检查是否可以使用赖子牌补齐 gaps
        return gaps <= wildCount;
    }

    // 测试代码
    public static void main(String[] args) {
        List<Integer> points1 = Arrays.asList(3, 3, 4, 4, -1);
        List<Integer> result1 = replaceWildCardsWithTrump(points1, 1, 5, 5);
        System.out.println(result1); // 输出调整后的牌型

        List<Integer> points2 = Arrays.asList(3, 4, 4, -1, -1);
        List<Integer> result2 = replaceWildCardsWithTrump(points2, 2, 5, 5);
        System.out.println(result2); // 输出调整后的牌型


      /*  int[] cards1 = {0x08, 0x04, 0x05, 0x06, 0x07}; // 顺子
        int[] cards2 = {0x33, 0x13, 0x23, 0x03};       // 四炸弹
        int[] cards8 = {0xEF, 0xEF, 0xFF, 0xFF};             // 四王炸
        int[] cards9 = {0x03, 0x13, 0x23, 0x33};             // 四炸弹
        int[] cards10 = {0x04, 0x14, 0x24, 0x34, 0x44};      // 五炸弹
        int[] cards11 = {0x05, 0x15, 0x25, 0x35, 0x45, 0x55}; // 六炸弹
        int[] cards12 = {0x06, 0x16, 0x26, 0x36, 0x46, 0x56, 0x66}; // 七炸弹
        int[] cards13 = {0x07, 0x17, 0x27, 0x37, 0x47, 0x57, 0x67, 0x77}; // 八炸弹
        int[] cards14 = {0x0e, 0x12, 0x23, 0x04, 0x05}; // 顺子
        int[] cards15 = {0x02, 0x02, 0x03, 0x03, 0x0e, 0x0e}; // A-A-2-2-3-3

        System.out.println(determineCardType(cards1, 0)); // STRAIGHT
        System.out.println(determineCardType(cards2, 0)); // BOMB
        System.out.println(determineCardType(cards8, 0)); // FOUR_KINGS
        System.out.println(determineCardType(cards9, 0)); // FOUR_BOMB
        System.out.println(determineCardType(cards10, 0)); // FIVE_BOMB
        System.out.println(determineCardType(cards11, 0)); // SIX_BOMB
        System.out.println(determineCardType(cards12, 0)); // SEVEN_BOMB
        System.out.println(determineCardType(cards13, 0)); // EIGHT_BOMB
        System.out.println(determineCardType(cards14, 0)); // STRAIGHT
        System.out.println(determineCardType(cards15, 0)); // PAIR_SEQUENCE*/
//        int[] cards10 = {0x08, 0x18, 0x29, 0x39, 0x45};      // 五炸弹

//        int[] cards20 = {0x02, 0x02, 0x02, 0x0e, 0x0e, 0x0e}; // A-A-A-2-2-2
//        int[] cards21 = {0x03, 0x03, 0x03, 0x02, 0x02, 0x02}; // 3-3-3-4-4-4

//        System.out.println(compareCardStrength(cards20, cards21, 3, CardType.TRIPLE_SEQUENCE, 0));
        int[] cards = {0x22, 0x23, 0x26, 0x45, 0x45}; // 红桃5-6-7-8-赖子
        System.out.println(determineCardType(cards, 0x45,null)); // FIVE_BOMB
    }
}