package com.incubator.game.util;

import cn.hutool.json.JSONArray;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Author:  xhb
 * Date:  2024-12-17 12:32
 */
public class JsonToMapUtil {
    public static List<Map<String, Object>> parseJsonStrToMapList(String jsonArrayStr) {
        JSONArray jsonArray = JSONUtil.parseArray(jsonArrayStr);
        List<Map<String, Object>> resultList = new ArrayList<>();

        for (Object obj : jsonArray) {
            JSONObject jsonObject = (JSONObject) obj;
            Map<String, Object> resultMap = parseJsonObject(jsonObject);
            resultList.add(resultMap);
        }

        return resultList;
    }

    public static List<Map<String, Object>> parseJsonArrayToMapList( JSONArray jsonArray ) {
        List<Map<String, Object>> resultList = new ArrayList<>();

        for (Object obj : jsonArray) {
            JSONObject jsonObject = (JSONObject) obj;
            Map<String, Object> resultMap = parseJsonObject(jsonObject);
            resultList.add(resultMap);
        }

        return resultList;
    }

    private static Map<String, Object> parseJsonObject(JSONObject jsonObject) {
        Map<String, Object> resultMap = new HashMap<>();

        for (String key : jsonObject.keySet()) {
            Object value = jsonObject.get(key);
            if (value instanceof JSONArray) {
                value = parseJsonArray((JSONArray) value);
            } else if (value instanceof JSONObject) {
                value = parseJsonObject((JSONObject) value);
            }
            resultMap.put(key, value);
        }

        return resultMap;
    }

    private static List<Object> parseJsonArray(JSONArray jsonArray) {
        List<Object> resultList = new ArrayList<>();

        for (Object obj : jsonArray) {
            if (obj instanceof JSONObject) {
                resultList.add(parseJsonObject((JSONObject) obj));
            } else {
                resultList.add(obj);
            }
        }

        return resultList;
    }






    public static Map<String, Object> jsonToMap(String jsonString) {
        JSONObject jsonObject = new JSONObject(jsonString);
        return jsonToMap(jsonObject);
    }

    private static Map<String, Object> jsonToMap(JSONObject jsonObject) {
        Map<String, Object> resultMap = new HashMap<>();

        for (String key : jsonObject.keySet()) {
            Object value = jsonObject.get(key);

            if (value == null) {
                resultMap.put(key, null);
            } else if (value instanceof JSONObject) {
                resultMap.put(key, jsonToMap((JSONObject) value));
            } else if (value instanceof JSONArray) {
                resultMap.put(key, jsonArrayToMapList((JSONArray) value));
            } else {
                resultMap.put(key, value);
            }
        }

        return resultMap;
    }

    private static List<Map<String, Object>> jsonArrayToMapList(JSONArray jsonArray) {
        List<Map<String, Object>> resultList = new ArrayList<>();

        for (Object obj : jsonArray) {
            if (obj instanceof JSONObject) {
                resultList.add(jsonToMap((JSONObject) obj));
            }
        }

        return resultList;
    }
}
