package com.incubator.game.util;
import java.lang.reflect.Field;
import java.util.Map;

/**
 * Author:  xhb
 * Date:  2024-12-25 10:25
 */
public class MapToObject {

    public static <T> T mapToObject(Map<String, String> map, Class<T> clazz) {
        try {
            T obj = clazz.newInstance();
            for (Field field : clazz.getDeclaredFields()) {
                field.setAccessible(true);
                String key = field.getName();
                if (map.containsKey(key)) {
                    String value = map.get(key);
                    Class<?> fieldType = field.getType();
                    if (fieldType == String.class) {
                        field.set(obj, value);
                    } else if (fieldType == int.class || fieldType == Integer.class) {
                        field.setInt(obj, Integer.parseInt(value));
                    } else if (fieldType == long.class || fieldType == Long.class) {
                        field.setLong(obj, Long.parseLong(value));
                    } else if (fieldType == double.class || fieldType == Double.class) {
                        field.setDouble(obj, Double.parseDouble(value));
                    } else if (fieldType == boolean.class || fieldType == Boolean.class) {
                        field.setBoolean(obj, Boolean.parseBoolean(value));
                    }
                }
            }
            return obj;
        } catch (InstantiationException | IllegalAccessException e) {
            e.printStackTrace();
            return null;
        }
    }
}
