package com.incubator.game.util;

import com.incubator.game.contest.Contest;
import com.incubator.game.contest.ContestService;
import com.incubator.game.player.Player;
import com.incubator.game.room.GDRoom;
import com.incubator.game.room.Room;

import java.util.*;

/**
 * 消息工具类
 */
public final class ProtoUtil {

    /**
     * 玩家信息转消息
     *
     * @param player
     * @return
     */
    public static Map<String, Object> playerInfoToMessage(Player player) {
        Map<String, Object> data = new HashMap<>();
        data.put("id", player.getId());
        data.put("name", player.data.name);
        data.put("avatar", player.data.avatarUrl);
        data.put("sex", player.data.sex);
        data.put("exp", player.data.exp);
        data.put("maxExp", player.data.maxExp);
        data.put("vipLevel", player.data.vipLevel);
        data.put("masterScore", player.data.masterScore);
        data.put("masterScoreLevel", player.data.masterScoreLevel);
        data.put("gold", player.data.gold);
        data.put("diamond", player.data.diamond);
        data.put("startTimeVip", player.data.startTimeVip);
        data.put("endTimeVip", player.data.endTimeVip);
        data.put("gps", player.data.gps);
        data.put("gpsAddr", player.data.gpsAddr);
        data.put("foca", player.data.foca);
        data.put("ranking", player.data.ranking);
        data.put("figureId", player.data.figureId);
        data.put("realName", player.data.realName);
        data.put("phoneNum", player.data.phoneNum);
        data.put("idCard", player.data.idCard);
        data.put("friendMasterNum", player.data.friendMasterNum);
        data.put("friendMasterRate", player.data.friendMasterRate);
        data.put("tradPoints", player.data.tradPoints);
        data.put("tradPointsRate", player.data.tradPointsRate);
        data.put("tradMatchNum", player.data.tradMatchNum);
        data.put("tradMatchRate", player.data.tradMatchRate);
        data.put("noReshNum", player.data.noReshNum);
        data.put("noReshRate", player.data.noReshRate);
        return data;
    }

    /**
     * 规则信息转消息
     *
     * @param gameRule 游戏规则
     * @return
     */
    public static Map<String, Object> gameRuleToMessage(GMStruct.GameRule gameRule) {
        Map<String, Object> data = new HashMap<>();
        // 1.掼蛋
        data.put("gameType", 1);
        // 玩法1 1:连续打，2:按局数，3:按时间
        data.put("playingTypeOne", new int[]{1,2,3});
        // 玩法2 1:A必打 2:J必打 3:8必打 4:5必打
        data.put("playingTypeTwo", new int[]{1,2,3,4});
        // 升级规则 1:123级，2:124级，3:135级，4:246级
        data.put("upgradeType", new int[]{1,2,3,4});
        // 翻山规则 1:有头游无末游过A结束，2:有头游过A结束，3:3局不过A判负结束，4:3局不过A从2开始
        data.put("flippingType", new int[]{1,2,3,4});
        // 进贡规则 1:进贡，2:不进贡（头游先出）
        data.put("advanceType", new int[]{1,2});
        // 超时规则 1:过牌 2:待机 3:托管
        data.put("timeoutType", new int[]{1,2,3});
        // 语音规则 1允许 2禁用
        data.put("voiceType", new int[]{1,2});
        // 记牌器 1允许 2禁用
        data.put("recordType", new int[]{1,2});
        // 观看规则 1第一视角 2上帝视角 3上帝&第一视角 4禁用
        data.put("watchType", new int[]{1,2,3,4});
        // 人数模式 1:2人 2:4人
        data.put("playerNum", new int[]{1,2});
        return data;
    }

    /**
     * 官方比赛集合转消息
     *
     * @return
     */
    public static List<Object> contestListToMessage() {
        List<Object> list = new ArrayList<>();
        for (Contest contest : ContestService.getInstance().contestMap.values()) {
            if (contest != null) {
                list.add(ProtoUtil.contestInfoToMessage(contest));
            }
        }
        return list;
    }

    /**
     * 官方比赛运行状态集合转消息
     *
     * @return
     */
    public static List<Object> contestListByRunToMessage() {
        List<Object> list = new ArrayList<>();
        for (Contest contest : ContestService.getInstance().contestMap.values()) {
            if (contest != null&&contest.prop.runStatus.equals("Y")) {
                list.add(ProtoUtil.contestInfoToMessage(contest));
            }
        }
        return list;
    }


    /**
     * 官方比赛集合转消息
     *
     * @return
     */
    public static Map<String, Object> contestInfoToMessage(Contest contest) {
        Map<String, Object> data = new HashMap<>();
        if (contest != null) {
            data.put("contestId", contest.data.contestId);
            data.put("contestName", contest.prop.desc);
            data.put("contestType", contest.data.contestType);
            data.put("contestPlayType", contest.prop.gameType);
            data.put("contestLevel", contest.data.level);
            data.put("contestTime", contest.data.startTime);
            data.put("contestState", contest.state);
            data.put("contestPhotoUrl", contest.prop.url);
//            data.put("regisCount", 0);
            data.put("regisCount", contest.actors.size());
            data.put("regisMoney", "");
            data.put("regisMoneyType", "");
            data.put("regisList", ProtoUtil.contestRegisListToMessage(contest));
        }
        return data;
    }

    /**
     * 官方比赛报名列表
     *
     * @param contest
     * @return
     */
    public static List<Map<String, Object>> contestRegisListToMessage(Contest contest) {
        List<Map<String, Object>> regisList = new ArrayList<>();
        if (!contest.actors.isEmpty()) {
            Map<String, Object> playerInfo;
            for (Player player : contest.actors.values()) {
                if (player == null) {
                    continue;
                }
                playerInfo = new HashMap<>();
                playerInfo.put("id", player.getId());
                playerInfo.put("name", player.data.name);
                playerInfo.put("avatar", "https://img95.699pic.com/element/40109/0194.png_300.png");
                playerInfo.put("sex", player.data.sex);
                playerInfo.put("masterScore", player.data.masterScore);
                playerInfo.put("ranking", player.data.ranking);
                regisList.add(playerInfo);
            }
        }
        return regisList;
    }

    /**
     * 活动列表
     * @param eventArray
     * @return
     */
    public static List<Map<String, Object>> eventToJsonArray(String eventArray) {
//        List<Map<String, Object>> list = new ArrayList<>();
//        for (Object o : eventArray) {
//            Map<String, Object> objectObjectHashMap = new HashMap<>();
//            cn.hutool.json.JSONObject object = (cn.hutool.json.JSONObject)o;
//            String type = (String)object.get("type");
//            objectObjectHashMap.put("type", type);
//
//            List<Object> objectList = new ArrayList<>();
//            JSONArray objects = object.getJSONArray("reward");
//            for (Object object1 : objects) {
//                Map<String, Object> objectObjectHashMap1 = new HashMap<>();
//                cn.hutool.json.JSONObject object2 = (cn.hutool.json.JSONObject)object1;
//                String eventId =(String) object2.get("eventId");
//                String eventName =(String) object2.get("eventName");
//                String eventType =(String) object2.get("eventType");
//                objectObjectHashMap1.put("eventId",eventId);
//                objectObjectHashMap1.put("eventName",eventName);
//                objectObjectHashMap1.put("eventType",eventType);
//
//                JSONArray objects1 = object2.getJSONArray("rewardList");
//                List<Object> objects2 = new ArrayList<>();
//                for (Object o1 : objects1) {
//                    HashMap<String, Object> stringObjectHashMap = new HashMap<>();
//                    cn.hutool.json.JSONObject o11 = (cn.hutool.json.JSONObject) o1;
//                    String periodNum = (String)o11.get("periodNum");
//                    String winProbability = (String)o11.get("winProbability");
//                    String goodsId = (String)o11.get("goodsId");
//                    String goodsName = (String)o11.get("goodsName");
//                    String baseReward = (String)o11.get("baseReward");
//                    String goodsImg = (String)o11.get("goodsImg");
//                    String lookEventRewardNum = (String)o11.get("lookEventRewardNum");
//                    String shareExtendRewardNum = (String)o11.get("shareExtendRewardNum");
//                    stringObjectHashMap.put("periodNum", periodNum);
//                    stringObjectHashMap.put("winProbability", winProbability);
//                    stringObjectHashMap.put("goodsId", goodsId);
//                    stringObjectHashMap.put("goodsName", goodsName);
//                    stringObjectHashMap.put("baseReward", baseReward);
//                    stringObjectHashMap.put("goodsImg", goodsImg);
//                    stringObjectHashMap.put("lookEventRewardNum", lookEventRewardNum);
//                    stringObjectHashMap.put("shareExtendRewardNum", shareExtendRewardNum);
//                    objects2.add(stringObjectHashMap);
//                }
//                objectObjectHashMap1.put("rewardList",objects2);
//                objectList.add(objectObjectHashMap1);
//            }
//            objectObjectHashMap.put("reward", objectList);
//
//            list.add(objectObjectHashMap);
//        }
        return JsonToMapUtil.parseJsonStrToMapList(eventArray);
    }
}
