package com.incubator.game.util;

import com.alibaba.fastjson2.JSONObject;

/**
 * 房间工具类
 *
 * @author johnc
 */
public final class RoomUtil {

    /**
     * 掼蛋房间规则
     *
     * @param type
     * @param mode
     * @param modev
     * @param lv
     * @param vic
     * @param jing
     * @param lp
     * @return
     */
    public static JSONObject gdRoomRule(int type, int mode, int modev, int lv, int vic, int jing, int lp) {
        JSONObject jsonData = new JSONObject();

        // 1掼蛋 2转蛋 3复式掼蛋
        jsonData.put("gameType", type);
        // 玩法1 1:连续打，2:按局数，3:按时间
        jsonData.put("mode", mode);

        switch (mode) {
            case 1: // 连续打
                // 玩法2 14:A必打 2:J必打 3:8必打 4:5必打
                jsonData.put("modev", modev);
                // 胜利规则 1:有头游无末游过A结束，2:有头游过A结束，3:3局不过A判负结束，4:3局不过A从2开始
                jsonData.put("flippingType", vic);
                break;
            case 2: // 按局数
                // 玩法2 4:4局 6:6局 8:8局 10:10局
                jsonData.put("modev", modev);
                // 套圈规则 1:套圈，2:不套圈
                jsonData.put("ferrule", lp);
                // 胜利规则 1:头游，2:有头游过A结束，3:3局不过A判负结束，4:3局不过A从2开始
                jsonData.put("flippingType", vic);
                break;
            case 3: // 按时间
                // 玩法2 20:20分钟 30:30分钟 40:40分钟 60:60分钟
                jsonData.put("modev", modev);
                // 套圈规则 1:套圈，2:不套圈
                jsonData.put("ferrule", lp);
                // 胜利规则 1:头游，2:有头游过A结束，3:3局不过A判负结束，4:3局不过A从2开始
                jsonData.put("flippingType", vic);
                break;
        }

        // 进贡规则 0:进贡，1:不进贡（头游先出）
        jsonData.put("advanceType", jing);
        // 升级规则 1:123级，2:124级，3:135级，4:246级
        jsonData.put("upgradeType", lv);


        return jsonData;
    }

    /**
     * 转蛋房间规则
     *
     * @param type
     * @param mode
     * @param modev
     * @param lv
     * @param team
     * @param tc
     * @param dv
     * @param pv
     * @return
     */
    public static JSONObject zdRoomRule(int type, int mode, int modev, int lv, int team, int tc, int dv, int pv) {
        JSONObject jsonData = new JSONObject();

        // 1掼蛋 2转蛋 3复式掼蛋
        jsonData.put("gameType", type);
        // 玩法1 2:按局数，3:按时间
        jsonData.put("mode", mode);
        // 玩法2 4:4局 6:6局 8:8局 10:10局
        jsonData.put("modev", modev);

        // 队友 0固定队友 1随机搭档 2均匀分配
        jsonData.put("team", team);
        // 级牌 0随机打几 1固定打2
        jsonData.put("tc", tc);
        // 翻倍 0不翻倍 1加倍 2翻倍
        jsonData.put("dv", dv);
        // 积分 0:123 1:124
        jsonData.put("pv", pv);

        return jsonData;
    }

    /**
     * 复式掼蛋房间规则
     *
     * @param type
     * @param mode
     * @param modev
     * @param lv
     * @param vic
     * @param lp
     * @param tc
     * @return
     */
    public static JSONObject fsgdRoomRule(int type, int mode, int modev, int lv, int vic, int lp, int tc) {
        JSONObject jsonData = new JSONObject();

        // 1掼蛋 2转蛋 3复式掼蛋
        jsonData.put("gameType", type);
        // 玩法1 1:连续打，2:按局数，3:按时间
        jsonData.put("mode", mode);
        // 玩法2 14:A必打 2:J必打 3:8必打 4:5必打
        jsonData.put("playingTypeTwo", modev);


        // 升级规则 1:123级，2:124级，3:135级，4:246级
        jsonData.put("upgradeType", lv);
        // 翻山规则 1:有头游无末游过A结束，2:有头游过A结束，3:3局不过A判负结束，4:3局不过A从2开始
        jsonData.put("flippingType", vic);
        // 进贡规则 0:进贡，1:不进贡（头游先出）
//        jsonData.put("advanceType", jing);

        return jsonData;
    }

}
